@@ -391,34 +391,6 @@ BOOL WINAPI GetOldestEventLogRecord( HANDLE hEventLog, PDWORD OldestRecord )
     return TRUE;
 }
 
-/******************************************************************************
- * GetTraceEnableFlags [ADVAPI32.@]
- */
-ULONG WINAPI GetTraceEnableFlags( TRACEHANDLE handle )
-{
-    FIXME("(%s) stub\n", wine_dbgstr_longlong(handle));
-    return 0;
-}
-
-/******************************************************************************
- * GetTraceEnableLevel [ADVAPI32.@]
- */
-UCHAR WINAPI GetTraceEnableLevel( TRACEHANDLE handle )
-{
-    FIXME("(%s) stub\n", wine_dbgstr_longlong(handle));
-    return TRACE_LEVEL_VERBOSE;
-}
-
-/******************************************************************************
- * GetTraceLoggerHandle [ADVAPI32.@]
- */
-TRACEHANDLE WINAPI GetTraceLoggerHandle( PVOID buf )
-{
-    FIXME("(%p) stub\n", buf);
-    SetLastError(ERROR_ACCESS_DENIED);
-    return INVALID_PROCESSTRACE_HANDLE;
-}
-
 /******************************************************************************
  * NotifyChangeEventLog [ADVAPI32.@]
  *
@@ -603,7 +575,7 @@ BOOL WINAPI ReadEventLogA( HANDLE hEventLog, DWORD dwReadFlags, DWORD dwRecordOf
     FIXME("(%p,0x%08x,0x%08x,%p,0x%08x,%p,%p) stub\n", hEventLog, dwReadFlags,
           dwRecordOffset, lpBuffer, nNumberOfBytesToRead, pnBytesRead, pnMinNumberOfBytesNeeded);
 
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    SetLastError(ERROR_HANDLE_EOF);
     return FALSE;
 }
 
@@ -618,7 +590,7 @@ BOOL WINAPI ReadEventLogW( HANDLE hEventLog, DWORD dwReadFlags, DWORD dwRecordOf
     FIXME("(%p,0x%08x,0x%08x,%p,0x%08x,%p,%p) stub\n", hEventLog, dwReadFlags,
           dwRecordOffset, lpBuffer, nNumberOfBytesToRead, pnBytesRead, pnMinNumberOfBytesNeeded);
 
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    SetLastError(ERROR_HANDLE_EOF);
     return FALSE;
 }
 
@@ -805,48 +777,6 @@ ULONG WINAPI StopTraceA( TRACEHANDLE session, LPCSTR session_name, PEVENT_TRACE_
     return ERROR_SUCCESS;
 }
 
-/******************************************************************************
- * TraceEvent [ADVAPI32.@]
- */
-ULONG WINAPI TraceEvent( TRACEHANDLE SessionHandle, PEVENT_TRACE_HEADER EventTrace )
-{
-    FIXME("%s %p\n", wine_dbgstr_longlong(SessionHandle), EventTrace);
-    return ERROR_CALL_NOT_IMPLEMENTED;
-}
-
-/******************************************************************************
- * EventProviderEnabled [ADVAPI32.@]
- *
- */
-BOOLEAN WINAPI EventProviderEnabled( REGHANDLE handle, UCHAR level, ULONGLONG keyword )
-{
-    FIXME("%s, %u, %s: stub\n", wine_dbgstr_longlong(handle), level, wine_dbgstr_longlong(keyword));
-    return FALSE;
-}
-
-/******************************************************************************
- * EventActivityIdControl [ADVAPI32.@]
- *
- */
-ULONG WINAPI EventActivityIdControl(ULONG code, GUID *guid)
-{
-    static int once;
-
-    if (!once++) FIXME("0x%x, %p: stub\n", code, guid);
-    return ERROR_SUCCESS;
-}
-
-/******************************************************************************
- * EventWriteTransfer [ADVAPI32.@]
- */
-ULONG WINAPI EventWriteTransfer( REGHANDLE handle, PCEVENT_DESCRIPTOR descriptor, LPCGUID activity,
-                                 LPCGUID related, ULONG count, PEVENT_DATA_DESCRIPTOR data )
-{
-    FIXME("%s, %p, %s, %s, %u, %p: stub\n", wine_dbgstr_longlong(handle), descriptor,
-          debugstr_guid(activity), debugstr_guid(related), count, data);
-    return ERROR_SUCCESS;
-}
-
 /******************************************************************************
  * QueryTraceW [ADVAPI32.@]
  */
@@ -889,30 +819,6 @@ ULONG WINAPI ProcessTrace( PTRACEHANDLE HandleArray, ULONG HandleCount, LPFILETI
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-/******************************************************************************
- * TraceMessage [ADVAPI32.@]
- */
-ULONG WINAPIV TraceMessage( TRACEHANDLE handle, ULONG flags, LPGUID guid, USHORT number, ... )
-{
-    __ms_va_list valist;
-    ULONG ret;
-
-    __ms_va_start( valist, number );
-    ret = TraceMessageVa( handle, flags, guid, number, valist );
-    __ms_va_end( valist );
-    return ret;
-}
-
-/******************************************************************************
- * TraceMessageVa [ADVAPI32.@]
- */
-ULONG WINAPI TraceMessageVa( TRACEHANDLE handle, ULONG flags, LPGUID guid, USHORT number,
-                            __ms_va_list args )
-{
-    FIXME("(%s %x %s %d) : stub\n", wine_dbgstr_longlong(handle), flags, debugstr_guid(guid), number);
-    return ERROR_SUCCESS;
-}
-
 /******************************************************************************
  * CloseTrace [ADVAPI32.@]
  */
