@@ -29,6 +29,7 @@
 #include "winbase.h"
 #include "winreg.h"
 #include "winternl.h"
+#include "sddl.h"
 #include "advapi32_misc.h"
 
 #include "wine/debug.h"
@@ -44,6 +45,12 @@ WINE_DEFAULT_DEBUG_CHANNEL(advapi);
         return FailureCode; \
 }
 
+static LPCSTR debugstr_us( const UNICODE_STRING *us )
+{
+    if (!us) return "(null)";
+    return debugstr_wn(us->Buffer, us->Length / sizeof(WCHAR));
+}
+
 static void dumpLsaAttributes(const LSA_OBJECT_ATTRIBUTES *oa)
 {
     if (oa)
@@ -480,6 +487,7 @@ NTSTATUS WINAPI LsaLookupSids(
     WCHAR *name_buffer;
     char *domain_data;
     SID_NAME_USE use;
+    WCHAR *strsid;
 
     TRACE("(%p, %u, %p, %p, %p)\n", PolicyHandle, Count, Sids, ReferencedDomains, Names);
 
@@ -502,7 +510,7 @@ NTSTATUS WINAPI LsaLookupSids(
     {
         (*Names)[i].Use = SidTypeUnknown;
         (*Names)[i].DomainIndex = -1;
-        (*Names)[i].Name.Buffer = NULL;
+        RtlInitUnicodeStringEx(&(*Names)[i].Name, NULL);
 
         memset(&(*ReferencedDomains)->Domains[i], 0, sizeof(LSA_TRUST_INFORMATION));
 
@@ -555,6 +563,14 @@ NTSTATUS WINAPI LsaLookupSids(
                 domain.MaximumLength = sizeof(WCHAR);
             }
         }
+        else if (ConvertSidToStringSidW(Sids[i], &strsid))
+        {
+            (*Names)[i].Name.Length = strlenW(strsid) * sizeof(WCHAR);
+            (*Names)[i].Name.MaximumLength = (strlenW(strsid) + 1) * sizeof(WCHAR);
+            name_fullsize += (strlenW(strsid) + 1) * sizeof(WCHAR);
+
+            LocalFree(strsid);
+        }
     }
 
     /* now we have full length needed for both */
@@ -571,6 +587,8 @@ NTSTATUS WINAPI LsaLookupSids(
     {
         name_size = domain_size = 0;
 
+        (*Names)[i].Name.Buffer = name_buffer;
+
         if (!LookupAccountSidW(NULL, Sids[i], NULL, &name_size, NULL, &domain_size, &use) &&
             GetLastError() == ERROR_INSUFFICIENT_BUFFER)
         {
@@ -590,15 +608,19 @@ NTSTATUS WINAPI LsaLookupSids(
 
             domain.Buffer = heap_alloc(domain.MaximumLength);
 
-            (*Names)[i].Name.Buffer = name_buffer;
             LookupAccountSidW(NULL, Sids[i], (*Names)[i].Name.Buffer, &name_size, domain.Buffer, &domain_size, &use);
             (*Names)[i].Use = use;
 
             (*Names)[i].DomainIndex = lsa_reflist_add_domain(*ReferencedDomains, &domain, &domain_data);
             heap_free(domain.Buffer);
         }
+        else if (ConvertSidToStringSidW(Sids[i], &strsid))
+        {
+            strcpyW((*Names)[i].Name.Buffer, strsid);
+            LocalFree(strsid);
+        }
 
-        name_buffer += name_size;
+        name_buffer += strlenW(name_buffer) + 1;
     }
     TRACE("mapped %u out of %u\n", mapped, Count);
 
@@ -757,16 +779,64 @@ NTSTATUS WINAPI LsaQueryInformationPolicy(
         break;
         case  PolicyDnsDomainInformation:	/* 12 (0xc) */
         {
-            /* Only the domain name is valid for the local computer.
-             * All other fields are zero.
-             */
-            PPOLICY_DNS_DOMAIN_INFO pinfo;
+            struct
+            {
+                POLICY_DNS_DOMAIN_INFO info;
+                struct
+                {
+                    SID sid;
+                    DWORD sid_subauthority[3];
+                } domain_sid;
+                WCHAR domain_name[256];
+                WCHAR dns_domain_name[256];
+                WCHAR dns_forest_name[256];
+            } *xdi;
+            struct
+            {
+                SID sid;
+                DWORD sid_subauthority[3];
+            } computer_sid;
+            DWORD dwSize;
 
-            pinfo = ADVAPI_GetDomainName(sizeof(*pinfo), offsetof(POLICY_DNS_DOMAIN_INFO, Name));
+            xdi = heap_alloc_zero(sizeof(*xdi));
+            if (!xdi) return STATUS_NO_MEMORY;
 
-            TRACE("setting domain to %s\n", debugstr_w(pinfo->Name.Buffer));
+            dwSize = 256;
+            if (GetComputerNameExW(ComputerNamePhysicalDnsDomain, xdi->domain_name, &dwSize))
+            {
+                WCHAR *dot;
 
-            *Buffer = pinfo;
+                dot = strrchrW(xdi->domain_name, '.');
+                if (dot) *dot = 0;
+                struprW(xdi->domain_name);
+                xdi->info.Name.Buffer = xdi->domain_name;
+                xdi->info.Name.Length = strlenW(xdi->domain_name) * sizeof(WCHAR);
+                xdi->info.Name.MaximumLength = xdi->info.Name.Length + sizeof(WCHAR);
+                TRACE("setting Name to %s\n", debugstr_w(xdi->info.Name.Buffer));
+            }
+
+            dwSize = 256;
+            if (GetComputerNameExW(ComputerNameDnsDomain, xdi->dns_domain_name, &dwSize))
+            {
+                xdi->info.DnsDomainName.Buffer = xdi->dns_domain_name;
+                xdi->info.DnsDomainName.Length = dwSize * sizeof(WCHAR);
+                xdi->info.DnsDomainName.MaximumLength = (dwSize + 1) * sizeof(WCHAR);
+                TRACE("setting DnsDomainName to %s\n", debugstr_w(xdi->info.DnsDomainName.Buffer));
+
+                xdi->info.DnsForestName.Buffer = xdi->dns_domain_name;
+                xdi->info.DnsForestName.Length = dwSize * sizeof(WCHAR);
+                xdi->info.DnsForestName.MaximumLength = (dwSize + 1) * sizeof(WCHAR);
+                TRACE("setting DnsForestName to %s\n", debugstr_w(xdi->info.DnsForestName.Buffer));
+            }
+
+            dwSize = sizeof(xdi->domain_sid);
+            if (ADVAPI_GetComputerSid(&computer_sid.sid) && GetWindowsAccountDomainSid(&computer_sid.sid, &xdi->domain_sid.sid, &dwSize))
+            {
+                xdi->info.Sid = &xdi->domain_sid.sid;
+                TRACE("setting SID to %s\n", debugstr_sid(&xdi->domain_sid.sid));
+            }
+
+            *Buffer = xdi;
         }
         break;
         case  PolicyAuditLogInformation:
@@ -978,11 +1048,55 @@ NTSTATUS WINAPI LsaUnregisterPolicyChangeNotification(
  * LsaLookupPrivilegeName [ADVAPI32.@]
  *
  */
-NTSTATUS WINAPI LsaLookupPrivilegeName(
-    LSA_HANDLE handle,
-    LUID *luid,
-    UNICODE_STRING **name)
+NTSTATUS WINAPI LsaLookupPrivilegeName(LSA_HANDLE handle, LUID *luid, LSA_UNICODE_STRING **name)
 {
-    FIXME("(%p,%p,%p) stub\n", handle, luid, name);
+    const WCHAR *privnameW;
+    DWORD length;
+    WCHAR *strW;
+
+    TRACE("(%p,%p,%p)\n", handle, luid, name);
+
+    if (!luid || !handle)
+        return STATUS_INVALID_PARAMETER;
+
+    *name = NULL;
+
+    if (!(privnameW = get_wellknown_privilege_name(luid)))
+        return STATUS_NO_SUCH_PRIVILEGE;
+
+    length = strlenW(privnameW);
+    *name = heap_alloc(sizeof(**name) + (length + 1) * sizeof(WCHAR));
+    if (!*name)
+        return STATUS_NO_MEMORY;
+
+    strW = (WCHAR *)(*name + 1);
+    memcpy(strW, privnameW, length * sizeof(WCHAR));
+    strW[length] = 0;
+    RtlInitUnicodeString(*name, strW);
+
+    return STATUS_SUCCESS;
+}
+
+/******************************************************************************
+ * LsaLookupPrivilegeDisplayName [ADVAPI32.@]
+ *
+ */
+NTSTATUS WINAPI LsaLookupPrivilegeDisplayName(LSA_HANDLE handle, LSA_UNICODE_STRING *name,
+    LSA_UNICODE_STRING **display_name, SHORT *language)
+{
+    FIXME("(%p, %s, %p, %p)\n", handle, debugstr_us(name), display_name, language);
+
     return STATUS_NO_SUCH_PRIVILEGE;
 }
+
+/******************************************************************************
+ * AuditQuerySystemPolicy [ADVAPI32.@]
+ *
+ */
+BOOLEAN WINAPI AuditQuerySystemPolicy(const GUID* guids, ULONG count, AUDIT_POLICY_INFORMATION** policy)
+{
+
+    FIXME("(%p, %d, %p)\n", guids, count, policy);
+    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    return FALSE;
+}
