@@ -26,20 +26,11 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(amstream);
 
-static const WCHAR sink_id[] = {'I','{','A','3','5','F','F','5','6','A',
-        '-','9','F','D','A','-','1','1','D','0','-','8','F','D','F',
-        '-','0','0','C','0','4','F','D','9','1','8','9','D','}',0};
+static const WCHAR sink_id[] = L"I{A35FF56A-9FDA-11D0-8FDF-00C04FD9189D}";
 
 static HRESULT ddrawstreamsample_create(IDirectDrawMediaStream *parent, IDirectDrawSurface *surface,
     const RECT *rect, IDirectDrawStreamSample **ddraw_stream_sample);
 
-struct ddraw_stream;
-
-typedef struct {
-    BaseInputPin pin;
-    struct ddraw_stream *parent;
-} DirectDrawMediaStreamInputPin;
-
 struct ddraw_stream
 {
     IAMMediaStream IAMMediaStream_iface;
@@ -751,7 +742,7 @@ static HRESULT WINAPI ddraw_sink_QueryPinInfo(IPin *iface, PIN_INFO *info)
 
     IBaseFilter_AddRef(info->pFilter = (IBaseFilter *)stream->filter);
     info->dir = PINDIR_INPUT;
-    lstrcpyW(info->achName, sink_id);
+    wcscpy(info->achName, sink_id);
 
     return S_OK;
 }
@@ -770,7 +761,7 @@ static HRESULT WINAPI ddraw_sink_QueryId(IPin *iface, WCHAR **id)
     if (!(*id = CoTaskMemAlloc(sizeof(sink_id))))
         return E_OUTOFMEMORY;
 
-    lstrcpyW(*id, sink_id);
+    wcscpy(*id, sink_id);
 
     return S_OK;
 }
