@@ -33,7 +33,6 @@
 #include "avifile_private.h"
 #include "extrachunk.h"
 
-#include "wine/unicode.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(avifile);
@@ -575,7 +574,7 @@ static HRESULT WINAPI IPersistFile_fnGetCurFile(IPersistFile *iface, LPOLESTR *p
     if (*ppszFileName == NULL)
       return AVIERR_MEMORY;
 
-    strcpyW(*ppszFileName, This->szFileName);
+    lstrcpyW(*ppszFileName, This->szFileName);
   }
 
   return AVIERR_OK;
