@@ -237,7 +237,7 @@ DATETIME_SetSystemTime (DATETIME_INFO *infoPtr, DWORD flag, const SYSTEMTIME *sy
  * *'dddddd' is handled as 'dddd' plus 'dd'.
  * *unrecognized formats are strings (here given the type DT_STRING;
  * start of the string is encoded in lower bits of DT_STRING.
- * Therefore, 'string' ends finally up as '<show seconds>tring'.
+ * Therefore, 'string' ends up as '<show seconds>tring'.
  *
  */
 static void
@@ -472,10 +472,10 @@ DATETIME_ReturnTxt (const DATETIME_INFO *infoPtr, int count, LPWSTR result, int
 	    result[1] = 0;
 	    break;
 	case ONEDIGITYEAR:
-	    wsprintfW (result, fmt_dW, date.wYear-10* (int) floor(date.wYear/10));
+	    wsprintfW (result, fmt_dW, date.wYear % 10);
 	    break;
 	case TWODIGITYEAR:
-	    wsprintfW (result, fmt__2dW, date.wYear-100* (int) floor(date.wYear/100));
+	    wsprintfW (result, fmt__2dW, date.wYear % 100);
 	    break;
         case INVALIDFULLYEAR:
 	case FULLYEAR:
