@@ -17,9 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include "d2d1_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d2d);
@@ -45,13 +42,8 @@ static HRESULT STDMETHODCALLTYPE d2d_bitmap_render_target_QueryInterface(ID2D1Bi
         *out = iface;
         return S_OK;
     }
-    else if (IsEqualGUID(iid, &IID_ID2D1GdiInteropRenderTarget))
-        return ID2D1RenderTarget_QueryInterface(render_target->dxgi_target, iid, out);
 
-    WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(iid));
-
-    *out = NULL;
-    return E_NOINTERFACE;
+    return IUnknown_QueryInterface(render_target->dxgi_inner, iid, out);
 }
 
 static ULONG STDMETHODCALLTYPE d2d_bitmap_render_target_AddRef(ID2D1BitmapRenderTarget *iface)
@@ -73,8 +65,9 @@ static ULONG STDMETHODCALLTYPE d2d_bitmap_render_target_Release(ID2D1BitmapRende
 
     if (!refcount)
     {
-        ID2D1RenderTarget_Release(render_target->dxgi_target);
-        ID2D1Bitmap_Release(render_target->bitmap);
+        IUnknown_Release(render_target->dxgi_inner);
+        if (render_target->bitmap)
+            ID2D1Bitmap_Release(render_target->bitmap);
         heap_free(render_target);
     }
 
@@ -223,8 +216,8 @@ static void STDMETHODCALLTYPE d2d_bitmap_render_target_DrawLine(ID2D1BitmapRende
 {
     struct d2d_bitmap_render_target *render_target = impl_from_ID2D1BitmapRenderTarget(iface);
 
-    TRACE("iface %p, p0 {%.8e, %.8e}, p1 {%.8e, %.8e}, brush %p, stroke_width %.8e, stroke_style %p.\n",
-            iface, p0.x, p0.y, p1.x, p1.y, brush, stroke_width, stroke_style);
+    TRACE("iface %p, p0 %s, p1 %s, brush %p, stroke_width %.8e, stroke_style %p.\n",
+            iface, debug_d2d_point_2f(&p0), debug_d2d_point_2f(&p1), brush, stroke_width, stroke_style);
 
     ID2D1RenderTarget_DrawLine(render_target->dxgi_target, p0, p1, brush, stroke_width, stroke_style);
 }
@@ -369,8 +362,8 @@ static void STDMETHODCALLTYPE d2d_bitmap_render_target_DrawTextLayout(ID2D1Bitma
 {
     struct d2d_bitmap_render_target *render_target = impl_from_ID2D1BitmapRenderTarget(iface);
 
-    TRACE("iface %p, origin {%.8e, %.8e}, layout %p, brush %p, options %#x.\n",
-            iface, origin.x, origin.y, layout, brush, options);
+    TRACE("iface %p, origin %s, layout %p, brush %p, options %#x.\n",
+            iface, debug_d2d_point_2f(&origin), layout, brush, options);
 
     ID2D1RenderTarget_DrawTextLayout(render_target->dxgi_target, origin, layout, brush, options);
 }
@@ -381,8 +374,8 @@ static void STDMETHODCALLTYPE d2d_bitmap_render_target_DrawGlyphRun(ID2D1BitmapR
 {
     struct d2d_bitmap_render_target *render_target = impl_from_ID2D1BitmapRenderTarget(iface);
 
-    TRACE("iface %p, baseline_origin {%.8e, %.8e}, glyph_run %p, brush %p, measuring_mode %#x.\n",
-            iface, baseline_origin.x, baseline_origin.y, glyph_run, brush, measuring_mode);
+    TRACE("iface %p, baseline_origin %s, glyph_run %p, brush %p, measuring_mode %#x.\n",
+            iface, debug_d2d_point_2f(&baseline_origin), glyph_run, brush, measuring_mode);
 
     ID2D1RenderTarget_DrawGlyphRun(render_target->dxgi_target,
             baseline_origin, glyph_run, brush, measuring_mode);
@@ -727,23 +720,26 @@ static const struct ID2D1BitmapRenderTargetVtbl d2d_bitmap_render_target_vtbl =
     d2d_bitmap_render_target_GetBitmap
 };
 
+static const struct d2d_device_context_ops d2d_bitmap_render_target_ops =
+{
+    NULL,
+};
+
 HRESULT d2d_bitmap_render_target_init(struct d2d_bitmap_render_target *render_target,
-        const struct d2d_d3d_render_target *parent_target, const D2D1_SIZE_F *size,
+        const struct d2d_device_context *parent_target, const D2D1_SIZE_F *size,
         const D2D1_SIZE_U *pixel_size, const D2D1_PIXEL_FORMAT *pixel_format,
         D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS options)
 {
     D2D1_RENDER_TARGET_PROPERTIES dxgi_rt_desc;
-    D2D1_BITMAP_PROPERTIES bitmap_desc;
-    D3D10_TEXTURE2D_DESC texture_desc;
-    IDXGISurface *dxgi_surface;
-    ID3D10Texture2D *texture;
+    D2D1_BITMAP_PROPERTIES1 bitmap_desc;
+    ID2D1DeviceContext *context;
+    D2D1_SIZE_U bitmap_size;
     HRESULT hr;
 
     if (options)
         FIXME("Compatible target options are ignored, %#x.\n", options);
 
     render_target->ID2D1BitmapRenderTarget_iface.lpVtbl = &d2d_bitmap_render_target_vtbl;
-    render_target->refcount = 1;
 
     dxgi_rt_desc.type = parent_target->desc.type;
     dxgi_rt_desc.usage = parent_target->desc.usage;
@@ -751,24 +747,24 @@ HRESULT d2d_bitmap_render_target_init(struct d2d_bitmap_render_target *render_ta
 
     if (pixel_size)
     {
-        texture_desc.Width = pixel_size->width;
-        texture_desc.Height = pixel_size->height;
+        bitmap_size.width = pixel_size->width;
+        bitmap_size.height = pixel_size->height;
     }
     else if (size)
     {
-        texture_desc.Width = ceilf((size->width * parent_target->desc.dpiX) / 96.0f);
-        texture_desc.Height = ceilf((size->height * parent_target->desc.dpiY) / 96.0f);
+        bitmap_size.width = ceilf((size->width * parent_target->desc.dpiX) / 96.0f);
+        bitmap_size.height = ceilf((size->height * parent_target->desc.dpiY) / 96.0f);
     }
     else
     {
-        texture_desc.Width = parent_target->pixel_size.width;
-        texture_desc.Height = parent_target->pixel_size.height;
+        bitmap_size.width = parent_target->pixel_size.width;
+        bitmap_size.height = parent_target->pixel_size.height;
     }
 
-    if (size)
+    if (size && size->width != 0.0f && size->height != 0.0f)
     {
-        dxgi_rt_desc.dpiX = (texture_desc.Width * 96.0f) / size->width;
-        dxgi_rt_desc.dpiY = (texture_desc.Height * 96.0f) / size->height;
+        dxgi_rt_desc.dpiX = (bitmap_size.width * 96.0f) / size->width;
+        dxgi_rt_desc.dpiY = (bitmap_size.height * 96.0f) / size->height;
     }
     else
     {
@@ -777,58 +773,51 @@ HRESULT d2d_bitmap_render_target_init(struct d2d_bitmap_render_target *render_ta
     }
 
     if (!pixel_format || pixel_format->format == DXGI_FORMAT_UNKNOWN)
-        texture_desc.Format = parent_target->desc.pixelFormat.format;
+        dxgi_rt_desc.pixelFormat.format = parent_target->desc.pixelFormat.format;
     else
-        texture_desc.Format = pixel_format->format;
-    dxgi_rt_desc.pixelFormat.format = texture_desc.Format;
+        dxgi_rt_desc.pixelFormat.format = pixel_format->format;
 
     if (!pixel_format || pixel_format->alphaMode == D2D1_ALPHA_MODE_UNKNOWN)
         dxgi_rt_desc.pixelFormat.alphaMode = D2D1_ALPHA_MODE_PREMULTIPLIED;
     else
         dxgi_rt_desc.pixelFormat.alphaMode = pixel_format->alphaMode;
 
-    texture_desc.MipLevels = 1;
-    texture_desc.ArraySize = 1;
-    texture_desc.SampleDesc.Count = 1;
-    texture_desc.SampleDesc.Quality = 0;
-    texture_desc.Usage = D3D10_USAGE_DEFAULT;
-    texture_desc.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
-    texture_desc.CPUAccessFlags = 0;
-    texture_desc.MiscFlags = 0;
-
-    if (FAILED(hr = ID3D10Device_CreateTexture2D(parent_target->device, &texture_desc, NULL, &texture)))
-    {
-        WARN("Failed to create texture, hr %#x.\n", hr);
-        return hr;
-    }
-
-    hr = ID3D10Texture2D_QueryInterface(texture, &IID_IDXGISurface, (void **)&dxgi_surface);
-    ID3D10Texture2D_Release(texture);
-    if (FAILED(hr))
-    {
-        WARN("Failed to get DXGI surface interface, hr %#x.\n", hr);
-        return hr;
-    }
-
-    if (FAILED(hr = d2d_d3d_create_render_target(parent_target->factory, dxgi_surface,
-            (IUnknown *)&render_target->ID2D1BitmapRenderTarget_iface, &dxgi_rt_desc, &render_target->dxgi_target)))
+    if (FAILED(hr = d2d_d3d_create_render_target(parent_target->device, NULL,
+            (IUnknown *)&render_target->ID2D1BitmapRenderTarget_iface,
+            parent_target->ops ? &d2d_bitmap_render_target_ops : NULL,
+            &dxgi_rt_desc, (void **)&render_target->dxgi_inner)))
     {
         WARN("Failed to create DXGI surface render target, hr %#x.\n", hr);
-        IDXGISurface_Release(dxgi_surface);
+        return hr;
+    }
+
+    /* Note that we should be a little careful with the "dxgi_target"
+     * reference we get here. Because the object is aggregated, releasing the
+     * interface in any error paths below would end up calling
+     * d2d_bitmap_render_target_Release(). */
+    if (FAILED(hr = IUnknown_QueryInterface(render_target->dxgi_inner,
+            &IID_ID2D1RenderTarget, (void **)&render_target->dxgi_target)))
+    {
+        WARN("Failed to retrieve ID2D1RenderTarget interface, hr %#x.\n", hr);
+        IUnknown_Release(render_target->dxgi_inner);
         return hr;
     }
 
     bitmap_desc.pixelFormat = dxgi_rt_desc.pixelFormat;
     bitmap_desc.dpiX = dxgi_rt_desc.dpiX;
     bitmap_desc.dpiY = dxgi_rt_desc.dpiY;
+    bitmap_desc.bitmapOptions = D2D1_BITMAP_OPTIONS_TARGET;
+    bitmap_desc.colorContext = NULL;
 
-    hr = ID2D1RenderTarget_CreateSharedBitmap(render_target->dxgi_target, &IID_IDXGISurface, dxgi_surface,
-            &bitmap_desc, &render_target->bitmap);
-    IDXGISurface_Release(dxgi_surface);
+    ID2D1RenderTarget_QueryInterface(render_target->dxgi_target, &IID_ID2D1DeviceContext, (void **)&context);
+    hr = ID2D1DeviceContext_CreateBitmap(context, bitmap_size, NULL, 0, &bitmap_desc,
+            (ID2D1Bitmap1 **)&render_target->bitmap);
+    ID2D1DeviceContext_SetTarget(context, (ID2D1Image *)render_target->bitmap);
+    ID2D1DeviceContext_Release(context);
     if (FAILED(hr))
     {
-        WARN("Failed to create shared bitmap, hr %#x.\n", hr);
-        ID2D1RenderTarget_Release(render_target->dxgi_target);
+        WARN("Failed to create target bitmap, hr %#x.\n", hr);
+        IUnknown_Release(render_target->dxgi_inner);
         return hr;
     }
 
