@@ -45,8 +45,9 @@ enum d2d_shape_type
 {
     D2D_SHAPE_TYPE_OUTLINE,
     D2D_SHAPE_TYPE_BEZIER_OUTLINE,
+    D2D_SHAPE_TYPE_ARC_OUTLINE,
     D2D_SHAPE_TYPE_TRIANGLE,
-    D2D_SHAPE_TYPE_BEZIER,
+    D2D_SHAPE_TYPE_CURVE,
     D2D_SHAPE_TYPE_COUNT,
 };
 
@@ -113,7 +114,8 @@ struct d2d_brush_cb
 struct d2d_ps_cb
 {
     BOOL outline;
-    BOOL pad[3];
+    BOOL is_arc;
+    BOOL pad[2];
     struct d2d_brush_cb colour_brush;
     struct d2d_brush_cb opacity_brush;
 };
@@ -301,7 +303,7 @@ HRESULT d2d_bitmap_brush_create(ID2D1Factory *factory, ID2D1Bitmap *bitmap,
         const D2D1_BITMAP_BRUSH_PROPERTIES1 *bitmap_brush_desc, const D2D1_BRUSH_PROPERTIES *brush_desc,
         struct d2d_brush **brush) DECLSPEC_HIDDEN;
 void d2d_brush_bind_resources(struct d2d_brush *brush, ID3D10Device *device, unsigned int brush_idx) DECLSPEC_HIDDEN;
-HRESULT d2d_brush_get_ps_cb(struct d2d_brush *brush, struct d2d_brush *opacity_brush, BOOL outline,
+HRESULT d2d_brush_get_ps_cb(struct d2d_brush *brush, struct d2d_brush *opacity_brush, BOOL outline, BOOL is_arc,
         struct d2d_device_context *render_target, ID3D10Buffer **ps_cb) DECLSPEC_HIDDEN;
 struct d2d_brush *unsafe_impl_from_ID2D1Brush(ID2D1Brush *iface) DECLSPEC_HIDDEN;
 
@@ -346,9 +348,10 @@ struct d2d_bitmap
     LONG refcount;
 
     ID2D1Factory *factory;
-    ID3D10ShaderResourceView *view;
+    ID3D10ShaderResourceView *srv;
     ID3D10RenderTargetView *rtv;
     IDXGISurface *surface;
+    ID3D10Resource *resource;
     D2D1_SIZE_U pixel_size;
     D2D1_PIXEL_FORMAT format;
     float dpi_x;
@@ -387,7 +390,7 @@ enum d2d_geometry_state
     D2D_GEOMETRY_STATE_FIGURE,
 };
 
-struct d2d_bezier_vertex
+struct d2d_curve_vertex
 {
     D2D1_POINT_2F position;
     struct
@@ -413,7 +416,7 @@ struct d2d_outline_vertex
     D2D1_POINT_2F next;
 };
 
-struct d2d_bezier_outline_vertex
+struct d2d_curve_outline_vertex
 {
     D2D1_POINT_2F position;
     D2D1_POINT_2F p0, p1, p2;
@@ -438,8 +441,13 @@ struct d2d_geometry
         size_t faces_size;
         size_t face_count;
 
-        struct d2d_bezier_vertex *bezier_vertices;
+        struct d2d_curve_vertex *bezier_vertices;
+        size_t bezier_vertices_size;
         size_t bezier_vertex_count;
+
+        struct d2d_curve_vertex *arc_vertices;
+        size_t arc_vertices_size;
+        size_t arc_vertex_count;
     } fill;
 
     struct
@@ -452,17 +460,29 @@ struct d2d_geometry
         size_t faces_size;
         size_t face_count;
 
-        struct d2d_bezier_outline_vertex *beziers;
+        struct d2d_curve_outline_vertex *beziers;
         size_t beziers_size;
         size_t bezier_count;
 
         struct d2d_face *bezier_faces;
         size_t bezier_faces_size;
         size_t bezier_face_count;
+
+        struct d2d_curve_outline_vertex *arcs;
+        size_t arcs_size;
+        size_t arc_count;
+
+        struct d2d_face *arc_faces;
+        size_t arc_faces_size;
+        size_t arc_face_count;
     } outline;
 
     union
     {
+        struct
+        {
+            D2D1_ELLIPSE ellipse;
+        } ellipse;
         struct
         {
             ID2D1GeometrySink ID2D1GeometrySink_iface;
@@ -482,6 +502,10 @@ struct d2d_geometry
             D2D1_RECT_F rect;
         } rectangle;
         struct
+        {
+            D2D1_ROUNDED_RECT rounded_rect;
+        } rounded_rectangle;
+        struct
         {
             ID2D1Geometry *src_geometry;
             D2D_MATRIX_3X2_F transform;
@@ -495,9 +519,13 @@ struct d2d_geometry
     } u;
 };
 
+HRESULT d2d_ellipse_geometry_init(struct d2d_geometry *geometry,
+        ID2D1Factory *factory, const D2D1_ELLIPSE *ellipse) DECLSPEC_HIDDEN;
 void d2d_path_geometry_init(struct d2d_geometry *geometry, ID2D1Factory *factory) DECLSPEC_HIDDEN;
 HRESULT d2d_rectangle_geometry_init(struct d2d_geometry *geometry,
         ID2D1Factory *factory, const D2D1_RECT_F *rect) DECLSPEC_HIDDEN;
+HRESULT d2d_rounded_rectangle_geometry_init(struct d2d_geometry *geometry,
+        ID2D1Factory *factory, const D2D1_ROUNDED_RECT *rounded_rect) DECLSPEC_HIDDEN;
 void d2d_transformed_geometry_init(struct d2d_geometry *geometry, ID2D1Factory *factory,
         ID2D1Geometry *src_geometry, const D2D_MATRIX_3X2_F *transform) DECLSPEC_HIDDEN;
 HRESULT d2d_geometry_group_init(struct d2d_geometry *geometry, ID2D1Factory *factory,
@@ -615,8 +643,25 @@ static inline const char *debug_d2d_point_2f(const D2D1_POINT_2F *point)
 
 static inline const char *debug_d2d_rect_f(const D2D1_RECT_F *rect)
 {
-    if (!rect) return "(null)";
-    return wine_dbg_sprintf("(%.8e,%.8e)-(%.8e,%.8e)", rect->left, rect->top, rect->right, rect->bottom );
+    if (!rect)
+        return "(null)";
+    return wine_dbg_sprintf("(%.8e, %.8e)-(%.8e, %.8e)", rect->left, rect->top, rect->right, rect->bottom);
+}
+
+static inline const char *debug_d2d_rounded_rect(const D2D1_ROUNDED_RECT *rounded_rect)
+{
+    if (!rounded_rect)
+        return "(null)";
+    return wine_dbg_sprintf("(%.8e, %.8e)-(%.8e, %.8e)[%.8e, %.8e]", rounded_rect->rect.left, rounded_rect->rect.top,
+            rounded_rect->rect.right, rounded_rect->rect.bottom, rounded_rect->radiusX, rounded_rect->radiusY);
+}
+
+static inline const char *debug_d2d_ellipse(const D2D1_ELLIPSE *ellipse)
+{
+    if (!ellipse)
+        return "(null)";
+    return wine_dbg_sprintf("(%.8e, %.8e)[%.8e, %.8e]",
+            ellipse->point.x, ellipse->point.y, ellipse->radiusX, ellipse->radiusY);
 }
 
 #endif /* __WINE_D2D1_PRIVATE_H */
