@@ -106,7 +106,7 @@ static HRESULT WINAPI d3d9_stateblock_Capture(IDirect3DStateBlock9 *iface)
         WARN("Trying to capture stateblock while recording, returning D3DERR_INVALIDCALL.\n");
         return D3DERR_INVALIDCALL;
     }
-    wined3d_stateblock_capture(stateblock->wined3d_stateblock);
+    wined3d_stateblock_capture(stateblock->wined3d_stateblock, device->state);
     wined3d_mutex_unlock();
 
     return D3D_OK;
@@ -116,13 +116,11 @@ static HRESULT WINAPI d3d9_stateblock_Apply(IDirect3DStateBlock9 *iface)
 {
     struct d3d9_stateblock *stateblock = impl_from_IDirect3DStateBlock9(iface);
     struct wined3d_texture *wined3d_texture;
-    unsigned int i, offset, stride, stage;
     struct wined3d_buffer *wined3d_buffer;
     struct d3d9_vertexbuffer *buffer;
-    enum wined3d_format_id format;
     struct d3d9_texture *texture;
     struct d3d9_device *device;
-    HRESULT hr;
+    unsigned int i;
 
     TRACE("iface %p.\n", iface);
 
@@ -134,26 +132,23 @@ static HRESULT WINAPI d3d9_stateblock_Apply(IDirect3DStateBlock9 *iface)
         WARN("Trying to apply stateblock while recording, returning D3DERR_INVALIDCALL.\n");
         return D3DERR_INVALIDCALL;
     }
-    wined3d_stateblock_apply(stateblock->wined3d_stateblock);
+    wined3d_stateblock_apply(stateblock->wined3d_stateblock, device->state);
     device->sysmem_vb = 0;
     for (i = 0; i < D3D9_MAX_STREAMS; ++i)
     {
-        if (FAILED(hr = wined3d_device_get_stream_source(device->wined3d_device,
-                i, &wined3d_buffer, &offset, &stride)))
+        if (!(wined3d_buffer = device->stateblock_state->streams[i].buffer))
             continue;
-        if (!wined3d_buffer || !(buffer = wined3d_buffer_get_parent(wined3d_buffer)))
+        if (!(buffer = wined3d_buffer_get_parent(wined3d_buffer)))
             continue;
         if (buffer->draw_buffer)
             device->sysmem_vb |= 1u << i;
     }
-    device->sysmem_ib = (wined3d_buffer = wined3d_device_get_index_buffer(device->wined3d_device, &format, &offset))
+    device->sysmem_ib = (wined3d_buffer = device->stateblock_state->index_buffer)
             && (buffer = wined3d_buffer_get_parent(wined3d_buffer)) && buffer->draw_buffer;
     device->auto_mipmaps = 0;
     for (i = 0; i < D3D9_MAX_TEXTURE_UNITS; ++i)
     {
-        stage = i >= 16 ? i - 16 + D3DVERTEXTEXTURESAMPLER0 : i;
-
-        if ((wined3d_texture = wined3d_device_get_texture(device->wined3d_device, stage))
+        if ((wined3d_texture = device->stateblock_state->textures[i])
                 && (texture = wined3d_texture_get_parent(wined3d_texture))
                 && texture->usage & D3DUSAGE_AUTOGENMIPMAP)
             device->auto_mipmaps |= 1u << i;
@@ -193,7 +188,7 @@ HRESULT stateblock_init(struct d3d9_stateblock *stateblock, struct d3d9_device *
     else
     {
         wined3d_mutex_lock();
-        hr = wined3d_stateblock_create(device->wined3d_device,
+        hr = wined3d_stateblock_create(device->wined3d_device, device->state,
                 (enum wined3d_stateblock_type)type, &stateblock->wined3d_stateblock);
         wined3d_mutex_unlock();
         if (FAILED(hr))
