@@ -111,6 +111,7 @@ static const GUID *d3dformat_to_wic_guid(D3DFORMAT format)
 #define DDS_PF_RGB 0x40
 #define DDS_PF_YUV 0x200
 #define DDS_PF_LUMINANCE 0x20000
+#define DDS_PF_BUMPLUMINANCE 0x40000
 #define DDS_PF_BUMPDUDV 0x80000
 
 struct dds_pixel_format
@@ -360,6 +361,17 @@ static D3DFORMAT dds_bump_to_d3dformat(const struct dds_pixel_format *pixel_form
     return D3DFMT_UNKNOWN;
 }
 
+static D3DFORMAT dds_bump_luminance_to_d3dformat(const struct dds_pixel_format *pixel_format)
+{
+    if (pixel_format->bpp == 32 && pixel_format->rmask == 0x000000ff && pixel_format->gmask == 0x0000ff00
+            && pixel_format->bmask == 0x00ff0000)
+        return D3DFMT_X8L8V8U8;
+
+    WARN("Unknown bump pixel format (%u, %#x, %#x, %#x, %#x).\n", pixel_format->bpp,
+        pixel_format->rmask, pixel_format->gmask, pixel_format->bmask, pixel_format->amask);
+    return D3DFMT_UNKNOWN;
+}
+
 static D3DFORMAT dds_pixel_format_to_d3dformat(const struct dds_pixel_format *pixel_format)
 {
     TRACE("pixel_format: size %u, flags %#x, fourcc %#x, bpp %u.\n", pixel_format->size,
@@ -377,6 +389,8 @@ static D3DFORMAT dds_pixel_format_to_d3dformat(const struct dds_pixel_format *pi
         return dds_alpha_to_d3dformat(pixel_format);
     if (pixel_format->flags & DDS_PF_BUMPDUDV)
         return dds_bump_to_d3dformat(pixel_format);
+    if (pixel_format->flags & DDS_PF_BUMPLUMINANCE)
+        return dds_bump_luminance_to_d3dformat(pixel_format);
 
     WARN("Unknown pixel format (flags %#x, fourcc %#x, bpp %u, r %#x, g %#x, b %#x, a %#x)\n",
         pixel_format->flags, pixel_format->fourcc, pixel_format->bpp,
@@ -1972,6 +1986,8 @@ HRESULT WINAPI D3DXLoadSurfaceFromMemory(IDirect3DSurface9 *dst_surface,
                     int i, int j, void *texel);
             unsigned int x, y;
 
+            src_pitch = src_pitch * srcformatdesc->block_width / srcformatdesc->block_byte_count;
+
             src_uncompressed = heap_alloc(src_size.width * src_size.height * sizeof(DWORD));
             if (!src_uncompressed)
             {
@@ -2003,8 +2019,7 @@ HRESULT WINAPI D3DXLoadSurfaceFromMemory(IDirect3DSurface9 *dst_surface,
                 DWORD *ptr = &src_uncompressed[y * src_size.width];
                 for (x = 0; x < src_size.width; ++x)
                 {
-                    fetch_dxt_texel(src_pitch / sizeof(DWORD), src_memory,
-                            x + src_rect->left, y + src_rect->top, ptr);
+                    fetch_dxt_texel(src_pitch, src_memory, x + src_rect->left, y + src_rect->top, ptr);
                     ++ptr;
                 }
             }
@@ -2080,7 +2095,8 @@ HRESULT WINAPI D3DXLoadSurfaceFromMemory(IDirect3DSurface9 *dst_surface,
                     ERR("Unexpected destination compressed format %u.\n", surfdesc.Format);
             }
             tx_compress_dxtn(4, dst_size_aligned.width, dst_size_aligned.height,
-                    dst_uncompressed, gl_format, lockrect.pBits, lockrect.Pitch);
+                    dst_uncompressed, gl_format, lockrect.pBits,
+                    lockrect.Pitch * destformatdesc->block_width / destformatdesc->block_byte_count);
             heap_free(dst_uncompressed);
         }
     }
