@@ -52,6 +52,9 @@
 #ifndef SHT_NULL
 #define SHT_NULL        0
 #endif
+#ifndef NT_GNU_BUILD_ID
+#define NT_GNU_BUILD_ID 3
+#endif
 #endif
 
 /* structure holding information while handling an ELF image
@@ -60,7 +63,9 @@
 struct image_file_map
 {
     enum module_type            modtype;
+    const struct image_file_map_ops *ops;
     unsigned                    addr_size;      /* either 16 (not used), 32 or 64 */
+    struct image_file_map*      alternate;      /* another file linked to this one */
     union
     {
         struct elf_file_map
@@ -69,7 +74,6 @@ struct image_file_map
             size_t                      elf_start;
             HANDLE                      handle;
             const char*	                shstrtab;
-            struct image_file_map*      alternate;      /* another ELF file (linked to this one) */
             char*                       target_copy;
 #ifdef __ELF__
             Elf64_Ehdr                  elfhdr;
@@ -84,7 +88,7 @@ struct image_file_map
         {
             size_t                      segs_size;
             size_t                      segs_start;
-            int                         fd;
+            HANDLE                      handle;
             struct image_file_map*      dsym;   /* the debug symbols file associated with this one */
 
 #ifdef HAVE_MACH_O_LOADER_H
@@ -126,86 +130,62 @@ struct image_file_map
 struct image_section_map
 {
     struct image_file_map*      fmap;
-    long                        sidx;
+    LONG_PTR                    sidx;
 };
 
-extern BOOL         elf_find_section(struct image_file_map* fmap, const char* name,
-                                     unsigned sht, struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern const char*  elf_map_section(struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern void         elf_unmap_section(struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern DWORD_PTR    elf_get_map_rva(const struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern unsigned     elf_get_map_size(const struct image_section_map* ism) DECLSPEC_HIDDEN;
+BOOL image_check_alternate(struct image_file_map* fmap, const struct module* module) DECLSPEC_HIDDEN;
 
-extern BOOL         macho_find_section(struct image_file_map* ifm, const char* segname,
-                                       const char* sectname, struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern const char*  macho_map_section(struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern void         macho_unmap_section(struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern DWORD_PTR    macho_get_map_rva(const struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern unsigned     macho_get_map_size(const struct image_section_map* ism) DECLSPEC_HIDDEN;
+BOOL elf_map_handle(HANDLE handle, struct image_file_map* fmap) DECLSPEC_HIDDEN;
+BOOL pe_map_file(HANDLE file, struct image_file_map* fmap, enum module_type mt) DECLSPEC_HIDDEN;
 
-extern BOOL         pe_find_section(struct image_file_map* fmap, const char* name,
-                                    struct image_section_map* ism) DECLSPEC_HIDDEN;
-extern const char*  pe_map_section(struct image_section_map* psm) DECLSPEC_HIDDEN;
-extern void         pe_unmap_section(struct image_section_map* psm) DECLSPEC_HIDDEN;
-extern DWORD_PTR    pe_get_map_rva(const struct image_section_map* psm) DECLSPEC_HIDDEN;
-extern unsigned     pe_get_map_size(const struct image_section_map* psm) DECLSPEC_HIDDEN;
+struct image_file_map_ops
+{
+    const char* (*map_section)(struct image_section_map* ism);
+    void  (*unmap_section)(struct image_section_map* ism);
+    BOOL (*find_section)(struct image_file_map* fmap, const char* name, struct image_section_map* ism);
+    DWORD_PTR (*get_map_rva)(const struct image_section_map* ism);
+    unsigned (*get_map_size)(const struct image_section_map* ism);
+    void (*unmap_file)(struct image_file_map *fmap);
+};
 
 static inline BOOL image_find_section(struct image_file_map* fmap, const char* name,
                                       struct image_section_map* ism)
 {
-    switch (fmap->modtype)
+    while (fmap)
     {
-    case DMT_ELF:   return elf_find_section(fmap, name, SHT_NULL, ism);
-    case DMT_MACHO: return macho_find_section(fmap, NULL, name, ism);
-    case DMT_PE:    return pe_find_section(fmap, name, ism);
-    default: assert(0); return FALSE;
+        if (fmap->ops->find_section(fmap, name, ism)) return TRUE;
+        fmap = fmap->alternate;
+    }
+    ism->fmap = NULL;
+    ism->sidx = -1;
+    return FALSE;
+}
+
+static inline void image_unmap_file(struct image_file_map* fmap)
+{
+    while (fmap)
+    {
+        fmap->ops->unmap_file(fmap);
+        fmap = fmap->alternate;
     }
 }
 
 static inline const char* image_map_section(struct image_section_map* ism)
 {
-    if (!ism->fmap) return NULL;
-    switch (ism->fmap->modtype)
-    {
-    case DMT_ELF:   return elf_map_section(ism);
-    case DMT_MACHO: return macho_map_section(ism);
-    case DMT_PE:    return pe_map_section(ism);
-    default: assert(0); return NULL;
-    }
+    return ism->fmap ? ism->fmap->ops->map_section(ism) : NULL;
 }
 
 static inline void image_unmap_section(struct image_section_map* ism)
 {
-    if (!ism->fmap) return;
-    switch (ism->fmap->modtype)
-    {
-    case DMT_ELF:   elf_unmap_section(ism); break;
-    case DMT_MACHO: macho_unmap_section(ism); break;
-    case DMT_PE:    pe_unmap_section(ism);   break;
-    default: assert(0); return;
-    }
+    if (ism->fmap) ism->fmap->ops->unmap_section(ism);
 }
 
 static inline DWORD_PTR image_get_map_rva(const struct image_section_map* ism)
 {
-    if (!ism->fmap) return 0;
-    switch (ism->fmap->modtype)
-    {
-    case DMT_ELF:   return elf_get_map_rva(ism);
-    case DMT_MACHO: return macho_get_map_rva(ism);
-    case DMT_PE:    return pe_get_map_rva(ism);
-    default: assert(0); return 0;
-    }
+    return ism->fmap ? ism->fmap->ops->get_map_rva(ism) : 0;
 }
 
 static inline unsigned image_get_map_size(const struct image_section_map* ism)
 {
-    if (!ism->fmap) return 0;
-    switch (ism->fmap->modtype)
-    {
-    case DMT_ELF:   return elf_get_map_size(ism);
-    case DMT_MACHO: return macho_get_map_size(ism);
-    case DMT_PE:    return pe_get_map_size(ism);
-    default: assert(0); return 0;
-    }
+    return ism->fmap ? ism->fmap->ops->get_map_size(ism) : 0;
 }
