@@ -26,6 +26,7 @@
 #include <assert.h>
 
 #include "dbghelp_private.h"
+#include "image_private.h"
 #include "psapi.h"
 #include "winternl.h"
 #include "wine/debug.h"
@@ -36,9 +37,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(dbghelp);
 const WCHAR        S_ElfW[]         = {'<','e','l','f','>','\0'};
 const WCHAR        S_WineLoaderW[]  = {'<','w','i','n','e','-','l','o','a','d','e','r','>','\0'};
 static const WCHAR S_DotSoW[]       = {'.','s','o','\0'};
-static const WCHAR S_DotDylibW[]    = {'.','d','y','l','i','b','\0'};
-static const WCHAR S_DotPdbW[]      = {'.','p','d','b','\0'};
-static const WCHAR S_DotDbgW[]      = {'.','d','b','g','\0'};
 const WCHAR        S_SlashW[]       = {'/','\0'};
 
 static const WCHAR S_AcmW[] = {'.','a','c','m','\0'};
@@ -81,14 +79,14 @@ static BOOL is_wine_loader(const WCHAR *module)
     static const WCHAR wineW[] = {'w','i','n','e',0};
     static const WCHAR suffixW[] = {'6','4',0};
     const WCHAR *filename = get_filename(module, NULL);
-    const char *ptr, *p;
+    const char *ptr;
     BOOL ret = FALSE;
     WCHAR *buffer;
     DWORD len;
 
     if ((ptr = getenv("WINELOADER")))
     {
-        if ((p = strrchr(ptr, '/'))) ptr = p + 1;
+        ptr = file_nameA(ptr);
         len = 2 + MultiByteToWideChar( CP_UNIXCP, 0, ptr, -1, NULL, 0 );
         buffer = heap_alloc( len * sizeof(WCHAR) );
         MultiByteToWideChar( CP_UNIXCP, 0, ptr, -1, buffer, len );
@@ -189,7 +187,7 @@ static const char*      get_module_type(enum module_type type, BOOL virtual)
 struct module* module_new(struct process* pcs, const WCHAR* name,
                           enum module_type type, BOOL virtual,
                           DWORD64 mod_addr, DWORD64 size,
-                          unsigned long stamp, unsigned long checksum)
+                          ULONG_PTR stamp, ULONG_PTR checksum)
 {
     struct module*      module;
     unsigned            i;
@@ -373,12 +371,8 @@ BOOL module_get_debug(struct module_pair* pair)
         BOOL ret;
         
         if (pair->effective->is_virtual) ret = FALSE;
-        else switch (pair->effective->type)
+        else if (pair->effective->type == DMT_PE)
         {
-        case DMT_ELF:
-            ret = elf_load_debug_info(pair->effective);
-            break;
-        case DMT_PE:
             idslW64.SizeOfStruct = sizeof(idslW64);
             idslW64.BaseOfImage = pair->effective->module.BaseOfImage;
             idslW64.CheckSum = pair->effective->module.CheckSum;
@@ -393,14 +387,9 @@ BOOL module_get_debug(struct module_pair* pair)
             pcs_callback(pair->pcs,
                          ret ? CBA_DEFERRED_SYMBOL_LOAD_COMPLETE : CBA_DEFERRED_SYMBOL_LOAD_FAILURE,
                          &idslW64);
-            break;
-        case DMT_MACHO:
-            ret = macho_load_debug_info(pair->pcs, pair->effective);
-            break;
-        default:
-            ret = FALSE;
-            break;
         }
+        else ret = pair->pcs->loader->load_debug_info(pair->pcs, pair->effective);
+
         if (!ret) pair->effective->module.SymType = SymNone;
         assert(pair->effective->module.SymType != SymDeferred);
         pair->effective->module.NumSyms = pair->effective->ht_symbols.num_elts;
@@ -475,66 +464,259 @@ static BOOL module_is_container_loaded(const struct process* pcs,
     return FALSE;
 }
 
-/******************************************************************
- *		module_get_type_by_name
- *
- * Guesses a filename type from its extension
- */
-enum module_type module_get_type_by_name(const WCHAR* name)
+static BOOL image_check_debug_link(const WCHAR* file, struct image_file_map* fmap, DWORD link_crc)
 {
-    int len = strlenW(name);
+    DWORD read_bytes;
+    HANDLE handle;
+    WCHAR *path;
+    WORD magic;
+    BOOL ret;
 
-    /* Skip all version extensions (.[digits]) regex: "(\.\d+)*$" */
-    do
+    path = get_dos_file_name(file);
+    handle = CreateFileW(path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
+    heap_free(path);
+    if (handle == INVALID_HANDLE_VALUE) return FALSE;
+
+    if (link_crc)
     {
-        int i = len;
-
-        while (i && name[i - 1] >= '0' && name[i - 1] <= '9') i--;
-
-        if (i && name[i - 1] == '.')
-            len = i - 1;
-        else
-            break;
-    } while (len);
-
-    /* check for terminating .so or .so.[digit] */
-    /* FIXME: Can't rely solely on extension; have to check magic or
-     *        stop using .so on Mac OS X.  For now, base on platform. */
-    if (len > 3 && !memcmp(name + len - 3, S_DotSoW, 3))
-#ifdef __APPLE__
-        return DMT_MACHO;
-#else
-        return DMT_ELF;
-#endif
-
-    if (len > 6 && !strncmpiW(name + len - 6, S_DotDylibW, 6))
-        return DMT_MACHO;
-
-    if (len > 4 && !strncmpiW(name + len - 4, S_DotPdbW, 4))
-        return DMT_PDB;
-
-    if (len > 4 && !strncmpiW(name + len - 4, S_DotDbgW, 4))
-        return DMT_DBG;
-
-    /* wine is also a native module (Mach-O on Mac OS X, ELF elsewhere) */
-    if (is_wine_loader(name))
-    {
-#ifdef __APPLE__
-        return DMT_MACHO;
-#else
-        return DMT_ELF;
-#endif
+        DWORD crc = calc_crc32(handle);
+        if (crc != link_crc)
+        {
+            WARN("Bad CRC for file %s (got %08x while expecting %08x)\n",  debugstr_w(file), crc, link_crc);
+            CloseHandle(handle);
+            return FALSE;
+        }
     }
-    return DMT_PE;
+
+    SetFilePointer(handle, 0, 0, FILE_BEGIN);
+    if (ReadFile(handle, &magic, sizeof(magic), &read_bytes, NULL) && magic == IMAGE_DOS_SIGNATURE)
+        ret = pe_map_file(handle, fmap, DMT_PE);
+    else
+        ret = elf_map_handle(handle, fmap);
+    CloseHandle(handle);
+    return ret;
 }
 
 /******************************************************************
- *		                refresh_module_list
+ *		image_locate_debug_link
+ *
+ * Locate a filename from a .gnu_debuglink section, using the same
+ * strategy as gdb:
+ * "If the full name of the directory containing the executable is
+ * execdir, and the executable has a debug link that specifies the
+ * name debugfile, then GDB will automatically search for the
+ * debugging information file in three places:
+ *  - the directory containing the executable file (that is, it
+ *    will look for a file named `execdir/debugfile',
+ *  - a subdirectory of that directory named `.debug' (that is, the
+ *    file `execdir/.debug/debugfile', and
+ *  - a subdirectory of the global debug file directory that includes
+ *    the executable's full path, and the name from the link (that is,
+ *    the file `globaldebugdir/execdir/debugfile', where globaldebugdir
+ *    is the global debug file directory, and execdir has been turned
+ *    into a relative path)." (from GDB manual)
  */
-static BOOL refresh_module_list(struct process* pcs)
+static BOOL image_locate_debug_link(const struct module* module, struct image_file_map* fmap, const char* filename, DWORD crc)
 {
-    /* force transparent ELF and Mach-O loading / unloading */
-    return elf_synchronize_module_list(pcs) || macho_synchronize_module_list(pcs);
+    static const WCHAR globalDebugDirW[] = {'/','u','s','r','/','l','i','b','/','d','e','b','u','g','/'};
+    static const WCHAR dotDebugW[] = {'.','d','e','b','u','g','/'};
+    const size_t globalDebugDirLen = ARRAY_SIZE(globalDebugDirW);
+    size_t filename_len, path_len;
+    WCHAR* p = NULL;
+    WCHAR* slash;
+    WCHAR* slash2;
+    struct image_file_map* fmap_link = NULL;
+
+    fmap_link = HeapAlloc(GetProcessHeap(), 0, sizeof(*fmap_link));
+    if (!fmap_link) return FALSE;
+
+    filename_len = MultiByteToWideChar(CP_UNIXCP, 0, filename, -1, NULL, 0);
+    path_len = strlenW(module->module.LoadedImageName);
+    if (module->real_path) path_len = max(path_len, strlenW(module->real_path));
+    p = HeapAlloc(GetProcessHeap(), 0,
+                  (globalDebugDirLen + path_len + 6 + 1 + filename_len + 1) * sizeof(WCHAR));
+    if (!p) goto found;
+
+    /* we prebuild the string with "execdir" */
+    strcpyW(p, module->module.LoadedImageName);
+    slash = p;
+    if ((slash2 = strrchrW(slash, '/'))) slash = slash2 + 1;
+    if ((slash2 = strrchrW(slash, '\\'))) slash = slash2 + 1;
+
+    /* testing execdir/filename */
+    MultiByteToWideChar(CP_UNIXCP, 0, filename, -1, slash, filename_len);
+    if (image_check_debug_link(p, fmap_link, crc)) goto found;
+
+    /* testing execdir/.debug/filename */
+    memcpy(slash, dotDebugW, sizeof(dotDebugW));
+    MultiByteToWideChar(CP_UNIXCP, 0, filename, -1, slash + ARRAY_SIZE(dotDebugW), filename_len);
+    if (image_check_debug_link(p, fmap_link, crc)) goto found;
+
+    if (module->real_path)
+    {
+        strcpyW(p, module->real_path);
+        slash = p;
+        if ((slash2 = strrchrW(slash, '/'))) slash = slash2 + 1;
+        if ((slash2 = strrchrW(slash, '\\'))) slash = slash2 + 1;
+        MultiByteToWideChar(CP_UNIXCP, 0, filename, -1, slash, filename_len);
+        if (image_check_debug_link(p, fmap_link, crc)) goto found;
+    }
+
+    /* testing globaldebugdir/execdir/filename */
+    memmove(p + globalDebugDirLen, p, (slash - p) * sizeof(WCHAR));
+    memcpy(p, globalDebugDirW, globalDebugDirLen * sizeof(WCHAR));
+    slash += globalDebugDirLen;
+    MultiByteToWideChar(CP_UNIXCP, 0, filename, -1, slash, filename_len);
+    if (image_check_debug_link(p, fmap_link, crc)) goto found;
+
+    /* finally testing filename */
+    if (image_check_debug_link(slash, fmap_link, crc)) goto found;
+
+
+    WARN("Couldn't locate or map %s\n", filename);
+    HeapFree(GetProcessHeap(), 0, p);
+    HeapFree(GetProcessHeap(), 0, fmap_link);
+    return FALSE;
+
+found:
+    TRACE("Located debug information file %s at %s\n", filename, debugstr_w(p));
+    HeapFree(GetProcessHeap(), 0, p);
+    fmap->alternate = fmap_link;
+    return TRUE;
+}
+
+/******************************************************************
+ *		image_locate_build_id_target
+ *
+ * Try to find the .so file containing the debug info out of the build-id note information
+ */
+static BOOL image_locate_build_id_target(struct image_file_map* fmap, const BYTE* id, unsigned idlen)
+{
+    static const WCHAR globalDebugDirW[] = {'/','u','s','r','/','l','i','b','/','d','e','b','u','g','/'};
+    static const WCHAR buildidW[] = {'.','b','u','i','l','d','-','i','d','/'};
+    static const WCHAR dotDebug0W[] = {'.','d','e','b','u','g',0};
+    struct image_file_map* fmap_link = NULL;
+    WCHAR* p;
+    WCHAR* z;
+    const BYTE* idend = id + idlen;
+
+    fmap_link = HeapAlloc(GetProcessHeap(), 0, sizeof(*fmap_link));
+    if (!fmap_link) return FALSE;
+
+    p = HeapAlloc(GetProcessHeap(), 0,
+                  sizeof(globalDebugDirW) + sizeof(buildidW) +
+                  (idlen * 2 + 1) * sizeof(WCHAR) + sizeof(dotDebug0W));
+    z = p;
+    memcpy(z, globalDebugDirW, sizeof(globalDebugDirW));
+    z += ARRAY_SIZE(globalDebugDirW);
+    memcpy(z, buildidW, sizeof(buildidW));
+    z += ARRAY_SIZE(buildidW);
+
+    if (id < idend)
+    {
+        *z++ = "0123456789abcdef"[*id >> 4  ];
+        *z++ = "0123456789abcdef"[*id & 0x0F];
+        id++;
+    }
+    if (id < idend)
+        *z++ = '/';
+    while (id < idend)
+    {
+        *z++ = "0123456789abcdef"[*id >> 4  ];
+        *z++ = "0123456789abcdef"[*id & 0x0F];
+        id++;
+    }
+    memcpy(z, dotDebug0W, sizeof(dotDebug0W));
+    TRACE("checking %s\n", wine_dbgstr_w(p));
+
+    if (image_check_debug_link(p, fmap_link, 0))
+    {
+        struct image_section_map buildid_sect;
+        if (image_find_section(fmap_link, ".note.gnu.build-id", &buildid_sect))
+        {
+            const UINT32* note;
+
+            note = (const UINT32*)image_map_section(&buildid_sect);
+            if (note != IMAGE_NO_MAP)
+            {
+                /* the usual ELF note structure: name-size desc-size type <name> <desc> */
+                if (note[2] == NT_GNU_BUILD_ID)
+                {
+                    if (note[1] == idlen &&
+                        !memcmp(note + 3 + ((note[0] + 3) >> 2), idend - idlen, idlen))
+                    {
+                        TRACE("Located debug information file at %s\n", debugstr_w(p));
+                        HeapFree(GetProcessHeap(), 0, p);
+                        fmap->alternate = fmap_link;
+                        return TRUE;
+                    }
+                    WARN("mismatch in buildid information for %s\n", wine_dbgstr_w(p));
+                }
+            }
+            image_unmap_section(&buildid_sect);
+        }
+        image_unmap_file(fmap_link);
+    }
+
+    TRACE("not found\n");
+    HeapFree(GetProcessHeap(), 0, p);
+    HeapFree(GetProcessHeap(), 0, fmap_link);
+    return FALSE;
+}
+
+/******************************************************************
+ *		image_check_alternate
+ *
+ * Load alternate files for a given image file, looking at either .note.gnu_build-id
+ * or .gnu_debuglink sections.
+ */
+BOOL image_check_alternate(struct image_file_map* fmap, const struct module* module)
+{
+    BOOL ret = FALSE;
+    BOOL found = FALSE;
+    struct image_section_map buildid_sect, debuglink_sect;
+
+    /* if present, add the .gnu_debuglink file as an alternate to current one */
+    if (image_find_section(fmap, ".note.gnu.build-id", &buildid_sect))
+    {
+        const UINT32* note;
+
+        found = TRUE;
+        note = (const UINT32*)image_map_section(&buildid_sect);
+        if (note != IMAGE_NO_MAP)
+        {
+            /* the usual ELF note structure: name-size desc-size type <name> <desc> */
+            if (note[2] == NT_GNU_BUILD_ID)
+            {
+                ret = image_locate_build_id_target(fmap, (const BYTE*)(note + 3 + ((note[0] + 3) >> 2)), note[1]);
+            }
+        }
+        image_unmap_section(&buildid_sect);
+    }
+    /* if present, add the .gnu_debuglink file as an alternate to current one */
+    if (!ret && image_find_section(fmap, ".gnu_debuglink", &debuglink_sect))
+    {
+        const char* dbg_link;
+
+        found = TRUE;
+        dbg_link = (const char*)image_map_section(&debuglink_sect);
+        if (dbg_link != IMAGE_NO_MAP)
+        {
+            /* The content of a debug link section is:
+             * 1/ a NULL terminated string, containing the file name for the
+             *    debug info
+             * 2/ padding on 4 byte boundary
+             * 3/ CRC of the linked file
+             */
+            DWORD crc = *(const DWORD*)(dbg_link + ((DWORD_PTR)(strlen(dbg_link) + 4) & ~3));
+            ret = image_locate_debug_link(module, fmap, dbg_link, crc);
+            if (!ret)
+                WARN("Couldn't load linked debug file for %s\n",
+                     debugstr_w(module->module.ModuleName));
+        }
+        image_unmap_section(&debuglink_sect);
+    }
+    return found ? ret : TRUE;
 }
 
 /***********************************************************************
@@ -608,8 +790,7 @@ DWORD64 WINAPI  SymLoadModuleExW(HANDLE hProcess, HANDLE hFile, PCWSTR wImageNam
     if (Flags & SLMFLAG_VIRTUAL)
     {
         if (!wImageName) return FALSE;
-        module = module_new(pcs, wImageName, module_get_type_by_name(wImageName),
-                            TRUE, BaseOfDll, SizeOfDll, 0, 0);
+        module = module_new(pcs, wImageName, DMT_PE, TRUE, BaseOfDll, SizeOfDll, 0, 0);
         if (!module) return FALSE;
         if (wModuleName) module_set_module(module, wModuleName);
         module->module.SymType = SymVirtual;
@@ -619,7 +800,7 @@ DWORD64 WINAPI  SymLoadModuleExW(HANDLE hProcess, HANDLE hFile, PCWSTR wImageNam
     if (Flags & ~(SLMFLAG_VIRTUAL))
         FIXME("Unsupported Flags %08x for %s\n", Flags, debugstr_w(wImageName));
 
-    refresh_module_list(pcs);
+    pcs->loader->synchronize_module_list(pcs);
 
     /* this is a Wine extension to the API just to redo the synchronisation */
     if (!wImageName && !hFile) return 0;
@@ -643,18 +824,7 @@ DWORD64 WINAPI  SymLoadModuleExW(HANDLE hProcess, HANDLE hFile, PCWSTR wImageNam
             wImageName)
         {
             /* and finally an ELF or Mach-O module */
-            switch (module_get_type_by_name(wImageName))
-            {
-                case DMT_ELF:
-                    module = elf_load_module(pcs, wImageName, BaseOfDll);
-                    break;
-                case DMT_MACHO:
-                    module = macho_load_module(pcs, wImageName, BaseOfDll);
-                    break;
-                default:
-                    /* Ignored */
-                    break;
-            }
+            module = pcs->loader->load_module(pcs, wImageName, BaseOfDll);
         }
     }
     if (!module)
@@ -1153,7 +1323,7 @@ BOOL WINAPI SymRefreshModuleList(HANDLE hProcess)
 
     if (!(pcs = process_find_by_handle(hProcess))) return FALSE;
 
-    return refresh_module_list(pcs);
+    return pcs->loader->synchronize_module_list(pcs);
 }
 
 /***********************************************************************
@@ -1178,3 +1348,38 @@ PVOID WINAPI SymFunctionTableAccess64(HANDLE hProcess, DWORD64 AddrBase)
 
     return dbghelp_current_cpu->find_runtime_function(module, AddrBase);
 }
+
+static BOOL native_synchronize_module_list(struct process* pcs)
+{
+    return FALSE;
+}
+
+static struct module* native_load_module(struct process* pcs, const WCHAR* name, ULONG_PTR addr)
+{
+    return NULL;
+}
+
+static BOOL native_load_debug_info(struct process* process, struct module* module)
+{
+    return FALSE;
+}
+
+static BOOL native_enum_modules(struct process *process, enum_modules_cb cb, void* user)
+{
+    return FALSE;
+}
+
+static BOOL native_fetch_file_info(struct process* process, const WCHAR* name, ULONG_PTR load_addr, DWORD_PTR* base,
+                                   DWORD* size, DWORD* checksum)
+{
+    return FALSE;
+}
+
+const struct loader_ops no_loader_ops =
+{
+    native_synchronize_module_list,
+    native_load_module,
+    native_load_debug_info,
+    native_enum_modules,
+    native_fetch_file_info,
+};
