@@ -73,7 +73,7 @@ static void pe_unmap_full(struct image_file_map* fmap)
  *
  * Maps a single section into memory from an PE file
  */
-const char* pe_map_section(struct image_section_map* ism)
+static const char* pe_map_section(struct image_section_map* ism)
 {
     void*       mapping;
     struct pe_file_map* fmap = &ism->fmap->u.pe;
@@ -111,8 +111,8 @@ const char* pe_map_section(struct image_section_map* ism)
  * Finds a section by name (and type) into memory from an PE file
  * or its alternate if any
  */
-BOOL pe_find_section(struct image_file_map* fmap, const char* name,
-                     struct image_section_map* ism)
+static BOOL pe_find_section(struct image_file_map* fmap, const char* name,
+                            struct image_section_map* ism)
 {
     const char*                 sectname;
     unsigned                    i;
@@ -148,7 +148,7 @@ BOOL pe_find_section(struct image_file_map* fmap, const char* name,
  *
  * Unmaps a single section from memory
  */
-void pe_unmap_section(struct image_section_map* ism)
+static void pe_unmap_section(struct image_section_map* ism)
 {
     if (ism->sidx >= 0 && ism->sidx < ism->fmap->u.pe.ntheader.FileHeader.NumberOfSections &&
         ism->fmap->u.pe.sect[ism->sidx].mapped != IMAGE_NO_MAP)
@@ -163,7 +163,7 @@ void pe_unmap_section(struct image_section_map* ism)
  *
  * Get the RVA of an PE section
  */
-DWORD_PTR pe_get_map_rva(const struct image_section_map* ism)
+static DWORD_PTR pe_get_map_rva(const struct image_section_map* ism)
 {
     if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.pe.ntheader.FileHeader.NumberOfSections)
         return 0;
@@ -175,13 +175,46 @@ DWORD_PTR pe_get_map_rva(const struct image_section_map* ism)
  *
  * Get the size of a PE section
  */
-unsigned pe_get_map_size(const struct image_section_map* ism)
+static unsigned pe_get_map_size(const struct image_section_map* ism)
 {
     if (ism->sidx < 0 || ism->sidx >= ism->fmap->u.pe.ntheader.FileHeader.NumberOfSections)
         return 0;
     return ism->fmap->u.pe.sect[ism->sidx].shdr.Misc.VirtualSize;
 }
 
+/******************************************************************
+ *		pe_unmap_file
+ *
+ * Unmaps an PE file from memory (previously mapped with pe_map_file)
+ */
+static void pe_unmap_file(struct image_file_map* fmap)
+{
+    if (fmap->u.pe.hMap != 0)
+    {
+        struct image_section_map  ism;
+        ism.fmap = fmap;
+        for (ism.sidx = 0; ism.sidx < fmap->u.pe.ntheader.FileHeader.NumberOfSections; ism.sidx++)
+        {
+            pe_unmap_section(&ism);
+        }
+        while (fmap->u.pe.full_count) pe_unmap_full(fmap);
+        HeapFree(GetProcessHeap(), 0, fmap->u.pe.sect);
+        HeapFree(GetProcessHeap(), 0, (void*)fmap->u.pe.strtable); /* FIXME ugly (see pe_map_file) */
+        CloseHandle(fmap->u.pe.hMap);
+        fmap->u.pe.hMap = NULL;
+    }
+}
+
+static const struct image_file_map_ops pe_file_map_ops =
+{
+    pe_map_section,
+    pe_unmap_section,
+    pe_find_section,
+    pe_get_map_rva,
+    pe_get_map_size,
+    pe_unmap_file,
+};
+
 /******************************************************************
  *		pe_is_valid_pointer_table
  *
@@ -206,11 +239,13 @@ static BOOL pe_is_valid_pointer_table(const IMAGE_NT_HEADERS* nthdr, const void*
  *
  * Maps an PE file into memory (and checks it's a real PE file)
  */
-static BOOL pe_map_file(HANDLE file, struct image_file_map* fmap, enum module_type mt)
+BOOL pe_map_file(HANDLE file, struct image_file_map* fmap, enum module_type mt)
 {
     void*       mapping;
 
     fmap->modtype = mt;
+    fmap->ops = &pe_file_map_ops;
+    fmap->alternate = NULL;
     fmap->u.pe.hMap = CreateFileMappingW(file, NULL, PAGE_READONLY, 0, 0, NULL);
     if (fmap->u.pe.hMap == 0) return FALSE;
     fmap->u.pe.full_count = 0;
@@ -285,29 +320,6 @@ error:
     return FALSE;
 }
 
-/******************************************************************
- *		pe_unmap_file
- *
- * Unmaps an PE file from memory (previously mapped with pe_map_file)
- */
-static void pe_unmap_file(struct image_file_map* fmap)
-{
-    if (fmap->u.pe.hMap != 0)
-    {
-        struct image_section_map  ism;
-        ism.fmap = fmap;
-        for (ism.sidx = 0; ism.sidx < fmap->u.pe.ntheader.FileHeader.NumberOfSections; ism.sidx++)
-        {
-            pe_unmap_section(&ism);
-        }
-        while (fmap->u.pe.full_count) pe_unmap_full(fmap);
-        HeapFree(GetProcessHeap(), 0, fmap->u.pe.sect);
-        HeapFree(GetProcessHeap(), 0, (void*)fmap->u.pe.strtable); /* FIXME ugly (see pe_map_file) */
-        CloseHandle(fmap->u.pe.hMap);
-        fmap->u.pe.hMap = NULL;
-    }
-}
-
 /******************************************************************
  *		pe_map_directory
  *
@@ -329,7 +341,7 @@ const char* pe_map_directory(struct module* module, int dirno, DWORD* size)
 
 static void pe_module_remove(struct process* pcs, struct module_format* modfmt)
 {
-    pe_unmap_file(&modfmt->u.pe_info->fmap);
+    image_unmap_file(&modfmt->u.pe_info->fmap);
     HeapFree(GetProcessHeap(), 0, modfmt);
 }
 
@@ -536,7 +548,7 @@ static BOOL pe_load_dbg_file(const struct process* pcs, struct module* module,
 
     TRACE("Processing DBG file %s\n", debugstr_a(dbg_name));
 
-    if (path_find_symbol_file(pcs, module, dbg_name, NULL, timestamp, 0, tmp, &module->module.DbgUnmatched) &&
+    if (path_find_symbol_file(pcs, module, dbg_name, DMT_DBG, NULL, timestamp, 0, tmp, &module->module.DbgUnmatched) &&
         (hFile = CreateFileW(tmp, GENERIC_READ, FILE_SHARE_READ, NULL,
                              OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)) != INVALID_HANDLE_VALUE &&
         ((hMap = CreateFileMappingW(hFile, NULL, PAGE_READONLY, 0, 0, NULL)) != 0) &&
@@ -711,7 +723,8 @@ BOOL pe_load_debug_info(const struct process* pcs, struct module* module)
 
     if (!(dbghelp_options & SYMOPT_PUBLICS_ONLY))
     {
-        ret = pe_load_stabs(pcs, module);
+        ret = image_check_alternate(&module->format_info[DFI_PE]->u.pe_info->fmap, module);
+        ret = pe_load_stabs(pcs, module) || ret;
         ret = pe_load_dwarf(module) || ret;
         ret = pe_load_msc_debug_info(pcs, module) || ret;
         ret = ret || pe_load_coff_symbol_table(module); /* FIXME */
@@ -727,68 +740,23 @@ BOOL pe_load_debug_info(const struct process* pcs, struct module* module)
     return ret;
 }
 
-static WCHAR *find_builtin_pe(const WCHAR *path, HANDLE *file)
+struct builtin_search
 {
-    const WCHAR *base_name;
-    size_t len, i;
-    WCHAR *buf;
+    WCHAR *path;
+    struct image_file_map fmap;
+};
 
-    static const WCHAR winebuilddirW[] = {'W','I','N','E','B','U','I','L','D','D','I','R',0};
-    static const WCHAR winedlldirW[] = {'W','I','N','E','D','L','L','D','I','R','%','u',0};
+static BOOL search_builtin_pe(void *param, HANDLE handle, const WCHAR *path)
+{
+    struct builtin_search *search = param;
+    size_t size;
 
-    if ((base_name = strrchrW(path, '\\'))) base_name++;
-    else base_name = path;
+    if (!pe_map_file(handle, &search->fmap, DMT_PE)) return FALSE;
 
-    if ((len = GetEnvironmentVariableW(winebuilddirW, NULL, 0)))
-    {
-        WCHAR *p, *end;
-        const WCHAR dllsW[] = { '\\','d','l','l','s','\\' };
-        const WCHAR programsW[] = { '\\','p','r','o','g','r','a','m','s','\\' };
-        const WCHAR dot_dllW[] = {'.','d','l','l',0};
-        const WCHAR dot_exeW[] = {'.','e','x','e',0};
-
-        if (!(buf = heap_alloc((len + 8 + 2 * lstrlenW(base_name)) * sizeof(WCHAR)))) return NULL;
-        end = buf + GetEnvironmentVariableW(winebuilddirW, buf, len);
-
-        memcpy(end, dllsW, sizeof(dllsW));
-        strcpyW(end + ARRAY_SIZE(dllsW), base_name);
-        if ((p = strchrW(end, '.')) && !lstrcmpW(p, dot_dllW)) *p = 0;
-        p = end + strlenW(end);
-        *p++ = '\\';
-        strcpyW(p, base_name);
-        *file = CreateFileW(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-        if (*file != INVALID_HANDLE_VALUE) return buf;
-
-        memcpy(end, programsW, sizeof(programsW));
-        end += ARRAY_SIZE(programsW);
-        strcpyW(end, base_name);
-        if ((p = strchrW(end, '.')) && !lstrcmpW(p, dot_exeW)) *p = 0;
-        p = end + strlenW(end);
-        *p++ = '\\';
-        strcpyW(p, base_name);
-        *file = CreateFileW(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-        if (*file != INVALID_HANDLE_VALUE) return buf;
-
-        heap_free(buf);
-    }
-
-    for (i = 0;; i++)
-    {
-        WCHAR name[64];
-        sprintfW(name, winedlldirW, i);
-        if (!(len = GetEnvironmentVariableW(name, NULL, 0))) break;
-        if (!(buf = heap_alloc((len + lstrlenW(base_name) + 2) * sizeof(WCHAR)))) return NULL;
-
-        GetEnvironmentVariableW(name, buf, len);
-        buf[len++] = '\\';
-        strcpyW(buf + len, base_name);
-        *file = CreateFileW(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
-        if (*file != INVALID_HANDLE_VALUE) return buf;
-
-        heap_free(buf);
-    }
-
-    return NULL;
+    size = (lstrlenW(path) + 1) * sizeof(WCHAR);
+    if ((search->path = heap_alloc(size)))
+        memcpy(search->path, path, size);
+    return TRUE;
 }
 
 /******************************************************************
@@ -820,18 +788,12 @@ struct module* pe_load_native_module(struct process* pcs, const WCHAR* name,
     modfmt->u.pe_info = (struct pe_module_info*)(modfmt + 1);
     if (pe_map_file(hFile, &modfmt->u.pe_info->fmap, DMT_PE))
     {
-        WCHAR *builtin_path = NULL;
-        HANDLE builtin_module;
-        if (modfmt->u.pe_info->fmap.u.pe.builtin && (builtin_path = find_builtin_pe(loaded_name, &builtin_module)))
+        struct builtin_search builtin = { NULL };
+        if (modfmt->u.pe_info->fmap.u.pe.builtin && search_dll_path(loaded_name, search_builtin_pe, &builtin))
         {
-            struct image_file_map builtin_fmap;
-            if (pe_map_file(builtin_module, &builtin_fmap, DMT_PE))
-            {
-                TRACE("reloaded %s from %s\n", debugstr_w(loaded_name), debugstr_w(builtin_path));
-                pe_unmap_file(&modfmt->u.pe_info->fmap);
-                modfmt->u.pe_info->fmap = builtin_fmap;
-            }
-            CloseHandle(builtin_module);
+            TRACE("reloaded %s from %s\n", debugstr_w(loaded_name), debugstr_w(builtin.path));
+            image_unmap_file(&modfmt->u.pe_info->fmap);
+            modfmt->u.pe_info->fmap = builtin.fmap;
         }
         if (!base) base = modfmt->u.pe_info->fmap.u.pe.ntheader.OptionalHeader.ImageBase;
         if (!size) size = modfmt->u.pe_info->fmap.u.pe.ntheader.OptionalHeader.SizeOfImage;
@@ -841,7 +803,7 @@ struct module* pe_load_native_module(struct process* pcs, const WCHAR* name,
                             modfmt->u.pe_info->fmap.u.pe.ntheader.OptionalHeader.CheckSum);
         if (module)
         {
-            module->real_path = builtin_path;
+            module->real_path = builtin.path;
             modfmt->module = module;
             modfmt->remove = pe_module_remove;
             modfmt->loc_compute = NULL;
@@ -856,8 +818,8 @@ struct module* pe_load_native_module(struct process* pcs, const WCHAR* name,
         else
         {
             ERR("could not load the module '%s'\n", debugstr_w(loaded_name));
-            heap_free(module->real_path);
-            pe_unmap_file(&modfmt->u.pe_info->fmap);
+            heap_free(builtin.path);
+            image_unmap_file(&modfmt->u.pe_info->fmap);
         }
     }
     if (!module) HeapFree(GetProcessHeap(), 0, modfmt);
