@@ -41,6 +41,7 @@
 #include <sys/mman.h>
 #endif
 #include <limits.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
@@ -111,11 +112,7 @@ struct stab_nlist
     unsigned char       n_type;
     char                n_other;
     short               n_desc;
-#if defined(__APPLE__) && defined(_WIN64)
-    unsigned long       n_value;
-#else
     unsigned            n_value;
-#endif
 };
 
 static void stab_strcpy(char* dest, int sz, const char* source)
@@ -156,7 +153,7 @@ static void stab_strcpy(char* dest, int sz, const char* source)
 typedef struct
 {
    char*		name;
-   unsigned long	value;
+   ULONG_PTR            value;
    struct symt**        vector;
    int			nrofentries;
 } include_def;
@@ -172,7 +169,7 @@ static struct symt**            cu_vector = NULL;
 static int 		        cu_nrofentries = 0;
 static struct symt_basic*       stabs_basic[36];
 
-static int stabs_new_include(const char* file, unsigned long val)
+static int stabs_new_include(const char* file, ULONG_PTR val)
 {
     if (num_include_def == num_alloc_include_def)
     {
@@ -197,7 +194,7 @@ static int stabs_new_include(const char* file, unsigned long val)
     return num_include_def++;
 }
 
-static int stabs_find_include(const char* file, unsigned long val)
+static int stabs_find_include(const char* file, ULONG_PTR val)
 {
     int		i;
 
@@ -251,7 +248,7 @@ static void stabs_free_includes(void)
     cu_nrofentries = 0;
 }
 
-static struct symt** stabs_find_ref(long filenr, long subnr)
+static struct symt** stabs_find_ref(LONG_PTR filenr, LONG_PTR subnr)
 {
     struct symt**       ret;
 
@@ -297,7 +294,7 @@ static struct symt** stabs_find_ref(long filenr, long subnr)
 
 static struct symt** stabs_read_type_enum(const char** x)
 {
-    long        filenr, subnr;
+    LONG_PTR    filenr, subnr;
     const char* iter;
     char*       end;
 
@@ -340,7 +337,7 @@ struct ParseTypedefData
 #ifdef PTS_DEBUG
 static void stabs_pts_push(struct ParseTypedefData* ptd, unsigned line)
 {
-    assert(ptd->err_idx < sizeof(ptd->errors) / sizeof(ptd->errors[0]));
+    assert(ptd->err_idx < ARRAY_SIZE(ptd->errors));
     ptd->errors[ptd->err_idx].line = line;
     ptd->errors[ptd->err_idx].ptr = ptd->ptr;
     ptd->err_idx++;
@@ -352,7 +349,7 @@ static void stabs_pts_push(struct ParseTypedefData* ptd, unsigned line)
 
 static int stabs_get_basic(struct ParseTypedefData* ptd, unsigned basic, struct symt** symt)
 {
-    PTS_ABORTIF(ptd, basic >= sizeof(stabs_basic) / sizeof(stabs_basic[0]));
+    PTS_ABORTIF(ptd, basic >= ARRAY_SIZE(stabs_basic));
 
     if (!stabs_basic[basic])
     {
@@ -421,7 +418,7 @@ static int stabs_pts_read_id(struct ParseTypedefData* ptd)
     return -1;
 }
 
-static int stabs_pts_read_number(struct ParseTypedefData* ptd, long* v)
+static int stabs_pts_read_number(struct ParseTypedefData* ptd, LONG_PTR* v)
 {
     char*	last;
 
@@ -432,7 +429,7 @@ static int stabs_pts_read_number(struct ParseTypedefData* ptd, long* v)
 }
 
 static int stabs_pts_read_type_reference(struct ParseTypedefData* ptd,
-                                         long* filenr, long* subnr)
+                                         LONG_PTR* filenr, LONG_PTR* subnr)
 {
     if (*ptd->ptr == '(')
     {
@@ -607,7 +604,7 @@ static inline int stabs_pts_read_method_info(struct ParseTypedefData* ptd)
         ptd->ptr++;
         if (mthd == '*')
         {
-            long int            ofs;
+            LONG_PTR            ofs;
 
             PTS_ABORTIF(ptd, stabs_pts_read_number(ptd, &ofs) == -1);
             PTS_ABORTIF(ptd, *ptd->ptr++ != ';');
@@ -623,7 +620,7 @@ static inline int stabs_pts_read_method_info(struct ParseTypedefData* ptd)
 static inline int stabs_pts_read_aggregate(struct ParseTypedefData* ptd, 
                                            struct symt_udt* sdt)
 {
-    long        	sz, ofs;
+    LONG_PTR    	sz, ofs;
     struct symt*        adt;
     struct symt*        dt = NULL;
     int			idx;
@@ -634,7 +631,7 @@ static inline int stabs_pts_read_aggregate(struct ParseTypedefData* ptd,
     doadd = symt_set_udt_size(ptd->module, sdt, sz);
     if (*ptd->ptr == '!') /* C++ inheritance */
     {
-        long     num_classes;
+        LONG_PTR num_classes;
 
         ptd->ptr++;
         PTS_ABORTIF(ptd, stabs_pts_read_number(ptd, &num_classes) == -1);
@@ -681,7 +678,7 @@ static inline int stabs_pts_read_aggregate(struct ParseTypedefData* ptd,
 
         if (ptd->ptr[0] == '$' && ptd->ptr[1] == 'v')
         {
-            long        x;
+            LONG_PTR    x;
 
             if (ptd->ptr[2] == 'f')
             {
@@ -765,7 +762,7 @@ static inline int stabs_pts_read_aggregate(struct ParseTypedefData* ptd,
 static inline int stabs_pts_read_enum(struct ParseTypedefData* ptd, 
                                       struct symt_enum* edt)
 {
-    long        value;
+    LONG_PTR    value;
     int		idx;
 
     while (*ptd->ptr != ';')
@@ -784,7 +781,7 @@ static inline int stabs_pts_read_enum(struct ParseTypedefData* ptd,
 static inline int stabs_pts_read_array(struct ParseTypedefData* ptd,
                                        struct symt** adt)
 {
-    long                lo, hi;
+    LONG_PTR            lo, hi;
     struct symt*        range_dt;
     struct symt*        base_dt;
 
@@ -809,10 +806,10 @@ static int stabs_pts_read_type_def(struct ParseTypedefData* ptd, const char* typ
                                    struct symt** ret_dt)
 {
     int			idx;
-    long		sz = -1;
+    LONG_PTR		sz = -1;
     struct symt*	new_dt = NULL;     /* newly created data type */
     struct symt*	ref_dt;		   /* referenced data type (pointer...) */
-    long		filenr1, subnr1, tmp;
+    LONG_PTR		filenr1, subnr1, tmp;
 
     /* things are a bit complicated because of the way the typedefs are stored inside
      * the file, because addresses can change when realloc is done, so we must call
@@ -973,8 +970,8 @@ static int stabs_pts_read_type_def(struct ParseTypedefData* ptd, const char* typ
             break;
         case 'R':
             {
-                long    type, len, unk;
-                int     basic;
+                LONG_PTR type, len, unk;
+                int      basic;
                 
                 PTS_ABORTIF(ptd, stabs_pts_read_number(ptd, &type) == -1);
                 PTS_ABORTIF(ptd, *ptd->ptr++ != ';');	/* ';' */
@@ -1125,8 +1122,8 @@ struct pending_line
 {
     int                 source_idx;
     int                 line_num;
-    unsigned long       offset;
-    unsigned long       load_offset;
+    ULONG_PTR           offset;
+    ULONG_PTR           load_offset;
 };
 
 struct pending_object
@@ -1178,8 +1175,8 @@ static inline void pending_add_var(struct pending_list* pending, const char* nam
 }
 
 static inline void pending_add_line(struct pending_list* pending, int source_idx,
-                                    int line_num, unsigned long offset,
-                                    unsigned long load_offset)
+                                    int line_num, ULONG_PTR offset,
+                                    ULONG_PTR load_offset)
 {
     pending_make_room(pending);
     pending->objs[pending->num].tag = PENDING_LINE;
@@ -1228,7 +1225,7 @@ static void pending_flush(struct pending_list* pending, struct module* module,
  *   function (assuming that current function ends where next function starts)
  */
 static void stabs_finalize_function(struct module* module, struct symt_function* func,
-                                    unsigned long size)
+                                    ULONG_PTR size)
 {
     IMAGEHLP_LINE64     il;
     struct location     loc;
@@ -1264,8 +1261,8 @@ static inline void stabbuf_append(char **buf, unsigned *buf_size, const char *st
     strcpy(*buf+buf_len, str);
 }
 
-BOOL stabs_parse(struct module* module, unsigned long load_offset, 
-                 const void* pv_stab_ptr, int stablen,
+BOOL stabs_parse(struct module* module, ULONG_PTR load_offset,
+                 const char* pv_stab_ptr, int stablen,
                  const char* strs, int strtablen,
                  stabs_def_cb callback, void* user)
 {
@@ -1278,7 +1275,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
     const char*                 ptr;
     char*                       stabbuff;
     unsigned int                stabbufflen;
-    const struct stab_nlist*    stab_ptr = pv_stab_ptr;
+    const struct stab_nlist*    stab_ptr;
     const char*                 strs_end;
     int                         strtabinc;
     char                        symname[4096];
@@ -1290,8 +1287,14 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
     BOOL                        ret = TRUE;
     struct location             loc;
     unsigned char               type;
+    size_t                      stabsize = sizeof(struct stab_nlist);
+    uint64_t                    n_value;
 
-    nstab = stablen / sizeof(struct stab_nlist);
+#ifdef __APPLE__
+    if (module->process->is_64bit)
+        stabsize = sizeof(struct nlist_64);
+#endif
+    nstab = stablen / stabsize;
     strs_end = strs + strtablen;
 
     memset(stabs_basic, 0, sizeof(stabs_basic));
@@ -1307,8 +1310,14 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
 
     strtabinc = 0;
     stabbuff[0] = '\0';
-    for (i = 0; i < nstab; i++, stab_ptr++)
+    for (i = 0; i < nstab; i++)
     {
+        stab_ptr = (struct stab_nlist *)(pv_stab_ptr + i * stabsize);
+        n_value = stab_ptr->n_value;
+#ifdef __APPLE__
+        if (module->process->is_64bit)
+            n_value = ((struct nlist_64 *)stab_ptr)->n_value;
+#endif
         ptr = strs + stab_ptr->n_strx;
         if ((ptr > strs_end) || (ptr + strlen(ptr) > strs_end))
         {
@@ -1382,7 +1391,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
             stab_strcpy(symname, sizeof(symname), ptr);
             loc.kind = loc_absolute;
             loc.reg = 0;
-            loc.offset = load_offset + stab_ptr->n_value;
+            loc.offset = load_offset + n_value;
             symt_new_global_variable(module, compiland, symname, TRUE /* FIXME */,
                                      loc, 0, stabs_parse_type(ptr));
             break;
@@ -1392,7 +1401,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
             stab_strcpy(symname, sizeof(symname), ptr);
             loc.kind = loc_absolute;
             loc.reg = 0;
-            loc.offset = load_offset + stab_ptr->n_value;
+            loc.offset = load_offset + n_value;
             symt_new_global_variable(module, compiland, symname, TRUE /* FIXME */,
                                      loc, 0, stabs_parse_type(ptr));
             break;
@@ -1400,14 +1409,14 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
             if (curr_func)
             {
                 block = symt_open_func_block(module, curr_func, block,
-                                             stab_ptr->n_value, 0);
+                                             n_value, 0);
                 pending_flush(&pending_block, module, curr_func, block);
             }
             break;
         case N_RBRAC:
             if (curr_func)
                 block = symt_close_func_block(module, curr_func, block,
-                                              stab_ptr->n_value);
+                                              n_value);
             break;
         case N_PSYM:
             /* These are function parameters. */
@@ -1417,9 +1426,9 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
                 stab_strcpy(symname, sizeof(symname), ptr);
                 loc.kind = loc_regrel;
                 loc.reg = dbghelp_current_cpu->frame_regno;
-                loc.offset = stab_ptr->n_value;
+                loc.offset = n_value;
                 symt_add_func_local(module, curr_func,
-                                    (int)stab_ptr->n_value >= 0 ? DataIsParam : DataIsLocal,
+                                    (int)n_value >= 0 ? DataIsParam : DataIsLocal,
                                     &loc, NULL, param_type, symname);
                 symt_add_function_signature_parameter(module, 
                                                       (struct symt_function_signature*)curr_func->type, 
@@ -1433,7 +1442,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
                 loc.kind = loc_register;
                 loc.offset = 0;
 
-                switch (stab_ptr->n_value)
+                switch (n_value)
                 {
                 case  0: loc.reg = CV_REG_EAX; break;
                 case  1: loc.reg = CV_REG_ECX; break;
@@ -1451,7 +1460,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
                 case 16:
                 case 17:
                 case 18:
-                case 19: loc.reg = CV_REG_ST0 + stab_ptr->n_value - 12; break;
+                case 19: loc.reg = CV_REG_ST0 + n_value - 12; break;
                 case 21:
                 case 22:
                 case 23:
@@ -1459,7 +1468,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
                 case 25:
                 case 26:
                 case 27:
-                case 28: loc.reg = CV_REG_XMM0 + stab_ptr->n_value - 21; break;
+                case 28: loc.reg = CV_REG_XMM0 + n_value - 21; break;
                 case 29:
                 case 30:
                 case 31:
@@ -1467,9 +1476,9 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
                 case 33:
                 case 34:
                 case 35:
-                case 36: loc.reg = CV_REG_MM0 + stab_ptr->n_value - 29; break;
+                case 36: loc.reg = CV_REG_MM0 + n_value - 29; break;
                 default:
-                    FIXME("Unknown register value (%lu)\n", (unsigned long)stab_ptr->n_value);
+                    FIXME("Unknown register value (%lu)\n", (ULONG_PTR)n_value);
                     loc.reg = CV_REG_NONE;
                     break;
                 }
@@ -1492,7 +1501,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
             /* These are local variables */
             loc.kind = loc_regrel;
             loc.reg = dbghelp_current_cpu->frame_regno;
-            loc.offset = stab_ptr->n_value;
+            loc.offset = n_value;
             if (curr_func != NULL) pending_add_var(&pending_block, ptr, DataIsLocal, &loc);
             break;
         case N_SLINE:
@@ -1503,14 +1512,14 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
             assert(source_idx >= 0);
             if (curr_func != NULL)
             {
-                unsigned long offset = stab_ptr->n_value;
+                ULONG_PTR offset = n_value;
                 if (module->type == DMT_MACHO)
                     offset -= curr_func->address - load_offset;
                 symt_add_func_line(module, curr_func, source_idx, 
                                    stab_ptr->n_desc, offset);
             }
             else pending_add_line(&pending_func, source_idx, stab_ptr->n_desc,
-                                  stab_ptr->n_value, load_offset);
+                                  n_value, load_offset);
             break;
         case N_FUN:
             /*
@@ -1537,13 +1546,13 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
                      * and offset of last function
                      */
                     stabs_finalize_function(module, curr_func, 
-                                            stab_ptr->n_value ?
-                                                (load_offset + stab_ptr->n_value - curr_func->address) : 0);
+                                            n_value ?
+                                                (load_offset + n_value - curr_func->address) : 0);
                 }
                 func_type = symt_new_function_signature(module, 
                                                         stabs_parse_type(ptr), -1);
                 curr_func = symt_new_function(module, compiland, symname, 
-                                              load_offset + stab_ptr->n_value, 0,
+                                              load_offset + n_value, 0,
                                               &func_type->symt);
                 pending_flush(&pending_func, module, curr_func, NULL);
             }
@@ -1552,7 +1561,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
                 /* some versions of GCC to use a N_FUN "" to mark the end of a function
                  * and n_value contains the size of the func
                  */
-                stabs_finalize_function(module, curr_func, stab_ptr->n_value);
+                stabs_finalize_function(module, curr_func, n_value);
                 curr_func = NULL;
             }
             break;
@@ -1594,7 +1603,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
             break;
         case N_UNDF:
             strs += strtabinc;
-            strtabinc = stab_ptr->n_value;
+            strtabinc = n_value;
             /* I'm not sure this is needed, so trace it before we obsolete it */
             if (curr_func)
             {
@@ -1607,8 +1616,8 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
             /* Ignore this. We don't care what it points to. */
             break;
         case N_BINCL:
-            stabs_add_include(stabs_new_include(ptr, stab_ptr->n_value));
-            assert(incl_stk < (int)(sizeof(incl) / sizeof(incl[0])) - 1);
+            stabs_add_include(stabs_new_include(ptr, n_value));
+            assert(incl_stk < (int) ARRAY_SIZE(incl) - 1);
             incl[++incl_stk] = source_idx;
             source_idx = source_new(module, NULL, ptr);
             break;
@@ -1617,9 +1626,9 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
             source_idx = incl[incl_stk--];
             break;
 	case N_EXCL:
-            if (stabs_add_include(stabs_find_include(ptr, stab_ptr->n_value)) < 0)
+            if (stabs_add_include(stabs_find_include(ptr, n_value)) < 0)
             {
-                ERR("Excluded header not found (%s,%ld)\n", ptr, (unsigned long)stab_ptr->n_value);
+                ERR("Excluded header not found (%s,%ld)\n", ptr, (ULONG_PTR)n_value);
                 module_reset_debug_info(module);
                 ret = FALSE;
                 goto done;
@@ -1656,7 +1665,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
                 if (*ptr == '_') ptr++;
                 stab_strcpy(symname, sizeof(symname), ptr);
 
-                callback(module, load_offset, symname, stab_ptr->n_value,
+                callback(module, load_offset, symname, n_value,
                          is_public, is_global, stab_ptr->n_other, compiland, user);
             }
             break;
@@ -1666,7 +1675,7 @@ BOOL stabs_parse(struct module* module, unsigned long load_offset,
         }
         stabbuff[0] = '\0';
         TRACE("0x%02x %lx %s\n",
-              stab_ptr->n_type, (unsigned long)stab_ptr->n_value, debugstr_a(strs + stab_ptr->n_strx));
+              stab_ptr->n_type, (ULONG_PTR)n_value, debugstr_a(strs + stab_ptr->n_strx));
     }
     module->module.SymType = SymDia;
     module->module.CVSig = 'S' | ('T' << 8) | ('A' << 16) | ('B' << 24);
