@@ -167,7 +167,9 @@ static ULONG WINAPI LinuxInputEffectImpl_AddRef(
 	LPDIRECTINPUTEFFECT iface)
 {
     LinuxInputEffectImpl *This = impl_from_IDirectInputEffect(iface);
-    return InterlockedIncrement(&(This->ref));
+    ULONG ref = InterlockedIncrement(&This->ref);
+    TRACE( "(%p) ref %d\n", This, ref );
+    return ref;
 }
 
 static HRESULT WINAPI LinuxInputEffectImpl_Download(
@@ -487,8 +489,6 @@ static HRESULT WINAPI LinuxInputEffectImpl_SetParameters(
 
     TRACE("(this=%p,%p,%d)\n", This, peff, dwFlags);
 
-    dump_DIEFFECT(peff, &This->guid, dwFlags);
-
     if ((dwFlags & ~DIEP_NORESTART & ~DIEP_NODOWNLOAD & ~DIEP_START) == 0) {
 	/* set everything */
 	dwFlags = DIEP_AXES | DIEP_DIRECTION | DIEP_DURATION | DIEP_ENVELOPE |
@@ -496,7 +496,12 @@ static HRESULT WINAPI LinuxInputEffectImpl_SetParameters(
 	    DIEP_TRIGGERREPEATINTERVAL | DIEP_TYPESPECIFICPARAMS;
     }
 
+    dump_DIEFFECT(peff, &This->guid, dwFlags);
+
     if (dwFlags & DIEP_AXES) {
+	if (!(peff->rgdwAxes))
+	    return DIERR_INVALIDPARAM;
+
 	/* the linux input effect system only supports one or two axes */
 	if (peff->cAxes > 2)
 	    return DIERR_INVALIDPARAM;
@@ -509,6 +514,9 @@ static HRESULT WINAPI LinuxInputEffectImpl_SetParameters(
      * different opinions about which way direction "0" is.  directx has 0 along the x
      * axis (left), linux has it along the y axis (down). */ 
     if (dwFlags & DIEP_DIRECTION) {
+	if (!(peff->rglDirection))
+	    return DIERR_INVALIDPARAM;
+
 	if (peff->cAxes == 1) {
 	    if (peff->dwFlags & DIEFF_CARTESIAN) {
 		if (dwFlags & DIEP_AXES) {
@@ -627,7 +635,7 @@ static HRESULT WINAPI LinuxInputEffectImpl_SetParameters(
     if (dwFlags & DIEP_TYPESPECIFICPARAMS)
     {
         if (!(peff->lpvTypeSpecificParams))
-            return DIERR_INCOMPLETEEFFECT;
+            return DIERR_INVALIDPARAM;
 
         if (type == DIEFT_PERIODIC)
         {
@@ -759,6 +767,8 @@ static ULONG WINAPI LinuxInputEffectImpl_Release(LPDIRECTINPUTEFFECT iface)
     LinuxInputEffectImpl *This = impl_from_IDirectInputEffect(iface);
     ULONG ref = InterlockedDecrement(&(This->ref));
 
+    TRACE( "(%p) ref %d\n", This, ref );
+
     if (ref == 0)
     {
         LinuxInputEffectImpl_Stop(iface);
