@@ -151,11 +151,17 @@ static HRESULT WINAPI IDirectMusicStyle8Impl_EnumBand(IDirectMusicStyle8 *iface,
 }
 
 static HRESULT WINAPI IDirectMusicStyle8Impl_GetDefaultBand(IDirectMusicStyle8 *iface,
-        IDirectMusicBand **ppBand)
+        IDirectMusicBand **band)
 {
-        IDirectMusicStyle8Impl *This = impl_from_IDirectMusicStyle8(iface);
-	FIXME("(%p, %p): stub\n", This, ppBand);
-	return S_OK;
+    IDirectMusicStyle8Impl *This = impl_from_IDirectMusicStyle8(iface);
+    FIXME("(%p, %p): stub\n", This, band);
+
+    if (!band)
+        return E_POINTER;
+
+    *band = NULL;
+
+    return S_FALSE;
 }
 
 static HRESULT WINAPI IDirectMusicStyle8Impl_EnumMotif(IDirectMusicStyle8 *iface, DWORD index,
@@ -778,23 +784,21 @@ static HRESULT parse_style_form(IDirectMusicStyle8Impl *This, DMUS_PRIVATE_CHUNK
 	ListCount[0] = 0;
 	switch (Chunk.fccID) {
 	case DMUS_FOURCC_BAND_FORM: { 
-	  LPSTREAM pClonedStream = NULL;
+          ULARGE_INTEGER save;
           struct style_band *pNewBand;
 
 	  TRACE_(dmfile)(": BAND RIFF\n");
-	  
-	  IStream_Clone (pStm, &pClonedStream);
-	    
+
+          /* Can be application provided IStream without Clone method */
 	  liMove.QuadPart = 0;
 	  liMove.QuadPart -= sizeof(FOURCC) + (sizeof(FOURCC)+sizeof(DWORD));
-	  IStream_Seek (pClonedStream, liMove, STREAM_SEEK_CUR, NULL);
+          IStream_Seek(pStm, liMove, STREAM_SEEK_CUR, &save);
 
-          hr = load_band(pClonedStream, &pBand);
+          hr = load_band(pStm, &pBand);
 	  if (FAILED(hr)) {
 	    ERR(": could not load track\n");
 	    return hr;
 	  }
-	  IStream_Release (pClonedStream);
 
           pNewBand = heap_alloc_zero(sizeof(*pNewBand));
 	  if (NULL == pNewBand) {
@@ -808,9 +812,9 @@ static HRESULT parse_style_form(IDirectMusicStyle8Impl *This, DMUS_PRIVATE_CHUNK
 	  IDirectMusicTrack_Release(pBand); pBand = NULL;  /* now we can release it as it's inserted */
 	
 	  /** now safely move the cursor */
-	  liMove.QuadPart = ListSize[0];
-	  IStream_Seek (pStm, liMove, STREAM_SEEK_CUR, NULL);
-	  
+          liMove.QuadPart = save.QuadPart - liMove.QuadPart + ListSize[0];
+          IStream_Seek(pStm, liMove, STREAM_SEEK_SET, NULL);
+
 	  break;
 	}
 	default: {
