@@ -30,7 +30,6 @@
 #include "windef.h"
 #include "winbase.h"
 #include "winuser.h"
-#include "winternl.h"
 #include "mmddk.h"
 #include "wingdi.h"
 #include "mmreg.h"
@@ -105,10 +104,12 @@ static void _dump_DSBCAPS(DWORD xmask) {
         FE(DSBCAPS_CTRLPAN)
         FE(DSBCAPS_CTRLVOLUME)
         FE(DSBCAPS_CTRLPOSITIONNOTIFY)
+        FE(DSBCAPS_CTRLFX)
         FE(DSBCAPS_STICKYFOCUS)
         FE(DSBCAPS_GLOBALFOCUS)
         FE(DSBCAPS_GETCURRENTPOSITION2)
         FE(DSBCAPS_MUTE3DATMAXDISTANCE)
+        FE(DSBCAPS_LOCDEFER)
 #undef FE
     };
     unsigned int     i;
@@ -179,7 +180,7 @@ static HRESULT DirectSoundDevice_Create(DirectSoundDevice ** ppDevice)
     InitializeCriticalSection(&(device->mixlock));
     device->mixlock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": DirectSoundDevice.mixlock");
 
-    RtlInitializeResource(&(device->buffer_list_lock));
+    InitializeSRWLock(&device->buffer_list_lock);
 
    *ppDevice = device;
 
@@ -236,7 +237,6 @@ static ULONG DirectSoundDevice_Release(DirectSoundDevice * device)
         HeapFree(GetProcessHeap(), 0, device->tmp_buffer);
         HeapFree(GetProcessHeap(), 0, device->cp_buffer);
         HeapFree(GetProcessHeap(), 0, device->buffer);
-        RtlDeleteResource(&device->buffer_list_lock);
         device->mixlock.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&device->mixlock);
         HeapFree(GetProcessHeap(),0,device);
@@ -449,6 +449,12 @@ static HRESULT DirectSoundDevice_CreateSoundBuffer(
             return DSERR_INVALIDPARAM;
         }
 
+        if (dsbd->dwFlags & DSBCAPS_CTRLFX)
+        {
+            WARN("Invalid parameter DSBCAPS_CTRLFX\n");
+            return DSERR_INVALIDPARAM;
+        }
+
         if (device->primary) {
             WARN("Primary Buffer already created\n");
             IDirectSoundBuffer8_AddRef(&device->primary->IDirectSoundBuffer8_iface);
@@ -604,7 +610,7 @@ HRESULT DirectSoundDevice_AddBuffer(
 
     TRACE("(%p, %p)\n", device, pDSB);
 
-    RtlAcquireResourceExclusive(&(device->buffer_list_lock), TRUE);
+    AcquireSRWLockExclusive(&device->buffer_list_lock);
 
     if (device->buffers)
         newbuffers = HeapReAlloc(GetProcessHeap(),0,device->buffers,sizeof(IDirectSoundBufferImpl*)*(device->nrofbuffers+1));
@@ -621,7 +627,7 @@ HRESULT DirectSoundDevice_AddBuffer(
         hr = DSERR_OUTOFMEMORY;
     }
 
-    RtlReleaseResource(&(device->buffer_list_lock));
+    ReleaseSRWLockExclusive(&device->buffer_list_lock);
 
     return hr;
 }
@@ -636,7 +642,7 @@ void DirectSoundDevice_RemoveBuffer(DirectSoundDevice * device, IDirectSoundBuff
 
     TRACE("(%p, %p)\n", device, pDSB);
 
-    RtlAcquireResourceExclusive(&(device->buffer_list_lock), TRUE);
+    AcquireSRWLockExclusive(&device->buffer_list_lock);
 
     if (device->nrofbuffers == 1) {
         assert(device->buffers[0] == pDSB);
@@ -654,7 +660,7 @@ void DirectSoundDevice_RemoveBuffer(DirectSoundDevice * device, IDirectSoundBuff
     device->nrofbuffers--;
     TRACE("buffer count is now %d\n", device->nrofbuffers);
 
-    RtlReleaseResource(&(device->buffer_list_lock));
+    ReleaseSRWLockExclusive(&device->buffer_list_lock);
 }
 
 /*******************************************************************************
@@ -861,14 +867,14 @@ static HRESULT WINAPI IDirectSound8Impl_SetCooperativeLevel(IDirectSound8 *iface
              level==DSSCL_PRIORITY ? "DSSCL_PRIORITY" : "DSSCL_EXCLUSIVE");
     }
 
-    RtlAcquireResourceExclusive(&device->buffer_list_lock, TRUE);
+    AcquireSRWLockExclusive(&device->buffer_list_lock);
     EnterCriticalSection(&device->mixlock);
     if ((level == DSSCL_WRITEPRIMARY) != (device->priolevel == DSSCL_WRITEPRIMARY))
         hr = DSOUND_ReopenDevice(device, level == DSSCL_WRITEPRIMARY);
     if (SUCCEEDED(hr))
         device->priolevel = level;
     LeaveCriticalSection(&device->mixlock);
-    RtlReleaseResource(&device->buffer_list_lock);
+    ReleaseSRWLockExclusive(&device->buffer_list_lock);
     return hr;
 }
 
