@@ -25,7 +25,7 @@
 #include "wine/heap.h"
 #include "wine/list.h"
 
-WINE_DEFAULT_DEBUG_CHANNEL(httpapi);
+WINE_DEFAULT_DEBUG_CHANNEL(http);
 
 static const WCHAR device_nameW[] = {'\\','D','e','v','i','c','e','\\','H','t','t','p','\\','R','e','q','Q','u','e','u','e',0};
 
@@ -62,7 +62,7 @@ ULONG WINAPI HttpInitialize(HTTPAPI_VERSION version, ULONG flags, void *reserved
     if (flags & ~HTTP_INITIALIZE_SERVER)
     {
         FIXME("Unhandled flags %#x.\n", flags);
-        return ERROR_CALL_NOT_IMPLEMENTED;
+        return ERROR_SUCCESS;
     }
 
     if (!(manager = OpenSCManagerW(NULL, NULL, SC_MANAGER_CONNECT)))
@@ -284,12 +284,41 @@ ULONG WINAPI HttpRemoveUrl(HANDLE queue, const WCHAR *url)
 ULONG WINAPI HttpReceiveRequestEntityBody(HANDLE queue, HTTP_REQUEST_ID id, ULONG flags,
         void *buffer, ULONG size, ULONG *ret_size, OVERLAPPED *ovl)
 {
-    *ret_size = 0;
+    struct http_receive_body_params params =
+    {
+        .id = id,
+        .bits = sizeof(void *) * 8,
+    };
+    ULONG ret = ERROR_SUCCESS;
+    OVERLAPPED sync_ovl;
 
-    FIXME("stub: queue %p, id %s, flags %#x, buffer %p, size %#x, ret_size %p, ovl %p.\n",
+    TRACE("queue %p, id %s, flags %#x, buffer %p, size %#x, ret_size %p, ovl %p.\n",
             queue, wine_dbgstr_longlong(id), flags, buffer, size, ret_size, ovl);
 
-    return ERROR_HANDLE_EOF;
+    if (flags)
+        FIXME("Ignoring flags %#x.\n", flags);
+
+    if (!ovl)
+    {
+        sync_ovl.hEvent = CreateEventW(NULL, TRUE, FALSE, NULL);
+        ovl = &sync_ovl;
+    }
+
+    if (!DeviceIoControl(queue, IOCTL_HTTP_RECEIVE_BODY, &params, sizeof(params), buffer, size, ret_size, ovl))
+        ret = GetLastError();
+
+    if (ovl == &sync_ovl)
+    {
+        if (ret == ERROR_IO_PENDING)
+        {
+            ret = ERROR_SUCCESS;
+            if (!GetOverlappedResult(queue, ovl, ret_size, TRUE))
+                ret = GetLastError();
+        }
+        CloseHandle(sync_ovl.hEvent);
+    }
+
+    return ret;
 }
 
 /***********************************************************************
@@ -323,14 +352,17 @@ ULONG WINAPI HttpReceiveHttpRequest(HANDLE queue, HTTP_REQUEST_ID id, ULONG flag
         ovl = &sync_ovl;
     }
 
-    if (!DeviceIoControl(queue, IOCTL_HTTP_RECEIVE_REQUEST, &params, sizeof(params), request, size, NULL, ovl))
+    if (!DeviceIoControl(queue, IOCTL_HTTP_RECEIVE_REQUEST, &params, sizeof(params), request, size, ret_size, ovl))
         ret = GetLastError();
 
     if (ovl == &sync_ovl)
     {
-        ret = ERROR_SUCCESS;
-        if (!GetOverlappedResult(queue, ovl, ret_size, TRUE))
-            ret = GetLastError();
+        if (ret == ERROR_IO_PENDING)
+        {
+            ret = ERROR_SUCCESS;
+            if (!GetOverlappedResult(queue, ovl, ret_size, TRUE))
+                ret = GetLastError();
+        }
         CloseHandle(sync_ovl.hEvent);
     }
 
@@ -625,25 +657,29 @@ ULONG WINAPI HttpCloseUrlGroup(HTTP_URL_GROUP_ID id)
 ULONG WINAPI HttpSetUrlGroupProperty(HTTP_URL_GROUP_ID id, HTTP_SERVER_PROPERTY property, void *value, ULONG length)
 {
     struct url_group *group = get_url_group(id);
-    const HTTP_BINDING_INFO *info = value;
 
     TRACE("id %s, property %u, value %p, length %u.\n",
             wine_dbgstr_longlong(id), property, value, length);
 
-    if (property != HttpServerBindingProperty)
+    switch (property)
     {
-        FIXME("Unhandled property %u.\n", property);
-        return ERROR_CALL_NOT_IMPLEMENTED;
+        case HttpServerBindingProperty:
+        {
+            const HTTP_BINDING_INFO *info = value;
+
+            TRACE("Binding to queue %p.\n", info->RequestQueueHandle);
+            group->queue = info->RequestQueueHandle;
+            if (group->url)
+                add_url(group->queue, group->url, group->context);
+            return ERROR_SUCCESS;
+        }
+        case HttpServerLoggingProperty:
+            WARN("Ignoring logging property.\n");
+            return ERROR_SUCCESS;
+        default:
+            FIXME("Unhandled property %u.\n", property);
+            return ERROR_CALL_NOT_IMPLEMENTED;
     }
-
-    TRACE("Binding to queue %p.\n", info->RequestQueueHandle);
-
-    group->queue = info->RequestQueueHandle;
-
-    if (group->url)
-        add_url(group->queue, group->url, group->context);
-
-    return ERROR_SUCCESS;
 }
 
 /***********************************************************************
@@ -735,3 +771,37 @@ ULONG WINAPI HttpCloseRequestQueue(HANDLE handle)
         return GetLastError();
     return ERROR_SUCCESS;
 }
+
+/***********************************************************************
+ *        HttpSetRequestQueueProperty     (HTTPAPI.@)
+ */
+ULONG WINAPI HttpSetRequestQueueProperty(HANDLE queue, HTTP_SERVER_PROPERTY property,
+        void *value, ULONG length, ULONG reserved1, void *reserved2)
+{
+    FIXME("queue %p, property %u, value %p, length %u, reserved1 %#x, reserved2 %p, stub!\n",
+            queue, property, value, length, reserved1, reserved2);
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+/***********************************************************************
+ *        HttpSetServerSessionProperty     (HTTPAPI.@)
+ */
+ULONG WINAPI HttpSetServerSessionProperty(HTTP_SERVER_SESSION_ID id,
+        HTTP_SERVER_PROPERTY property, void *value, ULONG length)
+{
+    TRACE("id %s, property %u, value %p, length %u.\n",
+            wine_dbgstr_longlong(id), property, value, length);
+
+    switch (property)
+    {
+        case HttpServerQosProperty:
+        {
+            const HTTP_QOS_SETTING_INFO *info = value;
+            FIXME("Ignoring QoS setting %u.\n", info->QosType);
+            return ERROR_SUCCESS;
+        }
+        default:
+            FIXME("Unhandled property %u.\n", property);
+            return ERROR_CALL_NOT_IMPLEMENTED;
+    }
+}
