@@ -17,8 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
 
 #include <limits.h>
 #include <math.h>
@@ -542,16 +540,16 @@ static inline HRESULT date_to_string(DOUBLE time, BOOL show_offset, int offset,
         }
 
         if(!show_offset)
-            sprintfW(buf, formatNoOffsetW, week, month, day,
+            swprintf(buf, ARRAY_SIZE(buf), formatNoOffsetW, week, month, day,
                     (int)hour_from_time(time), (int)min_from_time(time),
                     (int)sec_from_time(time), year, formatAD?ADW:BCW);
         else if(offset)
-            sprintfW(buf, formatW, week, month, day,
+            swprintf(buf, ARRAY_SIZE(buf), formatW, week, month, day,
                     (int)hour_from_time(time), (int)min_from_time(time),
                     (int)sec_from_time(time), sign, offset/60, offset%60,
                     year, formatAD?ADW:BCW);
         else
-            sprintfW(buf, formatUTCW, week, month, day,
+            swprintf(buf, ARRAY_SIZE(buf), formatUTCW, week, month, day,
                     (int)hour_from_time(time), (int)min_from_time(time),
                     (int)sec_from_time(time), year, formatAD?ADW:BCW);
 
@@ -584,7 +582,7 @@ static HRESULT Date_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     return dateobj_to_string(date, r);
 }
@@ -601,7 +599,7 @@ static HRESULT Date_toLocaleString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flag
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(isnan(date->time)) {
         if(r)
@@ -648,7 +646,7 @@ static HRESULT Date_toISOString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     year = year_from_time(date->time);
     if(isnan(year) || year > 999999 || year < -999999) {
@@ -658,15 +656,16 @@ static HRESULT Date_toISOString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
 
     if(year < 0) {
         *p++ = '-';
-        p += sprintfW(p, long_year_formatW, -(int)year);
+        p += swprintf(p, ARRAY_SIZE(buf) - 1, long_year_formatW, -(int)year);
     }else if(year > 9999) {
         *p++ = '+';
-        p += sprintfW(p, long_year_formatW, (int)year);
+        p += swprintf(p, ARRAY_SIZE(buf) - 1, long_year_formatW, (int)year);
     }else {
-        p += sprintfW(p, short_year_formatW, (int)year);
+        p += swprintf(p, ARRAY_SIZE(buf), short_year_formatW, (int)year);
     }
 
-    sprintfW(p, formatW, (int)month_from_time(date->time) + 1, (int)date_from_time(date->time),
+    swprintf(p, ARRAY_SIZE(buf) - (p - buf), formatW,
+             (int)month_from_time(date->time) + 1, (int)date_from_time(date->time),
              (int)hour_from_time(date->time), (int)min_from_time(date->time),
              (int)sec_from_time(date->time), (int)ms_from_time(date->time));
 
@@ -687,7 +686,7 @@ static HRESULT Date_valueOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(date->time);
@@ -720,7 +719,7 @@ static inline HRESULT create_utc_string(script_ctx_t *ctx, vdisp_t *jsthis, jsva
     DWORD lcid_en;
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(isnan(date->time)) {
         if(r)
@@ -745,7 +744,7 @@ static inline HRESULT create_utc_string(script_ctx_t *ctx, vdisp_t *jsthis, jsva
 
         day = date_from_time(date->time);
 
-        sprintfW(buf, formatAD ? formatADW : formatBCW, week, day, month, year,
+        swprintf(buf, ARRAY_SIZE(buf), formatAD ? formatADW : formatBCW, week, day, month, year,
                 (int)hour_from_time(date->time), (int)min_from_time(date->time),
                 (int)sec_from_time(date->time));
 
@@ -822,7 +821,7 @@ static HRESULT dateobj_to_date_string(DateInstance *date, jsval_t *r)
 
         day = date_from_time(time);
 
-        sprintfW(buf, formatAD ? formatADW : formatBCW, week, month, day, year);
+        swprintf(buf, ARRAY_SIZE(buf), formatAD ? formatADW : formatBCW, week, month, day, year);
 
         date_str = jsstr_alloc(buf);
         if(!date_str)
@@ -839,7 +838,7 @@ static HRESULT Date_toDateString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     DateInstance *date;
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     return dateobj_to_date_string(date, r);
 }
@@ -862,7 +861,7 @@ static HRESULT Date_toTimeString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(isnan(date->time)) {
         if(r)
@@ -883,11 +882,11 @@ static HRESULT Date_toTimeString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
         else sign = '-';
 
         if(offset)
-            sprintfW(buf, formatW, (int)hour_from_time(time),
+            swprintf(buf, ARRAY_SIZE(buf), formatW, (int)hour_from_time(time),
                     (int)min_from_time(time), (int)sec_from_time(time),
                     sign, offset/60, offset%60);
         else
-            sprintfW(buf, formatUTCW, (int)hour_from_time(time),
+            swprintf(buf, ARRAY_SIZE(buf), formatUTCW, (int)hour_from_time(time),
                     (int)min_from_time(time), (int)sec_from_time(time));
 
         date_str = jsstr_alloc(buf);
@@ -911,7 +910,7 @@ static HRESULT Date_toLocaleDateString(script_ctx_t *ctx, vdisp_t *jsthis, WORD
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(isnan(date->time)) {
         if(r)
@@ -950,7 +949,7 @@ static HRESULT Date_toLocaleTimeString(script_ctx_t *ctx, vdisp_t *jsthis, WORD
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(isnan(date->time)) {
         if(r)
@@ -986,7 +985,7 @@ static HRESULT Date_getTime(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(date->time);
@@ -1002,7 +1001,7 @@ static HRESULT Date_getFullYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r) {
         DOUBLE time = local_time(date->time, date);
@@ -1021,7 +1020,7 @@ static HRESULT Date_getUTCFullYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flag
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(year_from_time(date->time));
@@ -1036,7 +1035,7 @@ static HRESULT Date_getMonth(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(month_from_time(local_time(date->time, date)));
@@ -1052,7 +1051,7 @@ static HRESULT Date_getUTCMonth(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(month_from_time(date->time));
@@ -1067,7 +1066,7 @@ static HRESULT Date_getDate(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(date_from_time(local_time(date->time, date)));
@@ -1083,7 +1082,7 @@ static HRESULT Date_getUTCDate(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(date_from_time(date->time));
@@ -1099,7 +1098,7 @@ static HRESULT Date_getDay(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsig
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(week_day(local_time(date->time, date)));
@@ -1115,7 +1114,7 @@ static HRESULT Date_getUTCDay(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, un
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(week_day(date->time));
@@ -1131,7 +1130,7 @@ static HRESULT Date_getHours(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(hour_from_time(local_time(date->time, date)));
@@ -1147,7 +1146,7 @@ static HRESULT Date_getUTCHours(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(hour_from_time(date->time));
@@ -1163,7 +1162,7 @@ static HRESULT Date_getMinutes(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(min_from_time(local_time(date->time, date)));
@@ -1179,7 +1178,7 @@ static HRESULT Date_getUTCMinutes(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(min_from_time(date->time));
@@ -1194,7 +1193,7 @@ static HRESULT Date_getSeconds(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(sec_from_time(local_time(date->time, date)));
@@ -1210,7 +1209,7 @@ static HRESULT Date_getUTCSeconds(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(sec_from_time(date->time));
@@ -1226,7 +1225,7 @@ static HRESULT Date_getMilliseconds(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(ms_from_time(local_time(date->time, date)));
@@ -1242,7 +1241,7 @@ static HRESULT Date_getUTCMilliseconds(script_ctx_t *ctx, vdisp_t *jsthis, WORD
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(ms_from_time(date->time));
@@ -1258,7 +1257,7 @@ static HRESULT Date_getTimezoneOffset(script_ctx_t *ctx, vdisp_t *jsthis, WORD f
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(r)
         *r = jsval_number(floor((date->time-local_time(date->time, date))/MS_PER_MINUTE));
@@ -1276,10 +1275,10 @@ static HRESULT Date_setTime(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     hres = to_number(ctx, argv[0], &n);
     if(FAILED(hres))
@@ -1303,10 +1302,10 @@ static HRESULT Date_setMilliseconds(script_ctx_t *ctx, vdisp_t *jsthis, WORD fla
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     hres = to_number(ctx, argv[0], &n);
     if(FAILED(hres))
@@ -1333,10 +1332,10 @@ static HRESULT Date_setUTCMilliseconds(script_ctx_t *ctx, vdisp_t *jsthis, WORD
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     hres = to_number(ctx, argv[0], &n);
     if(FAILED(hres))
@@ -1363,10 +1362,10 @@ static HRESULT Date_setSeconds(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = local_time(date->time, date);
 
@@ -1402,10 +1401,10 @@ static HRESULT Date_setUTCSeconds(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = date->time;
 
@@ -1441,10 +1440,10 @@ static HRESULT Date_setMinutes(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = local_time(date->time, date);
 
@@ -1488,10 +1487,10 @@ static HRESULT Date_setUTCMinutes(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = date->time;
 
@@ -1535,10 +1534,10 @@ static HRESULT Date_setHours(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = local_time(date->time, date);
 
@@ -1589,10 +1588,10 @@ static HRESULT Date_setUTCHours(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = date->time;
 
@@ -1643,10 +1642,10 @@ static HRESULT Date_setDate(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     hres = to_number(ctx, argv[0], &n);
     if(FAILED(hres))
@@ -1672,10 +1671,10 @@ static HRESULT Date_setUTCDate(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     hres = to_number(ctx, argv[0], &n);
     if(FAILED(hres))
@@ -1701,10 +1700,10 @@ static HRESULT Date_setMonth(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, uns
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = local_time(date->time, date);
 
@@ -1740,10 +1739,10 @@ static HRESULT Date_setUTCMonth(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = date->time;
 
@@ -1779,10 +1778,10 @@ static HRESULT Date_setFullYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = local_time(date->time, date);
 
@@ -1825,10 +1824,10 @@ static HRESULT Date_setUTCFullYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flag
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = date->time;
 
@@ -1870,7 +1869,7 @@ static HRESULT Date_getYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     t = local_time(date->time, date);
     if(isnan(t)) {
@@ -1896,10 +1895,10 @@ static HRESULT Date_setYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     TRACE("\n");
 
     if(!(date = date_this(jsthis)))
-        return throw_type_error(ctx, JS_E_DATE_EXPECTED, NULL);
+        return JS_E_DATE_EXPECTED;
 
     if(!argc)
-        return throw_type_error(ctx, JS_E_MISSING_ARG, NULL);
+        return JS_E_MISSING_ARG;
 
     t = local_time(date->time, date);
 
@@ -2075,7 +2074,7 @@ static inline HRESULT date_parse(jsstr_t *input_str, double *ret) {
     for(i=0; i<input_len; i++) {
         if(input[i] == '(') nest_level++;
         else if(input[i] == ')') nest_level--;
-        else if(!nest_level) parse[parse_len++] = toupperW(input[i]);
+        else if(!nest_level) parse[parse_len++] = towupper(input[i]);
     }
     parse[parse_len] = 0;
 
@@ -2102,16 +2101,16 @@ static inline HRESULT date_parse(jsstr_t *input_str, double *ret) {
     }
 
     for(i=0; i<parse_len;) {
-        while(isspaceW(parse[i])) i++;
+        while(iswspace(parse[i])) i++;
         if(parse[i] == ',') {
             while(parse[i] == ',') i++;
             continue;
         }
 
         if(parse[i]>='0' && parse[i]<='9') {
-            int tmp = atoiW(&parse[i]);
+            int tmp = wcstol(&parse[i], NULL, 10);
             while(parse[i]>='0' && parse[i]<='9') i++;
-            while(isspaceW(parse[i])) i++;
+            while(iswspace(parse[i])) i++;
 
             if(parse[i] == ':') {
                 /* Time */
@@ -2121,17 +2120,17 @@ static inline HRESULT date_parse(jsstr_t *input_str, double *ret) {
                 hour = tmp;
 
                 while(parse[i] == ':') i++;
-                while(isspaceW(parse[i])) i++;
+                while(iswspace(parse[i])) i++;
                 if(parse[i]>='0' && parse[i]<='9') {
-                    min = atoiW(&parse[i]);
+                    min = wcstol(&parse[i], NULL, 10);
                     while(parse[i]>='0' && parse[i]<='9') i++;
                 }
 
-                while(isspaceW(parse[i])) i++;
+                while(iswspace(parse[i])) i++;
                 while(parse[i] == ':') i++;
-                while(isspaceW(parse[i])) i++;
+                while(iswspace(parse[i])) i++;
                 if(parse[i]>='0' && parse[i]<='9') {
-                    sec = atoiW(&parse[i]);
+                    sec = wcstol(&parse[i], NULL, 10);
                     while(parse[i]>='0' && parse[i]<='9') i++;
                 }
             }
@@ -2144,17 +2143,17 @@ static inline HRESULT date_parse(jsstr_t *input_str, double *ret) {
 
                 month = tmp-1;
 
-                while(isspaceW(parse[i])) i++;
+                while(iswspace(parse[i])) i++;
                 while(parse[i]=='-' || parse[i]=='/') i++;
-                while(isspaceW(parse[i])) i++;
+                while(iswspace(parse[i])) i++;
                 if(parse[i]<'0' || parse[i]>'9') break;
-                day = atoiW(&parse[i]);
+                day = wcstol(&parse[i], NULL, 10);
                 while(parse[i]>='0' && parse[i]<='9') i++;
 
                 while(parse[i]=='-' || parse[i]=='/') i++;
-                while(isspaceW(parse[i])) i++;
+                while(iswspace(parse[i])) i++;
                 if(parse[i]<'0' || parse[i]>'9') break;
-                year = atoiW(&parse[i]);
+                year = wcstol(&parse[i], NULL, 10);
                 while(parse[i]>='0' && parse[i]<='9') i++;
 
                 if(tmp >= 70){
@@ -2189,9 +2188,9 @@ static inline HRESULT date_parse(jsstr_t *input_str, double *ret) {
             if(parse[i] == '-')  positive = FALSE;
 
             i++;
-            while(isspaceW(parse[i])) i++;
+            while(iswspace(parse[i])) i++;
             if(parse[i]<'0' || parse[i]>'9') break;
-            offset = atoiW(&parse[i]);
+            offset = wcstol(&parse[i], NULL, 10);
             while(parse[i]>='0' && parse[i]<='9') i++;
 
             if(offset<24) offset *= 60;
@@ -2270,7 +2269,7 @@ static inline HRESULT date_parse(jsstr_t *input_str, double *ret) {
                 size -= i;
 
                 for(j=0; j<ARRAY_SIZE(string_ids); j++)
-                    if(!strncmpiW(&parse[i], strings[j], size)) break;
+                    if(!wcsnicmp(&parse[i], strings[j], size)) break;
 
                 if(j < 12) {
                     if(set_month) break;
