@@ -24,76 +24,50 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(jscript);
 
-static const WCHAR toStringW[] = {'t','o','S','t','r','i','n','g',0};
-static const WCHAR toLocaleStringW[] = {'t','o','L','o','c','a','l','e','S','t','r','i','n','g',0};
-static const WCHAR valueOfW[] = {'v','a','l','u','e','O','f',0};
-static const WCHAR hasOwnPropertyW[] = {'h','a','s','O','w','n','P','r','o','p','e','r','t','y',0};
-static const WCHAR propertyIsEnumerableW[] =
-    {'p','r','o','p','e','r','t','y','I','s','E','n','u','m','e','r','a','b','l','e',0};
-static const WCHAR isPrototypeOfW[] = {'i','s','P','r','o','t','o','t','y','p','e','O','f',0};
-
-static const WCHAR createW[] = {'c','r','e','a','t','e',0};
-static const WCHAR getOwnPropertyDescriptorW[] =
-    {'g','e','t','O','w','n','P','r','o','p','e','r','t','y','D','e','s','c','r','i','p','t','o','r',0};
-static const WCHAR getPrototypeOfW[] =
-    {'g','e','t','P','r','o','t','o','t','y','p','e','O','f',0};
-static const WCHAR definePropertyW[] = {'d','e','f','i','n','e','P','r','o','p','e','r','t','y',0};
-
-static const WCHAR definePropertiesW[] = {'d','e','f','i','n','e','P','r','o','p','e','r','t','i','e','s',0};
-
-static const WCHAR default_valueW[] = {'[','o','b','j','e','c','t',' ','O','b','j','e','c','t',']',0};
-
-static const WCHAR configurableW[] = {'c','o','n','f','i','g','u','r','a','b','l','e',0};
-static const WCHAR enumerableW[] = {'e','n','u','m','e','r','a','b','l','e',0};
-static const WCHAR valueW[] = {'v','a','l','u','e',0};
-static const WCHAR writableW[] = {'w','r','i','t','a','b','l','e',0};
-static const WCHAR getW[] = {'g','e','t',0};
-static const WCHAR setW[] = {'s','e','t',0};
-
 static HRESULT Object_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
         jsval_t *r)
 {
     jsdisp_t *jsdisp;
     const WCHAR *str;
 
-    static const WCHAR formatW[] = {'[','o','b','j','e','c','t',' ','%','s',']',0};
-
-    static const WCHAR arrayW[] = {'A','r','r','a','y',0};
-    static const WCHAR booleanW[] = {'B','o','o','l','e','a','n',0};
-    static const WCHAR dateW[] = {'D','a','t','e',0};
-    static const WCHAR errorW[] = {'E','r','r','o','r',0};
-    static const WCHAR functionW[] = {'F','u','n','c','t','i','o','n',0};
-    static const WCHAR mathW[] = {'M','a','t','h',0};
-    static const WCHAR numberW[] = {'N','u','m','b','e','r',0};
-    static const WCHAR objectW[] = {'O','b','j','e','c','t',0};
-    static const WCHAR regexpW[] = {'R','e','g','E','x','p',0};
-    static const WCHAR stringW[] = {'S','t','r','i','n','g',0};
     /* Keep in sync with jsclass_t enum */
-    static const WCHAR *names[] = {NULL, arrayW, booleanW, dateW, objectW, errorW,
-        functionW, NULL, mathW, numberW, objectW, regexpW, stringW, objectW, objectW, objectW};
+    static const WCHAR *names[] = {
+        NULL,
+        L"[object Array]",
+        L"[object Boolean]",
+        L"[object Date]",
+        L"[object Object]",
+        L"[object Error]",
+        L"[object Function]",
+        NULL,
+        L"[object Math]",
+        L"[object Number]",
+        L"[object Object]",
+        L"[object RegExp]",
+        L"[object String]",
+        L"[object Object]",
+        L"[object Object]",
+        L"[object Object]"
+    };
 
     TRACE("\n");
 
     jsdisp = get_jsdisp(jsthis);
     if(!jsdisp) {
-        str = objectW;
+        str = L"[object Object]";
     }else if(names[jsdisp->builtin_info->class]) {
         str = names[jsdisp->builtin_info->class];
     }else {
         assert(jsdisp->builtin_info->class != JSCLASS_NONE);
-        FIXME("jdisp->builtin_info->class = %d\n", jsdisp->builtin_info->class);
+        FIXME("jsdisp->builtin_info->class = %d\n", jsdisp->builtin_info->class);
         return E_FAIL;
     }
 
     if(r) {
         jsstr_t *ret;
-        WCHAR *ptr;
-
-        ret = jsstr_alloc_buf(9+strlenW(str), &ptr);
+        ret = jsstr_alloc(str);
         if(!ret)
             return E_OUTOFMEMORY;
-
-        sprintfW(ptr, formatW, str);
         *r = jsval_string(ret);
     }
 
@@ -110,7 +84,7 @@ static HRESULT Object_toLocaleString(script_ctx_t *ctx, vdisp_t *jsthis, WORD fl
         return E_FAIL;
     }
 
-    return jsdisp_call_name(jsthis->u.jsdisp, toStringW, DISPATCH_METHOD, 0, NULL, r);
+    return jsdisp_call_name(jsthis->u.jsdisp, L"toString", DISPATCH_METHOD, 0, NULL, r);
 }
 
 static HRESULT Object_valueOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
@@ -230,7 +204,7 @@ static HRESULT Object_get_value(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 
     TRACE("\n");
 
-    ret = jsstr_alloc(default_valueW);
+    ret = jsstr_alloc(L"[object Object]");
     if(!ret)
         return E_OUTOFMEMORY;
 
@@ -244,12 +218,12 @@ static void Object_destructor(jsdisp_t *dispex)
 }
 
 static const builtin_prop_t Object_props[] = {
-    {hasOwnPropertyW,        Object_hasOwnProperty,        PROPF_METHOD|1},
-    {isPrototypeOfW,         Object_isPrototypeOf,         PROPF_METHOD|1},
-    {propertyIsEnumerableW,  Object_propertyIsEnumerable,  PROPF_METHOD|1},
-    {toLocaleStringW,        Object_toLocaleString,        PROPF_METHOD},
-    {toStringW,              Object_toString,              PROPF_METHOD},
-    {valueOfW,               Object_valueOf,               PROPF_METHOD}
+    {L"hasOwnProperty",        Object_hasOwnProperty,        PROPF_METHOD|1},
+    {L"isPrototypeOf",         Object_isPrototypeOf,         PROPF_METHOD|1},
+    {L"propertyIsEnumerable",  Object_propertyIsEnumerable,  PROPF_METHOD|1},
+    {L"toLocaleString",        Object_toLocaleString,        PROPF_METHOD},
+    {L"toString",              Object_toString,              PROPF_METHOD},
+    {L"valueOf",               Object_valueOf,               PROPF_METHOD}
 };
 
 static const builtin_info_t Object_info = {
@@ -289,7 +263,7 @@ static HRESULT to_property_descriptor(script_ctx_t *ctx, jsdisp_t *attr_obj, pro
     memset(desc, 0, sizeof(*desc));
     desc->value = jsval_undefined();
 
-    hres = jsdisp_get_id(attr_obj, enumerableW, 0, &id);
+    hres = jsdisp_get_id(attr_obj, L"enumerable", 0, &id);
     if(SUCCEEDED(hres)) {
         desc->mask |= PROPF_ENUMERABLE;
         hres = jsdisp_propget(attr_obj, id, &v);
@@ -305,7 +279,7 @@ static HRESULT to_property_descriptor(script_ctx_t *ctx, jsdisp_t *attr_obj, pro
         return hres;
     }
 
-    hres = jsdisp_get_id(attr_obj, configurableW, 0, &id);
+    hres = jsdisp_get_id(attr_obj, L"configurable", 0, &id);
     if(SUCCEEDED(hres)) {
         desc->mask |= PROPF_CONFIGURABLE;
         hres = jsdisp_propget(attr_obj, id, &v);
@@ -321,7 +295,7 @@ static HRESULT to_property_descriptor(script_ctx_t *ctx, jsdisp_t *attr_obj, pro
         return hres;
     }
 
-    hres = jsdisp_get_id(attr_obj, valueW, 0, &id);
+    hres = jsdisp_get_id(attr_obj, L"value", 0, &id);
     if(SUCCEEDED(hres)) {
         hres = jsdisp_propget(attr_obj, id, &desc->value);
         if(FAILED(hres))
@@ -331,7 +305,7 @@ static HRESULT to_property_descriptor(script_ctx_t *ctx, jsdisp_t *attr_obj, pro
         return hres;
     }
 
-    hres = jsdisp_get_id(attr_obj, writableW, 0, &id);
+    hres = jsdisp_get_id(attr_obj, L"writable", 0, &id);
     if(SUCCEEDED(hres)) {
         desc->mask |= PROPF_WRITABLE;
         hres = jsdisp_propget(attr_obj, id, &v);
@@ -349,7 +323,7 @@ static HRESULT to_property_descriptor(script_ctx_t *ctx, jsdisp_t *attr_obj, pro
         return hres;
     }
 
-    hres = jsdisp_get_id(attr_obj, getW, 0, &id);
+    hres = jsdisp_get_id(attr_obj, L"get", 0, &id);
     if(SUCCEEDED(hres)) {
         desc->explicit_getter = TRUE;
         hres = jsdisp_propget(attr_obj, id, &v);
@@ -373,7 +347,7 @@ static HRESULT to_property_descriptor(script_ctx_t *ctx, jsdisp_t *attr_obj, pro
         return hres;
     }
 
-    hres = jsdisp_get_id(attr_obj, setW, 0, &id);
+    hres = jsdisp_get_id(attr_obj, L"set", 0, &id);
     if(SUCCEEDED(hres)) {
         desc->explicit_setter = TRUE;
         hres = jsdisp_propget(attr_obj, id, &v);
@@ -399,9 +373,9 @@ static HRESULT to_property_descriptor(script_ctx_t *ctx, jsdisp_t *attr_obj, pro
 
     if(desc->explicit_getter || desc->explicit_setter) {
         if(desc->explicit_value)
-            hres = throw_type_error(ctx, JS_E_PROP_DESC_MISMATCH, NULL);
+            hres = JS_E_PROP_DESC_MISMATCH;
         else if(desc->mask & PROPF_WRITABLE)
-            hres = throw_type_error(ctx, JS_E_INVALID_WRITABLE_PROP_DESC, NULL);
+            hres = JS_E_INVALID_WRITABLE_PROP_DESC;
     }
 
     if(FAILED(hres))
@@ -409,6 +383,57 @@ static HRESULT to_property_descriptor(script_ctx_t *ctx, jsdisp_t *attr_obj, pro
     return hres;
 }
 
+static HRESULT jsdisp_define_properties(script_ctx_t *ctx, jsdisp_t *obj, jsval_t list_val)
+{
+    DISPID id = DISPID_STARTENUM;
+    property_desc_t prop_desc;
+    IDispatch *list_disp;
+    jsdisp_t *list_obj, *desc_obj;
+    jsval_t desc_val;
+    BSTR name;
+    HRESULT hres;
+
+    hres = to_object(ctx, list_val, &list_disp);
+    if(FAILED(hres))
+        return hres;
+
+    if(!(list_obj = to_jsdisp(list_disp))) {
+        FIXME("non-JS list obj\n");
+        IDispatch_Release(list_disp);
+        return E_NOTIMPL;
+    }
+
+    while(1) {
+        hres = jsdisp_next_prop(list_obj, id, TRUE, &id);
+        if(hres != S_OK)
+            break;
+
+        hres = jsdisp_propget(list_obj, id, &desc_val);
+        if(FAILED(hres))
+            break;
+
+        if(!is_object_instance(desc_val) || !get_object(desc_val) || !(desc_obj = to_jsdisp(get_object(desc_val)))) {
+            jsval_release(desc_val);
+            break;
+        }
+
+        hres = to_property_descriptor(ctx, desc_obj, &prop_desc);
+        jsdisp_release(desc_obj);
+        if(FAILED(hres))
+            break;
+
+        hres = IDispatchEx_GetMemberName(&list_obj->IDispatchEx_iface, id, &name);
+        if(SUCCEEDED(hres))
+            hres = jsdisp_define_property(obj, name, &prop_desc);
+        release_property_descriptor(&prop_desc);
+        if(FAILED(hres))
+            break;
+    }
+
+    jsdisp_release(list_obj);
+    return FAILED(hres) ? hres : S_OK;
+}
+
 static HRESULT Object_defineProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
                                      unsigned argc, jsval_t *argv, jsval_t *r)
 {
@@ -421,7 +446,7 @@ static HRESULT Object_defineProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD fl
     TRACE("\n");
 
     if(argc < 1 || !is_object_instance(argv[0]))
-        return throw_type_error(ctx, JS_E_OBJECT_EXPECTED, NULL);
+        return JS_E_OBJECT_EXPECTED;
     obj = to_jsdisp(get_object(argv[0]));
     if(!obj) {
         FIXME("not implemented non-JS object\n");
@@ -441,7 +466,7 @@ static HRESULT Object_defineProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD fl
             hres = E_NOTIMPL;
         }
     }else {
-        hres = throw_type_error(ctx, JS_E_OBJECT_EXPECTED, NULL);
+        hres = JS_E_OBJECT_EXPECTED;
     }
     jsstr_release(name_str);
     if(FAILED(hres))
@@ -449,14 +474,28 @@ static HRESULT Object_defineProperty(script_ctx_t *ctx, vdisp_t *jsthis, WORD fl
 
     hres = jsdisp_define_property(obj, name, &prop_desc);
     release_property_descriptor(&prop_desc);
+    if(SUCCEEDED(hres) && r)
+        *r = jsval_obj(jsdisp_addref(obj));
     return hres;
 }
 
 static HRESULT Object_defineProperties(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
                                      unsigned argc, jsval_t *argv, jsval_t *r)
 {
-    FIXME("\n");
-    return E_NOTIMPL;
+    jsdisp_t *obj;
+    HRESULT hres;
+
+    if(argc < 1 || !is_object_instance(argv[0]) || !get_object(argv[0]) || !(obj = to_jsdisp(get_object(argv[0])))) {
+        FIXME("not an object\n");
+        return E_NOTIMPL;
+    }
+
+    TRACE("%p\n", obj);
+
+    hres = jsdisp_define_properties(ctx, obj, argc >= 2 ? argv[1] : jsval_undefined());
+    if(SUCCEEDED(hres) && r)
+        *r = jsval_obj(jsdisp_addref(obj));
+    return hres;
 }
 
 static HRESULT Object_getOwnPropertyDescriptor(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
@@ -471,7 +510,7 @@ static HRESULT Object_getOwnPropertyDescriptor(script_ctx_t *ctx, vdisp_t *jsthi
     TRACE("\n");
 
     if(argc < 1 || !is_object_instance(argv[0]))
-        return throw_type_error(ctx, JS_E_OBJECT_EXPECTED, NULL);
+        return JS_E_OBJECT_EXPECTED;
     obj = to_jsdisp(get_object(argv[0]));
     if(!obj) {
         FIXME("not implemented non-JS object\n");
@@ -496,22 +535,22 @@ static HRESULT Object_getOwnPropertyDescriptor(script_ctx_t *ctx, vdisp_t *jsthi
         return hres;
 
     if(prop_desc.explicit_getter || prop_desc.explicit_setter) {
-        hres = jsdisp_define_data_property(desc_obj, getW, PROPF_ALL,
+        hres = jsdisp_define_data_property(desc_obj, L"get", PROPF_ALL,
                 prop_desc.getter ? jsval_obj(prop_desc.getter) : jsval_undefined());
         if(SUCCEEDED(hres))
-            hres = jsdisp_define_data_property(desc_obj, setW, PROPF_ALL,
+            hres = jsdisp_define_data_property(desc_obj, L"set", PROPF_ALL,
                     prop_desc.setter ? jsval_obj(prop_desc.setter) : jsval_undefined());
     }else {
-        hres = jsdisp_propput_name(desc_obj, valueW, prop_desc.value);
+        hres = jsdisp_propput_name(desc_obj, L"value", prop_desc.value);
         if(SUCCEEDED(hres))
-            hres = jsdisp_define_data_property(desc_obj, writableW, PROPF_ALL,
+            hres = jsdisp_define_data_property(desc_obj, L"writable", PROPF_ALL,
                     jsval_bool(!!(prop_desc.flags & PROPF_WRITABLE)));
     }
     if(SUCCEEDED(hres))
-        hres = jsdisp_define_data_property(desc_obj, enumerableW, PROPF_ALL,
+        hres = jsdisp_define_data_property(desc_obj, L"enumerable", PROPF_ALL,
                 jsval_bool(!!(prop_desc.flags & PROPF_ENUMERABLE)));
     if(SUCCEEDED(hres))
-        hres = jsdisp_define_data_property(desc_obj, configurableW, PROPF_ALL,
+        hres = jsdisp_define_data_property(desc_obj, L"configurable", PROPF_ALL,
                 jsval_bool(!!(prop_desc.flags & PROPF_CONFIGURABLE)));
 
     release_property_descriptor(&prop_desc);
@@ -535,11 +574,6 @@ static HRESULT Object_create(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
 
     TRACE("(%s)\n", debugstr_jsval(argv[0]));
 
-    if(argc > 1) {
-        FIXME("Unsupported properties argument %s\n", debugstr_jsval(argv[1]));
-        return E_NOTIMPL;
-    }
-
     if(argc && is_object_instance(argv[0])) {
         if(get_object(argv[0]))
             proto = to_jsdisp(get_object(argv[0]));
@@ -552,13 +586,18 @@ static HRESULT Object_create(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
         return E_INVALIDARG;
     }
 
-    if(r) {
-        hres = create_dispex(ctx, NULL, proto, &obj);
-        if(FAILED(hres))
-            return hres;
+    hres = create_dispex(ctx, &ObjectInst_info, proto, &obj);
+    if(FAILED(hres))
+        return hres;
+
+    if(argc >= 2 && !is_undefined(argv[1]))
+        hres = jsdisp_define_properties(ctx, obj, argv[1]);
+
+    if(SUCCEEDED(hres) && r)
         *r = jsval_obj(obj);
-    }
-    return S_OK;
+    else
+        jsdisp_release(obj);
+    return hres;
 }
 
 static HRESULT Object_getPrototypeOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags,
@@ -587,11 +626,11 @@ static HRESULT Object_getPrototypeOf(script_ctx_t *ctx, vdisp_t *jsthis, WORD fl
 }
 
 static const builtin_prop_t ObjectConstr_props[] = {
-    {createW,                   Object_create,                      PROPF_ES5|PROPF_METHOD|2},
-    {definePropertiesW,         Object_defineProperties,            PROPF_ES5|PROPF_METHOD|2},
-    {definePropertyW,           Object_defineProperty,              PROPF_ES5|PROPF_METHOD|2},
-    {getOwnPropertyDescriptorW, Object_getOwnPropertyDescriptor,    PROPF_ES5|PROPF_METHOD|2},
-    {getPrototypeOfW,           Object_getPrototypeOf,              PROPF_ES5|PROPF_METHOD|1}
+    {L"create",                   Object_create,                      PROPF_ES5|PROPF_METHOD|2},
+    {L"defineProperties",         Object_defineProperties,            PROPF_ES5|PROPF_METHOD|2},
+    {L"defineProperty",           Object_defineProperty,              PROPF_ES5|PROPF_METHOD|2},
+    {L"getOwnPropertyDescriptor", Object_getOwnPropertyDescriptor,    PROPF_ES5|PROPF_METHOD|2},
+    {L"getPrototypeOf",           Object_getPrototypeOf,              PROPF_ES5|PROPF_METHOD|1}
 };
 
 static const builtin_info_t ObjectConstr_info = {
@@ -652,9 +691,7 @@ static HRESULT ObjectConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
 
 HRESULT create_object_constr(script_ctx_t *ctx, jsdisp_t *object_prototype, jsdisp_t **ret)
 {
-    static const WCHAR ObjectW[] = {'O','b','j','e','c','t',0};
-
-    return create_builtin_constructor(ctx, ObjectConstr_value, ObjectW, &ObjectConstr_info, PROPF_CONSTR,
+    return create_builtin_constructor(ctx, ObjectConstr_value, L"Object", &ObjectConstr_info, PROPF_CONSTR,
             object_prototype, ret);
 }
 
