@@ -710,6 +710,7 @@ static NTSTATUS status_gss_to_sspi( OM_uint32 status )
     case GSS_S_OLD_TOKEN:            return SEC_E_INVALID_TOKEN;
     case GSS_S_UNSEQ_TOKEN:          return SEC_E_OUT_OF_SEQUENCE;
     case GSS_S_GAP_TOKEN:            return SEC_E_OUT_OF_SEQUENCE;
+    case GSS_S_FAILURE:              return SEC_E_INTERNAL_ERROR;
 
     default:
         FIXME( "couldn't convert status 0x%08x to NTSTATUS\n", status );
@@ -750,12 +751,11 @@ static void trace_gss_status( OM_uint32 major_status, OM_uint32 minor_status )
 
 static void expirytime_gss_to_sspi( OM_uint32 expirytime, TimeStamp *timestamp )
 {
-    SYSTEMTIME time;
     FILETIME filetime;
     ULARGE_INTEGER tmp;
 
-    GetLocalTime( &time );
-    SystemTimeToFileTime( &time, &filetime );
+    GetSystemTimeAsFileTime( &filetime );
+    FileTimeToLocalFileTime( &filetime, &filetime );
     tmp.QuadPart = ((ULONGLONG)filetime.dwLowDateTime | (ULONGLONG)filetime.dwHighDateTime << 32) + expirytime;
     timestamp->LowPart  = tmp.QuadPart;
     timestamp->HighPart = tmp.QuadPart >> 32;
@@ -790,6 +790,7 @@ static ULONG flags_isc_req_to_gss( ULONG flags )
     if (flags & ISC_REQ_INTEGRITY)       ret |= GSS_C_INTEG_FLAG;
     if (flags & ISC_REQ_NULL_SESSION)    ret |= GSS_C_ANON_FLAG;
     if (flags & ISC_REQ_USE_DCE_STYLE)   ret |= GSS_C_DCE_STYLE;
+    if (flags & ISC_REQ_IDENTIFY)        ret |= GSS_C_IDENTIFY_FLAG;
     return ret;
 }
 
@@ -803,6 +804,8 @@ static ULONG flags_gss_to_isc_ret( ULONG flags )
     if (flags & GSS_C_CONF_FLAG)     ret |= ISC_RET_CONFIDENTIALITY;
     if (flags & GSS_C_INTEG_FLAG)    ret |= ISC_RET_INTEGRITY;
     if (flags & GSS_C_ANON_FLAG)     ret |= ISC_RET_NULL_SESSION;
+    if (flags & GSS_C_DCE_STYLE)     ret |= ISC_RET_USED_DCE_STYLE;
+    if (flags & GSS_C_IDENTIFY_FLAG) ret |= ISC_RET_IDENTIFY;
     return ret;
 }
 
@@ -816,6 +819,8 @@ static ULONG flags_gss_to_asc_ret( ULONG flags )
     if (flags & GSS_C_CONF_FLAG)     ret |= ASC_RET_CONFIDENTIALITY;
     if (flags & GSS_C_INTEG_FLAG)    ret |= ASC_RET_INTEGRITY;
     if (flags & GSS_C_ANON_FLAG)     ret |= ASC_RET_NULL_SESSION;
+    if (flags & GSS_C_DCE_STYLE)     ret |= ASC_RET_USED_DCE_STYLE;
+    if (flags & GSS_C_IDENTIFY_FLAG) ret |= ASC_RET_IDENTIFY;
     return ret;
 }
 
@@ -1013,7 +1018,8 @@ static NTSTATUS NTAPI kerberos_SpInitLsaModeContext( LSA_SEC_HANDLE credential,
 {
 #ifdef SONAME_LIBGSSAPI_KRB5
     static const ULONG supported = ISC_REQ_CONFIDENTIALITY | ISC_REQ_INTEGRITY | ISC_REQ_SEQUENCE_DETECT |
-                                   ISC_REQ_REPLAY_DETECT | ISC_REQ_MUTUAL_AUTH | ISC_REQ_USE_DCE_STYLE;
+                                   ISC_REQ_REPLAY_DETECT | ISC_REQ_MUTUAL_AUTH | ISC_REQ_USE_DCE_STYLE |
+                                   ISC_REQ_IDENTIFY | ISC_REQ_CONNECTION;
     OM_uint32 ret, minor_status, ret_flags = 0, expiry_time, req_flags = flags_isc_req_to_gss( context_req );
     gss_cred_id_t cred_handle;
     gss_ctx_id_t ctxt_handle;
@@ -1324,7 +1330,7 @@ NTSTATUS NTAPI SpLsaModeInitialize(ULONG lsa_version, PULONG package_version,
 
 static NTSTATUS NTAPI kerberos_SpInstanceInit(ULONG version, SECPKG_DLL_FUNCTIONS *dll_function_table, void **user_functions)
 {
-    FIXME("%u,%p,%p: stub\n", version, dll_function_table, user_functions);
+    TRACE("%#x,%p,%p\n", version, dll_function_table, user_functions);
 
     return STATUS_SUCCESS;
 }
@@ -1407,11 +1413,21 @@ static NTSTATUS NTAPI kerberos_SpVerifySignature( LSA_SEC_HANDLE context, SecBuf
 }
 
 #ifdef SONAME_LIBGSSAPI_KRB5
-static NTSTATUS seal_message_iov( gss_ctx_id_t ctxt_handle, SecBufferDesc *message )
+static NTSTATUS seal_message_iov( gss_ctx_id_t ctxt_handle, SecBufferDesc *message, ULONG quality_of_protection )
 {
     gss_iov_buffer_desc iov[4];
     OM_uint32 ret, minor_status;
-    int token_idx, data_idx, conf_state;
+    int token_idx, data_idx, conf_flag, conf_state;
+
+    if (!quality_of_protection)
+        conf_flag = 1; /* confidentiality + integrity */
+    else if (quality_of_protection == SECQOP_WRAP_NO_ENCRYPT)
+        conf_flag = 0; /* only integrity */
+    else
+    {
+        FIXME( "QOP %08x not supported\n", quality_of_protection );
+        return SEC_E_UNSUPPORTED_FUNCTION;
+    }
 
     /* FIXME: multiple data buffers, read-only buffers */
     if ((data_idx = get_buffer_index( message, SECBUFFER_DATA )) == -1) return SEC_E_INVALID_TOKEN;
@@ -1433,7 +1449,7 @@ static NTSTATUS seal_message_iov( gss_ctx_id_t ctxt_handle, SecBufferDesc *messa
     iov[3].buffer.length = 0;
     iov[3].buffer.value  = NULL;
 
-    ret = pgss_wrap_iov( &minor_status, ctxt_handle, 1, GSS_C_QOP_DEFAULT, &conf_state, iov, 4 );
+    ret = pgss_wrap_iov( &minor_status, ctxt_handle, conf_flag, GSS_C_QOP_DEFAULT, &conf_state, iov, 4 );
     TRACE( "gss_wrap_iov returned %08x minor status %08x\n", ret, minor_status );
     if (GSS_ERROR(ret)) trace_gss_status( ret, minor_status );
     if (ret == GSS_S_COMPLETE)
@@ -1446,11 +1462,21 @@ static NTSTATUS seal_message_iov( gss_ctx_id_t ctxt_handle, SecBufferDesc *messa
     return status_gss_to_sspi( ret );
 }
 
-static NTSTATUS seal_message( gss_ctx_id_t ctxt_handle, SecBufferDesc *message )
+static NTSTATUS seal_message( gss_ctx_id_t ctxt_handle, SecBufferDesc *message, ULONG quality_of_protection )
 {
     gss_buffer_desc input, output;
     OM_uint32 ret, minor_status;
-    int token_idx, data_idx, conf_state;
+    int token_idx, data_idx, conf_flag, conf_state;
+
+    if (!quality_of_protection)
+        conf_flag = 1; /* confidentiality + integrity */
+    else if (quality_of_protection == SECQOP_WRAP_NO_ENCRYPT)
+        conf_flag = 0; /* only integrity */
+    else
+    {
+        FIXME( "QOP %08x not supported\n", quality_of_protection );
+        return SEC_E_UNSUPPORTED_FUNCTION;
+    }
 
     /* FIXME: multiple data buffers, read-only buffers */
     if ((data_idx = get_buffer_index( message, SECBUFFER_DATA )) == -1) return SEC_E_INVALID_TOKEN;
@@ -1459,7 +1485,7 @@ static NTSTATUS seal_message( gss_ctx_id_t ctxt_handle, SecBufferDesc *message )
     input.length = message->pBuffers[data_idx].cbBuffer;
     input.value  = message->pBuffers[data_idx].pvBuffer;
 
-    ret = pgss_wrap( &minor_status, ctxt_handle, 1, GSS_C_QOP_DEFAULT, &input, &conf_state, &output );
+    ret = pgss_wrap( &minor_status, ctxt_handle, conf_flag, GSS_C_QOP_DEFAULT, &input, &conf_state, &output );
     TRACE( "gss_wrap returned %08x minor status %08x\n", ret, minor_status );
     if (GSS_ERROR(ret)) trace_gss_status( ret, minor_status );
     if (ret == GSS_S_COMPLETE)
@@ -1488,18 +1514,13 @@ static NTSTATUS NTAPI kerberos_SpSealMessage( LSA_SEC_HANDLE context, ULONG qual
     gss_ctx_id_t ctxt_handle;
 
     TRACE( "(%lx 0x%08x %p %u)\n", context, quality_of_protection, message, message_seq_no );
-    if (quality_of_protection)
-    {
-        FIXME( "flags %08x not supported\n", quality_of_protection );
-        return SEC_E_UNSUPPORTED_FUNCTION;
-    }
     if (message_seq_no) FIXME( "ignoring message_seq_no %u\n", message_seq_no );
 
     if (!context) return SEC_E_INVALID_HANDLE;
     ctxt_handle = ctxthandle_sspi_to_gss( context );
 
-    if (is_dce_style_context( ctxt_handle )) return seal_message_iov( ctxt_handle, message );
-    return seal_message( ctxt_handle, message );
+    if (is_dce_style_context( ctxt_handle )) return seal_message_iov( ctxt_handle, message, quality_of_protection );
+    return seal_message( ctxt_handle, message, quality_of_protection );
 #else
     FIXME( "(%lx 0x%08x %p %u)\n", context, quality_of_protection, message, message_seq_no );
     return SEC_E_UNSUPPORTED_FUNCTION;
