@@ -79,6 +79,21 @@ static const WCHAR conoutW[] = {'C','O','N','O','U','T','$',0};
 /* FIXME: this is not thread safe */
 static HANDLE console_wait_event;
 
+/* map input records to ASCII */
+static void input_records_WtoA( INPUT_RECORD *buffer, int count )
+{
+    UINT cp = GetConsoleCP();
+    int i;
+    char ch;
+
+    for (i = 0; i < count; i++)
+    {
+        if (buffer[i].EventType != KEY_EVENT) continue;
+        WideCharToMultiByte( cp, 0, &buffer[i].Event.KeyEvent.uChar.UnicodeChar, 1, &ch, 1, NULL, NULL );
+        buffer[i].Event.KeyEvent.uChar.AsciiChar = ch;
+    }
+}
+
 static struct termios S_termios;        /* saved termios for bare consoles */
 static BOOL S_termios_raw /* = FALSE */;
 
@@ -310,7 +325,7 @@ HANDLE WINAPI GetConsoleInputWaitHandle(void)
         SERVER_START_REQ(get_console_wait_event)
         {
             if (!wine_server_call_err( req ))
-                console_wait_event = wine_server_ptr_handle( reply->handle );
+                console_wait_event = wine_server_ptr_handle( reply->event );
         }
         SERVER_END_REQ;
     }
@@ -726,6 +741,30 @@ BOOL WINAPI AllocConsole(void)
 }
 
 
+/***********************************************************************
+ *            ReadConsoleA   (KERNEL32.@)
+ */
+BOOL WINAPI ReadConsoleA( HANDLE handle, LPVOID buffer, DWORD length, DWORD *ret_count, void *reserved )
+{
+    LPWSTR strW = HeapAlloc( GetProcessHeap(), 0, length * sizeof(WCHAR) );
+    DWORD count = 0;
+    BOOL ret;
+
+    if (!strW)
+    {
+        SetLastError( ERROR_NOT_ENOUGH_MEMORY );
+        return FALSE;
+    }
+    if ((ret = ReadConsoleW( handle, strW, length, &count, NULL )))
+    {
+        count = WideCharToMultiByte( GetConsoleCP(), 0, strW, count, buffer, length, NULL, NULL );
+        if (ret_count) *ret_count = count;
+    }
+    HeapFree( GetProcessHeap(), 0, strW );
+    return ret;
+}
+
+
 /***********************************************************************
  *            ReadConsoleW   (KERNEL32.@)
  */
@@ -801,6 +840,20 @@ BOOL WINAPI ReadConsoleW(HANDLE hConsoleInput, LPVOID lpBuffer,
 }
 
 
+/***********************************************************************
+ *            ReadConsoleInputA   (KERNEL32.@)
+ */
+BOOL WINAPI ReadConsoleInputA( HANDLE handle, INPUT_RECORD *buffer, DWORD length, DWORD *count )
+{
+    DWORD read;
+
+    if (!ReadConsoleInputW( handle, buffer, length, &read )) return FALSE;
+    input_records_WtoA( buffer, read );
+    if (count) *count = read;
+    return TRUE;
+}
+
+
 /***********************************************************************
  *            ReadConsoleInputW   (KERNEL32.@)
  */
@@ -1126,6 +1179,28 @@ static BOOL write_block(HANDLE hCon, CONSOLE_SCREEN_BUFFER_INFO* csbi,
     return TRUE;
 }
 
+
+/***********************************************************************
+ *            WriteConsoleA   (KERNEL32.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH WriteConsoleA( HANDLE handle, LPCVOID buffer, DWORD length,
+                                             DWORD *written, void *reserved )
+{
+    UINT cp = GetConsoleOutputCP();
+    LPWSTR strW;
+    DWORD lenW;
+    BOOL ret;
+
+    if (written) *written = 0;
+    lenW = MultiByteToWideChar( cp, 0, buffer, length, NULL, 0 );
+    if (!(strW = HeapAlloc( GetProcessHeap(), 0, lenW * sizeof(WCHAR) ))) return FALSE;
+    MultiByteToWideChar( cp, 0, buffer, length, strW, lenW );
+    ret = WriteConsoleW( handle, strW, lenW, written, 0 );
+    HeapFree( GetProcessHeap(), 0, strW );
+    return ret;
+}
+
+
 /***********************************************************************
  *            WriteConsoleW   (KERNEL32.@)
  */
@@ -1627,7 +1702,7 @@ DWORD WINAPI GetNumberOfConsoleFonts(void)
 BOOL WINAPI SetConsoleFont(HANDLE hConsole, DWORD index)
 {
     FIXME("(%p, %u): stub!\n", hConsole, index);
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    SetLastError(LOWORD(E_NOTIMPL) /* win10 1709+ */);
     return FALSE;
 }
 
@@ -1639,17 +1714,28 @@ BOOL WINAPI SetConsoleKeyShortcuts(BOOL set, BYTE keys, VOID *a, DWORD b)
 }
 
 
-BOOL WINAPI GetCurrentConsoleFont(HANDLE hConsole, BOOL maxwindow, LPCONSOLE_FONT_INFO fontinfo)
+BOOL WINAPI GetCurrentConsoleFontEx(HANDLE hConsole, BOOL maxwindow, CONSOLE_FONT_INFOEX *fontinfo)
 {
     BOOL ret;
+    struct
+    {
+        unsigned int color_map[16];
+        WCHAR face_name[LF_FACESIZE];
+    } data;
 
-    memset(fontinfo, 0, sizeof(CONSOLE_FONT_INFO));
+    if (fontinfo->cbSize != sizeof(CONSOLE_FONT_INFOEX))
+    {
+        SetLastError(ERROR_INVALID_PARAMETER);
+        return FALSE;
+    }
 
     SERVER_START_REQ(get_console_output_info)
     {
         req->handle = console_handle_unmap(hConsole);
+        wine_server_set_reply( req, &data, sizeof(data) - sizeof(WCHAR) );
         if ((ret = !wine_server_call_err(req)))
         {
+            fontinfo->nFont = 0;
             if (maxwindow)
             {
                 fontinfo->dwFontSize.X = min(reply->width, reply->max_width);
@@ -1660,12 +1746,39 @@ BOOL WINAPI GetCurrentConsoleFont(HANDLE hConsole, BOOL maxwindow, LPCONSOLE_FON
                 fontinfo->dwFontSize.X = reply->win_right - reply->win_left + 1;
                 fontinfo->dwFontSize.Y = reply->win_bottom - reply->win_top + 1;
             }
+            if (wine_server_reply_size( reply ) > sizeof(data.color_map))
+            {
+                data_size_t len = wine_server_reply_size( reply ) - sizeof(data.color_map);
+                memcpy( fontinfo->FaceName, data.face_name, len );
+                fontinfo->FaceName[len / sizeof(WCHAR)] = 0;
+            }
+            else
+                fontinfo->FaceName[0] = 0;
+            fontinfo->FontFamily = reply->font_pitch_family;
+            fontinfo->FontWeight = reply->font_weight;
         }
     }
     SERVER_END_REQ;
     return ret;
 }
 
+BOOL WINAPI GetCurrentConsoleFont(HANDLE hConsole, BOOL maxwindow, CONSOLE_FONT_INFO *fontinfo)
+{
+    BOOL ret;
+    CONSOLE_FONT_INFOEX res;
+
+    res.cbSize = sizeof(CONSOLE_FONT_INFOEX);
+
+    ret = GetCurrentConsoleFontEx(hConsole, maxwindow, &res);
+    if(ret)
+    {
+        fontinfo->nFont = res.nFont;
+        fontinfo->dwFontSize.X = res.dwFontSize.X;
+        fontinfo->dwFontSize.Y = res.dwFontSize.Y;
+    }
+    return ret;
+}
+
 static COORD get_console_font_size(HANDLE hConsole, DWORD index)
 {
     COORD c = {0,0};
@@ -1711,7 +1824,7 @@ COORD WINAPI GetConsoleFontSize(HANDLE hConsole, DWORD index)
 BOOL WINAPI GetConsoleFontInfo(HANDLE hConsole, BOOL maximize, DWORD numfonts, CONSOLE_FONT_INFO *info)
 {
     FIXME("(%p %d %u %p): stub!\n", hConsole, maximize, numfonts, info);
-    SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
+    SetLastError(LOWORD(E_NOTIMPL) /* win10 1709+ */);
     return FALSE;
 }
 
