@@ -50,28 +50,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(file);
 
-/* info structure for FindFirstFile handle */
-typedef struct
-{
-    DWORD             magic;       /* magic number */
-    HANDLE            handle;      /* handle to directory */
-    CRITICAL_SECTION  cs;          /* crit section protecting this structure */
-    FINDEX_SEARCH_OPS search_op;   /* Flags passed to FindFirst.  */
-    FINDEX_INFO_LEVELS level;      /* Level passed to FindFirst */
-    UNICODE_STRING    path;        /* NT path used to open the directory */
-    BOOL              is_root;     /* is directory the root of the drive? */
-    BOOL              wildcard;    /* did the mask contain wildcard characters? */
-    UINT              data_pos;    /* current position in dir data */
-    UINT              data_len;    /* length of dir data */
-    UINT              data_size;   /* size of data buffer, or 0 when everything has been read */
-    BYTE              data[1];     /* directory data */
-} FIND_FIRST_INFO;
-
-#define FIND_FIRST_MAGIC  0xc0ffee11
-
-static const UINT max_entry_size = offsetof( FILE_BOTH_DIRECTORY_INFORMATION, FileName[256] );
-
-static const WCHAR wildcardsW[] = { '*','?',0 };
 static const WCHAR krnl386W[] = {'k','r','n','l','3','8','6','.','e','x','e','1','6',0};
 
 /***********************************************************************
@@ -113,58 +91,6 @@ static HANDLE create_file_OF( LPCSTR path, INT mode )
 }
 
 
-/***********************************************************************
- *              check_dir_symlink
- *
- * Check if a dir symlink should be returned by FindNextFile.
- */
-static BOOL check_dir_symlink( FIND_FIRST_INFO *info, const FILE_BOTH_DIR_INFORMATION *file_info )
-{
-    UNICODE_STRING str;
-    ANSI_STRING unix_name;
-    struct stat st, parent_st;
-    BOOL ret = TRUE;
-    DWORD len;
-
-    str.MaximumLength = info->path.Length + sizeof(WCHAR) + file_info->FileNameLength;
-    if (!(str.Buffer = HeapAlloc( GetProcessHeap(), 0, str.MaximumLength ))) return TRUE;
-    memcpy( str.Buffer, info->path.Buffer, info->path.Length );
-    len = info->path.Length / sizeof(WCHAR);
-    if (!len || str.Buffer[len-1] != '\\') str.Buffer[len++] = '\\';
-    memcpy( str.Buffer + len, file_info->FileName, file_info->FileNameLength );
-    str.Length = len * sizeof(WCHAR) + file_info->FileNameLength;
-
-    unix_name.Buffer = NULL;
-    if (!wine_nt_to_unix_file_name( &str, &unix_name, OPEN_EXISTING, FALSE ) &&
-        !stat( unix_name.Buffer, &st ))
-    {
-        char *p = unix_name.Buffer + unix_name.Length - 1;
-
-        /* skip trailing slashes */
-        while (p > unix_name.Buffer && *p == '/') p--;
-
-        while (ret && p > unix_name.Buffer)
-        {
-            while (p > unix_name.Buffer && *p != '/') p--;
-            while (p > unix_name.Buffer && *p == '/') p--;
-            p[1] = 0;
-            if (!stat( unix_name.Buffer, &parent_st ) &&
-                parent_st.st_dev == st.st_dev &&
-                parent_st.st_ino == st.st_ino)
-            {
-                WARN( "suppressing dir symlink %s pointing to parent %s\n",
-                      debugstr_wn( str.Buffer, str.Length/sizeof(WCHAR) ),
-                      debugstr_a( unix_name.Buffer ));
-                ret = FALSE;
-            }
-        }
-    }
-    RtlFreeAnsiString( &unix_name );
-    RtlFreeUnicodeString( &str );
-    return ret;
-}
-
-
 /***********************************************************************
  *           FILE_SetDosError
  *
@@ -273,10 +199,25 @@ DWORD FILE_name_WtoA( LPCWSTR src, INT srclen, LPSTR dest, INT destlen )
     DWORD ret;
 
     if (srclen < 0) srclen = strlenW( src ) + 1;
-    if (!AreFileApisANSI())
-        RtlUnicodeToOemN( dest, destlen, &ret, src, srclen * sizeof(WCHAR) );
+    if (!destlen)
+    {
+        if (!AreFileApisANSI())
+        {
+            UNICODE_STRING strW;
+            strW.Buffer = (WCHAR *)src;
+            strW.Length = srclen * sizeof(WCHAR);
+            ret = RtlUnicodeStringToOemSize( &strW ) - 1;
+        }
+        else
+            RtlUnicodeToMultiByteSize( &ret, src, srclen * sizeof(WCHAR) );
+    }
     else
-        RtlUnicodeToMultiByteN( dest, destlen, &ret, src, srclen * sizeof(WCHAR) );
+    {
+        if (!AreFileApisANSI())
+            RtlUnicodeToOemN( dest, destlen, &ret, src, srclen * sizeof(WCHAR) );
+        else
+            RtlUnicodeToMultiByteN( dest, destlen, &ret, src, srclen * sizeof(WCHAR) );
+    }
     return ret;
 }
 
@@ -413,6 +354,61 @@ UINT WINAPI SetHandleCount( UINT count )
 }
 
 
+/*************************************************************************
+ *           ReadFile   (KERNEL32.@)
+ */
+BOOL WINAPI KERNEL32_ReadFile( HANDLE file, LPVOID buffer, DWORD count,
+                               LPDWORD result, LPOVERLAPPED overlapped )
+{
+    if (result) *result = 0;
+
+    if (is_console_handle( file ))
+    {
+        DWORD conread, mode;
+
+        if (!ReadConsoleA( file, buffer, count, &conread, NULL) || !GetConsoleMode( file, &mode ))
+            return FALSE;
+        /* ctrl-Z (26) means end of file on window (if at beginning of buffer)
+         * but Unix uses ctrl-D (4), and ctrl-Z is a bad idea on Unix :-/
+         * So map both ctrl-D ctrl-Z to EOF.
+         */
+        if ((mode & ENABLE_PROCESSED_INPUT) && conread > 0 &&
+            (((char *)buffer)[0] == 26 || ((char *)buffer)[0] == 4))
+        {
+            conread = 0;
+        }
+        if (result) *result = conread;
+        return TRUE;
+    }
+    return ReadFile( file, buffer, count, result, overlapped );
+}
+
+
+/*************************************************************************
+ *           WriteFile   (KERNEL32.@)
+ */
+BOOL WINAPI KERNEL32_WriteFile( HANDLE file, LPCVOID buffer, DWORD count,
+                                LPDWORD result, LPOVERLAPPED overlapped )
+{
+    if (is_console_handle( file )) return WriteConsoleA( file, buffer, count, result, NULL );
+    return WriteFile( file, buffer, count, result, overlapped );
+}
+
+
+/*************************************************************************
+ *           FlushFileBuffers   (KERNEL32.@)
+ */
+BOOL WINAPI KERNEL32_FlushFileBuffers( HANDLE file )
+{
+    IO_STATUS_BLOCK iosb;
+
+    /* this will fail (as expected) for an output handle */
+    if (is_console_handle( file )) return FlushConsoleInputBuffer( file );
+
+    return set_ntstatus( NtFlushBuffersFile( file, &iosb ));
+}
+
+
 /**************************************************************************
  *                      Operations on file names                          *
  **************************************************************************/
@@ -427,11 +423,7 @@ BOOL WINAPI ReplaceFileW(LPCWSTR lpReplacedFileName, LPCWSTR lpReplacementFileNa
                          LPVOID lpExclude, LPVOID lpReserved)
 {
     UNICODE_STRING nt_replaced_name, nt_replacement_name;
-    ANSI_STRING unix_replaced_name, unix_replacement_name, unix_backup_name;
-    HANDLE hReplaced = NULL, hReplacement = NULL, hBackup = NULL;
-    DWORD error = ERROR_SUCCESS;
-    UINT replaced_flags;
-    BOOL ret = FALSE;
+    HANDLE hReplacement = NULL;
     NTSTATUS status;
     IO_STATUS_BLOCK io;
     OBJECT_ATTRIBUTES attr;
@@ -451,10 +443,6 @@ BOOL WINAPI ReplaceFileW(LPCWSTR lpReplacedFileName, LPCWSTR lpReplacementFileNa
         return FALSE;
     }
 
-    unix_replaced_name.Buffer = NULL;
-    unix_replacement_name.Buffer = NULL;
-    unix_backup_name.Buffer = NULL;
-
     attr.Length = sizeof(attr);
     attr.RootDirectory = 0;
     attr.Attributes = OBJ_CASE_INSENSITIVE;
@@ -465,39 +453,21 @@ BOOL WINAPI ReplaceFileW(LPCWSTR lpReplacedFileName, LPCWSTR lpReplacementFileNa
     /* Open the "replaced" file for reading */
     if (!(RtlDosPathNameToNtPathName_U(lpReplacedFileName, &nt_replaced_name, NULL, NULL)))
     {
-        error = ERROR_PATH_NOT_FOUND;
-        goto fail;
+        SetLastError( ERROR_PATH_NOT_FOUND );
+        return FALSE;
     }
-    replaced_flags = lpBackupFileName ? FILE_OPEN : FILE_OPEN_IF;
     attr.ObjectName = &nt_replaced_name;
-    status = NtOpenFile(&hReplaced, GENERIC_READ|DELETE|SYNCHRONIZE,
-                        &attr, &io,
-                        FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SHARE_DELETE,
-                        FILE_SYNCHRONOUS_IO_NONALERT|FILE_NON_DIRECTORY_FILE);
-    if (status == STATUS_SUCCESS)
-        status = wine_nt_to_unix_file_name(&nt_replaced_name, &unix_replaced_name, replaced_flags, FALSE);
-    RtlFreeUnicodeString(&nt_replaced_name);
-    if (status != STATUS_SUCCESS)
-    {
-        if (status == STATUS_OBJECT_NAME_NOT_FOUND)
-            error = ERROR_FILE_NOT_FOUND;
-        else
-            error = ERROR_UNABLE_TO_REMOVE_REPLACED;
-        goto fail;
-    }
 
     /* Replacement should fail if replaced is READ_ONLY */
-    status = NtQueryInformationFile(hReplaced, &io, &info, sizeof(info), FileBasicInformation);
+    status = NtQueryAttributesFile(&attr, &info);
+    RtlFreeUnicodeString(&nt_replaced_name);
     if (status != STATUS_SUCCESS)
-    {
-        error = RtlNtStatusToDosError(status);
-        goto fail;
-    }
+        return set_ntstatus( status );
 
     if (info.FileAttributes & FILE_ATTRIBUTE_READONLY)
     {
-        error = ERROR_ACCESS_DENIED;
-        goto fail;
+        SetLastError( ERROR_ACCESS_DENIED );
+        return FALSE;
     }
 
     /*
@@ -506,65 +476,41 @@ BOOL WINAPI ReplaceFileW(LPCWSTR lpReplacedFileName, LPCWSTR lpReplacementFileNa
      */
     if (!(RtlDosPathNameToNtPathName_U(lpReplacementFileName, &nt_replacement_name, NULL, NULL)))
     {
-        error = ERROR_PATH_NOT_FOUND;
-        goto fail;
+        SetLastError( ERROR_PATH_NOT_FOUND );
+        return FALSE;
     }
     attr.ObjectName = &nt_replacement_name;
     status = NtOpenFile(&hReplacement,
                         GENERIC_READ|GENERIC_WRITE|DELETE|WRITE_DAC|SYNCHRONIZE,
                         &attr, &io, 0,
                         FILE_SYNCHRONOUS_IO_NONALERT|FILE_NON_DIRECTORY_FILE);
-    if (status == STATUS_SUCCESS)
-        status = wine_nt_to_unix_file_name(&nt_replacement_name, &unix_replacement_name, FILE_OPEN, FALSE);
     RtlFreeUnicodeString(&nt_replacement_name);
     if (status != STATUS_SUCCESS)
-    {
-        error = RtlNtStatusToDosError(status);
-        goto fail;
-    }
+        return set_ntstatus( status );
+    NtClose( hReplacement );
 
     /* If the user wants a backup then that needs to be performed first */
     if (lpBackupFileName)
     {
-        UNICODE_STRING nt_backup_name;
-        FILE_BASIC_INFORMATION replaced_info;
+        if (!MoveFileExW( lpReplacedFileName, lpBackupFileName, MOVEFILE_REPLACE_EXISTING ))
+            return FALSE;
+    }
+    else
+    {
+        /* ReplaceFile() can replace an open target. To do this, we need to move
+         * it out of the way first. */
+        static const WCHAR prefixW[] = {'r','f',0};
+        WCHAR temp_path[MAX_PATH], temp_file[MAX_PATH];
 
-        /* Obtain the file attributes from the "replaced" file */
-        status = NtQueryInformationFile(hReplaced, &io, &replaced_info,
-                                        sizeof(replaced_info),
-                                        FileBasicInformation);
-        if (status != STATUS_SUCCESS)
-        {
-            error = RtlNtStatusToDosError(status);
-            goto fail;
-        }
+        if (!GetTempPathW( ARRAY_SIZE(temp_path), temp_path )
+                || !GetTempFileNameW( temp_path, prefixW, 0, temp_file )
+                || !MoveFileExW( lpReplacedFileName, temp_file, MOVEFILE_REPLACE_EXISTING ))
+            return FALSE;
 
-        if (!(RtlDosPathNameToNtPathName_U(lpBackupFileName, &nt_backup_name, NULL, NULL)))
+        if (!DeleteFileW( temp_file ))
         {
-            error = ERROR_PATH_NOT_FOUND;
-            goto fail;
-        }
-        attr.ObjectName = &nt_backup_name;
-        /* Open the backup with permissions to write over it */
-        status = NtCreateFile(&hBackup, GENERIC_WRITE | SYNCHRONIZE,
-                              &attr, &io, NULL, replaced_info.FileAttributes,
-                              FILE_SHARE_WRITE, FILE_OPEN_IF,
-                              FILE_SYNCHRONOUS_IO_NONALERT|FILE_NON_DIRECTORY_FILE,
-                              NULL, 0);
-        if (status == STATUS_SUCCESS)
-            status = wine_nt_to_unix_file_name(&nt_backup_name, &unix_backup_name, FILE_OPEN_IF, FALSE);
-        RtlFreeUnicodeString(&nt_backup_name);
-        if (status != STATUS_SUCCESS)
-        {
-            error = RtlNtStatusToDosError(status);
-            goto fail;
-        }
-
-        /* If an existing backup exists then copy over it */
-        if (rename(unix_replaced_name.Buffer, unix_backup_name.Buffer) == -1)
-        {
-            error = ERROR_UNABLE_TO_REMOVE_REPLACED; /* is this correct? */
-            goto fail;
+            SetLastError( ERROR_UNABLE_TO_REMOVE_REPLACED );
+            return FALSE;
         }
     }
 
@@ -572,37 +518,17 @@ BOOL WINAPI ReplaceFileW(LPCWSTR lpReplacedFileName, LPCWSTR lpReplacementFileNa
      * Now that the backup has been performed (if requested), copy the replacement
      * into place
      */
-    if (rename(unix_replacement_name.Buffer, unix_replaced_name.Buffer) == -1)
+    if (!MoveFileExW( lpReplacementFileName, lpReplacedFileName, 0 ))
     {
-        if (errno == EACCES)
-        {
-            /* Inappropriate permissions on "replaced", rename will fail */
-            error = ERROR_UNABLE_TO_REMOVE_REPLACED;
-            goto fail;
-        }
         /* on failure we need to indicate whether a backup was made */
         if (!lpBackupFileName)
-            error = ERROR_UNABLE_TO_MOVE_REPLACEMENT;
+            SetLastError( ERROR_UNABLE_TO_MOVE_REPLACEMENT );
         else
-            error = ERROR_UNABLE_TO_MOVE_REPLACEMENT_2;
-        goto fail;
+            SetLastError( ERROR_UNABLE_TO_MOVE_REPLACEMENT_2 );
+        return FALSE;
     }
-    /* Success! */
-    ret = TRUE;
 
-    /* Perform resource cleanup */
-fail:
-    if (hBackup) CloseHandle(hBackup);
-    if (hReplaced) CloseHandle(hReplaced);
-    if (hReplacement) CloseHandle(hReplacement);
-    RtlFreeAnsiString(&unix_backup_name);
-    RtlFreeAnsiString(&unix_replacement_name);
-    RtlFreeAnsiString(&unix_replaced_name);
-
-    /* If there was an error, set the error code */
-    if(!ret)
-        SetLastError(error);
-    return ret;
+    return TRUE;
 }
 
 
@@ -647,405 +573,6 @@ BOOL WINAPI ReplaceFileA(LPCSTR lpReplacedFileName,LPCSTR lpReplacementFileName,
 }
 
 
-/*************************************************************************
- *           FindFirstFileExW  (KERNEL32.@)
- *
- * NOTE: The FindExSearchLimitToDirectories is ignored - it gives the same
- * results as FindExSearchNameMatch
- */
-HANDLE WINAPI FindFirstFileExW( LPCWSTR filename, FINDEX_INFO_LEVELS level,
-                                LPVOID data, FINDEX_SEARCH_OPS search_op,
-                                LPVOID filter, DWORD flags)
-{
-    WCHAR *mask;
-    BOOL has_wildcard = FALSE;
-    FIND_FIRST_INFO *info = NULL;
-    UNICODE_STRING nt_name;
-    OBJECT_ATTRIBUTES attr;
-    IO_STATUS_BLOCK io;
-    NTSTATUS status;
-    DWORD size, device = 0;
-
-    TRACE("%s %d %p %d %p %x\n", debugstr_w(filename), level, data, search_op, filter, flags);
-
-    if (flags & ~FIND_FIRST_EX_LARGE_FETCH)
-    {
-        FIXME("flags not implemented 0x%08x\n", flags );
-    }
-    if (search_op != FindExSearchNameMatch && search_op != FindExSearchLimitToDirectories)
-    {
-        FIXME("search_op not implemented 0x%08x\n", search_op);
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return INVALID_HANDLE_VALUE;
-    }
-    if (level != FindExInfoStandard && level != FindExInfoBasic)
-    {
-        FIXME("info level %d not implemented\n", level );
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return INVALID_HANDLE_VALUE;
-    }
-
-    if (!RtlDosPathNameToNtPathName_U( filename, &nt_name, &mask, NULL ))
-    {
-        SetLastError( ERROR_PATH_NOT_FOUND );
-        return INVALID_HANDLE_VALUE;
-    }
-
-    if (!mask && (device = RtlIsDosDeviceName_U( filename )))
-    {
-        static const WCHAR dotW[] = {'.',0};
-        WCHAR *dir = NULL;
-
-        /* we still need to check that the directory can be opened */
-
-        if (HIWORD(device))
-        {
-            if (!(dir = HeapAlloc( GetProcessHeap(), 0, HIWORD(device) + sizeof(WCHAR) )))
-            {
-                SetLastError( ERROR_NOT_ENOUGH_MEMORY );
-                goto error;
-            }
-            memcpy( dir, filename, HIWORD(device) );
-            dir[HIWORD(device)/sizeof(WCHAR)] = 0;
-        }
-        RtlFreeUnicodeString( &nt_name );
-        if (!RtlDosPathNameToNtPathName_U( dir ? dir : dotW, &nt_name, &mask, NULL ))
-        {
-            HeapFree( GetProcessHeap(), 0, dir );
-            SetLastError( ERROR_PATH_NOT_FOUND );
-            goto error;
-        }
-        HeapFree( GetProcessHeap(), 0, dir );
-        size = 0;
-    }
-    else if (!mask || !*mask)
-    {
-        SetLastError( ERROR_FILE_NOT_FOUND );
-        goto error;
-    }
-    else
-    {
-        nt_name.Length = (mask - nt_name.Buffer) * sizeof(WCHAR);
-        has_wildcard = strpbrkW( mask, wildcardsW ) != NULL;
-        size = has_wildcard ? 8192 : max_entry_size;
-    }
-
-    if (!(info = HeapAlloc( GetProcessHeap(), 0, offsetof( FIND_FIRST_INFO, data[size] ))))
-    {
-        SetLastError( ERROR_NOT_ENOUGH_MEMORY );
-        goto error;
-    }
-
-    /* check if path is the root of the drive, skipping the \??\ prefix */
-    info->is_root = FALSE;
-    if (nt_name.Length >= 6 * sizeof(WCHAR) && nt_name.Buffer[5] == ':')
-    {
-        DWORD pos = 6;
-        while (pos * sizeof(WCHAR) < nt_name.Length && nt_name.Buffer[pos] == '\\') pos++;
-        info->is_root = (pos * sizeof(WCHAR) >= nt_name.Length);
-    }
-
-    attr.Length = sizeof(attr);
-    attr.RootDirectory = 0;
-    attr.Attributes = OBJ_CASE_INSENSITIVE;
-    attr.ObjectName = &nt_name;
-    attr.SecurityDescriptor = NULL;
-    attr.SecurityQualityOfService = NULL;
-
-    status = NtOpenFile( &info->handle, GENERIC_READ | SYNCHRONIZE, &attr, &io,
-                         FILE_SHARE_READ | FILE_SHARE_WRITE,
-                         FILE_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT | FILE_OPEN_FOR_BACKUP_INTENT );
-
-    if (status != STATUS_SUCCESS)
-    {
-        if (status == STATUS_OBJECT_NAME_NOT_FOUND)
-            SetLastError( ERROR_PATH_NOT_FOUND );
-        else
-            SetLastError( RtlNtStatusToDosError(status) );
-        goto error;
-    }
-
-    RtlInitializeCriticalSection( &info->cs );
-    info->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": FIND_FIRST_INFO.cs");
-    info->path     = nt_name;
-    info->magic    = FIND_FIRST_MAGIC;
-    info->wildcard = has_wildcard;
-    info->data_pos = 0;
-    info->data_len = 0;
-    info->data_size = size;
-    info->search_op = search_op;
-    info->level     = level;
-
-    if (device)
-    {
-        WIN32_FIND_DATAW *wfd = data;
-
-        memset( wfd, 0, sizeof(*wfd) );
-        memcpy( wfd->cFileName, filename + HIWORD(device)/sizeof(WCHAR), LOWORD(device) );
-        wfd->dwFileAttributes = FILE_ATTRIBUTE_ARCHIVE;
-        CloseHandle( info->handle );
-        info->handle = 0;
-    }
-    else
-    {
-        UNICODE_STRING mask_str;
-
-        RtlInitUnicodeString( &mask_str, mask );
-        status = NtQueryDirectoryFile( info->handle, 0, NULL, NULL, &io, info->data, info->data_size,
-                                       FileBothDirectoryInformation, FALSE, &mask_str, TRUE );
-        if (status)
-        {
-            FindClose( info );
-            SetLastError( RtlNtStatusToDosError( status ) );
-            return INVALID_HANDLE_VALUE;
-        }
-
-        info->data_len = io.Information;
-        if (!has_wildcard) info->data_size = 0;  /* we read everything */
-
-        if (!FindNextFileW( info, data ))
-        {
-            TRACE( "%s not found\n", debugstr_w(filename) );
-            FindClose( info );
-            SetLastError( ERROR_FILE_NOT_FOUND );
-            return INVALID_HANDLE_VALUE;
-        }
-        if (!has_wildcard)  /* we can't find two files with the same name */
-        {
-            CloseHandle( info->handle );
-            info->handle = 0;
-        }
-    }
-    return info;
-
-error:
-    HeapFree( GetProcessHeap(), 0, info );
-    RtlFreeUnicodeString( &nt_name );
-    return INVALID_HANDLE_VALUE;
-}
-
-
-/*************************************************************************
- *           FindNextFileW   (KERNEL32.@)
- */
-BOOL WINAPI FindNextFileW( HANDLE handle, WIN32_FIND_DATAW *data )
-{
-    FIND_FIRST_INFO *info;
-    FILE_BOTH_DIR_INFORMATION *dir_info;
-    BOOL ret = FALSE;
-    NTSTATUS status;
-
-    TRACE("%p %p\n", handle, data);
-
-    if (!handle || handle == INVALID_HANDLE_VALUE)
-    {
-        SetLastError( ERROR_INVALID_HANDLE );
-        return ret;
-    }
-    info = handle;
-    if (info->magic != FIND_FIRST_MAGIC)
-    {
-        SetLastError( ERROR_INVALID_HANDLE );
-        return ret;
-    }
-
-    RtlEnterCriticalSection( &info->cs );
-
-    if (!info->handle) SetLastError( ERROR_NO_MORE_FILES );
-    else for (;;)
-    {
-        if (info->data_pos >= info->data_len)  /* need to read some more data */
-        {
-            IO_STATUS_BLOCK io;
-
-            if (info->data_size)
-                status = NtQueryDirectoryFile( info->handle, 0, NULL, NULL, &io, info->data, info->data_size,
-                                               FileBothDirectoryInformation, FALSE, NULL, FALSE );
-            else
-                status = STATUS_NO_MORE_FILES;
-
-            if (status)
-            {
-                SetLastError( RtlNtStatusToDosError( status ) );
-                if (status == STATUS_NO_MORE_FILES)
-                {
-                    CloseHandle( info->handle );
-                    info->handle = 0;
-                }
-                break;
-            }
-            info->data_len = io.Information;
-            info->data_pos = 0;
-        }
-
-        dir_info = (FILE_BOTH_DIR_INFORMATION *)(info->data + info->data_pos);
-
-        if (dir_info->NextEntryOffset) info->data_pos += dir_info->NextEntryOffset;
-        else info->data_pos = info->data_len;
-
-        /* don't return '.' and '..' in the root of the drive */
-        if (info->is_root)
-        {
-            if (dir_info->FileNameLength == sizeof(WCHAR) && dir_info->FileName[0] == '.') continue;
-            if (dir_info->FileNameLength == 2 * sizeof(WCHAR) &&
-                dir_info->FileName[0] == '.' && dir_info->FileName[1] == '.') continue;
-        }
-
-        /* check for dir symlink */
-        if ((dir_info->FileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
-            (dir_info->FileAttributes & FILE_ATTRIBUTE_REPARSE_POINT) &&
-            info->wildcard)
-        {
-            if (!check_dir_symlink( info, dir_info )) continue;
-        }
-
-        data->dwFileAttributes = dir_info->FileAttributes;
-        data->ftCreationTime   = *(FILETIME *)&dir_info->CreationTime;
-        data->ftLastAccessTime = *(FILETIME *)&dir_info->LastAccessTime;
-        data->ftLastWriteTime  = *(FILETIME *)&dir_info->LastWriteTime;
-        data->nFileSizeHigh    = dir_info->EndOfFile.QuadPart >> 32;
-        data->nFileSizeLow     = (DWORD)dir_info->EndOfFile.QuadPart;
-        data->dwReserved0      = 0;
-        data->dwReserved1      = 0;
-
-        memcpy( data->cFileName, dir_info->FileName, dir_info->FileNameLength );
-        data->cFileName[dir_info->FileNameLength/sizeof(WCHAR)] = 0;
-
-        if (info->level != FindExInfoBasic)
-        {
-            memcpy( data->cAlternateFileName, dir_info->ShortName, dir_info->ShortNameLength );
-            data->cAlternateFileName[dir_info->ShortNameLength/sizeof(WCHAR)] = 0;
-        }
-        else
-            data->cAlternateFileName[0] = 0;
-
-        TRACE("returning %s (%s)\n",
-              debugstr_w(data->cFileName), debugstr_w(data->cAlternateFileName) );
-
-        ret = TRUE;
-        break;
-    }
-
-    RtlLeaveCriticalSection( &info->cs );
-    return ret;
-}
-
-
-/*************************************************************************
- *           FindClose   (KERNEL32.@)
- */
-BOOL WINAPI DECLSPEC_HOTPATCH FindClose( HANDLE handle )
-{
-    FIND_FIRST_INFO *info = handle;
-
-    if (!handle || handle == INVALID_HANDLE_VALUE)
-    {
-        SetLastError( ERROR_INVALID_HANDLE );
-        return FALSE;
-    }
-
-    __TRY
-    {
-        if (info->magic == FIND_FIRST_MAGIC)
-        {
-            RtlEnterCriticalSection( &info->cs );
-            if (info->magic == FIND_FIRST_MAGIC)  /* in case someone else freed it in the meantime */
-            {
-                info->magic = 0;
-                if (info->handle) CloseHandle( info->handle );
-                info->handle = 0;
-                RtlFreeUnicodeString( &info->path );
-                info->data_pos = 0;
-                info->data_len = 0;
-                RtlLeaveCriticalSection( &info->cs );
-                info->cs.DebugInfo->Spare[0] = 0;
-                RtlDeleteCriticalSection( &info->cs );
-                HeapFree( GetProcessHeap(), 0, info );
-            }
-        }
-    }
-    __EXCEPT_PAGE_FAULT
-    {
-        WARN("Illegal handle %p\n", handle);
-        SetLastError( ERROR_INVALID_HANDLE );
-        return FALSE;
-    }
-    __ENDTRY
-
-    return TRUE;
-}
-
-
-/*************************************************************************
- *           FindFirstFileA   (KERNEL32.@)
- */
-HANDLE WINAPI FindFirstFileA( LPCSTR lpFileName, WIN32_FIND_DATAA *lpFindData )
-{
-    return FindFirstFileExA(lpFileName, FindExInfoStandard, lpFindData,
-                            FindExSearchNameMatch, NULL, 0);
-}
-
-/*************************************************************************
- *           FindFirstFileExA   (KERNEL32.@)
- */
-HANDLE WINAPI FindFirstFileExA( LPCSTR lpFileName, FINDEX_INFO_LEVELS fInfoLevelId,
-                                LPVOID lpFindFileData, FINDEX_SEARCH_OPS fSearchOp,
-                                LPVOID lpSearchFilter, DWORD dwAdditionalFlags)
-{
-    HANDLE handle;
-    WIN32_FIND_DATAA *dataA;
-    WIN32_FIND_DATAW dataW;
-    WCHAR *nameW;
-
-    if (!(nameW = FILE_name_AtoW( lpFileName, FALSE ))) return INVALID_HANDLE_VALUE;
-
-    handle = FindFirstFileExW(nameW, fInfoLevelId, &dataW, fSearchOp, lpSearchFilter, dwAdditionalFlags);
-    if (handle == INVALID_HANDLE_VALUE) return handle;
-
-    dataA = lpFindFileData;
-    dataA->dwFileAttributes = dataW.dwFileAttributes;
-    dataA->ftCreationTime   = dataW.ftCreationTime;
-    dataA->ftLastAccessTime = dataW.ftLastAccessTime;
-    dataA->ftLastWriteTime  = dataW.ftLastWriteTime;
-    dataA->nFileSizeHigh    = dataW.nFileSizeHigh;
-    dataA->nFileSizeLow     = dataW.nFileSizeLow;
-    FILE_name_WtoA( dataW.cFileName, -1, dataA->cFileName, sizeof(dataA->cFileName) );
-    FILE_name_WtoA( dataW.cAlternateFileName, -1, dataA->cAlternateFileName,
-                    sizeof(dataA->cAlternateFileName) );
-    return handle;
-}
-
-
-/*************************************************************************
- *           FindFirstFileW   (KERNEL32.@)
- */
-HANDLE WINAPI FindFirstFileW( LPCWSTR lpFileName, WIN32_FIND_DATAW *lpFindData )
-{
-    return FindFirstFileExW(lpFileName, FindExInfoStandard, lpFindData,
-                            FindExSearchNameMatch, NULL, 0);
-}
-
-
-/*************************************************************************
- *           FindNextFileA   (KERNEL32.@)
- */
-BOOL WINAPI FindNextFileA( HANDLE handle, WIN32_FIND_DATAA *data )
-{
-    WIN32_FIND_DATAW dataW;
-
-    if (!FindNextFileW( handle, &dataW )) return FALSE;
-    data->dwFileAttributes = dataW.dwFileAttributes;
-    data->ftCreationTime   = dataW.ftCreationTime;
-    data->ftLastAccessTime = dataW.ftLastAccessTime;
-    data->ftLastWriteTime  = dataW.ftLastWriteTime;
-    data->nFileSizeHigh    = dataW.nFileSizeHigh;
-    data->nFileSizeLow     = dataW.nFileSizeLow;
-    FILE_name_WtoA( dataW.cFileName, -1, data->cFileName, sizeof(data->cFileName) );
-    FILE_name_WtoA( dataW.cAlternateFileName, -1, data->cAlternateFileName,
-                    sizeof(data->cAlternateFileName) );
-    return TRUE;
-}
-
 /**************************************************************************
  *           FindFirstStreamW   (KERNEL32.@)
  */
