@@ -72,12 +72,6 @@ extern void ENV_CopyStartupInformation(void) DECLSPEC_HIDDEN;
 /* computername.c */
 extern void COMPUTERNAME_Init(void) DECLSPEC_HIDDEN;
 
-/* locale.c */
-extern void LOCALE_Init(void) DECLSPEC_HIDDEN;
-
-/* time.c */
-extern void TIMEZONE_InitRegistry(void) DECLSPEC_HIDDEN;
-
 /* oldconfig.c */
 extern void convert_old_config(void) DECLSPEC_HIDDEN;
 
