@@ -46,6 +46,7 @@
 
 #include "windef.h"
 #include "winbase.h"
+#include "winternl.h"
 #include "lzexpand.h"
 
 #include "wine/unicode.h"
@@ -138,7 +139,7 @@ static INT read_header(HFILE fd,struct lzfileheader *head)
 	/* We can't directly read the lzfileheader struct due to
 	 * structure element alignment
 	 */
-	if (_lread(fd,buf,LZ_HEADER_LEN)<LZ_HEADER_LEN)
+	if (_lread(fd,buf,LZ_HEADER_LEN) != LZ_HEADER_LEN)
 		return 0;
 	memcpy(head->magic,buf,LZ_MAGIC_LEN);
 	memcpy(&(head->compressiontype),buf+LZ_MAGIC_LEN,1);
@@ -285,7 +286,7 @@ INT WINAPI GetExpandedNameA( LPSTR in, LPSTR out )
 		if (fnislowercased)
 			head.lastchar=tolower(head.lastchar);
 		else
-			head.lastchar=toupper(head.lastchar);
+			head.lastchar=RtlUpperChar(head.lastchar);
 	}
 
 	/* now look where to replace the last character */
