@@ -38,17 +38,17 @@ static int    myARGC;
 static char** myARGV;
 
 static BOOL (WINAPI *pCheckRemoteDebuggerPresent)(HANDLE,PBOOL);
-static BOOL (WINAPI *pDebugActiveProcessStop)(DWORD);
-static BOOL (WINAPI *pDebugSetProcessKillOnExit)(BOOL);
-static BOOL (WINAPI *pIsDebuggerPresent)(void);
 
 static void (WINAPI *pDbgBreakPoint)(void);
 
+static NTSTATUS  (WINAPI *pNtSuspendProcess)(HANDLE process);
+static NTSTATUS  (WINAPI *pNtResumeProcess)(HANDLE process);
+
 static LONG child_failures;
 
 static HMODULE ntdll;
 
-static void WINAPIV WINETEST_PRINTF_ATTR(2, 3) test_child_ok(int condition, const char *msg, ...)
+static void WINAPIV __WINE_PRINTF_ATTR(2, 3) test_child_ok(int condition, const char *msg, ...)
 {
     __ms_va_list valist;
 
@@ -174,12 +174,7 @@ static void run_background_thread(void)
     CloseHandle(thread);
 }
 
-typedef struct
-{
-    DWORD pid;
-} crash_blackbox_t;
-
-static void doCrash(int argc,  char** argv)
+static void doCrash(void)
 {
     volatile char* p;
 
@@ -189,13 +184,6 @@ static void doCrash(int argc,  char** argv)
 
     run_background_thread();
 
-    if (argc >= 4)
-    {
-        crash_blackbox_t blackbox;
-        blackbox.pid=GetCurrentProcessId();
-        save_blackbox(argv[3], &blackbox, sizeof(blackbox), NULL);
-    }
-
     /* Just crash */
     trace("child: crashing...\n");
     p=NULL;
@@ -390,16 +378,16 @@ static void next_event_(unsigned line, struct debugger_context *ctx, unsigned ti
 #define wait_for_breakpoint(a) wait_for_breakpoint_(__LINE__,a)
 static void wait_for_breakpoint_(unsigned line, struct debugger_context *ctx)
 {
-    do next_event_(line, ctx, 2000);
+    do next_event_(line, ctx, WAIT_EVENT_TIMEOUT);
     while (ctx->ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT || ctx->ev.dwDebugEventCode == UNLOAD_DLL_DEBUG_EVENT
            || ctx->ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT);
 
-    ok(ctx->ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx->ev.dwDebugEventCode);
-    ok(ctx->ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
-       ctx->ev.u.Exception.ExceptionRecord.ExceptionCode);
+    ok_(__FILE__,line)(ctx->ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx->ev.dwDebugEventCode);
+    ok_(__FILE__,line)(ctx->ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
+                       ctx->ev.u.Exception.ExceptionRecord.ExceptionCode);
 }
 
-static void process_attach_events(struct debugger_context *ctx)
+static void process_attach_events(struct debugger_context *ctx, BOOL pass_exception)
 {
     DEBUG_EVENT ev;
     BOOL ret;
@@ -445,6 +433,13 @@ static void process_attach_events(struct debugger_context *ctx)
        ctx->ev.u.Exception.ExceptionRecord.ExceptionCode);
     ok(ctx->ev.u.Exception.ExceptionRecord.ExceptionAddress == pDbgBreakPoint, "ExceptionAddress != DbgBreakPoint\n");
 
+    if (pass_exception)
+    {
+        ret = ContinueDebugEvent(ctx->ev.dwProcessId, ctx->ev.dwThreadId, DBG_EXCEPTION_NOT_HANDLED);
+        ok(ret, "ContinueDebugEvent failed, last error %d.\n", GetLastError());
+        ctx->ev.dwDebugEventCode = -1;
+    }
+
     /* flush debug events */
     do next_event(ctx, POLL_EVENT_TIMEOUT);
     while (ctx->ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT || ctx->ev.dwDebugEventCode == UNLOAD_DLL_DEBUG_EVENT
@@ -477,7 +472,7 @@ static void doDebugger(int argc, char** argv)
     if (strstr(myARGV[2], "process"))
     {
         strcat(buf, "processing debug messages\n");
-        process_attach_events(&ctx);
+        process_attach_events(&ctx, FALSE);
     }
 
     debug_event=(argc >= 6 ? (HANDLE)(INT_PTR)atol(argv[5]) : NULL);
@@ -514,7 +509,7 @@ static void doDebugger(int argc, char** argv)
     blackbox.nokill_err=0;
     if (strstr(myARGV[2], "nokill"))
     {
-        blackbox.nokill_rc=pDebugSetProcessKillOnExit(FALSE);
+        blackbox.nokill_rc = DebugSetProcessKillOnExit(FALSE);
         if (!blackbox.nokill_rc)
             blackbox.nokill_err=GetLastError();
     }
@@ -524,7 +519,7 @@ static void doDebugger(int argc, char** argv)
     blackbox.detach_err=0;
     if (strstr(myARGV[2], "detach"))
     {
-        blackbox.detach_rc=pDebugActiveProcessStop(blackbox.pid);
+        blackbox.detach_rc = DebugActiveProcessStop(blackbox.pid);
         if (!blackbox.detach_rc)
             blackbox.detach_err=GetLastError();
     }
@@ -560,11 +555,9 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
     HANDLE start_event, done_event;
     char* cmd;
     char dbglog[MAX_PATH];
-    char childlog[MAX_PATH];
     PROCESS_INFORMATION	info;
     STARTUPINFOA startup;
     DWORD exit_code;
-    crash_blackbox_t crash_blackbox;
     debugger_blackbox_t dbg_blackbox;
     DWORD wait_code;
 
@@ -592,9 +585,8 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
     ok(ret == ERROR_SUCCESS, "unable to set AeDebug/debugger: ret=%d\n", ret);
     HeapFree(GetProcessHeap(), 0, cmd);
 
-    get_file_name(childlog);
-    cmd=HeapAlloc(GetProcessHeap(), 0, strlen(argv0)+16+strlen(dbglog)+2+1);
-    sprintf(cmd, "%s debugger crash \"%s\"", argv0, childlog);
+    cmd = HeapAlloc(GetProcessHeap(), 0, strlen(argv0) + 16);
+    sprintf(cmd, "%s debugger crash", argv0);
 
     trace("running %s...\n", dbgtasks);
     memset(&startup, 0, sizeof(startup));
@@ -618,7 +610,6 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
         WaitForSingleObject(info.hProcess, 5000);
         CloseHandle(info.hProcess);
         DeleteFileA(dbglog);
-        DeleteFileA(childlog);
         win_skip("Giving up on child process\n");
         return;
     }
@@ -654,18 +645,16 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
     if (skip_crash_and_debug)
     {
         DeleteFileA(dbglog);
-        DeleteFileA(childlog);
         win_skip("Giving up on debugger\n");
         return;
     }
 #endif
     ok(wait_code == WAIT_OBJECT_0, "Timed out waiting for the debugger\n");
 
-    ok(load_blackbox(childlog, &crash_blackbox, sizeof(crash_blackbox)), "failed to open: %s\n", childlog);
     ok(load_blackbox(dbglog, &dbg_blackbox, sizeof(dbg_blackbox)), "failed to open: %s\n", dbglog);
 
     ok(dbg_blackbox.argc == 6, "wrong debugger argument count: %d\n", dbg_blackbox.argc);
-    ok(dbg_blackbox.pid == crash_blackbox.pid, "the child and debugged pids don't match: %d != %d\n", crash_blackbox.pid, dbg_blackbox.pid);
+    ok(dbg_blackbox.pid == info.dwProcessId, "the child and debugged pids don't match: %d != %d\n", info.dwProcessId, dbg_blackbox.pid);
     ok(dbg_blackbox.debug_rc, "debugger: SetEvent(debug_event) failed err=%d\n", dbg_blackbox.debug_err);
     ok(dbg_blackbox.attach_rc, "DebugActiveProcess(%d) failed err=%d\n", dbg_blackbox.pid, dbg_blackbox.attach_err);
     ok(dbg_blackbox.nokill_rc, "DebugSetProcessKillOnExit(FALSE) failed err=%d\n", dbg_blackbox.nokill_err);
@@ -673,7 +662,6 @@ static void crash_and_debug(HKEY hkey, const char* argv0, const char* dbgtasks)
     ok(!dbg_blackbox.failures, "debugger reported %u failures\n", dbg_blackbox.failures);
 
     DeleteFileA(dbglog);
-    DeleteFileA(childlog);
 }
 
 static void crash_and_winedbg(HKEY hkey, const char* argv0)
@@ -776,17 +764,9 @@ static void test_ExitCode(void)
     ok(disposition == REG_OPENED_EXISTING_KEY, "expected REG_OPENED_EXISTING_KEY, got %d\n", disposition);
     crash_and_debug(hkey, test_exe, "dbg,event,order");
     crash_and_debug(hkey, test_exe, "dbg,attach,event,code2");
-    if (pDebugSetProcessKillOnExit)
-        crash_and_debug(hkey, test_exe, "dbg,attach,event,nokill");
-    else
-        win_skip("DebugSetProcessKillOnExit is not available\n");
-    if (pDebugActiveProcessStop)
-    {
-        crash_and_debug(hkey, test_exe, "dbg,attach,event,detach");
-        crash_and_debug(hkey, test_exe, "dbg,attach,detach,late");
-    }
-    else
-        win_skip("DebugActiveProcessStop is not available\n");
+    crash_and_debug(hkey, test_exe, "dbg,attach,event,nokill");
+    crash_and_debug(hkey, test_exe, "dbg,attach,event,detach");
+    crash_and_debug(hkey, test_exe, "dbg,attach,detach,late");
     crash_and_debug(hkey, test_exe, "dbg,attach,process,event,detach");
 
     if (disposition == REG_CREATED_NEW_KEY)
@@ -863,7 +843,7 @@ static void doChild(int argc, char **argv)
     child_ok(ret, "CheckRemoteDebuggerPresent failed, last error %#x.\n", GetLastError());
     child_ok(debug, "Expected debug != 0, got %#x.\n", debug);
 
-    ret = pDebugActiveProcessStop(ppid);
+    ret = DebugActiveProcessStop(ppid);
     child_ok(ret, "DebugActiveProcessStop failed, last error %#x.\n", GetLastError());
 
     ret = pCheckRemoteDebuggerPresent(parent, &debug);
@@ -873,7 +853,7 @@ static void doChild(int argc, char **argv)
     ret = CloseHandle(parent);
     child_ok(ret, "CloseHandle failed, last error %#x.\n", GetLastError());
 
-    ret = pIsDebuggerPresent();
+    ret = IsDebuggerPresent();
     child_ok(ret, "Expected ret != 0, got %#x.\n", ret);
     ret = pCheckRemoteDebuggerPresent(GetCurrentProcess(), &debug);
     child_ok(ret, "CheckRemoteDebuggerPresent failed, last error %#x.\n", GetLastError());
@@ -881,7 +861,7 @@ static void doChild(int argc, char **argv)
 
     NtCurrentTeb()->Peb->BeingDebugged = FALSE;
 
-    ret = pIsDebuggerPresent();
+    ret = IsDebuggerPresent();
     child_ok(!ret, "Expected ret != 0, got %#x.\n", ret);
     ret = pCheckRemoteDebuggerPresent(GetCurrentProcess(), &debug);
     child_ok(ret, "CheckRemoteDebuggerPresent failed, last error %#x.\n", GetLastError());
@@ -905,9 +885,9 @@ static void test_debug_loop(int argc, char **argv)
     char *cmd;
     BOOL ret;
 
-    if (!pDebugActiveProcessStop || !pCheckRemoteDebuggerPresent)
+    if (!pCheckRemoteDebuggerPresent)
     {
-        win_skip("DebugActiveProcessStop or CheckRemoteDebuggerPresent not available, skipping test.\n");
+        win_skip("CheckRemoteDebuggerPresent not available, skipping test.\n");
         return;
     }
 
@@ -1023,7 +1003,7 @@ static void doChildren(int argc, char **argv)
     HeapFree(GetProcessHeap(), 0, cmd);
 }
 
-static void test_debug_children(char *name, DWORD flag, BOOL debug_child)
+static void test_debug_children(const char *name, DWORD flag, BOOL debug_child, BOOL pass_exception)
 {
     const char *arguments = "debugger children";
     struct child_blackbox blackbox;
@@ -1036,9 +1016,9 @@ static void test_debug_children(char *name, DWORD flag, BOOL debug_child)
     BOOL debug, ret;
     struct debugger_context ctx = { 0 };
 
-    if (!pDebugActiveProcessStop || !pCheckRemoteDebuggerPresent)
+    if (!pCheckRemoteDebuggerPresent)
     {
-        win_skip("DebugActiveProcessStop or CheckRemoteDebuggerPresent not available, skipping test.\n");
+        win_skip("CheckRemoteDebuggerPresent not available, skipping test.\n");
         return;
     }
 
@@ -1096,7 +1076,7 @@ static void test_debug_children(char *name, DWORD flag, BOOL debug_child)
     {
         DWORD last_thread;
 
-        process_attach_events(&ctx);
+        process_attach_events(&ctx, pass_exception);
         ok(ctx.pid == pi.dwProcessId, "unexpected dwProcessId %x\n", ctx.pid);
 
         ret = DebugBreakProcess(pi.hProcess);
@@ -1118,6 +1098,13 @@ static void test_debug_children(char *name, DWORD flag, BOOL debug_child)
 
         ret = SetEvent(event_attach);
         ok(ret, "SetEvent failed, last error %d.\n", GetLastError());
+
+        if (pass_exception)
+        {
+            ret = ContinueDebugEvent(ctx.ev.dwProcessId, ctx.ev.dwThreadId, DBG_EXCEPTION_NOT_HANDLED);
+            ok(ret, "ContinueDebugEvent failed, last error %d.\n", GetLastError());
+            ctx.ev.dwDebugEventCode = -1;
+        }
     }
 
     do next_event(&ctx, WAIT_EVENT_TIMEOUT);
@@ -1267,9 +1254,10 @@ static void test_debugger(const char *argv0)
     struct debugger_context ctx = { 0 };
     PROCESS_INFORMATION pi;
     STARTUPINFOA si;
+    NTSTATUS status;
     HANDLE event, thread;
     BYTE *mem, buf[4096], *proc_code, *thread_proc, byte;
-    unsigned int i, worker_cnt, exception_cnt;
+    unsigned int i, worker_cnt, exception_cnt, skip_reply_later;
     struct debuggee_thread *debuggee_thread;
     char *cmd;
     BOOL ret;
@@ -1288,6 +1276,62 @@ static void test_debugger(const char *argv0)
 
     next_event(&ctx, WAIT_EVENT_TIMEOUT);
     ok(ctx.ev.dwDebugEventCode == CREATE_PROCESS_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+
+    if ((skip_reply_later = !ContinueDebugEvent(ctx.ev.dwProcessId, ctx.ev.dwThreadId, DBG_REPLY_LATER)))
+        win_skip("Skipping unsupported DBG_REPLY_LATER tests\n");
+    else
+    {
+        DEBUG_EVENT de;
+
+        de = ctx.ev;
+        ctx.ev.dwDebugEventCode = -1;
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        ok(de.dwDebugEventCode == ctx.ev.dwDebugEventCode,
+           "dwDebugEventCode differs: %x (was %x)\n", ctx.ev.dwDebugEventCode, de.dwDebugEventCode);
+        ok(de.dwProcessId == ctx.ev.dwProcessId,
+           "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de.dwProcessId);
+        ok(de.dwThreadId == ctx.ev.dwThreadId,
+           "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de.dwThreadId);
+
+        /* Suspending the thread should prevent other attach debug events
+         * to be received until it's resumed */
+        thread = OpenThread(THREAD_SUSPEND_RESUME, FALSE, ctx.ev.dwThreadId);
+        ok(thread != INVALID_HANDLE_VALUE, "OpenThread failed, last error:%u\n", GetLastError());
+
+        status = NtSuspendThread(thread, NULL);
+        ok(!status, "NtSuspendThread failed, last error:%u\n", GetLastError());
+
+        ok(ContinueDebugEvent(ctx.ev.dwProcessId, ctx.ev.dwThreadId, DBG_REPLY_LATER),
+           "ContinueDebugEvent failed, last error:%u\n", GetLastError());
+        ok(!WaitForDebugEvent(&ctx.ev, POLL_EVENT_TIMEOUT), "WaitForDebugEvent succeeded.\n");
+
+        status = NtResumeThread(thread, NULL);
+        ok(!status, "NtResumeThread failed, last error:%u\n", GetLastError());
+
+        ret = CloseHandle(thread);
+        ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+
+        ok(WaitForDebugEvent(&ctx.ev, POLL_EVENT_TIMEOUT), "WaitForDebugEvent failed.\n");
+        ok(de.dwDebugEventCode == ctx.ev.dwDebugEventCode,
+           "dwDebugEventCode differs: %x (was %x)\n", ctx.ev.dwDebugEventCode, de.dwDebugEventCode);
+
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        ok(ctx.ev.dwDebugEventCode == LOAD_DLL_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+        de = ctx.ev;
+
+        ok(ContinueDebugEvent(ctx.ev.dwProcessId, ctx.ev.dwThreadId, DBG_REPLY_LATER),
+           "ContinueDebugEvent failed, last error:%u\n", GetLastError());
+
+        ctx.ev.dwDebugEventCode = -1;
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        ok(de.dwDebugEventCode == ctx.ev.dwDebugEventCode,
+           "dwDebugEventCode differs: %x (was %x)\n", ctx.ev.dwDebugEventCode, de.dwDebugEventCode);
+        ok(de.dwProcessId == ctx.ev.dwProcessId,
+           "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de.dwProcessId);
+        ok(de.dwThreadId == ctx.ev.dwThreadId,
+           "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de.dwThreadId);
+    }
+
     wait_for_breakpoint(&ctx);
     do next_event(&ctx, POLL_EVENT_TIMEOUT);
     while(ctx.ev.dwDebugEventCode != -1);
@@ -1357,8 +1401,183 @@ static void test_debugger(const char *argv0)
     }
     else win_skip("call_debug_service_code not supported on this architecture\n");
 
+    if (skip_reply_later)
+        win_skip("Skipping unsupported DBG_REPLY_LATER tests\n");
+    else if (sizeof(loop_code) > 1)
+    {
+        HANDLE thread_a, thread_b;
+        DEBUG_EVENT de_a, de_b;
+
+        memset(buf, OP_BP, sizeof(buf));
+        memcpy(proc_code, &loop_code, sizeof(loop_code));
+        ret = WriteProcessMemory(pi.hProcess, mem, buf, sizeof(buf), NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        byte = OP_BP;
+        ret = WriteProcessMemory(pi.hProcess, thread_proc + 1, &byte, 1, NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        thread_a = CreateRemoteThread(pi.hProcess, NULL, 0, (void*)thread_proc, NULL, 0, NULL);
+        ok(thread_a != NULL, "CreateRemoteThread failed: %u\n", GetLastError());
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        ok(ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+        de_a = ctx.ev;
+
+        thread_b = CreateRemoteThread(pi.hProcess, NULL, 0, (void*)thread_proc, NULL, 0, NULL);
+        ok(thread_b != NULL, "CreateRemoteThread failed: %u\n", GetLastError());
+        do next_event(&ctx, POLL_EVENT_TIMEOUT);
+        while(ctx.ev.dwDebugEventCode != CREATE_THREAD_DEBUG_EVENT);
+        de_b = ctx.ev;
+
+        status = NtSuspendThread(thread_b, NULL);
+        ok(!status, "NtSuspendThread failed, last error:%u\n", GetLastError());
+        ok(ContinueDebugEvent(ctx.ev.dwProcessId, ctx.ev.dwThreadId, DBG_REPLY_LATER),
+           "ContinueDebugEvent failed, last error:%u\n", GetLastError());
+
+        ctx.ev.dwDebugEventCode = -1;
+        next_event(&ctx, WAIT_EVENT_TIMEOUT);
+        ok(ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT,
+           "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+        ok(de_a.dwProcessId == ctx.ev.dwProcessId,
+           "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de_a.dwProcessId);
+        ok(de_a.dwThreadId == ctx.ev.dwThreadId,
+           "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de_a.dwThreadId);
+        de_a = ctx.ev;
+
+        byte = 0xc3; /* ret */
+        ret = WriteProcessMemory(pi.hProcess, thread_proc + 1, &byte, 1, NULL);
+        ok(ret, "WriteProcessMemory failed: %u\n", GetLastError());
+
+        ok(pNtSuspendProcess != NULL, "NtSuspendProcess not found\n");
+        ok(pNtResumeProcess != NULL, "pNtResumeProcess not found\n");
+        if (pNtSuspendProcess && pNtResumeProcess)
+        {
+            status = pNtSuspendProcess(pi.hProcess);
+            ok(!status, "NtSuspendProcess failed, last error:%u\n", GetLastError());
+            ok(ContinueDebugEvent(ctx.ev.dwProcessId, ctx.ev.dwThreadId, DBG_REPLY_LATER),
+               "ContinueDebugEvent failed, last error:%u\n", GetLastError());
+            ok(!WaitForDebugEvent(&ctx.ev, POLL_EVENT_TIMEOUT), "WaitForDebugEvent succeeded.\n");
+
+            status = NtResumeThread(thread_b, NULL);
+            ok(!status, "NtResumeThread failed, last error:%u\n", GetLastError());
+            ok(!WaitForDebugEvent(&ctx.ev, POLL_EVENT_TIMEOUT), "WaitForDebugEvent succeeded.\n");
+
+            status = pNtResumeProcess(pi.hProcess);
+            ok(!status, "pNtResumeProcess failed, last error:%u\n", GetLastError());
+        }
+        else
+        {
+            status = NtResumeThread(thread_b, NULL);
+            ok(!status, "NtResumeThread failed, last error:%u\n", GetLastError());
+            ok(!WaitForDebugEvent(&ctx.ev, POLL_EVENT_TIMEOUT), "WaitForDebugEvent succeeded.\n");
+        }
+
+        /* Testing shows that on windows the debug event order between threads
+         * is not guaranteed.
+         *
+         * Now we expect thread_a to report:
+         * - its delayed EXCEPTION_DEBUG_EVENT
+         * - EXIT_THREAD_DEBUG_EVENT
+         *
+         * and thread_b to report:
+         * - its delayed CREATE_THREAD_DEBUG_EVENT
+         * - EXIT_THREAD_DEBUG_EVENT
+         *
+         * We should not get EXCEPTION_DEBUG_EVENT from thread_b as we updated
+         * its instructions before continuing CREATE_THREAD_DEBUG_EVENT.
+         */
+        ctx.ev.dwDebugEventCode = -1;
+        next_event(&ctx, POLL_EVENT_TIMEOUT);
+
+        if (ctx.ev.dwDebugEventCode == EXCEPTION_DEBUG_EVENT)
+        {
+            ok(de_a.dwDebugEventCode == ctx.ev.dwDebugEventCode,
+               "dwDebugEventCode differs: %x (was %x)\n", ctx.ev.dwDebugEventCode, de_a.dwDebugEventCode);
+            ok(de_a.dwProcessId == ctx.ev.dwProcessId,
+               "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de_a.dwProcessId);
+            ok(de_a.dwThreadId == ctx.ev.dwThreadId,
+               "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de_a.dwThreadId);
+
+            next_event(&ctx, POLL_EVENT_TIMEOUT);
+            if (ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT)
+            {
+                ok(de_a.dwProcessId == ctx.ev.dwProcessId,
+                   "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de_a.dwProcessId);
+                ok(de_a.dwThreadId == ctx.ev.dwThreadId,
+                   "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de_a.dwThreadId);
+
+                ret = CloseHandle(thread_a);
+                ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+                thread_a = NULL;
+
+                next_event(&ctx, POLL_EVENT_TIMEOUT);
+            }
+
+            ok(de_b.dwDebugEventCode == ctx.ev.dwDebugEventCode,
+               "dwDebugEventCode differs: %x (was %x)\n", ctx.ev.dwDebugEventCode, de_b.dwDebugEventCode);
+            ok(de_b.dwProcessId == ctx.ev.dwProcessId,
+               "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de_b.dwProcessId);
+            ok(de_b.dwThreadId == ctx.ev.dwThreadId,
+               "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de_b.dwThreadId);
+        }
+        else
+        {
+            ok(de_b.dwDebugEventCode == ctx.ev.dwDebugEventCode,
+               "dwDebugEventCode differs: %x (was %x)\n", ctx.ev.dwDebugEventCode, de_b.dwDebugEventCode);
+            ok(de_b.dwProcessId == ctx.ev.dwProcessId,
+               "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de_b.dwProcessId);
+            ok(de_b.dwThreadId == ctx.ev.dwThreadId,
+               "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de_b.dwThreadId);
+
+            next_event(&ctx, POLL_EVENT_TIMEOUT);
+            if (ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT)
+            {
+                ok(de_b.dwProcessId == ctx.ev.dwProcessId,
+                   "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de_b.dwProcessId);
+                ok(de_b.dwThreadId == ctx.ev.dwThreadId,
+                   "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de_b.dwThreadId);
+
+                ret = CloseHandle(thread_b);
+                ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+                thread_b = NULL;
+
+                next_event(&ctx, POLL_EVENT_TIMEOUT);
+            }
+
+            ok(de_a.dwDebugEventCode == ctx.ev.dwDebugEventCode,
+               "dwDebugEventCode differs: %x (was %x)\n", ctx.ev.dwDebugEventCode, de_a.dwDebugEventCode);
+            ok(de_a.dwProcessId == ctx.ev.dwProcessId,
+               "dwProcessId differs: %x (was %x)\n", ctx.ev.dwProcessId, de_a.dwProcessId);
+            ok(de_a.dwThreadId == ctx.ev.dwThreadId,
+               "dwThreadId differs: %x (was %x)\n", ctx.ev.dwThreadId, de_a.dwThreadId);
+        }
+
+        if (thread_a)
+        {
+            next_event(&ctx, POLL_EVENT_TIMEOUT);
+            ok(ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT,
+               "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+
+            ret = CloseHandle(thread_a);
+            ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+        }
+
+
+        if (thread_b)
+        {
+            next_event(&ctx, POLL_EVENT_TIMEOUT);
+            ok(ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT,
+               "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
+
+            ret = CloseHandle(thread_b);
+            ok(ret, "CloseHandle failed, last error %d.\n", GetLastError());
+        }
+    }
+
     if (sizeof(loop_code) > 1)
     {
+        struct debuggee_thread *prev_thread;
+
         memset(buf, OP_BP, sizeof(buf));
         memcpy(proc_code, &loop_code, sizeof(loop_code));
         ret = WriteProcessMemory(pi.hProcess, mem, buf, sizeof(buf), NULL);
@@ -1372,7 +1591,7 @@ static void test_debugger(const char *argv0)
             thread = CreateRemoteThread(pi.hProcess, NULL, 0, (void*)thread_proc, NULL, 0, NULL);
             ok(thread != NULL, "CreateRemoteThread failed: %u\n", GetLastError());
 
-            next_event(&ctx, 20000);
+            next_event(&ctx, WAIT_EVENT_TIMEOUT);
             ok(ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT, "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
 
             ret = CloseHandle(thread);
@@ -1386,10 +1605,8 @@ static void test_debugger(const char *argv0)
         expect_breakpoint_exception(&ctx, thread_proc + 1);
         exception_cnt = 1;
 
-        debuggee_thread = ctx.current_thread;
+        prev_thread = ctx.current_thread;
         fetch_process_context(&ctx);
-        ok(get_ip(&ctx.current_thread->ctx) == thread_proc + 2, "unexpected instruction pointer %p\n",
-           get_ip(&ctx.current_thread->ctx));
 
         byte = 0xc3; /* ret */
         ret = WriteProcessMemory(pi.hProcess, thread_proc + 1, &byte, 1, NULL);
@@ -1403,20 +1620,25 @@ static void test_debugger(const char *argv0)
             ret = WaitForDebugEvent(&ev, 10);
             ok(GetLastError() == ERROR_SEM_TIMEOUT, "WaitForDebugEvent returned %x(%u)\n", ret, GetLastError());
 
-            next_event(&ctx, 100);
+            next_event(&ctx, POLL_EVENT_TIMEOUT);
             if (ctx.ev.dwDebugEventCode != EXCEPTION_DEBUG_EVENT) break;
             trace("exception at %p in thread %04x\n", ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress, ctx.ev.dwThreadId);
             ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT, "ExceptionCode = %x\n",
                ctx.ev.u.Exception.ExceptionRecord.ExceptionCode);
             ok(ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress == thread_proc + 1,
                "ExceptionAddress = %p\n", ctx.ev.u.Exception.ExceptionRecord.ExceptionAddress);
-            ok(get_ip(&ctx.current_thread->ctx) == thread_proc + 2
-               || broken(get_ip(&ctx.current_thread->ctx) == thread_proc), /* sometimes observed on win10 */
-               "unexpected instruction pointer %p\n",
-               get_ip(&ctx.current_thread->ctx));
+            ok(get_ip(&prev_thread->ctx) == thread_proc + 2
+               || broken(get_ip(&prev_thread->ctx) == thread_proc), /* sometimes observed on win10 */
+               "unexpected instruction pointer %p\n", get_ip(&prev_thread->ctx));
+            prev_thread = ctx.current_thread;
             exception_cnt++;
         }
 
+        /* for some reason sometimes on Windows one thread has a different address. this is always the thread
+         * with the last reported exception, so we simply skip the check for the last exception unless it's the only one. */
+        if (exception_cnt == 1)
+            ok(get_ip(&prev_thread->ctx) == thread_proc + 2, "unexpected instruction pointer %p\n", get_ip(&prev_thread->ctx));
+
         trace("received %u exceptions\n", exception_cnt);
 
         for (;;)
@@ -1425,7 +1647,7 @@ static void test_debugger(const char *argv0)
                || broken(ctx.ev.dwDebugEventCode == CREATE_THREAD_DEBUG_EVENT), /* sometimes happens on vista */
                "dwDebugEventCode = %d\n", ctx.ev.dwDebugEventCode);
             if (ctx.ev.dwDebugEventCode == EXIT_THREAD_DEBUG_EVENT && !--worker_cnt) break;
-            next_event(&ctx, 2000);
+            next_event(&ctx, WAIT_EVENT_TIMEOUT);
         }
     }
 
@@ -1452,17 +1674,16 @@ START_TEST(debugger)
 
     hdll=GetModuleHandleA("kernel32.dll");
     pCheckRemoteDebuggerPresent=(void*)GetProcAddress(hdll, "CheckRemoteDebuggerPresent");
-    pDebugActiveProcessStop=(void*)GetProcAddress(hdll, "DebugActiveProcessStop");
-    pDebugSetProcessKillOnExit=(void*)GetProcAddress(hdll, "DebugSetProcessKillOnExit");
-    pIsDebuggerPresent=(void*)GetProcAddress(hdll, "IsDebuggerPresent");
 
     ntdll = GetModuleHandleA("ntdll.dll");
     pDbgBreakPoint = (void*)GetProcAddress(ntdll, "DbgBreakPoint");
+    pNtSuspendProcess = (void*)GetProcAddress(ntdll, "NtSuspendProcess");
+    pNtResumeProcess = (void*)GetProcAddress(ntdll, "NtResumeProcess");
 
     myARGC=winetest_get_mainargs(&myARGV);
     if (myARGC >= 3 && strcmp(myARGV[2], "crash") == 0)
     {
-        doCrash(myARGC, myARGV);
+        doCrash();
     }
     else if (myARGC >= 3 && strncmp(myARGV[2], "dbg,", 4) == 0)
     {
@@ -1487,10 +1708,11 @@ START_TEST(debugger)
         test_ExitCode();
         test_RemoteDebugger();
         test_debug_loop(myARGC, myARGV);
-        test_debug_children(myARGV[0], DEBUG_PROCESS, TRUE);
-        test_debug_children(myARGV[0], DEBUG_ONLY_THIS_PROCESS, FALSE);
-        test_debug_children(myARGV[0], DEBUG_PROCESS|DEBUG_ONLY_THIS_PROCESS, FALSE);
-        test_debug_children(myARGV[0], 0, FALSE);
+        test_debug_children(myARGV[0], DEBUG_PROCESS, TRUE, FALSE);
+        test_debug_children(myARGV[0], DEBUG_ONLY_THIS_PROCESS, FALSE, FALSE);
+        test_debug_children(myARGV[0], DEBUG_PROCESS|DEBUG_ONLY_THIS_PROCESS, FALSE, FALSE);
+        test_debug_children(myARGV[0], 0, FALSE, FALSE);
+        test_debug_children(myARGV[0], 0, FALSE, TRUE);
         test_debugger(myARGV[0]);
     }
 }
