@@ -1102,7 +1102,7 @@ static void test_debug_heap( const char *argv0, DWORD flags )
     ok( ret, "failed to create child process error %u\n", GetLastError() );
     if (ret)
     {
-        winetest_wait_child_process( info.hProcess );
+        wait_child_process( info.hProcess );
         CloseHandle( info.hThread );
         CloseHandle( info.hProcess );
     }
@@ -1206,7 +1206,7 @@ static void test_GetPhysicallyInstalledSystemMemory(void)
 
     total_memory = 0;
     ret = pGetPhysicallyInstalledSystemMemory(&total_memory);
-    ok(ret, "GetPhysicallyInstalledSystemMemory unexpectedly failed\n");
+    ok(ret, "GetPhysicallyInstalledSystemMemory unexpectedly failed (%u)\n", GetLastError());
     ok(total_memory != 0, "expected total_memory != 0\n");
 
     memstatus.dwLength = sizeof(memstatus);
