@@ -28,13 +28,12 @@
 #include "winternl.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "kernelbase.h"
+#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(kernelbase);
 
 
-HANDLE kernel32_handle = 0;
 BOOL is_wow64 = FALSE;
 
 /***********************************************************************
@@ -46,7 +45,7 @@ BOOL WINAPI DllMain( HINSTANCE hinst, DWORD reason, LPVOID reserved )
     {
         DisableThreadLibraryCalls( hinst );
         IsWow64Process( GetCurrentProcess(), &is_wow64 );
-        kernel32_handle = GetModuleHandleA( "kernel32.dll" );
+        init_locale();
         init_startup_info( NtCurrentTeb()->Peb->ProcessParameters );
     }
     return TRUE;
@@ -321,12 +320,6 @@ static HRESULT lcid_to_rfc1766(LCID lcid, WCHAR *rfc1766, INT len)
 
 HRESULT WINAPI GetAcceptLanguagesW(WCHAR *langbuf, DWORD *buflen)
 {
-    static const WCHAR keyW[] = {
-        'S','o','f','t','w','a','r','e','\\',
-        'M','i','c','r','o','s','o','f','t','\\',
-        'I','n','t','e','r','n','e','t',' ','E','x','p','l','o','r','e','r','\\',
-        'I','n','t','e','r','n','a','t','i','o','n','a','l',0};
-    static const WCHAR valueW[] = {'A','c','c','e','p','t','L','a','n','g','u','a','g','e',0};
     DWORD mystrlen, mytype;
     WCHAR *mystr;
     LCID mylcid;
@@ -343,8 +336,9 @@ HRESULT WINAPI GetAcceptLanguagesW(WCHAR *langbuf, DWORD *buflen)
     len = mystrlen * sizeof(WCHAR);
     mystr = heap_alloc(len);
     mystr[0] = 0;
-    RegOpenKeyExW(HKEY_CURRENT_USER, keyW, 0, KEY_QUERY_VALUE, &mykey);
-    lres = RegQueryValueExW(mykey, valueW, 0, &mytype, (PBYTE)mystr, &len);
+    RegOpenKeyExW(HKEY_CURRENT_USER, L"Software\\Microsoft\\Internet Explorer\\International",
+                  0, KEY_QUERY_VALUE, &mykey);
+    lres = RegQueryValueExW(mykey, L"AcceptLanguage", 0, &mytype, (PBYTE)mystr, &len);
     RegCloseKey(mykey);
     len = lstrlenW(mystr);
 
