@@ -27,6 +27,7 @@
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #define NONAMELESSUNION
+#define NONAMELESSSTRUCT
 #include "windef.h"
 #include "winbase.h"
 #include "winnls.h"
@@ -38,6 +39,7 @@
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(heap);
+WINE_DECLARE_DEBUG_CHANNEL(virtual);
 
 
 /***********************************************************************
@@ -57,6 +59,104 @@ BOOL WINAPI DECLSPEC_HOTPATCH FlushViewOfFile( const void *base, SIZE_T size )
 }
 
 
+/***********************************************************************
+ *          GetLargePageMinimum   (kernelbase.@)
+ */
+SIZE_T WINAPI GetLargePageMinimum(void)
+{
+    return 2 * 1024 * 1024;
+}
+
+
+/***********************************************************************
+ *          GetNativeSystemInfo   (kernelbase.@)
+ */
+void WINAPI DECLSPEC_HOTPATCH GetNativeSystemInfo( SYSTEM_INFO *si )
+{
+    GetSystemInfo( si );
+    if (!is_wow64) return;
+    switch (si->u.s.wProcessorArchitecture)
+    {
+    case PROCESSOR_ARCHITECTURE_INTEL:
+        si->u.s.wProcessorArchitecture = PROCESSOR_ARCHITECTURE_AMD64;
+        si->dwProcessorType = PROCESSOR_AMD_X8664;
+        break;
+    default:
+        FIXME( "Add the proper information for %d in wow64 mode\n", si->u.s.wProcessorArchitecture );
+    }
+}
+
+
+/***********************************************************************
+ *          GetSystemInfo   (kernelbase.@)
+ */
+void WINAPI DECLSPEC_HOTPATCH GetSystemInfo( SYSTEM_INFO *si )
+{
+    SYSTEM_BASIC_INFORMATION basic_info;
+    SYSTEM_CPU_INFORMATION cpu_info;
+
+    if (!set_ntstatus( NtQuerySystemInformation( SystemBasicInformation,
+                                                 &basic_info, sizeof(basic_info), NULL )) ||
+        !set_ntstatus( NtQuerySystemInformation( SystemCpuInformation,
+                                                 &cpu_info, sizeof(cpu_info), NULL )))
+        return;
+
+    si->u.s.wProcessorArchitecture  = cpu_info.Architecture;
+    si->u.s.wReserved               = 0;
+    si->dwPageSize                  = basic_info.PageSize;
+    si->lpMinimumApplicationAddress = basic_info.LowestUserAddress;
+    si->lpMaximumApplicationAddress = basic_info.HighestUserAddress;
+    si->dwActiveProcessorMask       = basic_info.ActiveProcessorsAffinityMask;
+    si->dwNumberOfProcessors        = basic_info.NumberOfProcessors;
+    si->dwAllocationGranularity     = basic_info.AllocationGranularity;
+    si->wProcessorLevel             = cpu_info.Level;
+    si->wProcessorRevision          = cpu_info.Revision;
+
+    switch (cpu_info.Architecture)
+    {
+    case PROCESSOR_ARCHITECTURE_INTEL:
+        switch (cpu_info.Level)
+        {
+        case 3:  si->dwProcessorType = PROCESSOR_INTEL_386;     break;
+        case 4:  si->dwProcessorType = PROCESSOR_INTEL_486;     break;
+        case 5:
+        case 6:  si->dwProcessorType = PROCESSOR_INTEL_PENTIUM; break;
+        default: si->dwProcessorType = PROCESSOR_INTEL_PENTIUM; break;
+        }
+        break;
+    case PROCESSOR_ARCHITECTURE_PPC:
+        switch (cpu_info.Level)
+        {
+        case 1:  si->dwProcessorType = PROCESSOR_PPC_601;       break;
+        case 3:
+        case 6:  si->dwProcessorType = PROCESSOR_PPC_603;       break;
+        case 4:  si->dwProcessorType = PROCESSOR_PPC_604;       break;
+        case 9:  si->dwProcessorType = PROCESSOR_PPC_604;       break;
+        case 20: si->dwProcessorType = PROCESSOR_PPC_620;       break;
+        default: si->dwProcessorType = 0;
+        }
+        break;
+    case PROCESSOR_ARCHITECTURE_AMD64:
+        si->dwProcessorType = PROCESSOR_AMD_X8664;
+        break;
+    case PROCESSOR_ARCHITECTURE_ARM:
+        switch (cpu_info.Level)
+        {
+        case 4:  si->dwProcessorType = PROCESSOR_ARM_7TDMI;     break;
+        default: si->dwProcessorType = PROCESSOR_ARM920;
+        }
+        break;
+    case PROCESSOR_ARCHITECTURE_ARM64:
+        si->dwProcessorType = 0;
+        break;
+    default:
+        FIXME( "Unknown processor architecture %x\n", cpu_info.Architecture );
+        si->dwProcessorType = 0;
+        break;
+    }
+}
+
+
 /***********************************************************************
  *          GetSystemFileCacheSize   (kernelbase.@)
  */
@@ -290,6 +390,42 @@ BOOL WINAPI DECLSPEC_HOTPATCH WriteProcessMemory( HANDLE process, void *addr, co
 }
 
 
+/* IsBadStringPtrA replacement for kernelbase, to catch exception in debug traces. */
+BOOL WINAPI IsBadStringPtrA( LPCSTR str, UINT_PTR max )
+{
+    if (!str) return TRUE;
+    __TRY
+    {
+        volatile const char *p = str;
+        while (p != str + max) if (!*p++) break;
+    }
+    __EXCEPT_PAGE_FAULT
+    {
+        return TRUE;
+    }
+    __ENDTRY
+    return FALSE;
+}
+
+
+/* IsBadStringPtrW replacement for kernelbase, to catch exception in debug traces. */
+BOOL WINAPI IsBadStringPtrW( LPCWSTR str, UINT_PTR max )
+{
+    if (!str) return TRUE;
+    __TRY
+    {
+        volatile const WCHAR *p = str;
+        while (p != str + max) if (!*p++) break;
+    }
+    __EXCEPT_PAGE_FAULT
+    {
+        return TRUE;
+    }
+    __ENDTRY
+    return FALSE;
+}
+
+
 /***********************************************************************
  * Heap functions
  ***********************************************************************/
@@ -552,7 +688,20 @@ LPVOID WINAPI DECLSPEC_HOTPATCH LocalLock( HLOCAL hmem )
 {
     void *ret = NULL;
 
-    if (is_pointer( hmem )) return IsBadReadPtr( hmem, 1 ) ? NULL : hmem;
+    if (is_pointer( hmem ))
+    {
+        __TRY
+        {
+            volatile char *p = hmem;
+            *p |= 0;
+        }
+        __EXCEPT_PAGE_FAULT
+        {
+            return NULL;
+        }
+        __ENDTRY
+        return hmem;
+    }
 
     RtlLockHeap( GetProcessHeap() );
     __TRY
@@ -739,12 +888,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH LocalUnlock( HLOCAL hmem )
  */
 HANDLE WINAPI DECLSPEC_HOTPATCH CreateMemoryResourceNotification( MEMORY_RESOURCE_NOTIFICATION_TYPE type )
 {
-    static const WCHAR lowmemW[] =
-        {'\\','K','e','r','n','e','l','O','b','j','e','c','t','s',
-         '\\','L','o','w','M','e','m','o','r','y','C','o','n','d','i','t','i','o','n',0};
-    static const WCHAR highmemW[] =
-        {'\\','K','e','r','n','e','l','O','b','j','e','c','t','s',
-         '\\','H','i','g','h','M','e','m','o','r','y','C','o','n','d','i','t','i','o','n',0};
     HANDLE ret;
     UNICODE_STRING nameW;
     OBJECT_ATTRIBUTES attr;
@@ -752,10 +895,10 @@ HANDLE WINAPI DECLSPEC_HOTPATCH CreateMemoryResourceNotification( MEMORY_RESOURC
     switch (type)
     {
     case LowMemoryResourceNotification:
-        RtlInitUnicodeString( &nameW, lowmemW );
+        RtlInitUnicodeString( &nameW, L"\\KernelObjects\\LowMemoryCondition" );
         break;
     case HighMemoryResourceNotification:
-        RtlInitUnicodeString( &nameW, highmemW );
+        RtlInitUnicodeString( &nameW, L"\\KernelObjects\\HighMemoryCondition" );
         break;
     default:
         SetLastError( ERROR_INVALID_PARAMETER );
@@ -816,6 +959,82 @@ BOOL WINAPI DECLSPEC_HOTPATCH FreeUserPhysicalPages( HANDLE process, ULONG_PTR *
 }
 
 
+/***********************************************************************
+ *             GetPhysicallyInstalledSystemMemory   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH GetPhysicallyInstalledSystemMemory( ULONGLONG *memory )
+{
+    MEMORYSTATUSEX status;
+
+    if (!memory)
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return FALSE;
+    }
+    status.dwLength = sizeof(status);
+    GlobalMemoryStatusEx( &status );
+    *memory = status.ullTotalPhys / 1024;
+    return TRUE;
+}
+
+
+/***********************************************************************
+ *             GlobalMemoryStatusEx   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH GlobalMemoryStatusEx( MEMORYSTATUSEX *status )
+{
+    static MEMORYSTATUSEX cached_status;
+    static DWORD last_check;
+    SYSTEM_BASIC_INFORMATION basic_info;
+    SYSTEM_PERFORMANCE_INFORMATION perf_info;
+
+    if (status->dwLength != sizeof(*status))
+    {
+        SetLastError( ERROR_INVALID_PARAMETER );
+        return FALSE;
+    }
+    if ((NtGetTickCount() - last_check) < 1000)
+    {
+	*status = cached_status;
+	return TRUE;
+    }
+    last_check = NtGetTickCount();
+
+    if (!set_ntstatus( NtQuerySystemInformation( SystemBasicInformation,
+                                                 &basic_info, sizeof(basic_info), NULL )) ||
+        !set_ntstatus( NtQuerySystemInformation( SystemPerformanceInformation,
+                                                 &perf_info, sizeof(perf_info), NULL)))
+        return FALSE;
+
+    status->dwMemoryLoad     = 0;
+    status->ullTotalPhys     = perf_info.TotalCommitLimit;
+    status->ullAvailPhys     = perf_info.AvailablePages;
+    status->ullTotalPageFile = perf_info.TotalCommitLimit + 1; /* Titan Quest refuses to run if TotalPageFile <= TotalPhys */
+    status->ullAvailPageFile = status->ullTotalPageFile - perf_info.TotalCommittedPages;
+    status->ullTotalVirtual  = (ULONG_PTR)basic_info.HighestUserAddress - (ULONG_PTR)basic_info.LowestUserAddress;
+    status->ullAvailVirtual  = status->ullTotalVirtual - 64 * 1024;  /* FIXME */
+    status->ullAvailExtendedVirtual = 0;
+
+    status->ullTotalPhys     *= basic_info.PageSize;
+    status->ullAvailPhys     *= basic_info.PageSize;
+    status->ullTotalPageFile *= basic_info.PageSize;
+    status->ullAvailPageFile *= basic_info.PageSize;
+
+    if (status->ullTotalPhys)
+        status->dwMemoryLoad = (status->ullTotalPhys - status->ullAvailPhys) / (status->ullTotalPhys / 100);
+
+    TRACE_(virtual)( "MemoryLoad %d, TotalPhys %s, AvailPhys %s, TotalPageFile %s,"
+                     "AvailPageFile %s, TotalVirtual %s, AvailVirtual %s\n",
+                    status->dwMemoryLoad, wine_dbgstr_longlong(status->ullTotalPhys),
+                    wine_dbgstr_longlong(status->ullAvailPhys), wine_dbgstr_longlong(status->ullTotalPageFile),
+                    wine_dbgstr_longlong(status->ullAvailPageFile), wine_dbgstr_longlong(status->ullTotalVirtual),
+                    wine_dbgstr_longlong(status->ullAvailVirtual) );
+
+    cached_status = *status;
+    return TRUE;
+}
+
+
 /***********************************************************************
  *             MapUserPhysicalPages   (kernelbase.@)
  */
@@ -948,3 +1167,48 @@ LPVOID WINAPI DECLSPEC_HOTPATCH VirtualAllocExNuma( HANDLE process, void *addr,
     if (node) FIXME( "Ignoring preferred node %u\n", node );
     return VirtualAllocEx( process, addr, size, type, protect );
 }
+
+
+/***********************************************************************
+ * Firmware functions
+ ***********************************************************************/
+
+
+/***********************************************************************
+ *             EnumSystemFirmwareTable   (kernelbase.@)
+ */
+UINT WINAPI EnumSystemFirmwareTables( DWORD provider, void *buffer, DWORD size )
+{
+    FIXME( "(0x%08x, %p, %d)\n", provider, buffer, size );
+    return 0;
+}
+
+
+/***********************************************************************
+ *             GetSystemFirmwareTable   (kernelbase.@)
+ */
+UINT WINAPI GetSystemFirmwareTable( DWORD provider, DWORD id, void *buffer, DWORD size )
+{
+    SYSTEM_FIRMWARE_TABLE_INFORMATION *info;
+    ULONG buffer_size = offsetof( SYSTEM_FIRMWARE_TABLE_INFORMATION, TableBuffer ) + size;
+
+    TRACE( "(0x%08x, 0x%08x, %p, %d)\n", provider, id, buffer, size );
+
+    if (!(info = RtlAllocateHeap( GetProcessHeap(), 0, buffer_size )))
+    {
+        SetLastError( ERROR_OUTOFMEMORY );
+        return 0;
+    }
+
+    info->ProviderSignature = provider;
+    info->Action = SystemFirmwareTable_Get;
+    info->TableID = id;
+
+    set_ntstatus( NtQuerySystemInformation( SystemFirmwareTableInformation,
+                                            info, buffer_size, &buffer_size ));
+    buffer_size -= offsetof( SYSTEM_FIRMWARE_TABLE_INFORMATION, TableBuffer );
+    if (buffer_size <= size) memcpy( buffer, info->TableBuffer, buffer_size );
+
+    HeapFree( GetProcessHeap(), 0, info );
+    return buffer_size;
+}
