@@ -797,7 +797,7 @@ static void test_PathCchAddBackslashEx(void)
         }
         else
         {
-            ok(ptrW == NULL, "%u: unexpecred end pointer.\n", i);
+            ok(ptrW == NULL, "%u: unexpected end pointer.\n", i);
             ok(remaining == 0, "%u: unexpected remaining buffer length.\n", i);
         }
     }
@@ -1110,7 +1110,7 @@ static void test_PathCchCanonicalize(void)
     hr = pPathCchCanonicalize(path_outW, ARRAY_SIZE(path_outW), path_inW);
     ok(hr == HRESULT_FROM_WIN32(ERROR_FILENAME_EXCED_RANGE), "expect hr %#x, got %#x %s\n",
        HRESULT_FROM_WIN32(ERROR_FILENAME_EXCED_RANGE), hr, wine_dbgstr_w(path_outW));
-    ok(lstrlenW(path_outW) == 0, "got %d\n", lstrlenW(path_outW));
+    ok(!*path_outW, "got %d\n", lstrlenW(path_outW));
 
     path_inW[0] = 'C';
     path_inW[1] = ':';
@@ -2337,6 +2337,26 @@ static void test_PathIsUNCEx(void)
     }
 }
 
+static void test_actctx(void)
+{
+    ACTCTX_SECTION_KEYED_DATA data = { sizeof(data) };
+    WCHAR exe_path[MAX_PATH];
+    char buf[1024];
+    ACTIVATION_CONTEXT_ASSEMBLY_DETAILED_INFORMATION *info = (void *)buf;
+    SIZE_T size;
+    BOOL b;
+
+    b = FindActCtxSectionStringW(FIND_ACTCTX_SECTION_KEY_RETURN_HACTCTX, NULL, ACTIVATION_CONTEXT_SECTION_DLL_REDIRECTION, L"testdll.dll", &data);
+    ok(b, "FindActCtxSectionString failed: %u\n", GetLastError());
+
+    b = QueryActCtxW(0, data.hActCtx, &data.ulAssemblyRosterIndex, AssemblyDetailedInformationInActivationContext, buf, sizeof(buf), &size);
+    ok(b, "QueryActCtx failed: %u\n", GetLastError());
+
+    GetModuleFileNameW(NULL, exe_path, ARRAY_SIZE(exe_path));
+    ok(!lstrcmpW(info->lpAssemblyManifestPath, exe_path), "lpAssemblyManifestPath = %s expected %s\n", debugstr_w(info->lpAssemblyManifestPath), debugstr_w(exe_path));
+    ok(!info->lpAssemblyDirectoryName, "lpAssemblyDirectoryName = %s\n", wine_dbgstr_w(info->lpAssemblyDirectoryName));
+}
+
 START_TEST(path)
 {
     HMODULE hmod = LoadLibraryA("kernelbase.dll");
@@ -2386,4 +2406,5 @@ START_TEST(path)
     test_PathCchStripPrefix();
     test_PathCchStripToRoot();
     test_PathIsUNCEx();
+    test_actctx();
 }
