@@ -151,7 +151,7 @@ static HRESULT WINAPI topology_QueryInterface(IMFTopology *iface, REFIID riid, v
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s %p)\n", iface, debugstr_guid(riid), out);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(riid), out);
 
     if (IsEqualIID(riid, &IID_IMFTopology) ||
             IsEqualIID(riid, &IID_IMFAttributes) ||
@@ -175,7 +175,7 @@ static ULONG WINAPI topology_AddRef(IMFTopology *iface)
     struct topology *topology = impl_from_IMFTopology(iface);
     ULONG refcount = InterlockedIncrement(&topology->refcount);
 
-    TRACE("(%p) refcount=%u\n", iface, refcount);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
     return refcount;
 }
@@ -285,7 +285,7 @@ static HRESULT WINAPI topology_GetItem(IMFTopology *iface, REFGUID key, PROPVARI
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetItem(topology->attributes, key, value);
 }
@@ -294,7 +294,7 @@ static HRESULT WINAPI topology_GetItemType(IMFTopology *iface, REFGUID key, MF_A
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), type);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), type);
 
     return IMFAttributes_GetItemType(topology->attributes, key, type);
 }
@@ -303,7 +303,7 @@ static HRESULT WINAPI topology_CompareItem(IMFTopology *iface, REFGUID key, REFP
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p, %p)\n", iface, debugstr_guid(key), value, result);
+    TRACE("%p, %s, %p, %p.\n", iface, debugstr_guid(key), value, result);
 
     return IMFAttributes_CompareItem(topology->attributes, key, value, result);
 }
@@ -313,7 +313,7 @@ static HRESULT WINAPI topology_Compare(IMFTopology *iface, IMFAttributes *theirs
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%p, %d, %p)\n", iface, theirs, type, result);
+    TRACE("%p, %p, %d, %p.\n", iface, theirs, type, result);
 
     return IMFAttributes_Compare(topology->attributes, theirs, type, result);
 }
@@ -322,7 +322,7 @@ static HRESULT WINAPI topology_GetUINT32(IMFTopology *iface, REFGUID key, UINT32
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetUINT32(topology->attributes, key, value);
 }
@@ -331,7 +331,7 @@ static HRESULT WINAPI topology_GetUINT64(IMFTopology *iface, REFGUID key, UINT64
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetUINT64(topology->attributes, key, value);
 }
@@ -340,7 +340,7 @@ static HRESULT WINAPI topology_GetDouble(IMFTopology *iface, REFGUID key, double
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetDouble(topology->attributes, key, value);
 }
@@ -349,7 +349,7 @@ static HRESULT WINAPI topology_GetGUID(IMFTopology *iface, REFGUID key, GUID *va
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetGUID(topology->attributes, key, value);
 }
@@ -358,7 +358,7 @@ static HRESULT WINAPI topology_GetStringLength(IMFTopology *iface, REFGUID key,
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), length);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), length);
 
     return IMFAttributes_GetStringLength(topology->attributes, key, length);
 }
@@ -368,7 +368,7 @@ static HRESULT WINAPI topology_GetString(IMFTopology *iface, REFGUID key, WCHAR
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p, %d, %p)\n", iface, debugstr_guid(key), value, size, length);
+    TRACE("%p, %s, %p, %d, %p.\n", iface, debugstr_guid(key), value, size, length);
 
     return IMFAttributes_GetString(topology->attributes, key, value, size, length);
 }
@@ -378,7 +378,7 @@ static HRESULT WINAPI topology_GetAllocatedString(IMFTopology *iface, REFGUID ke
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p, %p)\n", iface, debugstr_guid(key), value, length);
+    TRACE("%p, %s, %p, %p.\n", iface, debugstr_guid(key), value, length);
 
     return IMFAttributes_GetAllocatedString(topology->attributes, key, value, length);
 }
@@ -387,7 +387,7 @@ static HRESULT WINAPI topology_GetBlobSize(IMFTopology *iface, REFGUID key, UINT
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), size);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), size);
 
     return IMFAttributes_GetBlobSize(topology->attributes, key, size);
 }
@@ -397,7 +397,7 @@ static HRESULT WINAPI topology_GetBlob(IMFTopology *iface, REFGUID key, UINT8 *b
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p, %d, %p)\n", iface, debugstr_guid(key), buf, bufsize, blobsize);
+    TRACE("%p, %s, %p, %d, %p.\n", iface, debugstr_guid(key), buf, bufsize, blobsize);
 
     return IMFAttributes_GetBlob(topology->attributes, key, buf, bufsize, blobsize);
 }
@@ -406,7 +406,7 @@ static HRESULT WINAPI topology_GetAllocatedBlob(IMFTopology *iface, REFGUID key,
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p, %p)\n", iface, debugstr_guid(key), buf, size);
+    TRACE("%p, %s, %p, %p.\n", iface, debugstr_guid(key), buf, size);
 
     return IMFAttributes_GetAllocatedBlob(topology->attributes, key, buf, size);
 }
@@ -415,7 +415,7 @@ static HRESULT WINAPI topology_GetUnknown(IMFTopology *iface, REFGUID key, REFII
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %s, %p)\n", iface, debugstr_guid(key), debugstr_guid(riid), ppv);
+    TRACE("%p, %s, %s, %p.\n", iface, debugstr_guid(key), debugstr_guid(riid), ppv);
 
     return IMFAttributes_GetUnknown(topology->attributes, key, riid, ppv);
 }
@@ -424,7 +424,7 @@ static HRESULT WINAPI topology_SetItem(IMFTopology *iface, REFGUID key, REFPROPV
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_SetItem(topology->attributes, key, value);
 }
@@ -433,7 +433,7 @@ static HRESULT WINAPI topology_DeleteItem(IMFTopology *iface, REFGUID key)
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s)\n", topology, debugstr_guid(key));
+    TRACE("%p, %s.\n", topology, debugstr_guid(key));
 
     return IMFAttributes_DeleteItem(topology->attributes, key);
 }
@@ -442,7 +442,7 @@ static HRESULT WINAPI topology_DeleteAllItems(IMFTopology *iface)
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)\n", iface);
+    TRACE("%p.\n", iface);
 
     return IMFAttributes_DeleteAllItems(topology->attributes);
 }
@@ -451,7 +451,7 @@ static HRESULT WINAPI topology_SetUINT32(IMFTopology *iface, REFGUID key, UINT32
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %d)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %d.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_SetUINT32(topology->attributes, key, value);
 }
@@ -460,7 +460,7 @@ static HRESULT WINAPI topology_SetUINT64(IMFTopology *iface, REFGUID key, UINT64
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %s)\n", iface, debugstr_guid(key), wine_dbgstr_longlong(value));
+    TRACE("%p, %s, %s.\n", iface, debugstr_guid(key), wine_dbgstr_longlong(value));
 
     return IMFAttributes_SetUINT64(topology->attributes, key, value);
 }
@@ -469,7 +469,7 @@ static HRESULT WINAPI topology_SetDouble(IMFTopology *iface, REFGUID key, double
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %f)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %f.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_SetDouble(topology->attributes, key, value);
 }
@@ -478,7 +478,7 @@ static HRESULT WINAPI topology_SetGUID(IMFTopology *iface, REFGUID key, REFGUID
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %s)\n", iface, debugstr_guid(key), debugstr_guid(value));
+    TRACE("%p, %s, %s.\n", iface, debugstr_guid(key), debugstr_guid(value));
 
     return IMFAttributes_SetGUID(topology->attributes, key, value);
 }
@@ -487,7 +487,7 @@ static HRESULT WINAPI topology_SetString(IMFTopology *iface, REFGUID key, const
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %s)\n", iface, debugstr_guid(key), debugstr_w(value));
+    TRACE("%p, %s, %s.\n", iface, debugstr_guid(key), debugstr_w(value));
 
     return IMFAttributes_SetString(topology->attributes, key, value);
 }
@@ -496,7 +496,7 @@ static HRESULT WINAPI topology_SetBlob(IMFTopology *iface, REFGUID key, const UI
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p, %d)\n", iface, debugstr_guid(key), buf, size);
+    TRACE("%p, %s, %p, %d.\n", iface, debugstr_guid(key), buf, size);
 
     return IMFAttributes_SetBlob(topology->attributes, key, buf, size);
 }
@@ -505,7 +505,7 @@ static HRESULT WINAPI topology_SetUnknown(IMFTopology *iface, REFGUID key, IUnkn
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), unknown);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), unknown);
 
     return IMFAttributes_SetUnknown(topology->attributes, key, unknown);
 }
@@ -514,7 +514,7 @@ static HRESULT WINAPI topology_LockStore(IMFTopology *iface)
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)\n", iface);
+    TRACE("%p.\n", iface);
 
     return IMFAttributes_LockStore(topology->attributes);
 }
@@ -523,7 +523,7 @@ static HRESULT WINAPI topology_UnlockStore(IMFTopology *iface)
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)\n", iface);
+    TRACE("%p.\n", iface);
 
     return IMFAttributes_UnlockStore(topology->attributes);
 }
@@ -532,7 +532,7 @@ static HRESULT WINAPI topology_GetCount(IMFTopology *iface, UINT32 *count)
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%p)\n", iface, count);
+    TRACE("%p, %p.\n", iface, count);
 
     return IMFAttributes_GetCount(topology->attributes, count);
 }
@@ -541,7 +541,7 @@ static HRESULT WINAPI topology_GetItemByIndex(IMFTopology *iface, UINT32 index,
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%u, %p, %p)\n", iface, index, key, value);
+    TRACE("%p, %u, %p, %p.\n", iface, index, key, value);
 
     return IMFAttributes_GetItemByIndex(topology->attributes, index, key, value);
 }
@@ -550,7 +550,7 @@ static HRESULT WINAPI topology_CopyAllItems(IMFTopology *iface, IMFAttributes *d
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%p)\n", iface, dest);
+    TRACE("%p, %p.\n", iface, dest);
 
     return IMFAttributes_CopyAllItems(topology->attributes, dest);
 }
@@ -559,7 +559,7 @@ static HRESULT WINAPI topology_GetTopologyID(IMFTopology *iface, TOPOID *id)
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%p)\n", iface, id);
+    TRACE("%p, %p.\n", iface, id);
 
     if (!id)
         return E_POINTER;
@@ -792,7 +792,7 @@ static HRESULT WINAPI topology_GetSourceNodeCollection(IMFTopology *iface, IMFCo
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%p)\n", iface, collection);
+    TRACE("%p, %p.\n", iface, collection);
 
     return topology_get_node_collection(topology, MF_TOPOLOGY_SOURCESTREAM_NODE, collection);
 }
@@ -801,7 +801,7 @@ static HRESULT WINAPI topology_GetOutputNodeCollection(IMFTopology *iface, IMFCo
 {
     struct topology *topology = impl_from_IMFTopology(iface);
 
-    TRACE("(%p)->(%p)\n", iface, collection);
+    TRACE("%p, %p.\n", iface, collection);
 
     return topology_get_node_collection(topology, MF_TOPOLOGY_OUTPUT_NODE, collection);
 }
@@ -881,7 +881,7 @@ HRESULT WINAPI MFCreateTopology(IMFTopology **topology)
     struct topology *object;
     HRESULT hr;
 
-    TRACE("(%p)\n", topology);
+    TRACE("%p.\n", topology);
 
     if (!topology)
         return E_POINTER;
@@ -911,7 +911,7 @@ static HRESULT WINAPI topology_node_QueryInterface(IMFTopologyNode *iface, REFII
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s %p)\n", iface, debugstr_guid(riid), out);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(riid), out);
 
     if (IsEqualIID(riid, &IID_IMFTopologyNode) ||
             IsEqualIID(riid, &IID_IMFAttributes) ||
@@ -922,7 +922,7 @@ static HRESULT WINAPI topology_node_QueryInterface(IMFTopologyNode *iface, REFII
         return S_OK;
     }
 
-    WARN("Unsupported %s.\n", debugstr_guid(riid));
+    WARN("Unsupported interface %s.\n", debugstr_guid(riid));
     *out = NULL;
 
     return E_NOINTERFACE;
@@ -933,7 +933,7 @@ static ULONG WINAPI topology_node_AddRef(IMFTopologyNode *iface)
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
     ULONG refcount = InterlockedIncrement(&node->refcount);
 
-    TRACE("(%p) refcount=%u\n", iface, refcount);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
     return refcount;
 }
@@ -944,7 +944,7 @@ static ULONG WINAPI topology_node_Release(IMFTopologyNode *iface)
     ULONG refcount = InterlockedDecrement(&node->refcount);
     unsigned int i;
 
-    TRACE("(%p) refcount=%u\n", iface, refcount);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -974,7 +974,7 @@ static HRESULT WINAPI topology_node_GetItem(IMFTopologyNode *iface, REFGUID key,
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetItem(node->attributes, key, value);
 }
@@ -983,7 +983,7 @@ static HRESULT WINAPI topology_node_GetItemType(IMFTopologyNode *iface, REFGUID
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), type);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), type);
 
     return IMFAttributes_GetItemType(node->attributes, key, type);
 }
@@ -992,7 +992,7 @@ static HRESULT WINAPI topology_node_CompareItem(IMFTopologyNode *iface, REFGUID
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p, %p)\n", iface, debugstr_guid(key), value, result);
+    TRACE("%p, %s, %p, %p.\n", iface, debugstr_guid(key), value, result);
 
     return IMFAttributes_CompareItem(node->attributes, key, value, result);
 }
@@ -1002,7 +1002,7 @@ static HRESULT WINAPI topology_node_Compare(IMFTopologyNode *iface, IMFAttribute
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%p, %d, %p)\n", iface, theirs, type, result);
+    TRACE("%p, %p, %d, %p.\n", iface, theirs, type, result);
 
     return IMFAttributes_Compare(node->attributes, theirs, type, result);
 }
@@ -1011,7 +1011,7 @@ static HRESULT WINAPI topology_node_GetUINT32(IMFTopologyNode *iface, REFGUID ke
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetUINT32(node->attributes, key, value);
 }
@@ -1020,7 +1020,7 @@ static HRESULT WINAPI topology_node_GetUINT64(IMFTopologyNode *iface, REFGUID ke
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetUINT64(node->attributes, key, value);
 }
@@ -1029,7 +1029,7 @@ static HRESULT WINAPI topology_node_GetDouble(IMFTopologyNode *iface, REFGUID ke
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetDouble(node->attributes, key, value);
 }
@@ -1038,7 +1038,7 @@ static HRESULT WINAPI topology_node_GetGUID(IMFTopologyNode *iface, REFGUID key,
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_GetGUID(node->attributes, key, value);
 }
@@ -1047,7 +1047,7 @@ static HRESULT WINAPI topology_node_GetStringLength(IMFTopologyNode *iface, REFG
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), length);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), length);
 
     return IMFAttributes_GetStringLength(node->attributes, key, length);
 }
@@ -1057,7 +1057,7 @@ static HRESULT WINAPI topology_node_GetString(IMFTopologyNode *iface, REFGUID ke
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p, %d, %p)\n", iface, debugstr_guid(key), value, size, length);
+    TRACE("%p, %s, %p, %d, %p.\n", iface, debugstr_guid(key), value, size, length);
 
     return IMFAttributes_GetString(node->attributes, key, value, size, length);
 }
@@ -1067,7 +1067,7 @@ static HRESULT WINAPI topology_node_GetAllocatedString(IMFTopologyNode *iface, R
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p, %p)\n", iface, debugstr_guid(key), value, length);
+    TRACE("%p, %s, %p, %p.\n", iface, debugstr_guid(key), value, length);
 
     return IMFAttributes_GetAllocatedString(node->attributes, key, value, length);
 }
@@ -1076,7 +1076,7 @@ static HRESULT WINAPI topology_node_GetBlobSize(IMFTopologyNode *iface, REFGUID
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), size);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), size);
 
     return IMFAttributes_GetBlobSize(node->attributes, key, size);
 }
@@ -1086,7 +1086,7 @@ static HRESULT WINAPI topology_node_GetBlob(IMFTopologyNode *iface, REFGUID key,
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p, %d, %p)\n", iface, debugstr_guid(key), buf, bufsize, blobsize);
+    TRACE("%p, %s, %p, %d, %p.\n", iface, debugstr_guid(key), buf, bufsize, blobsize);
 
     return IMFAttributes_GetBlob(node->attributes, key, buf, bufsize, blobsize);
 }
@@ -1095,7 +1095,7 @@ static HRESULT WINAPI topology_node_GetAllocatedBlob(IMFTopologyNode *iface, REF
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p, %p)\n", iface, debugstr_guid(key), buf, size);
+    TRACE("%p, %s, %p, %p.\n", iface, debugstr_guid(key), buf, size);
 
     return IMFAttributes_GetAllocatedBlob(node->attributes, key, buf, size);
 }
@@ -1104,7 +1104,7 @@ static HRESULT WINAPI topology_node_GetUnknown(IMFTopologyNode *iface, REFGUID k
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %s, %p)\n", iface, debugstr_guid(key), debugstr_guid(riid), ppv);
+    TRACE("%p, %s, %s, %p.\n", iface, debugstr_guid(key), debugstr_guid(riid), ppv);
 
     return IMFAttributes_GetUnknown(node->attributes, key, riid, ppv);
 }
@@ -1113,7 +1113,7 @@ static HRESULT WINAPI topology_node_SetItem(IMFTopologyNode *iface, REFGUID key,
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_SetItem(node->attributes, key, value);
 }
@@ -1122,7 +1122,7 @@ static HRESULT WINAPI topology_node_DeleteItem(IMFTopologyNode *iface, REFGUID k
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s)\n", iface, debugstr_guid(key));
+    TRACE("%p, %s.\n", iface, debugstr_guid(key));
 
     return IMFAttributes_DeleteItem(node->attributes, key);
 }
@@ -1131,7 +1131,7 @@ static HRESULT WINAPI topology_node_DeleteAllItems(IMFTopologyNode *iface)
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)\n", iface);
+    TRACE("%p.\n", iface);
 
     return IMFAttributes_DeleteAllItems(node->attributes);
 }
@@ -1140,7 +1140,7 @@ static HRESULT WINAPI topology_node_SetUINT32(IMFTopologyNode *iface, REFGUID ke
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %d)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %d.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_SetUINT32(node->attributes, key, value);
 }
@@ -1149,7 +1149,7 @@ static HRESULT WINAPI topology_node_SetUINT64(IMFTopologyNode *iface, REFGUID ke
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %s)\n", iface, debugstr_guid(key), wine_dbgstr_longlong(value));
+    TRACE("%p, %s, %s.\n", iface, debugstr_guid(key), wine_dbgstr_longlong(value));
 
     return IMFAttributes_SetUINT64(node->attributes, key, value);
 }
@@ -1158,7 +1158,7 @@ static HRESULT WINAPI topology_node_SetDouble(IMFTopologyNode *iface, REFGUID ke
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %f)\n", iface, debugstr_guid(key), value);
+    TRACE("%p, %s, %f.\n", iface, debugstr_guid(key), value);
 
     return IMFAttributes_SetDouble(node->attributes, key, value);
 }
@@ -1167,7 +1167,7 @@ static HRESULT WINAPI topology_node_SetGUID(IMFTopologyNode *iface, REFGUID key,
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %s)\n", iface, debugstr_guid(key), debugstr_guid(value));
+    TRACE("%p, %s, %s.\n", iface, debugstr_guid(key), debugstr_guid(value));
 
     return IMFAttributes_SetGUID(node->attributes, key, value);
 }
@@ -1176,7 +1176,7 @@ static HRESULT WINAPI topology_node_SetString(IMFTopologyNode *iface, REFGUID ke
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %s)\n", iface, debugstr_guid(key), debugstr_w(value));
+    TRACE("%p, %s, %s.\n", iface, debugstr_guid(key), debugstr_w(value));
 
     return IMFAttributes_SetString(node->attributes, key, value);
 }
@@ -1185,7 +1185,7 @@ static HRESULT WINAPI topology_node_SetBlob(IMFTopologyNode *iface, REFGUID key,
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p, %d)\n", iface, debugstr_guid(key), buf, size);
+    TRACE("%p, %s, %p, %d.\n", iface, debugstr_guid(key), buf, size);
 
     return IMFAttributes_SetBlob(node->attributes, key, buf, size);
 }
@@ -1194,7 +1194,7 @@ static HRESULT WINAPI topology_node_SetUnknown(IMFTopologyNode *iface, REFGUID k
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(key), unknown);
+    TRACE("%p, %s, %p.\n", iface, debugstr_guid(key), unknown);
 
     return IMFAttributes_SetUnknown(node->attributes, key, unknown);
 }
@@ -1203,7 +1203,7 @@ static HRESULT WINAPI topology_node_LockStore(IMFTopologyNode *iface)
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)\n", iface);
+    TRACE("%p.\n", iface);
 
     return IMFAttributes_LockStore(node->attributes);
 }
@@ -1212,7 +1212,7 @@ static HRESULT WINAPI topology_node_UnlockStore(IMFTopologyNode *iface)
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)\n", iface);
+    TRACE("%p.\n", iface);
 
     return IMFAttributes_UnlockStore(node->attributes);
 }
@@ -1221,7 +1221,7 @@ static HRESULT WINAPI topology_node_GetCount(IMFTopologyNode *iface, UINT32 *cou
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%p)\n", iface, count);
+    TRACE("%p, %p.\n", iface, count);
 
     return IMFAttributes_GetCount(node->attributes, count);
 }
@@ -1230,7 +1230,7 @@ static HRESULT WINAPI topology_node_GetItemByIndex(IMFTopologyNode *iface, UINT3
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%u, %p, %p)\n", iface, index, key, value);
+    TRACE("%p, %u, %p, %p.\n", iface, index, key, value);
 
     return IMFAttributes_GetItemByIndex(node->attributes, index, key, value);
 }
@@ -1239,7 +1239,7 @@ static HRESULT WINAPI topology_node_CopyAllItems(IMFTopologyNode *iface, IMFAttr
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%p)\n", iface, dest);
+    TRACE("%p, %p.\n", iface, dest);
 
     return IMFAttributes_CopyAllItems(node->attributes, dest);
 }
@@ -1326,7 +1326,7 @@ static HRESULT WINAPI topology_node_GetNodeType(IMFTopologyNode *iface, MF_TOPOL
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%p)\n", iface, node_type);
+    TRACE("%p, %p.\n", iface, node_type);
 
     *node_type = node->node_type;
 
@@ -1337,7 +1337,7 @@ static HRESULT WINAPI topology_node_GetTopoNodeID(IMFTopologyNode *iface, TOPOID
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%p)\n", iface, id);
+    TRACE("%p, %p.\n", iface, id);
 
     *id = node->id;
 
@@ -1348,7 +1348,7 @@ static HRESULT WINAPI topology_node_SetTopoNodeID(IMFTopologyNode *iface, TOPOID
 {
     struct topology_node *node = impl_from_IMFTopologyNode(iface);
 
-    TRACE("(%p)->(%s)\n", iface, wine_dbgstr_longlong(id));
+    TRACE("%p, %s.\n", iface, wine_dbgstr_longlong(id));
 
     node->id = id;
 
@@ -1893,7 +1893,7 @@ static ULONG WINAPI topology_loader_AddRef(IMFTopoLoader *iface)
     struct topology_loader *loader = impl_from_IMFTopoLoader(iface);
     ULONG refcount = InterlockedIncrement(&loader->refcount);
 
-    TRACE("(%p) refcount=%u\n", iface, refcount);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
     return refcount;
 }
@@ -1903,7 +1903,7 @@ static ULONG WINAPI topology_loader_Release(IMFTopoLoader *iface)
     struct topology_loader *loader = impl_from_IMFTopoLoader(iface);
     ULONG refcount = InterlockedDecrement(&loader->refcount);
 
-    TRACE("(%p) refcount=%u\n", iface, refcount);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -2022,7 +2022,7 @@ static ULONG WINAPI seq_source_AddRef(IMFSequencerSource *iface)
     struct seq_source *seq_source = impl_from_IMFSequencerSource(iface);
     ULONG refcount = InterlockedIncrement(&seq_source->refcount);
 
-    TRACE("(%p) refcount=%u\n", iface, refcount);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
     return refcount;
 }
@@ -2032,7 +2032,7 @@ static ULONG WINAPI seq_source_Release(IMFSequencerSource *iface)
     struct seq_source *seq_source = impl_from_IMFSequencerSource(iface);
     ULONG refcount = InterlockedDecrement(&seq_source->refcount);
 
-    TRACE("(%p) refcount=%u\n", iface, refcount);
+    TRACE("%p, refcount %u.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -2045,14 +2045,14 @@ static ULONG WINAPI seq_source_Release(IMFSequencerSource *iface)
 static HRESULT WINAPI seq_source_AppendTopology(IMFSequencerSource *iface, IMFTopology *topology,
         DWORD flags, MFSequencerElementId *id)
 {
-    FIXME("%p, %p, %x, %p\n", iface, topology, flags, id);
+    FIXME("%p, %p, %x, %p.\n", iface, topology, flags, id);
 
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI seq_source_DeleteTopology(IMFSequencerSource *iface, MFSequencerElementId id)
 {
-    FIXME("%p, %#x\n", iface, id);
+    FIXME("%p, %#x.\n", iface, id);
 
     return E_NOTIMPL;
 }
@@ -2060,7 +2060,7 @@ static HRESULT WINAPI seq_source_DeleteTopology(IMFSequencerSource *iface, MFSeq
 static HRESULT WINAPI seq_source_GetPresentationContext(IMFSequencerSource *iface,
         IMFPresentationDescriptor *descriptor, MFSequencerElementId *id, IMFTopology **topology)
 {
-    FIXME("%p, %p, %p, %p\n", iface, descriptor, id, topology);
+    FIXME("%p, %p, %p, %p.\n", iface, descriptor, id, topology);
 
     return E_NOTIMPL;
 }
@@ -2068,14 +2068,14 @@ static HRESULT WINAPI seq_source_GetPresentationContext(IMFSequencerSource *ifac
 static HRESULT WINAPI seq_source_UpdateTopology(IMFSequencerSource *iface, MFSequencerElementId id,
         IMFTopology *topology)
 {
-    FIXME("%p, %#x, %p\n", iface, id, topology);
+    FIXME("%p, %#x, %p.\n", iface, id, topology);
 
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI seq_source_UpdateTopologyFlags(IMFSequencerSource *iface, MFSequencerElementId id, DWORD flags)
 {
-    FIXME("%p, %#x, %#x\n", iface, id, flags);
+    FIXME("%p, %#x, %#x.\n", iface, id, flags);
 
     return E_NOTIMPL;
 }
@@ -2134,7 +2134,7 @@ HRESULT WINAPI MFCreateSequencerSource(IUnknown *reserved, IMFSequencerSource **
 {
     struct seq_source *object;
 
-    TRACE("(%p, %p)\n", reserved, seq_source);
+    TRACE("%p, %p.\n", reserved, seq_source);
 
     if (!seq_source)
         return E_POINTER;
