@@ -139,16 +139,6 @@ HRESULT WINAPI MFInvokeCallback(IMFAsyncResult *result)
     return RtwqInvokeCallback((IRtwqAsyncResult *)result);
 }
 
-/***********************************************************************
- *      MFCreateAsyncResult (mfplat.@)
- */
-HRESULT WINAPI MFCreateAsyncResult(IUnknown *object, IMFAsyncCallback *callback, IUnknown *state, IMFAsyncResult **out)
-{
-    TRACE("%p, %p, %p, %p.\n", object, callback, state, out);
-
-    return RtwqCreateAsyncResult(object, (IRtwqAsyncCallback *)callback, state, (IRtwqAsyncResult **)out);
-}
-
 /***********************************************************************
  *      MFGetTimerPeriodicity (mfplat.@)
  */
