@@ -25,7 +25,6 @@
 #include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
-#include <sys/time.h>
 #ifdef SONAME_LIBDBUS_1
 # include <dbus/dbus.h>
 #endif
@@ -36,6 +35,14 @@
 #include "mountmgr.h"
 #include "winnls.h"
 #include "excpt.h"
+#define USE_WS_PREFIX
+#include "winsock2.h"
+#include "ws2ipdef.h"
+#include "nldef.h"
+#include "netioapi.h"
+#include "inaddr.h"
+#include "ip2string.h"
+#include "dhcpcsdk.h"
 
 #include "wine/library.h"
 #include "wine/exception.h"
@@ -45,9 +52,10 @@ WINE_DEFAULT_DEBUG_CHANNEL(mountmgr);
 
 #ifdef SONAME_LIBDBUS_1
 
-#define DBUS_FUNCS \
+#define DBUS_FUNCS               \
     DO_FUNC(dbus_bus_add_match); \
     DO_FUNC(dbus_bus_get); \
+    DO_FUNC(dbus_bus_get_private); \
     DO_FUNC(dbus_bus_remove_match); \
     DO_FUNC(dbus_connection_add_filter); \
     DO_FUNC(dbus_connection_read_write_dispatch); \
@@ -93,6 +101,7 @@ static DBusConnection *connection;
     DO_FUNC(libhal_ctx_set_device_removed); \
     DO_FUNC(libhal_ctx_shutdown); \
     DO_FUNC(libhal_device_get_property_bool); \
+    DO_FUNC(libhal_device_get_property_int); \
     DO_FUNC(libhal_device_get_property_string); \
     DO_FUNC(libhal_device_add_property_watch); \
     DO_FUNC(libhal_device_remove_property_watch); \
@@ -310,7 +319,7 @@ static void udisks_new_device( const char *udi )
 
     if (device)
     {
-        if (removable) add_dos_device( -1, udi, device, mount_point, drive_type, guid_ptr );
+        if (removable) add_dos_device( -1, udi, device, mount_point, drive_type, guid_ptr, NULL );
         else if (guid_ptr) add_volume( udi, device, mount_point, DEVICE_HARDDISK_VOL, guid_ptr );
     }
 
@@ -476,7 +485,7 @@ static void udisks2_add_device( const char *udi, DBusMessageIter *dict, DBusMess
     }
     if (device)
     {
-        if (removable) add_dos_device( -1, udi, device, mount_point, drive_type, guid_ptr );
+        if (removable) add_dos_device( -1, udi, device, mount_point, drive_type, guid_ptr, NULL );
         else if (guid_ptr) add_volume( udi, device, mount_point, DEVICE_HARDDISK_VOL, guid_ptr );
     }
 }
@@ -574,6 +583,178 @@ static DBusHandlerResult udisks_filter( DBusConnection *ctx, DBusMessage *msg, v
 
 #ifdef SONAME_LIBHAL
 
+static BOOL hal_get_ide_parameters( LibHalContext *ctx, const char *udi, SCSI_ADDRESS *scsi_addr, UCHAR *devtype, char *ident, size_t ident_size )
+{
+    DBusError error;
+    char *parent = NULL;
+    char *type = NULL;
+    char *model = NULL;
+    int host, chan;
+    BOOL ret = FALSE;
+
+    p_dbus_error_init( &error );
+
+    if (!(parent = p_libhal_device_get_property_string( ctx, udi, "info.parent", &error )))
+        goto done;
+    if ((host = p_libhal_device_get_property_int( ctx, parent, "ide.host", &error )) < 0)
+        goto done;
+    if ((chan = p_libhal_device_get_property_int( ctx, parent, "ide.channel", &error )) < 0)
+        goto done;
+
+    ret = TRUE;
+
+    if (devtype)
+    {
+        if (!(type = p_libhal_device_get_property_string( ctx, udi, "storage.drive_type", &error )))
+            *devtype = SCSI_UNKNOWN_PERIPHERAL;
+        else if (!strcmp( type, "disk" ) || !strcmp( type, "floppy" ))
+            *devtype = SCSI_DISK_PERIPHERAL;
+        else if (!strcmp( type, "tape" ))
+            *devtype = SCSI_TAPE_PERIPHERAL;
+        else if (!strcmp( type, "cdrom" ))
+            *devtype = SCSI_CDROM_PERIPHERAL;
+        else if (!strcmp( type, "raid" ))
+            *devtype = SCSI_ARRAY_PERIPHERAL;
+        else
+            *devtype = SCSI_UNKNOWN_PERIPHERAL;
+    }
+
+    if (ident)
+    {
+        if (!(model = p_libhal_device_get_property_string( ctx, udi, "storage.model", &error )))
+            p_dbus_error_free( &error );  /* ignore error */
+        else
+            lstrcpynA( ident, model, ident_size );
+    }
+
+    scsi_addr->PortNumber = host;
+    scsi_addr->PathId = 0;
+    scsi_addr->TargetId = chan;
+    scsi_addr->Lun = 0;
+
+done:
+    if (model) p_libhal_free_string( model );
+    if (type) p_libhal_free_string( type );
+    if (parent) p_libhal_free_string( parent );
+    p_dbus_error_free( &error );
+    return ret;
+}
+
+static BOOL hal_get_scsi_parameters( LibHalContext *ctx, const char *udi, SCSI_ADDRESS *scsi_addr, UCHAR *devtype, char *ident, size_t ident_size )
+{
+    DBusError error;
+    char *type = NULL;
+    char *vendor = NULL;
+    char *model = NULL;
+    int host, bus, target, lun;
+    BOOL ret = FALSE;
+
+    p_dbus_error_init( &error );
+
+    if ((host = p_libhal_device_get_property_int( ctx, udi, "scsi.host", &error )) < 0)
+        goto done;
+    if ((bus = p_libhal_device_get_property_int( ctx, udi, "scsi.bus", &error )) < 0)
+        goto done;
+    if ((target = p_libhal_device_get_property_int( ctx, udi, "scsi.target", &error )) < 0)
+        goto done;
+    if ((lun = p_libhal_device_get_property_int( ctx, udi, "scsi.lun", &error )) < 0)
+        goto done;
+
+    ret = TRUE;
+    scsi_addr->PortNumber = host;
+    scsi_addr->PathId = bus;
+    scsi_addr->TargetId = target;
+    scsi_addr->Lun = lun;
+
+    if (ident)
+    {
+        if (!(vendor = p_libhal_device_get_property_string( ctx, udi, "scsi.vendor", &error )))
+            p_dbus_error_free( &error );  /* ignore error */
+        if (!(model = p_libhal_device_get_property_string( ctx, udi, "scsi.model", &error )))
+            p_dbus_error_free( &error );  /* ignore error */
+        snprintf( ident, ident_size, "%-8s%-16s", vendor ? vendor : "WINE", model ? model : "SCSI" );
+    }
+
+    if (devtype)
+    {
+        if (!(type = p_libhal_device_get_property_string( ctx, udi, "scsi.type", &error )))
+        {
+            *devtype = SCSI_UNKNOWN_PERIPHERAL;
+            goto done;
+        }
+        if (!strcmp( type, "disk" ))
+            *devtype = SCSI_DISK_PERIPHERAL;
+        else if (!strcmp( type, "tape" ))
+            *devtype = SCSI_TAPE_PERIPHERAL;
+        else if (!strcmp( type, "printer" ))
+            *devtype = SCSI_PRINTER_PERIPHERAL;
+        else if (!strcmp( type, "processor" ))
+            *devtype = SCSI_PROCESSOR_PERIPHERAL;
+        else if (!strcmp( type, "cdrom" ))
+            *devtype = SCSI_CDROM_PERIPHERAL;
+        else if (!strcmp( type, "scanner" ))
+            *devtype = SCSI_SCANNER_PERIPHERAL;
+        else if (!strcmp( type, "medium_changer" ))
+            *devtype = SCSI_MEDIUM_CHANGER_PERIPHERAL;
+        else if (!strcmp( type, "comm" ))
+            *devtype = SCSI_COMMS_PERIPHERAL;
+        else if (!strcmp( type, "raid" ))
+            *devtype = SCSI_ARRAY_PERIPHERAL;
+        else
+            *devtype = SCSI_UNKNOWN_PERIPHERAL;
+    }
+
+done:
+    if (type) p_libhal_free_string( type );
+    if (vendor) p_libhal_free_string( vendor );
+    if (model) p_libhal_free_string( model );
+    p_dbus_error_free( &error );
+    return ret;
+}
+
+static void hal_new_ide_device( LibHalContext *ctx, const char *udi )
+{
+    SCSI_ADDRESS scsi_addr;
+    UCHAR devtype;
+    char ident[40];
+
+    if (!hal_get_ide_parameters( ctx, udi, &scsi_addr, &devtype, ident, sizeof(ident) )) return;
+    create_scsi_entry( &scsi_addr, 255, devtype == SCSI_CDROM_PERIPHERAL ? "atapi" : "WINE SCSI", devtype, ident, NULL );
+}
+
+static void hal_set_device_name( LibHalContext *ctx, const char *udi, const UNICODE_STRING *devname )
+{
+    DBusError error;
+    SCSI_ADDRESS scsi_addr;
+    char *parent = NULL;
+
+    p_dbus_error_init( &error );
+
+    if (!hal_get_ide_parameters( ctx, udi, &scsi_addr, NULL, NULL, 0 ))
+    {
+        if (!(parent = p_libhal_device_get_property_string( ctx, udi, "info.parent", &error )))
+            goto done;
+        if (!hal_get_scsi_parameters( ctx, parent, &scsi_addr, NULL, NULL, 0 ))
+            goto done;
+    }
+    set_scsi_device_name( &scsi_addr, devname );
+
+done:
+    if (parent) p_libhal_free_string( parent );
+    p_dbus_error_free( &error );
+}
+
+static void hal_new_scsi_device( LibHalContext *ctx, const char *udi )
+{
+    SCSI_ADDRESS scsi_addr;
+    UCHAR devtype;
+    char ident[40];
+
+    if (!hal_get_scsi_parameters( ctx, udi, &scsi_addr, &devtype, ident, sizeof(ident) )) return;
+    /* FIXME: get real controller Id for SCSI */
+    create_scsi_entry( &scsi_addr, 255, "WINE SCSI", devtype, ident, NULL );
+}
+
 /* HAL callback for new device */
 static void hal_new_device( LibHalContext *ctx, const char *udi )
 {
@@ -588,6 +769,9 @@ static void hal_new_device( LibHalContext *ctx, const char *udi )
 
     p_dbus_error_init( &error );
 
+    hal_new_scsi_device( ctx, udi );
+    hal_new_ide_device( ctx, udi );
+
     if (!(device = p_libhal_device_get_property_string( ctx, udi, "block.device", &error )))
         goto done;
 
@@ -611,7 +795,10 @@ static void hal_new_device( LibHalContext *ctx, const char *udi )
 
     if (p_libhal_device_get_property_bool( ctx, parent, "storage.removable", &error ))
     {
-        add_dos_device( -1, udi, device, mount_point, drive_type, guid_ptr );
+        UNICODE_STRING devname;
+
+        add_dos_device( -1, udi, device, mount_point, drive_type, guid_ptr, &devname );
+        hal_set_device_name( ctx, parent, &devname );
         /* add property watch for mount point */
         p_libhal_device_add_property_watch( ctx, udi, &error );
     }
@@ -762,6 +949,272 @@ void initialize_dbus(void)
     CloseHandle( handle );
 }
 
+#if !defined(HAVE_SYSTEMCONFIGURATION_SCDYNAMICSTORECOPYDHCPINFO_H) || !defined(HAVE_SYSTEMCONFIGURATION_SCNETWORKCONFIGURATION_H)
+
+/* The udisks dispatch loop will block all threads using the same connection, so we'll
+   use a private connection. Multiple threads can make methods calls at the same time
+   on the same connection, according to the documentation.
+ */
+static DBusConnection *dhcp_connection;
+static DBusConnection *get_dhcp_connection(void)
+{
+    if (!dhcp_connection)
+    {
+        DBusError error;
+        p_dbus_error_init( &error );
+        if (!(dhcp_connection = p_dbus_bus_get_private( DBUS_BUS_SYSTEM, &error )))
+        {
+            WARN( "failed to get system dbus connection: %s\n", error.message );
+            p_dbus_error_free( &error );
+        }
+    }
+    return dhcp_connection;
+}
+
+static DBusMessage *device_by_iface_request( const char *iface )
+{
+    DBusMessage *request, *reply;
+    DBusMessageIter iter;
+    DBusError error;
+
+    request = p_dbus_message_new_method_call( "org.freedesktop.NetworkManager", "/org/freedesktop/NetworkManager",
+                                              "org.freedesktop.NetworkManager", "GetDeviceByIpIface" );
+    if (!request) return NULL;
+
+    p_dbus_message_iter_init_append( request, &iter );
+    p_dbus_message_iter_append_basic( &iter, DBUS_TYPE_STRING, &iface );
+
+    p_dbus_error_init( &error );
+    reply = p_dbus_connection_send_with_reply_and_block( get_dhcp_connection(), request, -1, &error );
+    p_dbus_message_unref( request );
+    if (!reply)
+    {
+        WARN( "failed: %s\n", error.message );
+        p_dbus_error_free( &error );
+        return NULL;
+    }
+
+    p_dbus_error_free( &error );
+    return reply;
+}
+
+#define IF_NAMESIZE 16
+static BOOL map_adapter_name( const WCHAR *name, char *unix_name, DWORD len )
+{
+    WCHAR unix_nameW[IF_NAMESIZE];
+    UNICODE_STRING str;
+    GUID guid;
+
+    RtlInitUnicodeString( &str, name );
+    if (!RtlGUIDFromString( &str, &guid ))
+    {
+        NET_LUID luid;
+        if (ConvertInterfaceGuidToLuid( &guid, &luid ) ||
+            ConvertInterfaceLuidToNameW( &luid, unix_nameW, ARRAY_SIZE(unix_nameW) )) return FALSE;
+
+        name = unix_nameW;
+    }
+    return WideCharToMultiByte( CP_UNIXCP, 0, name, -1, unix_name, len, NULL, NULL ) != 0;
+}
+
+static DBusMessage *dhcp4_config_request( const WCHAR *adapter )
+{
+    static const char *device = "org.freedesktop.NetworkManager.Device";
+    static const char *dhcp4_config = "Dhcp4Config";
+    char iface[IF_NAMESIZE];
+    DBusMessage *request, *reply;
+    DBusMessageIter iter;
+    DBusError error;
+    const char *path = NULL;
+
+    if (!map_adapter_name( adapter, iface, sizeof(iface) )) return NULL;
+    if (!(reply = device_by_iface_request( iface ))) return NULL;
+
+    p_dbus_message_iter_init( reply, &iter );
+    if (p_dbus_message_iter_get_arg_type( &iter ) == DBUS_TYPE_OBJECT_PATH) p_dbus_message_iter_get_basic( &iter, &path );
+    p_dbus_message_unref( reply );
+    if (!path) return NULL;
+
+    request = p_dbus_message_new_method_call( "org.freedesktop.NetworkManager", path,
+                                              "org.freedesktop.DBus.Properties", "Get" );
+    if (!request) return NULL;
+
+    p_dbus_message_iter_init_append( request, &iter );
+    p_dbus_message_iter_append_basic( &iter, DBUS_TYPE_STRING, &device );
+    p_dbus_message_iter_append_basic( &iter, DBUS_TYPE_STRING, &dhcp4_config );
+
+    p_dbus_error_init( &error );
+    reply = p_dbus_connection_send_with_reply_and_block( get_dhcp_connection(), request, -1, &error );
+    p_dbus_message_unref( request );
+    if (!reply)
+    {
+        WARN( "failed: %s\n", error.message );
+        p_dbus_error_free( &error );
+        return NULL;
+    }
+
+    p_dbus_error_free( &error );
+    return reply;
+}
+
+static DBusMessage *dhcp4_config_options_request( const WCHAR *adapter )
+{
+    static const char *dhcp4_config = "org.freedesktop.NetworkManager.DHCP4Config";
+    static const char *options = "Options";
+    DBusMessage *request, *reply;
+    DBusMessageIter iter, sub;
+    DBusError error;
+    const char *path = NULL;
+
+    if (!(reply = dhcp4_config_request( adapter ))) return NULL;
+
+    p_dbus_message_iter_init( reply, &iter );
+    if (p_dbus_message_iter_get_arg_type( &iter ) == DBUS_TYPE_VARIANT)
+    {
+        p_dbus_message_iter_recurse( &iter, &sub );
+        p_dbus_message_iter_get_basic( &sub, &path );
+    }
+    if (!path)
+    {
+        p_dbus_message_unref( reply );
+        return NULL;
+    }
+
+    request = p_dbus_message_new_method_call( "org.freedesktop.NetworkManager", path,
+                                              "org.freedesktop.DBus.Properties", "Get" );
+    p_dbus_message_unref( reply );
+    if (!request) return NULL;
+
+    p_dbus_message_iter_init_append( request, &iter );
+    p_dbus_message_iter_append_basic( &iter, DBUS_TYPE_STRING, &dhcp4_config );
+    p_dbus_message_iter_append_basic( &iter, DBUS_TYPE_STRING, &options );
+
+    p_dbus_error_init( &error );
+    reply = p_dbus_connection_send_with_reply_and_block( get_dhcp_connection(), request, -1, &error );
+    p_dbus_message_unref( request );
+    if (!reply)
+    {
+        p_dbus_error_free( &error );
+        return NULL;
+    }
+
+    p_dbus_error_free( &error );
+    return reply;
+}
+
+static const char *dhcp4_config_option_next_dict_entry( DBusMessageIter *iter, DBusMessageIter *variant )
+{
+    DBusMessageIter sub;
+    const char *name;
+
+    if (p_dbus_message_iter_get_arg_type( iter ) != DBUS_TYPE_DICT_ENTRY) return NULL;
+    p_dbus_message_iter_recurse( iter, &sub );
+    p_dbus_message_iter_next( iter );
+    p_dbus_message_iter_get_basic( &sub, &name );
+    p_dbus_message_iter_next( &sub );
+    p_dbus_message_iter_recurse( &sub, variant );
+    return name;
+}
+
+static DBusMessage *dhcp4_config_option_request( const WCHAR *adapter, const char *option, const char **value )
+{
+    DBusMessage *reply;
+    DBusMessageIter iter, variant;
+    const char *name;
+
+    if (!(reply = dhcp4_config_options_request( adapter ))) return NULL;
+
+    *value = NULL;
+    p_dbus_message_iter_init( reply, &iter );
+    if (p_dbus_message_iter_get_arg_type( &iter ) == DBUS_TYPE_VARIANT)
+    {
+        p_dbus_message_iter_recurse( &iter, &iter );
+        if (p_dbus_message_iter_get_arg_type( &iter ) == DBUS_TYPE_ARRAY)
+        {
+            p_dbus_message_iter_recurse( &iter, &iter );
+            while ((name = dhcp4_config_option_next_dict_entry( &iter, &variant )))
+            {
+                if (!strcmp( name, option ))
+                {
+                    p_dbus_message_iter_get_basic( &variant, value );
+                    break;
+                }
+            }
+        }
+    }
+
+    return reply;
+}
+
+static const char *map_option( ULONG option )
+{
+    switch (option)
+    {
+    case OPTION_SUBNET_MASK:         return "subnet_mask";
+    case OPTION_ROUTER_ADDRESS:      return "next_server";
+    case OPTION_HOST_NAME:           return "host_name";
+    case OPTION_DOMAIN_NAME:         return "domain_name";
+    case OPTION_BROADCAST_ADDRESS:   return "broadcast_address";
+    case OPTION_MSFT_IE_PROXY:       return "wpad";
+    default:
+        FIXME( "unhandled option %u\n", option );
+        return "";
+    }
+}
+
+ULONG get_dhcp_request_param( const WCHAR *adapter, struct mountmgr_dhcp_request_param *param, char *buf, ULONG offset,
+                              ULONG size )
+{
+    DBusMessage *reply;
+    const char *value;
+    ULONG ret = 0;
+
+    param->offset = param->size = 0;
+
+    if (!(reply = dhcp4_config_option_request( adapter, map_option(param->id), &value ))) return 0;
+
+    switch (param->id)
+    {
+    case OPTION_SUBNET_MASK:
+    case OPTION_ROUTER_ADDRESS:
+    case OPTION_BROADCAST_ADDRESS:
+    {
+        IN_ADDR *ptr = (IN_ADDR *)(buf + offset);
+        if (value && size >= sizeof(IN_ADDR) && !RtlIpv4StringToAddressA( value, TRUE, NULL, ptr ))
+        {
+            param->offset = offset;
+            param->size   = sizeof(*ptr);
+            TRACE( "returning %08x\n", *(DWORD *)ptr );
+        }
+        ret = sizeof(*ptr);
+        break;
+    }
+    case OPTION_HOST_NAME:
+    case OPTION_DOMAIN_NAME:
+    case OPTION_MSFT_IE_PROXY:
+    {
+        char *ptr = buf + offset;
+        int len = value ? strlen( value ) : 0;
+        if (len && size >= len)
+        {
+            memcpy( ptr, value, len );
+            param->offset = offset;
+            param->size   = len;
+            TRACE( "returning %s\n", debugstr_an(ptr, len) );
+        }
+        ret = len;
+        break;
+    }
+    default:
+        FIXME( "option %u not supported\n", param->id );
+        break;
+    }
+
+    p_dbus_message_unref( reply );
+    return ret;
+}
+#endif
+
 #else  /* SONAME_LIBDBUS_1 */
 
 void initialize_dbus(void)
