@@ -37,47 +37,56 @@
 DEFINE_GUID(WMMEDIATYPE_Audio, 0x73647561,0x0000,0x0010,0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71);
 DEFINE_GUID(WMMEDIASUBTYPE_MP3,0x00000055,0x0000,0x0010,0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71);
 DEFINE_GUID(WMMEDIASUBTYPE_PCM,0x00000001,0x0000,0x0010,0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71);
+DEFINE_GUID(WMFORMAT_WaveFormatEx,  0x05589f81,0xc356,0x11ce,0xbf,0x01,0x00,0xaa,0x00,0x55,0x59,0x5a);
 
 WINE_DEFAULT_DEBUG_CHANNEL(mp3dmod);
 
 static HINSTANCE mp3dmod_instance;
 
-struct mp3_decoder {
+struct mp3_decoder
+{
+    IUnknown IUnknown_inner;
     IMediaObject IMediaObject_iface;
+    IUnknown *outer;
     LONG ref;
     mpg123_handle *mh;
-    DMO_MEDIA_TYPE outtype;
+
+    DMO_MEDIA_TYPE intype, outtype;
+    BOOL intype_set, outtype_set;
+
     IMediaBuffer *buffer;
     REFERENCE_TIME timestamp;
 };
 
-static inline struct mp3_decoder *impl_from_IMediaObject(IMediaObject *iface)
+static inline struct mp3_decoder *impl_from_IUnknown(IUnknown *iface)
 {
-    return CONTAINING_RECORD(iface, struct mp3_decoder, IMediaObject_iface);
+    return CONTAINING_RECORD(iface, struct mp3_decoder, IUnknown_inner);
 }
 
-static HRESULT WINAPI MediaObject_QueryInterface(IMediaObject *iface, REFIID iid, void **ppv)
+static HRESULT WINAPI Unknown_QueryInterface(IUnknown *iface, REFIID iid, void **obj)
 {
-    struct mp3_decoder *This = impl_from_IMediaObject(iface);
+    struct mp3_decoder *This = impl_from_IUnknown(iface);
 
-    TRACE("(%p)->(%s, %p)\n", This, debugstr_guid(iid), ppv);
+    TRACE("(%p)->(%s, %p)\n", This, debugstr_guid(iid), obj);
 
-    if (IsEqualIID(iid, &IID_IUnknown) || IsEqualIID(iid, &IID_IMediaObject))
-        *ppv = &This->IMediaObject_iface;
+    if (IsEqualGUID(iid, &IID_IUnknown))
+        *obj = &This->IUnknown_inner;
+    else if (IsEqualGUID(iid, &IID_IMediaObject))
+        *obj = &This->IMediaObject_iface;
     else
     {
         FIXME("no interface for %s\n", debugstr_guid(iid));
-        *ppv = NULL;
+        *obj = NULL;
         return E_NOINTERFACE;
     }
 
-    IMediaObject_AddRef(iface);
+    IUnknown_AddRef((IUnknown *)*obj);
     return S_OK;
 }
 
-static ULONG WINAPI MediaObject_AddRef(IMediaObject *iface)
+static ULONG WINAPI Unknown_AddRef(IUnknown *iface)
 {
-    struct mp3_decoder *This = impl_from_IMediaObject(iface);
+    struct mp3_decoder *This = impl_from_IUnknown(iface);
     ULONG refcount = InterlockedIncrement(&This->ref);
 
     TRACE("(%p) AddRef from %d\n", This, refcount - 1);
@@ -85,59 +94,160 @@ static ULONG WINAPI MediaObject_AddRef(IMediaObject *iface)
     return refcount;
 }
 
-static ULONG WINAPI MediaObject_Release(IMediaObject *iface)
+static ULONG WINAPI Unknown_Release(IUnknown *iface)
 {
-    struct mp3_decoder *This = impl_from_IMediaObject(iface);
+    struct mp3_decoder *This = impl_from_IUnknown(iface);
     ULONG refcount = InterlockedDecrement(&This->ref);
 
     TRACE("(%p) Release from %d\n", This, refcount + 1);
 
     if (!refcount)
     {
+        if (This->buffer)
+            IMediaBuffer_Release(This->buffer);
+        if (This->intype_set)
+            MoFreeMediaType(&This->intype);
+        MoFreeMediaType(&This->outtype);
         mpg123_delete(This->mh);
         heap_free(This);
     }
     return refcount;
 }
 
+static const IUnknownVtbl Unknown_vtbl = {
+    Unknown_QueryInterface,
+    Unknown_AddRef,
+    Unknown_Release,
+};
+
+static inline struct mp3_decoder *impl_from_IMediaObject(IMediaObject *iface)
+{
+    return CONTAINING_RECORD(iface, struct mp3_decoder, IMediaObject_iface);
+}
+
+static HRESULT WINAPI MediaObject_QueryInterface(IMediaObject *iface, REFIID iid, void **obj)
+{
+    struct mp3_decoder *This = impl_from_IMediaObject(iface);
+    return IUnknown_QueryInterface(This->outer, iid, obj);
+}
+
+static ULONG WINAPI MediaObject_AddRef(IMediaObject *iface)
+{
+    struct mp3_decoder *This = impl_from_IMediaObject(iface);
+    return IUnknown_AddRef(This->outer);
+}
+
+static ULONG WINAPI MediaObject_Release(IMediaObject *iface)
+{
+    struct mp3_decoder *This = impl_from_IMediaObject(iface);
+    return IUnknown_Release(This->outer);
+}
+
 static HRESULT WINAPI MediaObject_GetStreamCount(IMediaObject *iface, DWORD *input, DWORD *output)
 {
-    FIXME("(%p)->(%p, %p) stub!\n", iface, input, output);
+    TRACE("iface %p, input %p, output %p.\n", iface, input, output);
 
-    return E_NOTIMPL;
+    *input = *output = 1;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_GetInputStreamInfo(IMediaObject *iface, DWORD index, DWORD *flags)
 {
-    FIXME("(%p)->(%d, %p) stub!\n", iface, index, flags);
+    TRACE("iface %p, index %u, flags %p.\n", iface, index, flags);
 
-    return E_NOTIMPL;
+    *flags = 0;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_GetOutputStreamInfo(IMediaObject *iface, DWORD index, DWORD *flags)
 {
-    FIXME("(%p)->(%d, %p) stub!\n", iface, index, flags);
+    TRACE("iface %p, index %u, flags %p.\n", iface, index, flags);
 
-    return E_NOTIMPL;
+    *flags = 0;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_GetInputType(IMediaObject *iface, DWORD index, DWORD type_index, DMO_MEDIA_TYPE *type)
 {
-    FIXME("(%p)->(%d, %d, %p) stub!\n", iface, index, type_index, type);
+    TRACE("iface %p, index %u, type_index %u, type %p.\n", iface, index, type_index, type);
 
-    return E_NOTIMPL;
+    if (type_index)
+        return DMO_E_NO_MORE_ITEMS;
+
+    type->majortype = WMMEDIATYPE_Audio;
+    type->subtype = WMMEDIASUBTYPE_MP3;
+    type->formattype = GUID_NULL;
+    type->pUnk = NULL;
+    type->cbFormat = 0;
+    type->pbFormat = NULL;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_GetOutputType(IMediaObject *iface, DWORD index, DWORD type_index, DMO_MEDIA_TYPE *type)
 {
-    FIXME("(%p)->(%d, %d, %p) stub!\n", iface, index, type_index, type);
+    struct mp3_decoder *dmo = impl_from_IMediaObject(iface);
+    const WAVEFORMATEX *input_format;
+    WAVEFORMATEX *format;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, index %u, type_index %u, type %p.\n", iface, index, type_index, type);
+
+    if (!dmo->intype_set)
+        return DMO_E_TYPE_NOT_SET;
+
+    input_format = (WAVEFORMATEX *)dmo->intype.pbFormat;
+
+    if (type_index >= (2 * input_format->nChannels))
+        return DMO_E_NO_MORE_ITEMS;
+
+    type->majortype = WMMEDIATYPE_Audio;
+    type->subtype = WMMEDIASUBTYPE_PCM;
+    type->formattype = WMFORMAT_WaveFormatEx;
+    type->pUnk = NULL;
+    type->cbFormat = sizeof(WAVEFORMATEX);
+    if (!(type->pbFormat = CoTaskMemAlloc(sizeof(WAVEFORMATEX))))
+        return E_OUTOFMEMORY;
+    format = (WAVEFORMATEX *)type->pbFormat;
+    format->wFormatTag = WAVE_FORMAT_PCM;
+    format->nSamplesPerSec = input_format->nSamplesPerSec;
+    format->nChannels = (type_index / 2) ? 1 : input_format->nChannels;
+    format->wBitsPerSample = (type_index % 2) ? 8 : 16;
+    format->nBlockAlign = format->nChannels * format->wBitsPerSample / 8;
+    format->nAvgBytesPerSec = format->nSamplesPerSec * format->nBlockAlign;
+    format->cbSize = 0;
+
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_SetInputType(IMediaObject *iface, DWORD index, const DMO_MEDIA_TYPE *type, DWORD flags)
 {
-    FIXME("(%p)->(%d, %p, %#x) stub!\n", iface, index, type, flags);
+    struct mp3_decoder *dmo = impl_from_IMediaObject(iface);
+
+    TRACE("iface %p, index %u, type %p, flags %#x.\n", iface, index, type, flags);
+
+    if (flags & DMO_SET_TYPEF_CLEAR)
+    {
+        if (dmo->intype_set)
+            MoFreeMediaType(&dmo->intype);
+        dmo->intype_set = FALSE;
+        return S_OK;
+    }
+
+    if (!IsEqualGUID(&type->majortype, &WMMEDIATYPE_Audio)
+            || !IsEqualGUID(&type->subtype, &WMMEDIASUBTYPE_MP3)
+            || !IsEqualGUID(&type->formattype, &WMFORMAT_WaveFormatEx))
+        return DMO_E_TYPE_NOT_ACCEPTED;
+
+    if (!(flags & DMO_SET_TYPEF_TEST_ONLY))
+    {
+        if (dmo->intype_set)
+            MoFreeMediaType(&dmo->intype);
+        MoCopyMediaType(&dmo->intype, type);
+        dmo->intype_set = TRUE;
+    }
 
     return S_OK;
 }
@@ -154,9 +264,13 @@ static HRESULT WINAPI MediaObject_SetOutputType(IMediaObject *iface, DWORD index
     if (flags & DMO_SET_TYPEF_CLEAR)
     {
         MoFreeMediaType(&This->outtype);
+        This->outtype_set = FALSE;
         return S_OK;
     }
 
+    if (!IsEqualGUID(&type->formattype, &WMFORMAT_WaveFormatEx))
+        return DMO_E_TYPE_NOT_ACCEPTED;
+
     format = (WAVEFORMATEX *)type->pbFormat;
 
     if (format->wBitsPerSample == 8)
@@ -179,6 +293,7 @@ static HRESULT WINAPI MediaObject_SetOutputType(IMediaObject *iface, DWORD index
             return DMO_E_TYPE_NOT_ACCEPTED;
         }
         MoCopyMediaType(&This->outtype, type);
+        This->outtype_set = TRUE;
     }
 
     return S_OK;
@@ -198,18 +313,33 @@ static HRESULT WINAPI MediaObject_GetOutputCurrentType(IMediaObject *iface, DWOR
     return E_NOTIMPL;
 }
 
-static HRESULT WINAPI MediaObject_GetInputSizeInfo(IMediaObject *iface, DWORD index, DWORD *size, DWORD *max_lookahead, DWORD *alignment)
+static HRESULT WINAPI MediaObject_GetInputSizeInfo(IMediaObject *iface,
+        DWORD index, DWORD *size, DWORD *lookahead, DWORD *alignment)
 {
-    FIXME("(%p)->(%d, %p, %p, %p) stub!\n", iface, index, size, max_lookahead, alignment);
+    struct mp3_decoder *dmo = impl_from_IMediaObject(iface);
 
-    return E_NOTIMPL;
+    TRACE("iface %p, index %u, size %p, lookahead %p, alignment %p.\n", iface, index, size, lookahead, alignment);
+
+    if (!dmo->intype_set || !dmo->outtype_set)
+        return DMO_E_TYPE_NOT_SET;
+
+    *size = 0;
+    *alignment = 1;
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_GetOutputSizeInfo(IMediaObject *iface, DWORD index, DWORD *size, DWORD *alignment)
 {
-    FIXME("(%p)->(%d, %p, %p) stub!\n", iface, index, size, alignment);
+    struct mp3_decoder *dmo = impl_from_IMediaObject(iface);
 
-    return E_NOTIMPL;
+    TRACE("iface %p, index %u, size %p, alignment %p.\n", iface, index, size, alignment);
+
+    if (!dmo->intype_set || !dmo->outtype_set)
+        return DMO_E_TYPE_NOT_SET;
+
+    *size = 2 * 1152 * ((WAVEFORMATEX *)dmo->outtype.pbFormat)->wBitsPerSample / 8;
+    *alignment = 1;
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_GetInputMaxLatency(IMediaObject *iface, DWORD index, REFERENCE_TIME *latency)
@@ -228,16 +358,27 @@ static HRESULT WINAPI MediaObject_SetInputMaxLatency(IMediaObject *iface, DWORD
 
 static HRESULT WINAPI MediaObject_Flush(IMediaObject *iface)
 {
-    FIXME("(%p)->() stub!\n", iface);
+    struct mp3_decoder *dmo = impl_from_IMediaObject(iface);
 
-    return E_NOTIMPL;
+    TRACE("iface %p.\n", iface);
+
+    if (dmo->buffer)
+        IMediaBuffer_Release(dmo->buffer);
+    dmo->buffer = NULL;
+    dmo->timestamp = 0;
+
+    /* mpg123 doesn't give us a way to flush, so just close and reopen the feed. */
+    mpg123_close(dmo->mh);
+    mpg123_open_feed(dmo->mh);
+
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_Discontinuity(IMediaObject *iface, DWORD index)
 {
-    FIXME("(%p)->(%d) stub!\n", iface, index);
+    TRACE("iface %p.\n", iface);
 
-    return E_NOTIMPL;
+    return S_OK;
 }
 
 static HRESULT WINAPI MediaObject_AllocateStreamingResources(IMediaObject *iface)
@@ -326,6 +467,20 @@ static HRESULT WINAPI MediaObject_ProcessOutput(IMediaObject *iface, DWORD flags
 
     buffers[0].dwStatus = 0;
 
+    if (!buffers[0].pBuffer)
+    {
+        while ((err = mpg123_read(This->mh, NULL, 0, &written)) == MPG123_NEW_FORMAT);
+        if (err == MPG123_NEED_MORE)
+            return S_OK;
+        else if (err == MPG123_ERR)
+            ERR("mpg123_read() failed: %s\n", mpg123_strerror(This->mh));
+        else if (err != MPG123_OK)
+            ERR("mpg123_read() returned %d\n", err);
+
+        buffers[0].dwStatus = DMO_OUTPUT_DATA_BUFFERF_INCOMPLETE;
+        return S_OK;
+    }
+
     if (!This->buffer)
         return S_FALSE;
 
@@ -389,7 +544,7 @@ static HRESULT WINAPI MediaObject_Lock(IMediaObject *iface, LONG lock)
     return E_NOTIMPL;
 }
 
-static const IMediaObjectVtbl IMediaObject_vtbl = {
+static const IMediaObjectVtbl MediaObject_vtbl = {
     MediaObject_QueryInterface,
     MediaObject_AddRef,
     MediaObject_Release,
@@ -416,23 +571,28 @@ static const IMediaObjectVtbl IMediaObject_vtbl = {
     MediaObject_Lock,
 };
 
-static HRESULT create_mp3_decoder(REFIID iid, void **obj)
+static HRESULT create_mp3_decoder(IUnknown *outer, REFIID iid, void **obj)
 {
     struct mp3_decoder *This;
+    HRESULT hr;
     int err;
 
     if (!(This = heap_alloc_zero(sizeof(*This))))
         return E_OUTOFMEMORY;
 
-    This->IMediaObject_iface.lpVtbl = &IMediaObject_vtbl;
-    This->ref = 0;
+    This->IUnknown_inner.lpVtbl = &Unknown_vtbl;
+    This->IMediaObject_iface.lpVtbl = &MediaObject_vtbl;
+    This->ref = 1;
+    This->outer = outer ? outer : &This->IUnknown_inner;
 
     mpg123_init();
     This->mh = mpg123_new(NULL, &err);
     mpg123_open_feed(This->mh);
     mpg123_format_none(This->mh);
 
-    return IMediaObject_QueryInterface(&This->IMediaObject_iface, iid, obj);
+    hr = IUnknown_QueryInterface(&This->IUnknown_inner, iid, obj);
+    IUnknown_Release(&This->IUnknown_inner);
+    return hr;
 }
 
 static HRESULT WINAPI ClassFactory_QueryInterface(IClassFactory *iface, REFIID iid, void **obj)
@@ -466,13 +626,13 @@ static HRESULT WINAPI ClassFactory_CreateInstance(IClassFactory *iface, IUnknown
 {
     TRACE("(%p, %s, %p)\n", outer, debugstr_guid(iid), obj);
 
-    if (outer)
+    if (outer && !IsEqualGUID(iid, &IID_IUnknown))
     {
         *obj = NULL;
-        return CLASS_E_NOAGGREGATION;
+        return E_NOINTERFACE;
     }
 
-    return create_mp3_decoder(iid, obj);
+    return create_mp3_decoder(outer, iid, obj);
 }
 
 static HRESULT WINAPI ClassFactory_LockServer(IClassFactory *iface, BOOL lock)
