@@ -127,54 +127,19 @@ static void domain_restore(MonoDomain *prev_domain)
         mono_domain_set(prev_domain, FALSE);
 }
 
-static HRESULT RuntimeHost_AddDefaultDomain(RuntimeHost *This, MonoDomain **result)
-{
-    struct DomainEntry *entry;
-    HRESULT res=S_OK;
-
-    EnterCriticalSection(&This->lock);
-
-    entry = HeapAlloc(GetProcessHeap(), 0, sizeof(*entry));
-    if (!entry)
-    {
-        res = E_OUTOFMEMORY;
-        goto end;
-    }
-
-    /* FIXME: Use exe filename to name the domain? */
-    entry->domain = mono_jit_init_version("mscorlib.dll", "v4.0.30319");
-
-    if (!entry->domain)
-    {
-        HeapFree(GetProcessHeap(), 0, entry);
-        res = E_FAIL;
-        goto end;
-    }
-
-    is_mono_started = TRUE;
-
-    list_add_tail(&This->domains, &entry->entry);
-
-    *result = entry->domain;
-
-end:
-    LeaveCriticalSection(&This->lock);
-
-    return res;
-}
-
 static HRESULT RuntimeHost_GetDefaultDomain(RuntimeHost *This, const WCHAR *config_path, MonoDomain **result)
 {
     WCHAR config_dir[MAX_PATH];
     WCHAR base_dir[MAX_PATH];
     char *base_dirA, *config_pathA, *slash;
     HRESULT res=S_OK;
+    static BOOL configured_domain;
+
+    *result = get_root_domain();
 
     EnterCriticalSection(&This->lock);
 
-    if (This->default_domain) goto end;
-
-    res = RuntimeHost_AddDefaultDomain(This, &This->default_domain);
+    if (configured_domain) goto end;
 
     if (!config_path)
     {
@@ -213,40 +178,20 @@ static HRESULT RuntimeHost_GetDefaultDomain(RuntimeHost *This, const WCHAR *conf
         *(slash + 1) = 0;
 
     TRACE("setting base_dir: %s, config_path: %s\n", base_dirA, config_pathA);
-    mono_domain_set_config(This->default_domain, base_dirA, config_pathA);
+    mono_domain_set_config(*result, base_dirA, config_pathA);
 
     HeapFree(GetProcessHeap(), 0, config_pathA);
     HeapFree(GetProcessHeap(), 0, base_dirA);
 
 end:
-    *result = This->default_domain;
+
+    configured_domain = TRUE;
 
     LeaveCriticalSection(&This->lock);
 
     return res;
 }
 
-static void RuntimeHost_DeleteDomain(RuntimeHost *This, MonoDomain *domain)
-{
-    struct DomainEntry *entry;
-
-    EnterCriticalSection(&This->lock);
-
-    LIST_FOR_EACH_ENTRY(entry, &This->domains, struct DomainEntry, entry)
-    {
-        if (entry->domain == domain)
-        {
-            list_remove(&entry->entry);
-            if (This->default_domain == domain)
-                This->default_domain = NULL;
-            HeapFree(GetProcessHeap(), 0, entry);
-            break;
-        }
-    }
-
-    LeaveCriticalSection(&This->lock);
-}
-
 static BOOL RuntimeHost_GetMethod(MonoDomain *domain, const char *assemblyname,
     const char *namespace, const char *typename, const char *methodname, int arg_count,
     MonoMethod **method)
@@ -1478,21 +1423,22 @@ __int32 WINAPI _CorExeMain(void)
     wcscat(config_file, dotconfig);
 
     hr = parse_config_file(config_file, &parsed_config);
-    if (SUCCEEDED(hr) && parsed_config.private_path)
+    if (SUCCEEDED(hr) && parsed_config.private_path && parsed_config.private_path[0])
     {
+        WCHAR *save;
         for(i = 0; parsed_config.private_path[i] != 0; i++)
             if (parsed_config.private_path[i] == ';') number_of_private_paths++;
         if (parsed_config.private_path[wcslen(parsed_config.private_path) - 1] != ';') number_of_private_paths++;
         config_file_dir_size = (wcsrchr(config_file, '\\') - config_file) + 1;
         priv_path = HeapAlloc(GetProcessHeap(), 0, (number_of_private_paths + 1) * sizeof(WCHAR *));
         /* wcstok ignores trailing semicolons */
-        temp = wcstok(parsed_config.private_path, scW);
+        temp = wcstok_s(parsed_config.private_path, scW, &save);
         for (i = 0; i < number_of_private_paths; i++)
         {
             priv_path[i] = HeapAlloc(GetProcessHeap(), 0, (config_file_dir_size + wcslen(temp) + 1) * sizeof(WCHAR));
             memcpy(priv_path[i], config_file, config_file_dir_size * sizeof(WCHAR));
             wcscpy(priv_path[i] + config_file_dir_size, temp);
-            temp = wcstok(NULL, scW);
+            temp = wcstok_s(NULL, scW, &save);
         }
         priv_path[number_of_private_paths] = NULL;
         if (InterlockedCompareExchangePointer((void **)&private_path, priv_path, NULL))
@@ -1529,8 +1475,6 @@ __int32 WINAPI _CorExeMain(void)
                 ERR("couldn't load %s, status=%d\n", debugstr_w(filename), status);
                 exit_code = -1;
             }
-
-            RuntimeHost_DeleteDomain(host, domain);
         }
         else
             exit_code = -1;
@@ -1544,8 +1488,6 @@ __int32 WINAPI _CorExeMain(void)
 
     if (domain)
     {
-        int dummy;
-        mono_threads_enter_gc_unsafe_region(&dummy);
         mono_thread_manage();
         mono_runtime_quit();
     }
@@ -1618,8 +1560,6 @@ HRESULT RuntimeHost_Construct(CLRRuntimeInfo *runtime_version, RuntimeHost** res
 
     This->ref = 1;
     This->version = runtime_version;
-    list_init(&This->domains);
-    This->default_domain = NULL;
     InitializeCriticalSection(&This->lock);
     This->lock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": RuntimeHost.lock");
 
