@@ -18,9 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #define COBJMACROS
 
 #include <stdarg.h>
@@ -35,9 +32,9 @@
 
 #include "msipriv.h"
 #include "winemsi.h"
+#include "wine/asm.h"
 #include "wine/heap.h"
 #include "wine/debug.h"
-#include "wine/unicode.h"
 #include "wine/exception.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
@@ -128,7 +125,7 @@ BOOL msi_action_is_unique( const MSIPACKAGE *package, const WCHAR *action )
 
     for (i = 0; i < package->unique_actions_count; i++)
     {
-        if (!strcmpW( package->unique_actions[i], action )) return TRUE;
+        if (!wcscmp( package->unique_actions[i], action )) return TRUE;
     }
     return FALSE;
 }
@@ -190,7 +187,7 @@ static LPWSTR msi_get_deferred_action(LPCWSTR action, LPCWSTR actiondata,
           lstrlenW(format) - 7;
     deferred = msi_alloc(len * sizeof(WCHAR));
 
-    sprintfW(deferred, format, actiondata, usersid, prodcode, action);
+    swprintf(deferred, len, format, actiondata, usersid, prodcode, action);
     return deferred;
 }
 
@@ -199,15 +196,15 @@ static void set_deferred_action_props( MSIPACKAGE *package, const WCHAR *deferre
     static const WCHAR sep[] = {'<','=','>',0};
     const WCHAR *end, *beg = deferred_data + 1;
 
-    end = strstrW(beg, sep);
+    end = wcsstr(beg, sep);
     msi_set_property( package->db, szCustomActionData, beg, end - beg );
     beg = end + 3;
 
-    end = strstrW(beg, sep);
+    end = wcsstr(beg, sep);
     msi_set_property( package->db, szUserSID, beg, end - beg );
     beg = end + 3;
 
-    end = strchrW(beg, ']');
+    end = wcschr(beg, ']');
     msi_set_property( package->db, szProductCode, beg, end - beg );
 }
 
@@ -228,7 +225,7 @@ WCHAR *msi_create_temp_file( MSIDATABASE *db )
         if (!(db->tempfolder = strdupW( tmp ))) return NULL;
     }
 
-    if ((ret = msi_alloc( (strlenW( db->tempfolder ) + 20) * sizeof(WCHAR) )))
+    if ((ret = msi_alloc( (lstrlenW( db->tempfolder ) + 20) * sizeof(WCHAR) )))
     {
         if (!GetTempFileNameW( db->tempfolder, szMsi, 0, ret ))
         {
@@ -298,7 +295,7 @@ static MSIBINARY *get_temp_binary(MSIPACKAGE *package, LPCWSTR source)
 
     LIST_FOR_EACH_ENTRY( binary, &package->binaries, MSIBINARY, entry )
     {
-        if (!strcmpW( binary->source, source ))
+        if (!wcscmp( binary->source, source ))
             return binary;
     }
 
@@ -468,7 +465,7 @@ static void handle_msi_break(LPCSTR target)
     if (!GetEnvironmentVariableA("MsiBreak", val, MAX_PATH))
         return;
 
-    if (strcasecmp(val, target))
+    if (strcmp(val, target))
         return;
 
     sprintf(msg, format, GetCurrentProcessId(), GetCurrentProcessId());
@@ -521,7 +518,7 @@ UINT CDECL __wine_msi_call_dll_function(DWORD client_pid, const GUID *guid)
     {
         WCHAR endpoint[12];
 
-        sprintfW(endpoint, endpoint_fmtW, client_pid);
+        swprintf(endpoint, ARRAY_SIZE(endpoint), endpoint_fmtW, client_pid);
         status = RpcStringBindingComposeW(NULL, ncalrpcW, NULL, endpoint, NULL, &binding_str);
         if (status != RPC_S_OK)
         {
@@ -583,7 +580,8 @@ UINT CDECL __wine_msi_call_dll_function(DWORD client_pid, const GUID *guid)
 
     midl_user_free(dll);
     midl_user_free(proc);
-    MsiCloseHandle(hPackage);
+
+    MsiCloseAllHandles();
 
     return r;
 }
@@ -606,7 +604,8 @@ static DWORD custom_start_server(MSIPACKAGE *package, DWORD arch)
         (arch == SCS_64BIT_BINARY && package->custom_server_64_process))
         return ERROR_SUCCESS;
 
-    sprintfW(buffer, pipe_name, GetCurrentProcessId(), arch == SCS_32BIT_BINARY ? 32 : 64);
+    swprintf(buffer, ARRAY_SIZE(buffer), pipe_name,
+             GetCurrentProcessId(), arch == SCS_32BIT_BINARY ? 32 : 64);
     pipe = CreateNamedPipeW(buffer, PIPE_ACCESS_DUPLEX, 0, 1, sizeof(DWORD64),
         sizeof(GUID), 0, NULL);
     if (pipe == INVALID_HANDLE_VALUE)
@@ -619,8 +618,8 @@ static DWORD custom_start_server(MSIPACKAGE *package, DWORD arch)
         GetSystemWow64DirectoryW(path, MAX_PATH - ARRAY_SIZE(msiexecW));
     else
         GetSystemDirectoryW(path, MAX_PATH - ARRAY_SIZE(msiexecW));
-    strcatW(path, msiexecW);
-    sprintfW(cmdline, argsW, path, GetCurrentProcessId());
+    lstrcatW(path, msiexecW);
+    swprintf(cmdline, ARRAY_SIZE(cmdline), argsW, path, GetCurrentProcessId());
 
     if (wow64 && arch == SCS_64BIT_BINARY)
     {
@@ -745,7 +744,7 @@ static msi_custom_action_info *do_msidbCustomActionTypeDll(
     {
         WCHAR endpoint[12];
 
-        sprintfW(endpoint, endpoint_fmtW, GetCurrentProcessId());
+        swprintf(endpoint, ARRAY_SIZE(endpoint), endpoint_fmtW, GetCurrentProcessId());
         status = RpcServerUseProtseqEpW(ncalrpcW, RPC_C_PROTSEQ_MAX_REQS_DEFAULT,
             endpoint, NULL);
         if (status != RPC_S_OK)
@@ -824,14 +823,14 @@ static HANDLE execute_command( const WCHAR *app, WCHAR *arg, const WCHAR *dir )
             return INVALID_HANDLE_VALUE;
         }
 
-        if (arg) len_arg = strlenW( arg );
+        if (arg) len_arg = lstrlenW( arg );
         if (!(cmd = msi_alloc( (len_exe + len_arg + 4) * sizeof(WCHAR) )))
         {
             msi_free( exe );
             return INVALID_HANDLE_VALUE;
         }
         p = cmd;
-        if (strchrW( exe, ' ' ))
+        if (wcschr( exe, ' ' ))
         {
             *p++ = '\"';
             memcpy( p, exe, len_exe * sizeof(WCHAR) );
@@ -841,7 +840,7 @@ static HANDLE execute_command( const WCHAR *app, WCHAR *arg, const WCHAR *dir )
         }
         else
         {
-            strcpyW( p, exe );
+            lstrcpyW( p, exe );
             p += len_exe;
         }
         if (arg)
@@ -951,50 +950,136 @@ static UINT HANDLE_CustomType19( MSIPACKAGE *package, const WCHAR *source, const
     return ERROR_INSTALL_FAILURE;
 }
 
+static WCHAR *build_msiexec_args( const WCHAR *filename, const WCHAR *params )
+{
+    UINT len_filename = lstrlenW( filename ), len_params = lstrlenW( params );
+    UINT len = ARRAY_SIZE(L"/qb /i ") - 1;
+    WCHAR *ret;
+
+    if (!(ret = msi_alloc( (len + len_filename + len_params + 4) * sizeof(WCHAR) ))) return NULL;
+    memcpy( ret, L"/qb /i ", sizeof(L"/qb /i ") );
+    ret[len++] = '"';
+    memcpy( ret + len, filename, len_filename * sizeof(WCHAR) );
+    len += len_filename;
+    ret[len++] = '"';
+    ret[len++] = ' ';
+    lstrcpyW( ret + len, params );
+    return ret;
+}
+
 static UINT HANDLE_CustomType23( MSIPACKAGE *package, const WCHAR *source, const WCHAR *target,
                                  INT type, const WCHAR *action )
 {
-    static const WCHAR msiexecW[] = {'m','s','i','e','x','e','c',0};
-    static const WCHAR paramsW[] = {'/','q','b',' ','/','i',' '};
-    WCHAR *dir, *arg, *p;
-    UINT len_src, len_dir, len_tgt, len = ARRAY_SIZE( paramsW );
+    WCHAR *dir, *filename, *args, *p;
+    UINT len_dir, len_source = lstrlenW( source );
     HANDLE handle;
 
     if (!(dir = msi_dup_property( package->db, szOriginalDatabase ))) return ERROR_OUTOFMEMORY;
-    if (!(p = strrchrW( dir, '\\' )) && !(p = strrchrW( dir, '/' )))
+    if (!(p = wcsrchr( dir, '\\' )) && !(p = wcsrchr( dir, '/' )))
     {
         msi_free( dir );
         return ERROR_FUNCTION_FAILED;
     }
     *p = 0;
     len_dir = p - dir;
-    len_src = strlenW( source );
-    len_tgt = strlenW( target );
-    if (!(arg = msi_alloc( (len + len_dir + len_src + len_tgt + 5) * sizeof(WCHAR) )))
+    if (!(filename = msi_alloc( (len_dir + len_source + 2) * sizeof(WCHAR) )))
+    {
+        msi_free( dir );
+        return ERROR_OUTOFMEMORY;
+    }
+    memcpy( filename, dir, len_dir * sizeof(WCHAR) );
+    filename[len_dir++] = '\\';
+    memcpy( filename + len_dir, source, len_source * sizeof(WCHAR) );
+    filename[len_dir + len_source] = 0;
+
+    if (!(args = build_msiexec_args( filename, target )))
     {
         msi_free( dir );
         return ERROR_OUTOFMEMORY;
     }
-    memcpy( arg, paramsW, sizeof(paramsW) );
-    arg[len++] = '"';
-    memcpy( arg + len, dir, len_dir * sizeof(WCHAR) );
-    len += len_dir;
-    arg[len++] = '\\';
-    memcpy( arg + len, source, len_src * sizeof(WCHAR) );
-    len += len_src;
-    arg[len++] = '"';
-    arg[len++] = ' ';
-    strcpyW( arg + len, target );
 
     TRACE("installing %s concurrently\n", debugstr_w(source));
 
-    handle = execute_command( msiexecW, arg, dir );
+    handle = execute_command( L"msiexec", args, dir );
     msi_free( dir );
-    msi_free( arg );
+    msi_free( args );
     if (handle == INVALID_HANDLE_VALUE) return ERROR_SUCCESS;
     return wait_process_handle( package, type, handle, action );
 }
 
+static UINT write_substorage_to_file( MSIPACKAGE *package, const WCHAR *source, const WCHAR *filename )
+{
+    IStorage *src = NULL, *dst = NULL;
+    UINT r = ERROR_FUNCTION_FAILED;
+    HRESULT hr;
+
+    hr = StgCreateDocfile( filename, STGM_CREATE|STGM_TRANSACTED|STGM_WRITE|STGM_SHARE_EXCLUSIVE, 0, &dst );
+    if (FAILED( hr ))
+    {
+        WARN( "can't open destination storage %s (%08x)\n", debugstr_w(filename), hr );
+        goto done;
+    }
+
+    hr = IStorage_OpenStorage( package->db->storage, source, NULL, STGM_SHARE_EXCLUSIVE, NULL, 0, &src );
+    if (FAILED( hr ))
+    {
+        WARN( "can't open source storage %s (%08x)\n", debugstr_w(source), hr );
+        goto done;
+    }
+
+    hr = IStorage_CopyTo( src, 0, NULL, NULL, dst );
+    if (FAILED( hr ))
+    {
+        ERR( "failed to copy storage %s (%08x)\n", debugstr_w(source), hr );
+        goto done;
+    }
+
+    hr = IStorage_Commit( dst, 0 );
+    if (FAILED( hr ))
+        ERR( "failed to commit storage (%08x)\n", hr );
+    else
+        r = ERROR_SUCCESS;
+
+done:
+    if (src) IStorage_Release( src );
+    if (dst) IStorage_Release( dst );
+    return r;
+}
+
+static UINT HANDLE_CustomType7( MSIPACKAGE *package, const WCHAR *source, const WCHAR *target,
+                                INT type, const WCHAR *action )
+{
+    WCHAR *tmpfile, *args;
+    MSIBINARY *binary = NULL;
+    HANDLE handle;
+    UINT r;
+
+    if (!(tmpfile = msi_create_temp_file( package->db ))) return ERROR_FUNCTION_FAILED;
+
+    r = write_substorage_to_file( package, source, tmpfile );
+    if (r != ERROR_SUCCESS)
+        goto error;
+
+    if (!(binary = msi_alloc( sizeof(*binary) ))) goto error;
+    binary->source  = NULL;
+    binary->tmpfile = tmpfile;
+    list_add_tail( &package->binaries, &binary->entry );
+
+    if (!(args = build_msiexec_args( tmpfile, target ))) return ERROR_OUTOFMEMORY;
+
+    TRACE("installing %s concurrently\n", debugstr_w(source));
+
+    handle = execute_command( L"msiexec", args, L"C:\\" );
+    msi_free( args );
+    if (handle == INVALID_HANDLE_VALUE) return ERROR_SUCCESS;
+    return wait_process_handle( package, type, handle, action );
+
+error:
+    DeleteFileW( tmpfile );
+    msi_free( tmpfile );
+    return ERROR_FUNCTION_FAILED;
+}
+
 static UINT HANDLE_CustomType50( MSIPACKAGE *package, const WCHAR *source, const WCHAR *target,
                                  INT type, const WCHAR *action )
 {
@@ -1317,7 +1402,7 @@ UINT ACTION_CustomAction(MSIPACKAGE *package, const WCHAR *action)
     int len;
 
     /* deferred action: [properties]Action */
-    if ((ptr = strrchrW(action, ']')))
+    if ((ptr = wcsrchr(action, ']')))
     {
         deferred_data = action;
         action = ptr + 1;
@@ -1379,66 +1464,68 @@ UINT ACTION_CustomAction(MSIPACKAGE *package, const WCHAR *action)
 
     switch (type & CUSTOM_ACTION_TYPE_MASK)
     {
-        case 1: /* DLL file stored in a Binary table stream */
-            rc = HANDLE_CustomType1(package,source,target,type,action);
-            break;
-        case 2: /* EXE file stored in a Binary table stream */
-            rc = HANDLE_CustomType2(package,source,target,type,action);
-            break;
-        case 18: /*EXE file installed with package */
-            rc = HANDLE_CustomType18(package,source,target,type,action);
-            break;
-        case 19: /* Error that halts install */
-            rc = HANDLE_CustomType19(package,source,target,type,action);
-            break;
-        case 17:
-            rc = HANDLE_CustomType17(package,source,target,type,action);
-            break;
-        case 23: /* installs another package in the source tree */
-            deformat_string(package,target,&deformated);
-            rc = HANDLE_CustomType23(package,source,deformated,type,action);
-            msi_free(deformated);
-            break;
-        case 50: /*EXE file specified by a property value */
-            rc = HANDLE_CustomType50(package,source,target,type,action);
-            break;
-        case 34: /*EXE to be run in specified directory */
-            rc = HANDLE_CustomType34(package,source,target,type,action);
-            break;
-        case 35: /* Directory set with formatted text. */
-            deformat_string(package,target,&deformated);
-            MSI_SetTargetPathW(package, source, deformated);
-            msi_free(deformated);
-            break;
-        case 51: /* Property set with formatted text. */
-            if (!source)
-                break;
-
-            len = deformat_string( package, target, &deformated );
-            rc = msi_set_property( package->db, source, deformated, len );
-            if (rc == ERROR_SUCCESS && !strcmpW( source, szSourceDir ))
-                msi_reset_source_folders( package );
-            msi_free(deformated);
-            break;
-    case 37: /* JScript/VBScript text stored in target column. */
-    case 38:
-        rc = HANDLE_CustomType37_38(package,source,target,type,action);
+    case 1: /* DLL file stored in a Binary table stream */
+        rc = HANDLE_CustomType1( package, source, target, type, action );
+        break;
+    case 2: /* EXE file stored in a Binary table stream */
+        rc = HANDLE_CustomType2( package, source, target, type, action );
         break;
     case 5:
-    case 6: /* JScript/VBScript file stored in a Binary table stream. */
-        rc = HANDLE_CustomType5_6(package,source,target,type,action);
+    case 6: /* JScript/VBScript file stored in a Binary table stream */
+        rc = HANDLE_CustomType5_6( package, source, target, type, action );
         break;
-    case 21: /* JScript/VBScript file installed with the product. */
+    case 7: /* Concurrent install from substorage */
+        deformat_string( package, target, &deformated );
+        rc = HANDLE_CustomType7( package, source, target, type, action );
+        msi_free( deformated );
+        break;
+    case 17:
+        rc = HANDLE_CustomType17( package, source, target, type, action );
+        break;
+    case 18: /* EXE file installed with package */
+        rc = HANDLE_CustomType18( package, source, target, type, action );
+        break;
+    case 19: /* Error that halts install */
+        rc = HANDLE_CustomType19( package, source, target, type, action );
+        break;
+    case 21: /* JScript/VBScript file installed with the product */
     case 22:
-        rc = HANDLE_CustomType21_22(package,source,target,type,action);
+        rc = HANDLE_CustomType21_22( package, source, target, type, action );
         break;
-    case 53: /* JScript/VBScript text specified by a property value. */
+    case 23: /* Installs another package in the source tree */
+        deformat_string( package, target, &deformated );
+        rc = HANDLE_CustomType23( package, source, deformated, type, action );
+        msi_free( deformated );
+        break;
+    case 34: /* EXE to be run in specified directory */
+        rc = HANDLE_CustomType34( package, source, target, type, action );
+        break;
+    case 35: /* Directory set with formatted text */
+        deformat_string( package, target, &deformated );
+        MSI_SetTargetPathW( package, source, deformated );
+        msi_free( deformated );
+        break;
+    case 37: /* JScript/VBScript text stored in target column */
+    case 38:
+        rc = HANDLE_CustomType37_38( package, source, target, type, action );
+        break;
+    case 50: /* EXE file specified by a property value */
+        rc = HANDLE_CustomType50( package, source, target, type, action );
+        break;
+    case 51: /* Property set with formatted text */
+        if (!source) break;
+        len = deformat_string( package, target, &deformated );
+        rc = msi_set_property( package->db, source, deformated, len );
+        if (rc == ERROR_SUCCESS && !wcscmp( source, szSourceDir )) msi_reset_source_folders( package );
+        msi_free( deformated );
+        break;
+    case 53: /* JScript/VBScript text specified by a property value */
     case 54:
-        rc = HANDLE_CustomType53_54(package,source,target,type,action);
+        rc = HANDLE_CustomType53_54( package, source, target, type, action );
         break;
     default:
-        FIXME("unhandled action type %u (%s %s)\n", type & CUSTOM_ACTION_TYPE_MASK,
-              debugstr_w(source), debugstr_w(target));
+        FIXME( "unhandled action type %u (%s %s)\n", type & CUSTOM_ACTION_TYPE_MASK, debugstr_w(source),
+               debugstr_w(target) );
     }
 
 end:
