@@ -36,7 +36,7 @@
 #include "winemsi.h"
 #include "wine/heap.h"
 #include "wine/debug.h"
-#include "wine/unicode.h"
+#include "wine/exception.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(msi);
 
@@ -80,7 +80,17 @@ UINT WINAPI MsiDoActionW( MSIHANDLE hInstall, LPCWSTR szAction )
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        return remote_DoAction(remote, szAction);
+        __TRY
+        {
+            ret = remote_DoAction(remote, szAction);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
+
+        return ret;
     }
  
     ret = ACTION_PerformAction(package, szAction);
@@ -129,7 +139,17 @@ UINT WINAPI MsiSequenceW( MSIHANDLE hInstall, LPCWSTR szTable, INT iSequenceMode
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        return remote_Sequence(remote, szTable, iSequenceMode);
+        __TRY
+        {
+            ret = remote_Sequence(remote, szTable, iSequenceMode);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
+
+        return ret;
     }
     ret = MSI_Sequence( package, szTable );
     msiobj_release( &package->hdr );
@@ -145,7 +165,7 @@ UINT msi_strcpy_to_awstring( const WCHAR *str, int len, awstring *awbuf, DWORD *
     if (!sz)
         return ERROR_SUCCESS;
 
-    if (len < 0) len = strlenW( str );
+    if (len < 0) len = lstrlenW( str );
  
     if (awbuf->unicode && awbuf->str.w)
     {
@@ -176,7 +196,7 @@ UINT msi_strncpyWtoA(const WCHAR *str, int lenW, char *buf, DWORD *sz, BOOL remo
     if (!sz)
         return buf ? ERROR_INVALID_PARAMETER : ERROR_SUCCESS;
 
-    if (lenW < 0) lenW = strlenW(str);
+    if (lenW < 0) lenW = lstrlenW(str);
     lenA = WideCharToMultiByte(CP_ACP, 0, str, lenW + 1, NULL, 0, NULL, NULL);
     WideCharToMultiByte(CP_ACP, 0, str, lenW + 1, buf, *sz, NULL, NULL);
     lenA--;
@@ -198,7 +218,7 @@ UINT msi_strncpyW(const WCHAR *str, int len, WCHAR *buf, DWORD *sz)
     if (!sz)
         return buf ? ERROR_INVALID_PARAMETER : ERROR_SUCCESS;
 
-    if (len < 0) len = strlenW(str);
+    if (len < 0) len = lstrlenW(str);
     if (buf)
         memcpy(buf, str, min(len + 1, *sz) * sizeof(WCHAR));
     if (buf && len >= *sz)
@@ -218,7 +238,7 @@ const WCHAR *msi_get_target_folder( MSIPACKAGE *package, const WCHAR *name )
     if (!folder->ResolvedTarget)
     {
         MSIFOLDER *parent = folder;
-        while (parent->Parent && strcmpW( parent->Parent, parent->Directory ))
+        while (parent->Parent && wcscmp( parent->Parent, parent->Directory ))
         {
             parent = msi_get_loaded_folder( package, parent->Parent );
         }
@@ -257,7 +277,16 @@ UINT WINAPI MsiGetTargetPathA(MSIHANDLE hinst, const char *folder, char *buf, DW
             return ERROR_INVALID_HANDLE;
         }
 
-        r = remote_GetTargetPath(remote, folderW, &path);
+        __TRY
+        {
+            r = remote_GetTargetPath(remote, folderW, &path);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = GetExceptionCode();
+        }
+        __ENDTRY
+
         if (!r)
             r = msi_strncpyWtoA(path, -1, buf, sz, TRUE);
 
@@ -300,7 +329,16 @@ UINT WINAPI MsiGetTargetPathW(MSIHANDLE hinst, const WCHAR *folder, WCHAR *buf,
         if (!(remote = msi_get_remote(hinst)))
             return ERROR_INVALID_HANDLE;
 
-        r = remote_GetTargetPath(remote, folder, &path);
+        __TRY
+        {
+            r = remote_GetTargetPath(remote, folder, &path);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = GetExceptionCode();
+        }
+        __ENDTRY
+
         if (!r)
             r = msi_strncpyW(path, -1, buf, sz);
 
@@ -331,11 +369,11 @@ WCHAR *msi_resolve_source_folder( MSIPACKAGE *package, const WCHAR *name, MSIFOL
 
     TRACE("working to resolve %s\n", debugstr_w(name));
 
-    if (!strcmpW( name, szSourceDir )) name = szTargetDir;
+    if (!wcscmp( name, szSourceDir )) name = szTargetDir;
     if (!(f = msi_get_loaded_folder( package, name ))) return NULL;
 
     /* special resolving for root dir */
-    if (!strcmpW( name, szTargetDir ) && !f->ResolvedSource)
+    if (!wcscmp( name, szTargetDir ) && !f->ResolvedSource)
     {
         f->ResolvedSource = get_source_root( package );
     }
@@ -372,8 +410,7 @@ WCHAR *msi_resolve_source_folder( MSIPACKAGE *package, const WCHAR *name, MSIFOL
 UINT WINAPI MsiGetSourcePathA(MSIHANDLE hinst, const char *folder, char *buf, DWORD *sz)
 {
     MSIPACKAGE *package;
-    const WCHAR *path;
-    WCHAR *folderW;
+    WCHAR *path, *folderW;
     UINT r;
 
     TRACE("%s %p %p\n", debugstr_a(folder), buf, sz);
@@ -396,7 +433,16 @@ UINT WINAPI MsiGetSourcePathA(MSIHANDLE hinst, const char *folder, char *buf, DW
             return ERROR_INVALID_HANDLE;
         }
 
-        r = remote_GetSourcePath(remote, folderW, &path);
+        __TRY
+        {
+            r = remote_GetSourcePath(remote, folderW, &path);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = GetExceptionCode();
+        }
+        __ENDTRY
+
         if (!r)
             r = msi_strncpyWtoA(path, -1, buf, sz, TRUE);
 
@@ -411,6 +457,7 @@ UINT WINAPI MsiGetSourcePathA(MSIHANDLE hinst, const char *folder, char *buf, DW
     else
         r = ERROR_DIRECTORY;
 
+    heap_free(path);
     heap_free(folderW);
     msiobj_release(&package->hdr);
     return r;
@@ -422,7 +469,7 @@ UINT WINAPI MsiGetSourcePathA(MSIHANDLE hinst, const char *folder, char *buf, DW
 UINT WINAPI MsiGetSourcePathW(MSIHANDLE hinst, const WCHAR *folder, WCHAR *buf, DWORD *sz)
 {
     MSIPACKAGE *package;
-    const WCHAR *path;
+    WCHAR *path;
     UINT r;
 
     TRACE("%s %p %p\n", debugstr_w(folder), buf, sz);
@@ -439,7 +486,16 @@ UINT WINAPI MsiGetSourcePathW(MSIHANDLE hinst, const WCHAR *folder, WCHAR *buf,
         if (!(remote = msi_get_remote(hinst)))
             return ERROR_INVALID_HANDLE;
 
-        r = remote_GetSourcePath(remote, folder, &path);
+        __TRY
+        {
+            r = remote_GetSourcePath(remote, folder, &path);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = GetExceptionCode();
+        }
+        __ENDTRY
+
         if (!r)
             r = msi_strncpyW(path, -1, buf, sz);
 
@@ -453,6 +509,7 @@ UINT WINAPI MsiGetSourcePathW(MSIHANDLE hinst, const WCHAR *folder, WCHAR *buf,
     else
         r = ERROR_DIRECTORY;
 
+    heap_free(path);
     msiobj_release(&package->hdr);
     return r;
 }
@@ -490,7 +547,7 @@ static void set_target_path( MSIPACKAGE *package, MSIFOLDER *folder, const WCHAR
     WCHAR *target_path;
 
     if (!(target_path = msi_normalize_path( path ))) return;
-    if (strcmpW( target_path, folder->ResolvedTarget ))
+    if (wcscmp( target_path, folder->ResolvedTarget ))
     {
         msi_free( folder->ResolvedTarget );
         folder->ResolvedTarget = target_path;
@@ -513,7 +570,7 @@ UINT MSI_SetTargetPathW( MSIPACKAGE *package, LPCWSTR szFolder, LPCWSTR szFolder
 
     TRACE("%p %s %s\n", package, debugstr_w(szFolder), debugstr_w(szFolderPath));
 
-    attrib = GetFileAttributesW(szFolderPath);
+    attrib = msi_get_file_attributes( package, szFolderPath );
     /* native MSI tests writeability by making temporary files at each drive */
     if (attrib != INVALID_FILE_ATTRIBUTES &&
         (attrib & FILE_ATTRIBUTE_OFFLINE || attrib & FILE_ATTRIBUTE_READONLY))
@@ -560,7 +617,17 @@ UINT WINAPI MsiSetTargetPathW(MSIHANDLE hInstall, LPCWSTR szFolder,
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        return remote_SetTargetPath(remote, szFolder, szFolderPath);
+        __TRY
+        {
+            ret = remote_SetTargetPath(remote, szFolder, szFolderPath);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
+
+        return ret;
     }
 
     ret = MSI_SetTargetPathW( package, szFolder, szFolderPath );
@@ -616,7 +683,17 @@ BOOL WINAPI MsiGetMode(MSIHANDLE hInstall, MSIRUNMODE iRunMode)
         if (!(remote = msi_get_remote(hInstall)))
             return FALSE;
 
-        return remote_GetMode(remote, iRunMode);
+        __TRY
+        {
+            r = remote_GetMode(remote, iRunMode);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = FALSE;
+        }
+        __ENDTRY
+
+        return r;
     }
 
     switch (iRunMode)
@@ -699,7 +776,17 @@ UINT WINAPI MsiSetMode(MSIHANDLE hInstall, MSIRUNMODE iRunMode, BOOL fState)
         if (!(remote = msi_get_remote(hInstall)))
             return FALSE;
 
-        return remote_SetMode(remote, iRunMode, fState);
+        __TRY
+        {
+            r = remote_SetMode(remote, iRunMode, fState);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = GetExceptionCode();
+        }
+        __ENDTRY
+
+        return r;
     }
 
     switch (iRunMode)
@@ -848,7 +935,7 @@ UINT MSI_SetFeatureStateW( MSIPACKAGE *package, LPCWSTR szFeature, INSTALLSTATE
     /* update all the features that are children of this feature */
     LIST_FOR_EACH_ENTRY( child, &package->features, MSIFEATURE, entry )
     {
-        if (child->Feature_Parent && !strcmpW( szFeature, child->Feature_Parent ))
+        if (child->Feature_Parent && !wcscmp( szFeature, child->Feature_Parent ))
             MSI_SetFeatureStateW(package, child->Feature, iState);
     }
     
@@ -877,7 +964,17 @@ UINT WINAPI MsiSetFeatureStateW(MSIHANDLE hInstall, LPCWSTR szFeature,
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        return remote_SetFeatureState(remote, szFeature, iState);
+        __TRY
+        {
+            rc = remote_SetFeatureState(remote, szFeature, iState);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            rc = GetExceptionCode();
+        }
+        __ENDTRY
+
+        return rc;
     }
 
     rc = MSI_SetFeatureStateW(package,szFeature,iState);
@@ -933,7 +1030,7 @@ UINT WINAPI MsiSetFeatureAttributesW( MSIHANDLE handle, LPCWSTR name, DWORD attr
         return ERROR_INVALID_HANDLE;
 
     costing = msi_dup_property( package->db, szCostingComplete );
-    if (!costing || !strcmpW( costing, szOne ))
+    if (!costing || !wcscmp( costing, szOne ))
     {
         msi_free( costing );
         msiobj_release( &package->hdr );
@@ -1008,11 +1105,17 @@ UINT WINAPI MsiGetFeatureStateW(MSIHANDLE hInstall, LPCWSTR szFeature,
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        /* FIXME: should use SEH */
-        if (!piInstalled || !piAction)
-            return RPC_X_NULL_REF_POINTER;
+        __TRY
+        {
+            ret = remote_GetFeatureState(remote, szFeature, piInstalled, piAction);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
 
-        return remote_GetFeatureState(remote, szFeature, piInstalled, piAction);
+        return ret;
     }
 
     ret = MSI_GetFeatureStateW(package, szFeature, piInstalled, piAction);
@@ -1123,11 +1226,17 @@ UINT WINAPI MsiGetFeatureCostW(MSIHANDLE hInstall, LPCWSTR szFeature,
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        /* FIXME: should use SEH */
-        if (!piCost)
-            return RPC_X_NULL_REF_POINTER;
+        __TRY
+        {
+            ret = remote_GetFeatureCost(remote, szFeature, iCostTree, iState, piCost);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
 
-        return remote_GetFeatureCost(remote, szFeature, iCostTree, iState, piCost);
+        return ret;
     }
 
     if (!piCost)
@@ -1208,7 +1317,7 @@ static UINT MSI_GetFeatureInfo( MSIPACKAGE *package, LPCWSTR name, LPDWORD attrs
     if (attrs) *attrs = map_feature_attributes( feature->Attributes );
     if (title_len)
     {
-        if (feature->Title) len = strlenW( feature->Title );
+        if (feature->Title) len = lstrlenW( feature->Title );
         else len = 0;
         if (*title_len <= len)
         {
@@ -1217,14 +1326,14 @@ static UINT MSI_GetFeatureInfo( MSIPACKAGE *package, LPCWSTR name, LPDWORD attrs
         }
         else if (title)
         {
-            if (feature->Title) strcpyW( title, feature->Title );
+            if (feature->Title) lstrcpyW( title, feature->Title );
             else *title = 0;
             *title_len = len;
         }
     }
     if (help_len)
     {
-        if (feature->Description) len = strlenW( feature->Description );
+        if (feature->Description) len = lstrlenW( feature->Description );
         else len = 0;
         if (*help_len <= len)
         {
@@ -1233,7 +1342,7 @@ static UINT MSI_GetFeatureInfo( MSIPACKAGE *package, LPCWSTR name, LPDWORD attrs
         }
         else if (help)
         {
-            if (feature->Description) strcpyW( help, feature->Description );
+            if (feature->Description) lstrcpyW( help, feature->Description );
             else *help = 0;
             *help_len = len;
         }
@@ -1370,7 +1479,17 @@ UINT WINAPI MsiSetComponentStateW(MSIHANDLE hInstall, LPCWSTR szComponent,
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        return remote_SetComponentState(remote, szComponent, iState);
+        __TRY
+        {
+            ret = remote_SetComponentState(remote, szComponent, iState);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
+
+        return ret;
     }
 
     ret = MSI_SetComponentStateW(package, szComponent, iState);
@@ -1401,11 +1520,17 @@ UINT WINAPI MsiGetComponentStateW(MSIHANDLE hInstall, LPCWSTR szComponent,
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        /* FIXME: should use SEH */
-        if (!piInstalled || !piAction)
-            return RPC_X_NULL_REF_POINTER;
+        __TRY
+        {
+            ret = remote_GetComponentState(remote, szComponent, piInstalled, piAction);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            ret = GetExceptionCode();
+        }
+        __ENDTRY
 
-        return remote_GetComponentState(remote, szComponent, piInstalled, piAction);
+        return ret;
     }
 
     ret = MSI_GetComponentStateW( package, szComponent, piInstalled, piAction);
@@ -1429,7 +1554,17 @@ LANGID WINAPI MsiGetLanguage(MSIHANDLE hInstall)
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        return remote_GetLanguage(remote);
+        __TRY
+        {
+            langid = remote_GetLanguage(remote);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            langid = 0;
+        }
+        __ENDTRY
+
+        return langid;
     }
 
     langid = msi_get_property_int( package->db, szProductLanguage, 0 );
@@ -1452,7 +1587,7 @@ UINT MSI_SetInstallLevel( MSIPACKAGE *package, int iInstallLevel )
     if (iInstallLevel < 1)
         return MSI_SetFeatureStates( package );
 
-    len = sprintfW( level, fmt, iInstallLevel );
+    len = swprintf( level, ARRAY_SIZE(level), fmt, iInstallLevel );
     r = msi_set_property( package->db, szInstallLevel, level, len );
     if ( r == ERROR_SUCCESS )
         r = MSI_SetFeatureStates( package );
@@ -1478,7 +1613,17 @@ UINT WINAPI MsiSetInstallLevel(MSIHANDLE hInstall, int iInstallLevel)
         if (!(remote = msi_get_remote(hInstall)))
             return ERROR_INVALID_HANDLE;
 
-        return remote_SetInstallLevel(remote, iInstallLevel);
+        __TRY
+        {
+            r = remote_SetInstallLevel(remote, iInstallLevel);
+        }
+        __EXCEPT(rpc_filter)
+        {
+            r = GetExceptionCode();
+        }
+        __ENDTRY
+
+        return r;
     }
 
     r = MSI_SetInstallLevel( package, iInstallLevel );
