@@ -27,7 +27,6 @@
 #include "winuser.h"
 #include "winerror.h"
 #include "wine/debug.h"
-#include "wine/unicode.h"
 #include "msi.h"
 #include "msiquery.h"
 #include "msipriv.h"
@@ -46,7 +45,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(msidb);
 #define MSIFIELD_INT    1
 #define MSIFIELD_WSTR   3
 #define MSIFIELD_STREAM 4
-#define MSIFIELD_INTPTR 5
 
 static void MSI_FreeField( MSIFIELD *field )
 {
@@ -54,7 +52,6 @@ static void MSI_FreeField( MSIFIELD *field )
     {
     case MSIFIELD_NULL:
     case MSIFIELD_INT:
-    case MSIFIELD_INTPTR:
         break;
     case MSIFIELD_WSTR:
         msi_free( field->u.szwVal);
@@ -190,9 +187,6 @@ UINT MSI_RecordCopyField( MSIRECORD *in_rec, UINT in_n,
         case MSIFIELD_INT:
             out->u.iVal = in->u.iVal;
             break;
-        case MSIFIELD_INTPTR:
-            out->u.pVal = in->u.pVal;
-            break;
         case MSIFIELD_WSTR:
             if ((str = msi_strdupW( in->u.szwVal, in->len )))
             {
@@ -216,32 +210,6 @@ UINT MSI_RecordCopyField( MSIRECORD *in_rec, UINT in_n,
     return r;
 }
 
-INT_PTR MSI_RecordGetIntPtr( MSIRECORD *rec, UINT iField )
-{
-    int ret;
-
-    TRACE( "%p %d\n", rec, iField );
-
-    if( iField > rec->count )
-        return MININT_PTR;
-
-    switch( rec->fields[iField].type )
-    {
-    case MSIFIELD_INT:
-        return rec->fields[iField].u.iVal;
-    case MSIFIELD_INTPTR:
-        return rec->fields[iField].u.pVal;
-    case MSIFIELD_WSTR:
-        if( string2intW( rec->fields[iField].u.szwVal, &ret ) )
-            return ret;
-        return MININT_PTR;
-    default:
-        break;
-    }
-
-    return MININT_PTR;
-}
-
 int MSI_RecordGetInteger( MSIRECORD *rec, UINT iField)
 {
     int ret = 0;
@@ -255,8 +223,6 @@ int MSI_RecordGetInteger( MSIRECORD *rec, UINT iField)
     {
     case MSIFIELD_INT:
         return rec->fields[iField].u.iVal;
-    case MSIFIELD_INTPTR:
-        return rec->fields[iField].u.pVal;
     case MSIFIELD_WSTR:
         if( string2intW( rec->fields[iField].u.szwVal, &ret ) )
             return ret;
@@ -311,20 +277,6 @@ UINT WINAPI MsiRecordClearData( MSIHANDLE handle )
     return ERROR_SUCCESS;
 }
 
-UINT MSI_RecordSetIntPtr( MSIRECORD *rec, UINT iField, INT_PTR pVal )
-{
-    TRACE("%p %u %ld\n", rec, iField, pVal);
-
-    if( iField > rec->count )
-        return ERROR_INVALID_PARAMETER;
-
-    MSI_FreeField( &rec->fields[iField] );
-    rec->fields[iField].type = MSIFIELD_INTPTR;
-    rec->fields[iField].u.pVal = pVal;
-
-    return ERROR_SUCCESS;
-}
-
 UINT MSI_RecordSetInteger( MSIRECORD *rec, UINT iField, int iVal )
 {
     TRACE("%p %u %d\n", rec, iField, iVal);
@@ -629,7 +581,7 @@ UINT msi_record_set_string( MSIRECORD *rec, UINT field, const WCHAR *value, int
 
     MSI_FreeField( &rec->fields[field] );
 
-    if (value && len < 0) len = strlenW( value );
+    if (value && len < 0) len = lstrlenW( value );
 
     if (value && len)
     {
@@ -1095,10 +1047,121 @@ void dump_record(MSIRECORD *rec)
         case MSIFIELD_NULL: TRACE("(null)"); break;
         case MSIFIELD_INT: TRACE("%d", rec->fields[i].u.iVal); break;
         case MSIFIELD_WSTR: TRACE("%s", debugstr_w(rec->fields[i].u.szwVal)); break;
-        case MSIFIELD_INTPTR: TRACE("%ld", rec->fields[i].u.pVal); break;
         case MSIFIELD_STREAM: TRACE("%p", rec->fields[i].u.stream); break;
         }
         if (i < rec->count) TRACE(", ");
     }
     TRACE("]\n");
 }
+
+UINT copy_remote_record(const struct wire_record *in, MSIHANDLE out)
+{
+    MSIRECORD *rec;
+    unsigned int i;
+    UINT r = ERROR_SUCCESS;
+
+    if (!(rec = msihandle2msiinfo(out, MSIHANDLETYPE_RECORD)))
+        return ERROR_INVALID_HANDLE;
+
+    rec->cookie = in->cookie;
+    for (i = 0; i <= in->count; i++)
+    {
+        switch (in->fields[i].type)
+        {
+        case MSIFIELD_NULL:
+            MSI_FreeField(&rec->fields[i]);
+            rec->fields[i].type = MSIFIELD_NULL;
+            break;
+        case MSIFIELD_INT:
+            r = MSI_RecordSetInteger(rec, i, in->fields[i].u.iVal);
+            break;
+        case MSIFIELD_WSTR:
+            r = MSI_RecordSetStringW(rec, i, in->fields[i].u.szwVal);
+            break;
+        case MSIFIELD_STREAM:
+            r = MSI_RecordSetIStream(rec, i, in->fields[i].u.stream);
+            break;
+        default:
+            ERR("invalid field type %d\n", in->fields[i].type);
+            break;
+        }
+
+        if (r)
+        {
+            msiobj_release(&rec->hdr);
+            return r;
+        }
+    }
+
+    msiobj_release(&rec->hdr);
+    return ERROR_SUCCESS;
+}
+
+UINT unmarshal_record(const struct wire_record *in, MSIHANDLE *out)
+{
+    if (!in)
+    {
+        *out = 0;
+        return ERROR_SUCCESS;
+    }
+
+    *out = MsiCreateRecord(in->count);
+    if (!*out) return ERROR_OUTOFMEMORY;
+
+    return copy_remote_record(in, *out);
+}
+
+struct wire_record *marshal_record(MSIHANDLE handle)
+{
+    struct wire_record *ret;
+    unsigned int i;
+    MSIRECORD *rec;
+
+    if (!(rec = msihandle2msiinfo(handle, MSIHANDLETYPE_RECORD)))
+        return NULL;
+
+    ret = midl_user_allocate(sizeof(*ret) + rec->count * sizeof(ret->fields[0]));
+    ret->count = rec->count;
+    ret->cookie = rec->cookie;
+
+    for (i = 0; i <= rec->count; i++)
+    {
+        switch (rec->fields[i].type)
+        {
+        case MSIFIELD_NULL:
+            break;
+        case MSIFIELD_INT:
+            ret->fields[i].u.iVal = rec->fields[i].u.iVal;
+            break;
+        case MSIFIELD_WSTR:
+            ret->fields[i].u.szwVal = strdupW(rec->fields[i].u.szwVal);
+            break;
+        case MSIFIELD_STREAM:
+            IStream_AddRef(rec->fields[i].u.stream);
+            ret->fields[i].u.stream = rec->fields[i].u.stream;
+            break;
+        default:
+            ERR("invalid field type %d\n", rec->fields[i].type);
+            break;
+        }
+        ret->fields[i].type = rec->fields[i].type;
+    }
+
+    msiobj_release(&rec->hdr);
+    return ret;
+}
+
+void free_remote_record(struct wire_record *rec)
+{
+    int i;
+
+    for (i = 0; i <= rec->count; i++)
+    {
+        if (rec->fields[i].type == MSIFIELD_WSTR)
+            midl_user_free(rec->fields[i].u.szwVal);
+        else if (rec->fields[i].type == MSIFIELD_STREAM)
+            IStream_Release(rec->fields[i].u.stream);
+    }
+
+    midl_user_free(rec);
+}
