@@ -23,6 +23,7 @@
 #define COBJMACROS
 #define NONAMELESSUNION
 
+#include "stdio.h"
 #include "windef.h"
 #include "winbase.h"
 #include "winreg.h"
@@ -30,7 +31,6 @@
 #include "shlwapi.h"
 #include "wine/debug.h"
 #include "wine/exception.h"
-#include "wine/unicode.h"
 #include "msi.h"
 #include "msiquery.h"
 #include "msidefs.h"
@@ -984,31 +984,31 @@ static void parse_filetime( LPCWSTR str, FILETIME *ft )
 
     /* YYYY/MM/DD hh:mm:ss */
 
-    while (isspaceW( *p )) p++;
+    while (iswspace( *p )) p++;
 
-    lt.wYear = strtolW( p, &end, 10 );
+    lt.wYear = wcstol( p, &end, 10 );
     if (*end != '/') return;
     p = end + 1;
 
-    lt.wMonth = strtolW( p, &end, 10 );
+    lt.wMonth = wcstol( p, &end, 10 );
     if (*end != '/') return;
     p = end + 1;
 
-    lt.wDay = strtolW( p, &end, 10 );
+    lt.wDay = wcstol( p, &end, 10 );
     if (*end != ' ') return;
     p = end + 1;
 
-    while (isspaceW( *p )) p++;
+    while (iswspace( *p )) p++;
 
-    lt.wHour = strtolW( p, &end, 10 );
+    lt.wHour = wcstol( p, &end, 10 );
     if (*end != ':') return;
     p = end + 1;
 
-    lt.wMinute = strtolW( p, &end, 10 );
+    lt.wMinute = wcstol( p, &end, 10 );
     if (*end != ':') return;
     p = end + 1;
 
-    lt.wSecond = strtolW( p, &end, 10 );
+    lt.wSecond = wcstol( p, &end, 10 );
 
     TzSpecificLocalTimeToSystemTime( NULL, &lt, &utc );
     SystemTimeToFileTime( &utc, ft );
@@ -1017,7 +1017,7 @@ static void parse_filetime( LPCWSTR str, FILETIME *ft )
 static UINT parse_prop( LPCWSTR prop, LPCWSTR value, UINT *pid, INT *int_value,
                         FILETIME *ft_value, awcstring *str_value )
 {
-    *pid = atoiW( prop );
+    *pid = wcstol( prop, NULL, 10 );
     switch (*pid)
     {
     case PID_CODEPAGE:
@@ -1025,7 +1025,7 @@ static UINT parse_prop( LPCWSTR prop, LPCWSTR value, UINT *pid, INT *int_value,
     case PID_CHARCOUNT:
     case PID_SECURITY:
     case PID_PAGECOUNT:
-        *int_value = atoiW( value );
+        *int_value = wcstol( value, NULL, 10 );
         break;
 
     case PID_LASTPRINTED:
@@ -1096,6 +1096,114 @@ end:
     return r;
 }
 
+static UINT save_prop( MSISUMMARYINFO *si, HANDLE handle, UINT row )
+{
+    static const char fmt_systemtime[] = "%04u/%02u/%02u %02u:%02u:%02u";
+    char data[36]; /* largest string: YYYY/MM/DD hh:mm:ss */
+    static const char fmt_begin[] = "%u\t";
+    static const char data_end[] = "\r\n";
+    static const char fmt_int[] = "%u";
+    UINT r, data_type, len;
+    SYSTEMTIME system_time;
+    FILETIME file_time;
+    INT int_value;
+    awstring str;
+    DWORD sz;
+
+    str.unicode = FALSE;
+    str.str.a = NULL;
+    len = 0;
+    r = get_prop( si, row, &data_type, &int_value, &file_time, &str, &len );
+    if (r != ERROR_SUCCESS && r != ERROR_MORE_DATA)
+        return r;
+    if (data_type == VT_EMPTY)
+        return ERROR_SUCCESS; /* property not set */
+    sz = sprintf( data, fmt_begin, row );
+    if (!WriteFile( handle, data, sz, &sz, NULL ))
+        return ERROR_WRITE_FAULT;
+
+    switch( data_type )
+    {
+    case VT_I2:
+    case VT_I4:
+        sz = sprintf( data, fmt_int, int_value );
+        if (!WriteFile( handle, data, sz, &sz, NULL ))
+            return ERROR_WRITE_FAULT;
+        break;
+    case VT_LPSTR:
+        len++;
+        if (!(str.str.a = msi_alloc( len )))
+            return ERROR_OUTOFMEMORY;
+        r = get_prop( si, row, NULL, NULL, NULL, &str, &len );
+        if (r != ERROR_SUCCESS)
+        {
+            msi_free( str.str.a );
+            return r;
+        }
+        sz = len;
+        if (!WriteFile( handle, str.str.a, sz, &sz, NULL ))
+        {
+            msi_free( str.str.a );
+            return ERROR_WRITE_FAULT;
+        }
+        msi_free( str.str.a );
+        break;
+    case VT_FILETIME:
+        if (!FileTimeToSystemTime( &file_time, &system_time ))
+            return ERROR_FUNCTION_FAILED;
+        sz = sprintf( data, fmt_systemtime, system_time.wYear, system_time.wMonth,
+                      system_time.wDay, system_time.wHour, system_time.wMinute,
+                      system_time.wSecond );
+        if (!WriteFile( handle, data, sz, &sz, NULL ))
+            return ERROR_WRITE_FAULT;
+        break;
+    case VT_EMPTY:
+        /* cannot reach here, property not set */
+        break;
+    default:
+        FIXME( "Unknown property variant type\n" );
+        return ERROR_FUNCTION_FAILED;
+    }
+
+    sz = ARRAY_SIZE(data_end) - 1;
+    if (!WriteFile( handle, data_end, sz, &sz, NULL ))
+        return ERROR_WRITE_FAULT;
+
+    return ERROR_SUCCESS;
+}
+
+UINT msi_export_suminfo( MSIDATABASE *db, HANDLE handle )
+{
+    UINT i, r, num_rows;
+    MSISUMMARYINFO *si;
+
+    r = msi_get_suminfo( db->storage, 0, &si );
+    if (r != ERROR_SUCCESS)
+        r = msi_get_db_suminfo( db, 0, &si );
+    if (r != ERROR_SUCCESS)
+        return r;
+
+    num_rows = get_property_count( si->property );
+    if (!num_rows)
+    {
+        msiobj_release( &si->hdr );
+        return ERROR_FUNCTION_FAILED;
+    }
+
+    for (i = 0; i < num_rows; i++)
+    {
+        r = save_prop( si, handle, i );
+        if (r != ERROR_SUCCESS)
+        {
+            msiobj_release( &si->hdr );
+            return r;
+        }
+    }
+
+    msiobj_release( &si->hdr );
+    return ERROR_SUCCESS;
+}
+
 UINT WINAPI MsiSummaryInfoPersist( MSIHANDLE handle )
 {
     MSISUMMARYINFO *si;
