@@ -20,11 +20,13 @@
 #include "windef.h"
 #include "cxx.h"
 
+#define CXX_EXCEPTION       0xe06d7363
 #define ALIGNED_SIZE(size, alignment) (((size)+((alignment)-1))/(alignment)*(alignment))
 
 typedef unsigned char MSVCP_bool;
 typedef SIZE_T MSVCP_size_t;
 
+#define MSVCP_SIZE_T_MAX (~(MSVCP_size_t)0)
 #if _MSVCP_VER >= 100
 typedef __int64 DECLSPEC_ALIGN(8) streamoff;
 typedef __int64 DECLSPEC_ALIGN(8) streamsize;
@@ -47,10 +49,6 @@ extern void* (__cdecl *MSVCRT_operator_new)(MSVCP_size_t);
 extern void (__cdecl *MSVCRT_operator_delete)(void*);
 extern void* (__cdecl *MSVCRT_set_new_handler)(void*);
 
-#if _MSVCP_VER >= 140
-extern int* (__cdecl *UCRTBASE___processing_throw)(void);
-#endif
-
 #if _MSVCP_VER >= 110
 /* keep in sync with msvcrt/lock.c */
 typedef struct cs_queue
@@ -295,6 +293,14 @@ ctype_char* ctype_char_use_facet(const locale*);
 ctype_wchar* ctype_wchar_use_facet(const locale*);
 ctype_wchar* ctype_short_use_facet(const locale*);
 
+/* class _Init_locks */
+typedef struct {
+    char empty_struct;
+} _Init_locks;
+
+void __cdecl _Init_locks__Init_locks_ctor(_Init_locks*);
+void __cdecl _Init_locks__Init_locks_dtor(_Init_locks*);
+
 /* class _Lockit */
 typedef struct {
 #if _MSVCP_VER >= 70
@@ -310,8 +316,6 @@ typedef struct {
 #define _LOCK_DEBUG 3
 #define _MAX_LOCK 4
 
-void init_lockit(void);
-void free_lockit(void);
 _Lockit* __thiscall _Lockit_ctor_locktype(_Lockit*, int);
 void __thiscall _Lockit_dtor(_Lockit*);
 
@@ -660,9 +664,3 @@ static inline int mbstowcs_wrapper( size_t *ret, wchar_t *wcs, size_t size, cons
 #endif
 
 void free_misc(void);
-
-#if _MSVCP_VER >= 140
-#define UCRTBASE_PRINTF_STANDARD_SNPRINTF_BEHAVIOUR      (0x0002)
-int __cdecl __stdio_common_vsprintf(unsigned __int64 options, char *str, size_t len, const char *format,
-                                    _locale_t locale, __ms_va_list valist);
-#endif
