@@ -136,6 +136,9 @@ static int (__cdecl *p__memicmp_l)(const char*, const char*, size_t, _locale_t);
 static int (__cdecl *p__vsnwprintf)(wchar_t *buffer,size_t count, const wchar_t *format, __ms_va_list valist);
 static size_t (__cdecl *p___strncnt)(const char *str, size_t count);
 static int (__cdecl *p_swscanf)(const wchar_t *str, const wchar_t* format, ...);
+static int (__cdecl *p____mb_cur_max_l_func)(_locale_t locale);
+static _locale_t (__cdecl *p__create_locale)(int, const char*);
+static void (__cdecl *p__free_locale)(_locale_t);
 
 /* make sure we use the correct errno */
 #undef errno
@@ -234,11 +237,6 @@ static int cb_called[4];
 static int g_qsort_s_context_counter;
 static int g_bsearch_s_context_counter;
 
-static inline int almost_equal_f(float f1, float f2)
-{
-    return f1-f2 > -1e-30 && f1-f2 < 1e-30;
-}
-
 /* ########## */
 
 /* thiscall emulation */
@@ -407,6 +405,9 @@ static BOOL init(void)
     SET(p__vsnwprintf, "_vsnwprintf");
     SET(p___strncnt, "__strncnt");
     SET(p_swscanf, "swscanf");
+    SET(p____mb_cur_max_l_func, "___mb_cur_max_l_func");
+    SET(p__create_locale, "_create_locale");
+    SET(p__free_locale, "_free_locale");
 
     if (sizeof(void *) == 8)
     {
@@ -1009,7 +1010,7 @@ if (0)
             _atoflt_testdata[i].ret, _atoflt_testdata[i].str);
 
         if (ret == 0)
-          ok(almost_equal_f(flt.f, _atoflt_testdata[i].flt), "got %f, expected %f, for %s\n", flt.f,
+          ok(flt.f == _atoflt_testdata[i].flt, "got %f, expected %f, for %s\n", flt.f,
               _atoflt_testdata[i].flt, _atoflt_testdata[i].str);
 
         i++;
@@ -1475,9 +1476,12 @@ static void test_is_exception_typeof(void)
 
 static void test__AdjustPointer(void)
 {
-    int off = 0xf0;
-    void *obj1 = &off;
-    void *obj2 = (char*)&off - 2;
+    struct {
+        int vbase;
+        int off;
+    } obj = { 0, 0xf0 };
+    void *obj1 = &obj.off;
+    void *obj2 = &obj;
     struct test_data {
         void *ptr;
         void *ret;
@@ -1490,10 +1494,10 @@ static void test__AdjustPointer(void)
         {NULL, NULL, {0, -1, 0}},
         {(void*)0xbeef, (void*)0xbef0, {1, -1, 1}},
         {(void*)0xbeef, (void*)0xbeee, {-1, -1, 0}},
-        {&obj1, (char*)&obj1 + off, {0, 0, 0}},
-        {(char*)&obj1 - 5, (char*)&obj1 + off, {0, 5, 0}},
-        {(char*)&obj1 - 3, (char*)&obj1 + off + 24, {24, 3, 0}},
-        {(char*)&obj2 - 17, (char*)&obj2 + off + 4, {4, 17, 2}}
+        {&obj1, (char*)&obj1 + obj.off, {0, 0, 0}},
+        {(char*)&obj1 - 5, (char*)&obj1 + obj.off, {0, 5, 0}},
+        {(char*)&obj1 - 3, (char*)&obj1 + obj.off + 24, {24, 3, 0}},
+        {(char*)&obj2 - 17, (char*)&obj2 + obj.off + 4, {4, 17, sizeof(int)}}
     };
     void *ret;
     int i;
@@ -1938,6 +1942,25 @@ static void test_swscanf(void)
     ok( ret == (short)WEOF, "ret = %d\n", ret );
 }
 
+static void test____mb_cur_max_l_func(void)
+{
+    int ret;
+    _locale_t l;
+
+    ret = p____mb_cur_max_l_func(NULL);
+    ok( ret == 1, "MB_CUR_MAX_L(NULL) = %d\n", ret );
+
+    l = p__create_locale(LC_ALL, "chinese-traditional");
+    if (!l)
+    {
+        skip("DBCS locale not available\n");
+        return;
+    }
+    ret = p____mb_cur_max_l_func(l);
+    ok( ret == 2, "MB_CUR_MAX_L(cht) = %d\n", ret );
+    p__free_locale(l);
+}
+
 START_TEST(msvcr90)
 {
     if(!init())
@@ -1977,4 +2000,5 @@ START_TEST(msvcr90)
 #endif
     test___strncnt();
     test_swscanf();
+    test____mb_cur_max_l_func();
 }
