@@ -125,14 +125,24 @@ static MSVCRT_size_t msvcrt_heap_size(void *ptr)
     return HeapSize(heap, 0, ptr);
 }
 
+/*********************************************************************
+ *		_callnewh (MSVCRT.@)
+ */
+int CDECL _callnewh(MSVCRT_size_t size)
+{
+  int ret = 0;
+  MSVCRT_new_handler_func handler = MSVCRT_new_handler;
+  if(handler)
+    ret = (*handler)(size) ? 1 : 0;
+  return ret;
+}
+
 /*********************************************************************
  *		??2@YAPAXI@Z (MSVCRT.@)
  */
 void* CDECL DECLSPEC_HOTPATCH MSVCRT_operator_new(MSVCRT_size_t size)
 {
   void *retval;
-  int freed;
-  MSVCRT_new_handler_func handler;
 
   do
   {
@@ -142,13 +152,7 @@ void* CDECL DECLSPEC_HOTPATCH MSVCRT_operator_new(MSVCRT_size_t size)
       TRACE("(%ld) returning %p\n", size, retval);
       return retval;
     }
-
-    handler = MSVCRT_new_handler;
-    if(handler)
-      freed = (*handler)(size);
-    else
-      freed = 0;
-  } while(freed);
+  } while(_callnewh(size));
 
   TRACE("(%ld) out of memory\n", size);
 #if _MSVCR_VER >= 80
@@ -222,24 +226,8 @@ MSVCRT_new_handler_func CDECL MSVCRT_set_new_handler(void *func)
  */
 int CDECL MSVCRT__set_new_mode(int mode)
 {
-  int old_mode;
-  LOCK_HEAP;
-  old_mode = MSVCRT_new_mode;
-  MSVCRT_new_mode = mode;
-  UNLOCK_HEAP;
-  return old_mode;
-}
-
-/*********************************************************************
- *		_callnewh (MSVCRT.@)
- */
-int CDECL _callnewh(MSVCRT_size_t size)
-{
-  int ret = 0;
-  MSVCRT_new_handler_func handler = MSVCRT_new_handler;
-  if(handler)
-    ret = (*handler)(size) ? 1 : 0;
-  return ret;
+  if(!MSVCRT_CHECK_PMT(mode == 0 || mode == 1)) return -1;
+  return InterlockedExchange(&MSVCRT_new_mode, mode);
 }
 
 /*********************************************************************
@@ -441,10 +429,18 @@ void CDECL _free_base(void* ptr)
  */
 void* CDECL MSVCRT_malloc(MSVCRT_size_t size)
 {
-  void *ret = msvcrt_heap_alloc(0, size);
-  if (!ret)
-      *MSVCRT__errno() = MSVCRT_ENOMEM;
-  return ret;
+    void *ret;
+
+    do
+    {
+        ret = msvcrt_heap_alloc(0, size);
+        if (ret || !MSVCRT_new_mode)
+            break;
+    } while(_callnewh(size));
+
+    if (!ret)
+        *MSVCRT__errno() = MSVCRT_ENOMEM;
+    return ret;
 }
 
 #if _MSVCR_VER>=140
