@@ -18,19 +18,26 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-/* this function is part of the import lib for compatibility with the Mingw runtime */
+/* this function is part of the import lib for compatibility with ucrt runtime */
 #if 0
 #pragma makedep implib
 #endif
 
-#include "msvcrt.h"
+#include <stdio.h>
 
-extern MSVCRT_FILE * CDECL __p__iob(void);
+#undef __iob_func
+extern FILE * __cdecl __iob_func(void);
 
 /*********************************************************************
  *		__acrt_iob_func(UCRTBASE.@)
  */
-MSVCRT_FILE * CDECL __acrt_iob_func(unsigned idx)
+FILE * __cdecl __acrt_iob_func(unsigned idx)
 {
-    return __p__iob() + idx;
+    return __iob_func() + idx;
 }
+
+#ifdef __i386__
+void *_imp____acrt_iob_func = __acrt_iob_func;
+#else
+void *__imp___acrt_iob_func = __acrt_iob_func;
+#endif
