@@ -22,23 +22,8 @@
 #include <errno.h>
 #include <stdio.h>
 #include <math.h>
-#include "msvcrt.h"
 #include <process.h>
 
-static inline float __port_infinity(void)
-{
-    static const unsigned __inf_bytes = 0x7f800000;
-    return *(const float *)&__inf_bytes;
-}
-#define INFINITY __port_infinity()
-
-static inline float __port_nan(void)
-{
-    static const unsigned __nan_bytes = 0x7fc00000;
-    return *(const float *)&__nan_bytes;
-}
-#define NAN __port_nan()
-
 static inline BOOL almost_equal(double d1, double d2) {
     if(d1-d2>-1e-30 && d1-d2<1e-30)
         return TRUE;
@@ -47,14 +32,14 @@ static inline BOOL almost_equal(double d1, double d2) {
 
 static int (__cdecl *prand_s)(unsigned int *);
 static int (__cdecl *pI10_OUTPUT)(long double, int, int, void*);
-static int (__cdecl *pstrerror_s)(char *, MSVCRT_size_t, int);
+static int (__cdecl *pstrerror_s)(char *, size_t, int);
 static int (__cdecl *p_get_doserrno)(int *);
 static int (__cdecl *p_get_errno)(int *);
 static int (__cdecl *p_set_doserrno)(int);
 static int (__cdecl *p_set_errno)(int);
 static void (__cdecl *p__invalid_parameter)(const wchar_t*,
         const wchar_t*, const wchar_t*, unsigned int, uintptr_t);
-static void (__cdecl *p_qsort_s)(void*, MSVCRT_size_t, MSVCRT_size_t,
+static void (__cdecl *p_qsort_s)(void*, size_t, size_t,
         int (__cdecl*)(void*, const void*, const void*), void*);
 static double (__cdecl *p_atan)(double);
 static double (__cdecl *p_exp)(double);
@@ -164,7 +149,7 @@ static void test_I10_OUTPUT(void)
     if (j != 12)
         trace("sizeof(long double) = %d on this machine\n", j);
 
-    for(i=0; i<sizeof(I10_OUTPUT_tests)/sizeof(I10_OUTPUT_test); i++) {
+    for(i=0; i<ARRAY_SIZE(I10_OUTPUT_tests); i++) {
         memset(out.str, '#', sizeof(out.str));
 
         if (sizeof(long double) == 12)
@@ -225,7 +210,7 @@ static void test_strerror_s(void)
     memset(buf, 'X', sizeof(buf));
     ret = pstrerror_s(buf, 1, 0);
     ok(ret == 0, "Expected strerror_s to return 0, got %d\n", ret);
-    ok(strlen(buf) == 0, "Expected output buffer to be null terminated\n");
+    ok(buf[0] == 0, "Expected output buffer to be null terminated\n");
 
     memset(buf, 'X', sizeof(buf));
     ret = pstrerror_s(buf, 2, 0);
@@ -562,7 +547,7 @@ static void test_thread_handle_close(void)
     DWORD ret;
 
     /* _beginthread: handle is not closed on ExitThread and _endthreadex */
-    hThread = (HANDLE)_beginthread(test_thread_func, 0, (void*)0);
+    hThread = (HANDLE)_beginthread(test_thread_func, 0, NULL);
     ok(hThread != INVALID_HANDLE_VALUE, "_beginthread failed (%d)\n", errno);
     WaitForSingleObject(hThread, INFINITE);
     ret = CloseHandle(hThread);
@@ -621,7 +606,7 @@ static void test__lfind_s(void)
     }
 
     key = 1234;
-    num = sizeof(tests)/sizeof(tests[0]);
+    num = ARRAY_SIZE(tests);
 
     errno = 0xdeadbeef;
     found = p_lfind_s(NULL, tests, &num, sizeof(int), _lfind_s_comp, NULL);
