@@ -1967,6 +1967,14 @@ INT CDECL MSVCRT__iswctype_l( MSVCRT_wchar_t wc, MSVCRT_wctype_t type, MSVCRT__l
     return (get_char_typeW(wc) & 0xffff) & type;
 }
 
+/*********************************************************************
+ *		iswctype    (MSVCRT.@)
+ */
+INT CDECL MSVCRT_iswctype( MSVCRT_wchar_t wc, MSVCRT_wctype_t type )
+{
+    return (get_char_typeW(wc) & 0xfff) & type;
+}
+
 /*********************************************************************
  *		_iswblank_l (MSVCRT.@)
  */
