@@ -759,7 +759,7 @@ static HRESULT BindStatusCallback_create(httprequest* This, BindStatusCallback *
         case VT_ARRAY|VT_UI1:
         {
             sa = V_ARRAY(body);
-            if ((hr = SafeArrayAccessData(sa, (void **)&ptr)) != S_OK)
+            if ((hr = SafeArrayAccessData(sa, &ptr)) != S_OK)
             {
                 heap_free(bsc);
                 return hr;
