@@ -1296,9 +1296,8 @@ BOOL DIR_is_hidden_file( const UNICODE_STRING *name )
 /***********************************************************************
  *           hash_short_file_name
  *
- * Transform a Unix file name into a hashed DOS name. If the name is a valid
- * DOS name, it is converted to upper-case; otherwise it is replaced by a
- * hashed version that fits in 8.3 format.
+ * Transform a Unix file name into a hashed DOS name. If the name is not a valid
+ * DOS name, it is replaced by a hashed version that fits in 8.3 format.
  * 'buffer' must be at least 12 characters long.
  * Returns length of short name in bytes; short name is NOT null-terminated.
  */
@@ -1334,7 +1333,7 @@ static ULONG hash_short_file_name( const UNICODE_STRING *name, LPWSTR buffer )
     for (i = 4, p = name->Buffer, dst = buffer; i > 0; i--, p++)
     {
         if (p == end || p == ext) break;
-        *dst++ = is_invalid_dos_char(*p) ? '_' : toupperW(*p);
+        *dst++ = is_invalid_dos_char(*p) ? '_' : *p;
     }
     /* Pad to 5 chars with '~' */
     while (i-- >= 0) *dst++ = '~';
@@ -1349,7 +1348,7 @@ static ULONG hash_short_file_name( const UNICODE_STRING *name, LPWSTR buffer )
     {
         *dst++ = '.';
         for (i = 3, ext++; (i > 0) && ext < end; i--, ext++)
-            *dst++ = is_invalid_dos_char(*ext) ? '_' : toupperW(*ext);
+            *dst++ = is_invalid_dos_char(*ext) ? '_' : *ext;
     }
     return dst - buffer;
 }
@@ -1446,13 +1445,13 @@ static BOOLEAN match_filename( const UNICODE_STRING *name_str, const UNICODE_STR
 static BOOL append_entry( struct dir_data *data, const char *long_name,
                           const char *short_name, const UNICODE_STRING *mask )
 {
-    int i, long_len, short_len;
+    int long_len, short_len;
     WCHAR long_nameW[MAX_DIR_ENTRY_LEN + 1];
     WCHAR short_nameW[13];
     UNICODE_STRING str;
 
-    long_len = ntdll_umbstowcs( 0, long_name, strlen(long_name), long_nameW, MAX_DIR_ENTRY_LEN );
-    if (long_len == -1) return TRUE;
+    long_len = ntdll_umbstowcs( long_name, strlen(long_name), long_nameW, ARRAY_SIZE(long_nameW) );
+    if (long_len == ARRAY_SIZE(long_nameW)) return TRUE;
     long_nameW[long_len] = 0;
 
     str.Buffer = long_nameW;
@@ -1461,10 +1460,8 @@ static BOOL append_entry( struct dir_data *data, const char *long_name,
 
     if (short_name)
     {
-        short_len = ntdll_umbstowcs( 0, short_name, strlen(short_name),
+        short_len = ntdll_umbstowcs( short_name, strlen(short_name),
                                      short_nameW, ARRAY_SIZE( short_nameW ) - 1 );
-        if (short_len == -1) short_len = ARRAY_SIZE( short_nameW ) - 1;
-        for (i = 0; i < short_len; i++) short_nameW[i] = toupperW( short_nameW[i] );
     }
     else  /* generate a short name if necessary */
     {
@@ -1475,6 +1472,7 @@ static BOOL append_entry( struct dir_data *data, const char *long_name,
             short_len = hash_short_file_name( &str, short_nameW );
     }
     short_nameW[short_len] = 0;
+    wcsupr( short_nameW );
 
     TRACE( "long %s short %s mask %s\n",
            debugstr_w( long_nameW ), debugstr_w( short_nameW ), debugstr_us( mask ));
@@ -1795,12 +1793,10 @@ static NTSTATUS read_directory_data( struct dir_data *data, int fd, const UNICOD
     if (!has_wildcard( mask ))
     {
         /* convert the mask to a Unix name and check for it */
-        int ret, used_default;
         char unix_name[MAX_DIR_ENTRY_LEN * 3 + 1];
-
-        ret = ntdll_wcstoumbs( 0, mask->Buffer, mask->Length / sizeof(WCHAR),
-                               unix_name, sizeof(unix_name) - 1, NULL, &used_default );
-        if (ret > 0 && !used_default)
+        int ret = ntdll_wcstoumbs( mask->Buffer, mask->Length / sizeof(WCHAR),
+                                   unix_name, sizeof(unix_name) - 1, TRUE );
+        if (ret > 0)
         {
             unix_name[ret] = 0;
 #ifdef HAVE_GETATTRLIST
@@ -2047,16 +2043,13 @@ static NTSTATUS find_file_in_dir( char *unix_name, int pos, const WCHAR *name, i
     DIR *dir;
     struct dirent *de;
     struct stat st;
-    int ret, used_default;
+    int ret;
 
     /* try a shortcut for this directory */
 
     unix_name[pos++] = '/';
-    ret = ntdll_wcstoumbs( 0, name, length, unix_name + pos, MAX_DIR_ENTRY_LEN,
-                           NULL, &used_default );
-    /* if we used the default char, the Unix name won't round trip properly back to Unicode */
-    /* so it cannot match the file we are looking for */
-    if (ret >= 0 && !used_default)
+    ret = ntdll_wcstoumbs( name, length, unix_name + pos, MAX_DIR_ENTRY_LEN + 1, TRUE );
+    if (ret >= 0 && ret <= MAX_DIR_ENTRY_LEN)
     {
         unix_name[pos + ret] = 0;
         if (!stat( unix_name, &st ))
@@ -2106,9 +2099,9 @@ static NTSTATUS find_file_in_dir( char *unix_name, int pos, const WCHAR *name, i
 
                     if (kde[1].d_name[0])
                     {
-                        ret = ntdll_umbstowcs( 0, kde[1].d_name, strlen(kde[1].d_name),
+                        ret = ntdll_umbstowcs( kde[1].d_name, strlen(kde[1].d_name),
                                                buffer, MAX_DIR_ENTRY_LEN );
-                        if (ret == length && !strncmpiW( buffer, name, length))
+                        if (ret == length && !RtlCompareUnicodeStrings( buffer, ret, name, ret, TRUE ))
                         {
                             strcpy( unix_name + pos, kde[1].d_name );
                             RtlLeaveCriticalSection( &dir_section );
@@ -2116,9 +2109,9 @@ static NTSTATUS find_file_in_dir( char *unix_name, int pos, const WCHAR *name, i
                             goto success;
                         }
                     }
-                    ret = ntdll_umbstowcs( 0, kde[0].d_name, strlen(kde[0].d_name),
+                    ret = ntdll_umbstowcs( kde[0].d_name, strlen(kde[0].d_name),
                                            buffer, MAX_DIR_ENTRY_LEN );
-                    if (ret == length && !strncmpiW( buffer, name, length))
+                    if (ret == length && !RtlCompareUnicodeStrings( buffer, ret, name, ret, TRUE ))
                     {
                         strcpy( unix_name + pos,
                                 kde[1].d_name[0] ? kde[1].d_name : kde[0].d_name );
@@ -2151,8 +2144,8 @@ static NTSTATUS find_file_in_dir( char *unix_name, int pos, const WCHAR *name, i
     str.MaximumLength = sizeof(buffer);
     while ((de = readdir( dir )))
     {
-        ret = ntdll_umbstowcs( 0, de->d_name, strlen(de->d_name), buffer, MAX_DIR_ENTRY_LEN );
-        if (ret == length && !strncmpiW( buffer, name, length ))
+        ret = ntdll_umbstowcs( de->d_name, strlen(de->d_name), buffer, MAX_DIR_ENTRY_LEN );
+        if (ret == length && !RtlCompareUnicodeStrings( buffer, ret, name, ret, TRUE ))
         {
             strcpy( unix_name + pos, de->d_name );
             closedir( dir );
@@ -2166,7 +2159,7 @@ static NTSTATUS find_file_in_dir( char *unix_name, int pos, const WCHAR *name, i
         {
             WCHAR short_nameW[12];
             ret = hash_short_file_name( &str, short_nameW );
-            if (ret == length && !strncmpiW( short_nameW, name, length ))
+            if (ret == length && !wcsnicmp( short_nameW, name, length ))
             {
                 strcpy( unix_name + pos, de->d_name );
                 closedir( dir );
@@ -2351,7 +2344,7 @@ static NTSTATUS get_dos_device( const WCHAR *name, UINT name_len, ANSI_STRING *u
     strcat( unix_name, "/dosdevices/" );
     dev = unix_name + strlen(unix_name);
 
-    for (i = 0; i < name_len; i++) dev[i] = (char)tolowerW(name[i]);
+    for (i = 0; i < name_len; i++) dev[i] = (name[i] >= 'A' && name[i] <= 'Z' ? name[i] + 32 : name[i]);
     dev[i] = 0;
 
     /* special case for drive devices */
@@ -2415,7 +2408,7 @@ static inline int get_dos_prefix_len( const UNICODE_STRING *name )
         return ARRAY_SIZE( nt_prefixW );
 
     if (name->Length >= sizeof(dosdev_prefixW) &&
-        !strncmpiW( name->Buffer, dosdev_prefixW, ARRAY_SIZE( dosdev_prefixW )))
+        !wcsnicmp( name->Buffer, dosdev_prefixW, ARRAY_SIZE( dosdev_prefixW )))
         return ARRAY_SIZE( dosdev_prefixW );
 
     return 0;
@@ -2564,26 +2557,25 @@ static NTSTATUS lookup_unix_name( const WCHAR *name, int name_len, char **buffer
                                   UINT disposition, BOOLEAN check_case )
 {
     NTSTATUS status;
-    int ret, used_default, len;
+    int ret, len;
     struct stat st;
     char *unix_name = *buffer;
     const BOOL redirect = nb_redirects && ntdll_get_thread_data()->wow64_redir;
 
     /* try a shortcut first */
 
-    ret = ntdll_wcstoumbs( 0, name, name_len, unix_name + pos, unix_len - pos - 1,
-                           NULL, &used_default );
-
     while (name_len && IS_SEPARATOR(*name))
     {
         name++;
         name_len--;
     }
 
-    if (ret >= 0 && !used_default)  /* if we used the default char the name didn't convert properly */
+    unix_name[pos] = '/';
+    ret = ntdll_wcstoumbs( name, name_len, unix_name + pos + 1, unix_len - pos - 1, TRUE );
+    if (ret >= 0 && ret < unix_len - pos - 1)
     {
         char *p;
-        unix_name[pos + ret] = 0;
+        unix_name[pos + 1 + ret] = 0;
         for (p = unix_name + pos ; *p; p++) if (*p == '\\') *p = '/';
         if (!name_len || !redirect || (!strstr( unix_name, "/windows/") && strncmp( unix_name, "windows/", 8 )))
         {
@@ -2636,9 +2628,8 @@ static NTSTATUS lookup_unix_name( const WCHAR *name, int name_len, char **buffer
                 status = STATUS_OBJECT_NAME_NOT_FOUND;
                 if (disposition != FILE_OPEN && disposition != FILE_OVERWRITE)
                 {
-                    ret = ntdll_wcstoumbs( 0, name, end - name, unix_name + pos + 1,
-                                           MAX_DIR_ENTRY_LEN, NULL, &used_default );
-                    if (ret > 0 && !used_default)
+                    ret = ntdll_wcstoumbs( name, end - name, unix_name + pos + 1, MAX_DIR_ENTRY_LEN + 1, TRUE );
+                    if (ret > 0 && ret <= MAX_DIR_ENTRY_LEN)
                     {
                         unix_name[pos] = '/';
                         unix_name[pos + 1 + ret] = 0;
@@ -2698,8 +2689,7 @@ NTSTATUS nt_to_unix_file_name_attr( const OBJECT_ATTRIBUTES *attr, ANSI_STRING *
     for (p = name; p < name + name_len; p++)
         if (*p < 32 || strchrW( invalid_charsW, *p )) return STATUS_OBJECT_NAME_INVALID;
 
-    unix_len = ntdll_wcstoumbs( 0, name, name_len, NULL, 0, NULL, NULL );
-    unix_len += MAX_DIR_ENTRY_LEN + 3;
+    unix_len = name_len * 3 + MAX_DIR_ENTRY_LEN + 3;
     if (!(unix_name = RtlAllocateHeap( GetProcessHeap(), 0, unix_len )))
         return STATUS_NO_MEMORY;
     unix_name[0] = '.';
@@ -2764,8 +2754,8 @@ NTSTATUS CDECL wine_nt_to_unix_file_name( const UNICODE_STRING *nameW, ANSI_STRI
     const WCHAR *name, *p;
     struct stat st;
     char *unix_name;
-    int pos, ret, name_len, unix_len, prefix_len, used_default;
-    WCHAR prefix[MAX_DIR_ENTRY_LEN];
+    int pos, ret, name_len, unix_len, prefix_len;
+    WCHAR prefix[MAX_DIR_ENTRY_LEN + 1];
     BOOLEAN is_unix = FALSE;
 
     name     = nameW->Buffer;
@@ -2794,8 +2784,10 @@ NTSTATUS CDECL wine_nt_to_unix_file_name( const UNICODE_STRING *nameW, ANSI_STRI
     if (pos == name_len)  /* no subdir, plain DOS device */
         return get_dos_device( name, name_len, unix_name_ret );
 
-    for (prefix_len = 0; prefix_len < pos; prefix_len++)
-        prefix[prefix_len] = tolowerW(name[prefix_len]);
+    prefix_len = pos;
+    memcpy( prefix, name, prefix_len * sizeof(WCHAR) );
+    prefix[prefix_len] = 0;
+    wcslwr( prefix );
 
     name += prefix_len;
     name_len -= prefix_len;
@@ -2814,9 +2806,7 @@ NTSTATUS CDECL wine_nt_to_unix_file_name( const UNICODE_STRING *nameW, ANSI_STRI
             if (*p < 32 || strchrW( invalid_charsW, *p )) return STATUS_OBJECT_NAME_INVALID;
     }
 
-    unix_len = ntdll_wcstoumbs( 0, prefix, prefix_len, NULL, 0, NULL, NULL );
-    unix_len += ntdll_wcstoumbs( 0, name, name_len, NULL, 0, NULL, NULL );
-    unix_len += MAX_DIR_ENTRY_LEN + 3;
+    unix_len = (prefix_len + name_len) * 3 + MAX_DIR_ENTRY_LEN + 3;
     unix_len += strlen(config_dir) + sizeof("/dosdevices/");
     if (!(unix_name = RtlAllocateHeap( GetProcessHeap(), 0, unix_len )))
         return STATUS_NO_MEMORY;
@@ -2824,9 +2814,8 @@ NTSTATUS CDECL wine_nt_to_unix_file_name( const UNICODE_STRING *nameW, ANSI_STRI
     strcat( unix_name, "/dosdevices/" );
     pos = strlen(unix_name);
 
-    ret = ntdll_wcstoumbs( 0, prefix, prefix_len, unix_name + pos, unix_len - pos - 1,
-                           NULL, &used_default );
-    if (!ret || used_default)
+    ret = ntdll_wcstoumbs( prefix, prefix_len, unix_name + pos, unix_len - pos - 1, TRUE );
+    if (ret <= 0)
     {
         RtlFreeHeap( GetProcessHeap(), 0, unix_name );
         return STATUS_OBJECT_NAME_INVALID;
