@@ -144,7 +144,7 @@ static void set_registry_variables( WCHAR **env, HANDLE hkey, ULONG type )
         }
         /* PATH is magic */
         if (env_name.Length == sizeof(pathW) &&
-            !strncmpiW( env_name.Buffer, pathW, ARRAY_SIZE( pathW )) &&
+            !wcsnicmp( env_name.Buffer, pathW, ARRAY_SIZE( pathW )) &&
             !RtlQueryEnvironmentVariable_U( *env, &env_name, &tmp ))
         {
             RtlAppendUnicodeToString( &tmp, sep );
@@ -478,7 +478,7 @@ static WCHAR *build_initial_environment( char **env )
     for (e = env; *e; e++)
     {
         if (is_special_env_var( *e )) continue;
-        size += ntdll_umbstowcs( 0, *e, strlen(*e) + 1, NULL, 0 );
+        size += strlen(*e) + 1;
     }
 
     if (!(ptr = RtlAllocateHeap( GetProcessHeap(), 0, size * sizeof(WCHAR) ))) return NULL;
@@ -498,7 +498,7 @@ static WCHAR *build_initial_environment( char **env )
         }
         else if (is_special_env_var( str )) continue;  /* skip it */
 
-        ntdll_umbstowcs( 0, str, strlen(str) + 1, p, size - (p - ptr) );
+        ntdll_umbstowcs( str, strlen(str) + 1, p, size - (p - ptr) );
         p += strlenW(p) + 1;
     }
     *p = 0;
@@ -522,9 +522,8 @@ char **build_envp( const WCHAR *envW )
     unsigned int i;
 
     lenW = get_env_length( envW );
-    length = ntdll_wcstoumbs( 0, envW, lenW, NULL, 0, NULL, NULL );
-    if (!(env = RtlAllocateHeap( GetProcessHeap(), 0, length ))) return NULL;
-    ntdll_wcstoumbs( 0, envW, lenW, env, length, NULL, NULL );
+    if (!(env = RtlAllocateHeap( GetProcessHeap(), 0, lenW * 3 ))) return NULL;
+    length = ntdll_wcstoumbs( envW, lenW, env, lenW * 3, FALSE );
 
     for (p = env; *p; p += strlen(p) + 1, count++)
         if (is_special_env_var( p )) length += 4; /* prefix it with "WINE" */
@@ -622,7 +621,7 @@ static void get_current_directory( UNICODE_STRING *dir )
         if (!wine_unix_to_nt_file_name( &unix_name, &nt_name ))
         {
             /* skip the \??\ prefix */
-            if (nt_name.Length > 6 * sizeof(WCHAR*) && nt_name.Buffer[5] == ':')
+            if (nt_name.Length > 6 * sizeof(WCHAR) && nt_name.Buffer[5] == ':')
             {
                 dir->Length = nt_name.Length - 4 * sizeof(WCHAR);
                 memcpy( dir->Buffer, nt_name.Buffer + 4, dir->Length );
@@ -663,7 +662,7 @@ static inline BOOL is_path_prefix( const WCHAR *prefix, const WCHAR *path, const
 {
     DWORD len = strlenW( prefix );
 
-    if (strncmpiW( path, prefix, len )) return FALSE;
+    if (wcsnicmp( path, prefix, len )) return FALSE;
     while (path[len] == '\\') len++;
     return path + len == file;
 }
@@ -676,11 +675,10 @@ static void get_image_path( const char *argv0, UNICODE_STRING *path )
 {
     static const WCHAR exeW[] = {'.','e','x','e',0};
     WCHAR *load_path, *file_part, *name, full_name[MAX_PATH];
-    DWORD len;
+    DWORD len = strlen(argv0) + 1;
 
-    len = ntdll_umbstowcs( 0, argv0, strlen(argv0) + 1, NULL, 0 );
     if (!(name = RtlAllocateHeap( GetProcessHeap(), 0, len * sizeof(WCHAR) ))) goto failed;
-    ntdll_umbstowcs( 0, argv0, strlen(argv0) + 1, name, len );
+    ntdll_umbstowcs( argv0, len, name, len );
 
     if (RtlDetermineDosPathNameType_U( name ) != RELATIVE_PATH ||
         strchrW( name, '/' ) || strchrW( name, '\\' ))
@@ -739,8 +737,7 @@ static void set_library_wargv( char **argv, const UNICODE_STRING *image )
     DWORD total = 0;
 
     if (image) total += 1 + image->Length / sizeof(WCHAR);
-    for (argc = (image != NULL); argv[argc]; argc++)
-        total += ntdll_umbstowcs( 0, argv[argc], strlen(argv[argc]) + 1, NULL, 0 );
+    for (argc = (image != NULL); argv[argc]; argc++) total += strlen(argv[argc]) + 1;
 
     wargv = RtlAllocateHeap( GetProcessHeap(), 0,
                              total * sizeof(WCHAR) + (argc + 1) * sizeof(*wargv) );
@@ -754,7 +751,7 @@ static void set_library_wargv( char **argv, const UNICODE_STRING *image )
     }
     for (argc = (image != NULL); argv[argc]; argc++)
     {
-        DWORD reslen = ntdll_umbstowcs( 0, argv[argc], strlen(argv[argc]) + 1, p, total );
+        DWORD reslen = ntdll_umbstowcs( argv[argc], strlen(argv[argc]) + 1, p, total );
         wargv[argc] = p;
         p += reslen;
         total -= reslen;
@@ -906,16 +903,20 @@ NTSTATUS WINAPI RtlDestroyEnvironment(PWSTR env)
 
 static LPCWSTR ENV_FindVariable(PCWSTR var, PCWSTR name, unsigned namelen)
 {
-    for (; *var; var += strlenW(var) + 1)
+    while (*var)
     {
         /* match var names, but avoid setting a var with a name including a '='
          * (a starting '=' is valid though)
          */
-        if (strncmpiW(var, name, namelen) == 0 && var[namelen] == '=' &&
-            strchrW(var + 1, '=') == var + namelen) 
+        unsigned int len = strlenW( var );
+        if (len > namelen &&
+            var[namelen] == '=' &&
+            !RtlCompareUnicodeStrings( var, namelen, name, namelen, TRUE ) &&
+            strchrW(var + 1, '=') == var + namelen)
         {
             return var + namelen + 1;
         }
+        var += len + 1;
     }
     return NULL;
 }
@@ -990,7 +991,7 @@ void WINAPI RtlSetCurrentEnvironment(PWSTR new_env, PWSTR* old_env)
 NTSTATUS WINAPI RtlSetEnvironmentVariable(PWSTR* penv, PUNICODE_STRING name, 
                                           PUNICODE_STRING value)
 {
-    INT         len, old_size;
+    INT varlen, len, old_size;
     LPWSTR      p, env;
     NTSTATUS    nts = STATUS_VARIABLE_NOT_FOUND;
 
@@ -1014,9 +1015,11 @@ NTSTATUS WINAPI RtlSetEnvironmentVariable(PWSTR* penv, PUNICODE_STRING name,
     old_size = get_env_length( env );
 
     /* Find a place to insert the string */
-    for (p = env; *p; p += strlenW(p) + 1)
+    for (p = env; *p; p += varlen + 1)
     {
-        if (!strncmpiW(name->Buffer, p, len) && (p[len] == '=')) break;
+        varlen = strlenW(p);
+        if (varlen > len && p[len] == '=' &&
+            !RtlCompareUnicodeStrings( name->Buffer, len, p, len, TRUE )) break;
     }
     if (!value && !*p) goto done;  /* Value to remove doesn't exist */
 
@@ -1546,5 +1549,6 @@ done:
         RtlInitUnicodeString( &curdir, windows_dir );
         RtlSetCurrentDirectory_U( &curdir );
     }
+    if (!params->CurrentDirectory.Handle) chdir("/"); /* avoid locking removable devices */
     set_wow64_environment( &params->Environment );
 }
