@@ -814,9 +814,9 @@ static char **build_argv( const UNICODE_STRING *cmdlineW, int reserved )
     char *arg, *s, *d, *cmdline;
     int in_quotes, bcount, len;
 
-    len = ntdll_wcstoumbs( 0, cmdlineW->Buffer, cmdlineW->Length / sizeof(WCHAR), NULL, 0, NULL, NULL );
-    if (!(cmdline = RtlAllocateHeap( GetProcessHeap(), 0, len + 1 ))) return NULL;
-    ntdll_wcstoumbs( 0, cmdlineW->Buffer, cmdlineW->Length / sizeof(WCHAR), cmdline, len, NULL, NULL );
+    len = cmdlineW->Length / sizeof(WCHAR);
+    if (!(cmdline = RtlAllocateHeap( GetProcessHeap(), 0, len * 3 + 1 ))) return NULL;
+    len = ntdll_wcstoumbs( cmdlineW->Buffer, len, cmdline, len * 3, FALSE );
     cmdline[len++] = 0;
 
     argc = reserved + 1;
@@ -1244,13 +1244,13 @@ static BOOL is_builtin_path( UNICODE_STRING *path, BOOL *is_64bit )
                                    's','y','s','w','o','w','6','4'};
 
     *is_64bit = is_win64;
-    if (path->Length > sizeof(systemW) && !strncmpiW( path->Buffer, systemW, ARRAY_SIZE(systemW) ))
+    if (path->Length > sizeof(systemW) && !wcsnicmp( path->Buffer, systemW, ARRAY_SIZE(systemW) ))
     {
         if (is_wow64 && !ntdll_get_thread_data()->wow64_redir) *is_64bit = TRUE;
         return TRUE;
     }
     if ((is_win64 || is_wow64) && path->Length > sizeof(wow64W) &&
-        !strncmpiW( path->Buffer, wow64W, ARRAY_SIZE(wow64W) ))
+        !wcsnicmp( path->Buffer, wow64W, ARRAY_SIZE(wow64W) ))
     {
         *is_64bit = FALSE;
         return TRUE;
@@ -1436,9 +1436,9 @@ static ULONG get_env_size( const RTL_USER_PROCESS_PARAMETERS *params, char **win
         static const WCHAR WINEDEBUG[] = {'W','I','N','E','D','E','B','U','G','=',0};
         if (!*winedebug && !strncmpW( ptr, WINEDEBUG, ARRAY_SIZE( WINEDEBUG ) - 1 ))
         {
-            DWORD len = ntdll_wcstoumbs( 0, ptr, strlenW(ptr) + 1, NULL, 0, NULL, NULL );
+            DWORD len = strlenW(ptr) * 3 + 1;
             if ((*winedebug = RtlAllocateHeap( GetProcessHeap(), 0, len )))
-                ntdll_wcstoumbs( 0, ptr, strlenW(ptr) + 1, *winedebug, len, NULL, NULL );
+                ntdll_wcstoumbs( ptr, strlenW(ptr) + 1, *winedebug, len, FALSE );
         }
         ptr += strlenW(ptr) + 1;
     }
@@ -1579,7 +1579,7 @@ NTSTATUS restart_process( RTL_USER_PROCESS_PARAMETERS *params, NTSTATUS status )
     /* check for .com or .pif extension */
     if (status == STATUS_INVALID_IMAGE_NOT_MZ &&
         (p = strrchrW( params->ImagePathName.Buffer, '.' )) &&
-        (!strcmpiW( p, comW ) || !strcmpiW( p, pifW )))
+        (!wcsicmp( p, comW ) || !wcsicmp( p, pifW )))
         status = STATUS_INVALID_IMAGE_WIN_16;
 
     switch (status)
@@ -1709,6 +1709,7 @@ NTSTATUS WINAPI RtlCreateUserProcess( UNICODE_STRING *path, ULONG attributes,
 
     SERVER_START_REQ( new_process )
     {
+        req->parent_process = wine_server_obj_handle(parent);
         req->inherit_all    = inherit;
         req->create_flags   = params->DebugFlags; /* hack: creation flags stored in DebugFlags for now */
         req->socket_fd      = socketfd[1];
