@@ -42,7 +42,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(relay);
 
 struct relay_descr  /* descriptor for a module */
 {
-    void               *magic;               /* signature */
+    ULONG_PTR           magic;               /* signature */
     void               *relay_call;          /* functions to call from relay thunks */
     void               *private;             /* reserved for the relay code private data */
     const char         *entry_point_base;    /* base address of entry point thunks */
@@ -50,7 +50,13 @@ struct relay_descr  /* descriptor for a module */
     const char         *args_string;         /* string describing the arguments */
 };
 
-#define RELAY_DESCR_MAGIC  ((void *)0xdeb90002)
+struct relay_descr_rva  /* RVA to the descriptor for PE dlls */
+{
+    DWORD magic;
+    DWORD descr;
+};
+
+#define RELAY_DESCR_MAGIC  0xdeb90002
 #define IS_INTARG(x)       (((ULONG_PTR)(x) >> 16) == 0)
 
 /* private data built at dll load time */
@@ -87,15 +93,6 @@ static inline int strcmpAW( const char *strA, const WCHAR *strW )
     return (unsigned char)*strA - *strW;
 }
 
-/* compare an ASCII and a Unicode string without depending on the current codepage */
-static inline int strncmpiAW( const char *strA, const WCHAR *strW, int n )
-{
-    int ret = 0;
-    for ( ; n > 0; n--, strA++, strW++)
-        if ((ret = toupperW((unsigned char)*strA) - toupperW(*strW)) || !*strA) break;
-    return ret;
-}
-
 /***********************************************************************
  *           build_list
  *
@@ -219,7 +216,7 @@ static DWORD WINAPI init_debug_lists( RTL_RUN_ONCE *once, void *param, void **co
  *
  * Check if a given module and function is in the list.
  */
-static BOOL check_list( const char *module, int ordinal, const char *func, const WCHAR *const *list )
+static BOOL check_list( const WCHAR *module, int ordinal, const char *func, const WCHAR *const *list )
 {
     char ord_str[10];
 
@@ -230,7 +227,7 @@ static BOOL check_list( const char *module, int ordinal, const char *func, const
         if (p && p > *list)  /* check module and function */
         {
             int len = p - *list;
-            if (strncmpiAW( module, *list, len-1 ) || module[len]) continue;
+            if (wcsnicmp( module, *list, len - 1 ) || module[len]) continue;
             if (p[1] == '*' && !p[2]) return TRUE;
             if (!strcmpAW( ord_str, p + 1 )) return TRUE;
             if (func && !strcmpAW( func, p + 1 )) return TRUE;
@@ -249,7 +246,7 @@ static BOOL check_list( const char *module, int ordinal, const char *func, const
  *
  * Check if a given function must be included in the relay output.
  */
-static BOOL check_relay_include( const char *module, int ordinal, const char *func )
+static BOOL check_relay_include( const WCHAR *module, int ordinal, const char *func )
 {
     if (debug_relay_excludelist && check_list( module, ordinal, func, debug_relay_excludelist ))
         return FALSE;
@@ -286,9 +283,9 @@ static BOOL check_from_module( const WCHAR **includelist, const WCHAR **excludel
     {
         int len;
 
-        if (!strcmpiW( *listitem, module )) return !show;
+        if (!wcsicmp( *listitem, module )) return !show;
         len = strlenW( *listitem );
-        if (!strncmpiW( *listitem, module, len ) && !strcmpiW( module + len, dllW ))
+        if (!wcsnicmp( *listitem, module, len ) && !wcsicmp( module + len, dllW ))
             return !show;
     }
     return show;
@@ -771,9 +768,12 @@ DECLSPEC_HIDDEN void WINAPI relay_trace_exit( struct relay_descr *descr, unsigne
 extern INT_PTR WINAPI relay_call( struct relay_descr *descr, unsigned int idx, const INT_PTR *stack );
 __ASM_GLOBAL_FUNC( relay_call,
                    "pushq %rbp\n\t"
+                   __ASM_SEH(".seh_pushreg %rbp\n\t")
                    __ASM_CFI(".cfi_adjust_cfa_offset 8\n\t")
                    __ASM_CFI(".cfi_rel_offset %rbp,0\n\t")
                    "movq %rsp,%rbp\n\t"
+                   __ASM_SEH(".seh_setframe %rbp,0\n\t")
+                   __ASM_SEH(".seh_endprologue\n\t")
                    __ASM_CFI(".cfi_def_cfa_register %rbp\n\t")
                    "leaq -0x48(%rbp),%rsp\n\t"
                    "andq $~15,%rsp\n\t"
@@ -825,7 +825,7 @@ __ASM_GLOBAL_FUNC( relay_call,
                    __ASM_CFI(".cfi_same_value %rsi\n\t")
                    "movq -8(%rbp),%rdi\n\t"
                    __ASM_CFI(".cfi_same_value %rdi\n\t")
-                   "movq %rbp,%rsp\n\t"
+                   "leaq 0(%rbp),%rsp\n\t"
                    __ASM_CFI(".cfi_def_cfa_register %rsp\n\t")
                    "popq %rbp\n\t"
                    __ASM_CFI(".cfi_adjust_cfa_offset -8\n\t")
@@ -837,6 +837,26 @@ __ASM_GLOBAL_FUNC( relay_call,
 #endif
 
 
+static struct relay_descr *get_relay_descr( HMODULE module, const IMAGE_EXPORT_DIRECTORY *exports,
+                                            DWORD exp_size )
+{
+    struct relay_descr *descr;
+    struct relay_descr_rva *rva;
+    ULONG_PTR ptr = (ULONG_PTR)module + exports->Name;
+
+    /* sanity checks */
+    if (ptr <= (ULONG_PTR)(exports + 1)) return NULL;
+    if (ptr > (ULONG_PTR)exports + exp_size) return NULL;
+    if (ptr % sizeof(DWORD)) return NULL;
+
+    rva = (struct relay_descr_rva *)ptr - 1;
+    if (rva->magic != RELAY_DESCR_MAGIC) return NULL;
+    if (rva->descr) descr = (struct relay_descr *)((char *)module + rva->descr);
+    else descr = (struct relay_descr *)((const char *)exports + exp_size);
+    if (descr->magic != RELAY_DESCR_MAGIC) return NULL;
+    return descr;
+}
+
 /***********************************************************************
  *           RELAY_GetProcAddress
  *
@@ -846,9 +866,9 @@ FARPROC RELAY_GetProcAddress( HMODULE module, const IMAGE_EXPORT_DIRECTORY *expo
                               DWORD exp_size, FARPROC proc, DWORD ordinal, const WCHAR *user )
 {
     struct relay_private_data *data;
-    const struct relay_descr *descr = (const struct relay_descr *)((const char *)exports + exp_size);
+    const struct relay_descr *descr = get_relay_descr( module, exports, exp_size );
 
-    if (descr->magic != RELAY_DESCR_MAGIC || !(data = descr->private)) return proc;  /* no relay data */
+    if (!descr || !(data = descr->private)) return proc;  /* no relay data */
     if (!data->entry_points[ordinal].orig_func) return proc;  /* not a relayed function */
     if (check_from_module( debug_from_relay_includelist, debug_from_relay_excludelist, user ))
         return proc;  /* we want to relay it */
@@ -866,18 +886,20 @@ void RELAY_SetupDLL( HMODULE module )
     IMAGE_EXPORT_DIRECTORY *exports;
     DWORD *funcs;
     unsigned int i, len;
-    DWORD size, entry_point_rva;
+    DWORD size, entry_point_rva, old_prot;
     struct relay_descr *descr;
     struct relay_private_data *data;
+    WCHAR dllnameW[sizeof(data->dllname)];
     const WORD *ordptr;
+    void *func_base;
+    SIZE_T func_size;
 
     RtlRunOnceExecuteOnce( &init_once, init_debug_lists, NULL, NULL );
 
     exports = RtlImageDirectoryEntryToData( module, TRUE, IMAGE_DIRECTORY_ENTRY_EXPORT, &size );
     if (!exports) return;
 
-    descr = (struct relay_descr *)((char *)exports + size);
-    if (descr->magic != RELAY_DESCR_MAGIC) return;
+    if (!(descr = get_relay_descr( module, exports, size ))) return;
 
     if (!(data = RtlAllocateHeap( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*data) +
                                   (exports->NumberOfFunctions-1) * sizeof(data->entry_points) )))
@@ -893,6 +915,7 @@ void RELAY_SetupDLL( HMODULE module )
     len = min( len, sizeof(data->dllname) - 1 );
     memcpy( data->dllname, (char *)module + exports->Name, len );
     data->dllname[len] = 0;
+    ascii_to_unicode( dllnameW, data->dllname, len + 1 );
 
     /* fetch name pointer for all entry points and store them in the private structure */
 
@@ -907,15 +930,21 @@ void RELAY_SetupDLL( HMODULE module )
 
     funcs = (DWORD *)((char *)module + exports->AddressOfFunctions);
     entry_point_rva = descr->entry_point_base - (const char *)module;
+
+    func_base = funcs;
+    func_size = exports->NumberOfFunctions * sizeof(*funcs);
+    NtProtectVirtualMemory( NtCurrentProcess(), &func_base, &func_size, PAGE_READWRITE, &old_prot );
     for (i = 0; i < exports->NumberOfFunctions; i++, funcs++)
     {
         if (!descr->entry_point_offsets[i]) continue;   /* not a normal function */
-        if (!check_relay_include( data->dllname, i + exports->Base, data->entry_points[i].name ))
+        if (!check_relay_include( dllnameW, i + exports->Base, data->entry_points[i].name ))
             continue;  /* don't include this entry point */
 
         data->entry_points[i].orig_func = (char *)module + *funcs;
         *funcs = entry_point_rva + descr->entry_point_offsets[i];
     }
+    if (old_prot != PAGE_READWRITE)
+        NtProtectVirtualMemory( NtCurrentProcess(), &func_base, &func_size, old_prot, &old_prot );
 }
 
 #else  /* __i386__ || __x86_64__ || __arm__ || __aarch64__ */
@@ -999,9 +1028,14 @@ static SNOOP_RETURNENTRIES *firstrets;
  */
 static BOOL SNOOP_ShowDebugmsgSnoop(const char *module, int ordinal, const char *func)
 {
-    if (debug_snoop_excludelist && check_list( module, ordinal, func, debug_snoop_excludelist ))
+    WCHAR moduleW[40];
+    int len = strlen(module);
+
+    if (len >= ARRAY_SIZE( moduleW )) return FALSE;
+    ascii_to_unicode( moduleW, module, len + 1 );
+    if (debug_snoop_excludelist && check_list( moduleW, ordinal, func, debug_snoop_excludelist ))
         return FALSE;
-    if (debug_snoop_includelist && !check_list( module, ordinal, func, debug_snoop_includelist ))
+    if (debug_snoop_includelist && !check_list( moduleW, ordinal, func, debug_snoop_includelist ))
         return FALSE;
     return TRUE;
 }
@@ -1320,7 +1354,7 @@ void WINAPI DECLSPEC_HIDDEN __regs_SNOOP_Return( void **stack )
                         "leal 12(%esp),%eax\n\t"                        \
                         "pushl %eax\n\t"                                \
                         __ASM_CFI(".cfi_adjust_cfa_offset 4\n\t")       \
-                        "call " __ASM_NAME("__regs_" #name) __ASM_STDCALL(4) "\n\t" \
+                        "call " __ASM_STDCALL("__regs_" #name,4) "\n\t" \
                         __ASM_CFI(".cfi_adjust_cfa_offset -4\n\t")      \
                         "popl %edx\n\t"                                 \
                         __ASM_CFI(".cfi_adjust_cfa_offset -4\n\t")      \
