@@ -2143,8 +2143,8 @@ void WINAPI RtlReleaseSRWLockShared( RTL_SRWLOCK *lock )
  *              RtlTryAcquireSRWLockExclusive (NTDLL.@)
  *
  * NOTES
- *  Similar to AcquireSRWLockExclusive recusive calls are not allowed
- *  and will fail with return value FALSE.
+ *  Similarly to AcquireSRWLockExclusive, recursive calls are not allowed
+ *  and will fail with a FALSE return value.
  */
 BOOLEAN WINAPI RtlTryAcquireSRWLockExclusive( RTL_SRWLOCK *lock )
 {
@@ -2457,6 +2457,7 @@ NTSTATUS WINAPI RtlWaitOnAddress( const void *addr, const void *cmp, SIZE_T size
     apc_call_t call;
     apc_result_t result;
     timeout_t abs_timeout = timeout ? timeout->QuadPart : TIMEOUT_INFINITE;
+    sigset_t old_set;
 
     if (size != 1 && size != 2 && size != 4 && size != 8)
         return STATUS_INVALID_PARAMETER;
@@ -2470,7 +2471,7 @@ NTSTATUS WINAPI RtlWaitOnAddress( const void *addr, const void *cmp, SIZE_T size
 
     memset( &result, 0, sizeof(result) );
 
-    for (;;)
+    do
     {
         RtlEnterCriticalSection( &addr_section );
         if (!compare_addr( addr, cmp, size ))
@@ -2479,34 +2480,43 @@ NTSTATUS WINAPI RtlWaitOnAddress( const void *addr, const void *cmp, SIZE_T size
             return STATUS_SUCCESS;
         }
 
-        SERVER_START_REQ( select )
+        pthread_sigmask( SIG_BLOCK, &server_block_set, &old_set );
+        for (;;)
         {
-            req->flags    = SELECT_INTERRUPTIBLE;
-            req->cookie   = wine_server_client_ptr( &cookie );
-            req->prev_apc = apc_handle;
-            req->timeout  = abs_timeout;
-            wine_server_add_data( req, &result, sizeof(result) );
-            wine_server_add_data( req, &select_op, sizeof(select_op.keyed_event) );
-            ret = wine_server_call( req );
-            abs_timeout = reply->timeout;
-            apc_handle  = reply->apc_handle;
-            call        = reply->call;
+            SERVER_START_REQ( select )
+            {
+                req->flags    = SELECT_INTERRUPTIBLE;
+                req->cookie   = wine_server_client_ptr( &cookie );
+                req->prev_apc = apc_handle;
+                req->timeout  = abs_timeout;
+                wine_server_add_data( req, &result, sizeof(result) );
+                wine_server_add_data( req, &select_op, sizeof(select_op.keyed_event) );
+                ret = server_call_unlocked( req );
+                abs_timeout = reply->timeout;
+                apc_handle  = reply->apc_handle;
+                call        = reply->call;
+            }
+            SERVER_END_REQ;
+
+            if (ret != STATUS_KERNEL_APC) break;
+            invoke_apc( &call, &result );
         }
-        SERVER_END_REQ;
+        pthread_sigmask( SIG_SETMASK, &old_set, NULL );
 
         RtlLeaveCriticalSection( &addr_section );
 
-        if (ret == STATUS_PENDING) ret = wait_select_reply( &cookie );
-        if (ret != STATUS_USER_APC) break;
-        if (invoke_apc( &call, &result ))
+        if (ret == STATUS_USER_APC)
         {
+            invoke_apc( &call, &result );
             /* if we ran a user apc we have to check once more if additional apcs are queued,
              * but we don't want to wait */
             abs_timeout = 0;
             user_apc = TRUE;
-            size = 0;
         }
+
+        if (ret == STATUS_PENDING) ret = wait_select_reply( &cookie );
     }
+    while (ret == STATUS_USER_APC || ret == STATUS_KERNEL_APC);
 
     if (ret == STATUS_TIMEOUT && user_apc) ret = STATUS_USER_APC;
 
