@@ -18,6 +18,7 @@
 
 
 #include <stdarg.h>
+#include <stdlib.h>
 
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
@@ -28,7 +29,6 @@
 #include "winternl.h"
 #include "powrprof.h"
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(powrprof);
 
@@ -178,7 +178,7 @@ BOOLEAN WINAPI GetPwrDiskSpindownRange(PUINT RangeMax, PUINT RangeMin)
       TRACE("Using default: 3600\n");
       *RangeMax = 3600;
    } else {
-      *RangeMax = atoiW((LPCWSTR)lpValue);
+      *RangeMax = wcstol((LPCWSTR)lpValue, NULL, 10);
    }
 
    cbValue = sizeof(lpValue);
@@ -189,7 +189,7 @@ BOOLEAN WINAPI GetPwrDiskSpindownRange(PUINT RangeMax, PUINT RangeMin)
       TRACE("Using default: 3\n");
       *RangeMin = 3;
    } else {
-      *RangeMin = atoiW((LPCWSTR)lpValue);
+      *RangeMin = wcstol((LPCWSTR)lpValue, NULL, 10);
    }
 
    RegCloseKey(hKey);
@@ -315,6 +315,14 @@ DWORD WINAPI PowerReadDCValue(HKEY RootPowerKey, const GUID *Scheme, const GUID
    return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
+DWORD WINAPI PowerReadFriendlyName(HKEY RootPowerKey, const GUID *Scheme,
+	const GUID *SubGroup, const GUID *PowerSettings, UCHAR *Buffer,
+	DWORD *BufferSize)
+{
+   FIXME("(%p,%s,%s,%s,%p,%p) stub!\n", RootPowerKey, debugstr_guid(Scheme), debugstr_guid(SubGroup), debugstr_guid(PowerSettings), Buffer, BufferSize);
+   return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
 POWER_PLATFORM_ROLE WINAPI PowerDeterminePlatformRole(void)
 {
    FIXME("stub\n");
