@@ -49,24 +49,6 @@ static inline AudioRecord *impl_from_IPersistPropertyBag(IPersistPropertyBag *if
     return CONTAINING_RECORD(iface, AudioRecord, IPersistPropertyBag_iface);
 }
 
-static const IBaseFilterVtbl AudioRecordVtbl = {
-    BaseFilterImpl_QueryInterface,
-    BaseFilterImpl_AddRef,
-    BaseFilterImpl_Release,
-    BaseFilterImpl_GetClassID,
-    BaseFilterImpl_Stop,
-    BaseFilterImpl_Pause,
-    BaseFilterImpl_Run,
-    BaseFilterImpl_GetState,
-    BaseFilterImpl_SetSyncSource,
-    BaseFilterImpl_GetSyncSource,
-    BaseFilterImpl_EnumPins,
-    BaseFilterImpl_FindPin,
-    BaseFilterImpl_QueryFilterInfo,
-    BaseFilterImpl_JoinFilterGraph,
-    BaseFilterImpl_QueryVendorInfo
-};
-
 static struct strmbase_pin *audio_record_get_pin(struct strmbase_filter *iface, unsigned int index)
 {
     FIXME("iface %p, index %u, stub!\n", iface, index);
@@ -172,22 +154,18 @@ static const IPersistPropertyBagVtbl PersistPropertyBagVtbl =
     PPB_Save
 };
 
-IUnknown* WINAPI QCAP_createAudioCaptureFilter(IUnknown *outer, HRESULT *phr)
+HRESULT audio_record_create(IUnknown *outer, IUnknown **out)
 {
-    AudioRecord *This = NULL;
+    AudioRecord *object;
 
-    FIXME("(%p, %p): the entire CLSID_AudioRecord implementation is just stubs\n", outer, phr);
+    if (!(object = CoTaskMemAlloc(sizeof(*object))))
+        return E_OUTOFMEMORY;
+    memset(object, 0, sizeof(*object));
 
-    This = CoTaskMemAlloc(sizeof(*This));
-    if (This == NULL) {
-        *phr = E_OUTOFMEMORY;
-        return NULL;
-    }
-    memset(This, 0, sizeof(*This));
-    This->IPersistPropertyBag_iface.lpVtbl = &PersistPropertyBagVtbl;
+    object->IPersistPropertyBag_iface.lpVtbl = &PersistPropertyBagVtbl;
+    strmbase_filter_init(&object->filter, outer, &CLSID_AudioRecord, &filter_ops);
 
-    strmbase_filter_init(&This->filter, &AudioRecordVtbl, outer, &CLSID_AudioRecord, &filter_ops);
-
-    *phr = S_OK;
-    return &This->filter.IUnknown_inner;
+    TRACE("Created audio recorder %p.\n", object);
+    *out = &object->filter.IUnknown_inner;
+    return S_OK;
 }
