@@ -62,11 +62,6 @@ static inline VfwCapture *impl_from_strmbase_filter(struct strmbase_filter *ifac
     return CONTAINING_RECORD(iface, VfwCapture, filter);
 }
 
-static inline VfwCapture *impl_from_IBaseFilter(IBaseFilter *iface)
-{
-    return CONTAINING_RECORD(iface, VfwCapture, filter.IBaseFilter_iface);
-}
-
 static inline VfwCapture *impl_from_IAMStreamConfig(IAMStreamConfig *iface)
 {
     return CONTAINING_RECORD(iface, VfwCapture, IAMStreamConfig_iface);
@@ -99,7 +94,7 @@ static void vfw_capture_destroy(struct strmbase_filter *iface)
     if (filter->init)
     {
         if (filter->filter.state != State_Stopped)
-            qcap_driver_stop(filter->driver_info, &filter->filter.state);
+            qcap_driver_stop_stream(filter->driver_info);
         qcap_driver_destroy(filter->driver_info);
     }
 
@@ -129,55 +124,47 @@ static HRESULT vfw_capture_query_interface(struct strmbase_filter *iface, REFIID
     return S_OK;
 }
 
+static HRESULT vfw_capture_init_stream(struct strmbase_filter *iface)
+{
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
+
+    qcap_driver_init_stream(filter->driver_info);
+    return VFW_S_CANT_CUE;
+}
+
+static HRESULT vfw_capture_start_stream(struct strmbase_filter *iface, REFERENCE_TIME time)
+{
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
+
+    qcap_driver_start_stream(filter->driver_info);
+    return S_OK;
+}
+
+static HRESULT vfw_capture_stop_stream(struct strmbase_filter *iface)
+{
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
+
+    qcap_driver_stop_stream(filter->driver_info);
+    return VFW_S_CANT_CUE;
+}
+
+static HRESULT vfw_capture_cleanup_stream(struct strmbase_filter *iface)
+{
+    VfwCapture *filter = impl_from_strmbase_filter(iface);
+
+    qcap_driver_cleanup_stream(filter->driver_info);
+    return S_OK;
+}
+
 static const struct strmbase_filter_ops filter_ops =
 {
     .filter_get_pin = vfw_capture_get_pin,
     .filter_destroy = vfw_capture_destroy,
     .filter_query_interface = vfw_capture_query_interface,
-};
-
-/** IMediaFilter methods **/
-
-static HRESULT WINAPI VfwCapture_Stop(IBaseFilter * iface)
-{
-    VfwCapture *This = impl_from_IBaseFilter(iface);
-
-    TRACE("()\n");
-    return qcap_driver_stop(This->driver_info, &This->filter.state);
-}
-
-static HRESULT WINAPI VfwCapture_Pause(IBaseFilter * iface)
-{
-    VfwCapture *This = impl_from_IBaseFilter(iface);
-
-    TRACE("()\n");
-    return qcap_driver_pause(This->driver_info, &This->filter.state);
-}
-
-static HRESULT WINAPI VfwCapture_Run(IBaseFilter * iface, REFERENCE_TIME tStart)
-{
-    VfwCapture *This = impl_from_IBaseFilter(iface);
-    TRACE("(%s)\n", wine_dbgstr_longlong(tStart));
-    return qcap_driver_run(This->driver_info, &This->filter.state);
-}
-
-static const IBaseFilterVtbl VfwCapture_Vtbl =
-{
-    BaseFilterImpl_QueryInterface,
-    BaseFilterImpl_AddRef,
-    BaseFilterImpl_Release,
-    BaseFilterImpl_GetClassID,
-    VfwCapture_Stop,
-    VfwCapture_Pause,
-    VfwCapture_Run,
-    BaseFilterImpl_GetState,
-    BaseFilterImpl_SetSyncSource,
-    BaseFilterImpl_GetSyncSource,
-    BaseFilterImpl_EnumPins,
-    BaseFilterImpl_FindPin,
-    BaseFilterImpl_QueryFilterInfo,
-    BaseFilterImpl_JoinFilterGraph,
-    BaseFilterImpl_QueryVendorInfo
+    .filter_init_stream = vfw_capture_init_stream,
+    .filter_start_stream = vfw_capture_start_stream,
+    .filter_stop_stream = vfw_capture_stop_stream,
+    .filter_cleanup_stream = vfw_capture_cleanup_stream,
 };
 
 /* AMStreamConfig interface, we only need to implement {G,S}etFormat */
@@ -229,9 +216,9 @@ AMStreamConfig_SetFormat(IAMStreamConfig *iface, AM_MEDIA_TYPE *pmt)
     }
 
     hr = qcap_driver_set_format(This->driver_info, pmt);
-    if (SUCCEEDED(hr) && This->filter.filterInfo.pGraph && This->source.pin.peer)
+    if (SUCCEEDED(hr) && This->filter.graph && This->source.pin.peer)
     {
-        hr = IFilterGraph_Reconnect(This->filter.filterInfo.pGraph, &This->source.pin.IPin_iface);
+        hr = IFilterGraph_Reconnect(This->filter.graph, &This->source.pin.IPin_iface);
         if (SUCCEEDED(hr))
             TRACE("Reconnection completed, with new media format..\n");
     }
@@ -576,53 +563,27 @@ static const struct strmbase_source_ops source_ops =
     .pfnDecideAllocator = BaseOutputPinImpl_DecideAllocator,
 };
 
-static const IPinVtbl VfwPin_Vtbl =
-{
-    BasePinImpl_QueryInterface,
-    BasePinImpl_AddRef,
-    BasePinImpl_Release,
-    BaseOutputPinImpl_Connect,
-    BaseOutputPinImpl_ReceiveConnection,
-    BaseOutputPinImpl_Disconnect,
-    BasePinImpl_ConnectedTo,
-    BasePinImpl_ConnectionMediaType,
-    BasePinImpl_QueryPinInfo,
-    BasePinImpl_QueryDirection,
-    BasePinImpl_QueryId,
-    BasePinImpl_QueryAccept,
-    BasePinImpl_EnumMediaTypes,
-    BasePinImpl_QueryInternalConnections,
-    BaseOutputPinImpl_EndOfStream,
-    BaseOutputPinImpl_BeginFlush,
-    BaseOutputPinImpl_EndFlush,
-    BasePinImpl_NewSegment
-};
-
-IUnknown * WINAPI QCAP_createVFWCaptureFilter(IUnknown *outer, HRESULT *phr)
+HRESULT vfw_capture_create(IUnknown *outer, IUnknown **out)
 {
     static const WCHAR source_name[] = {'O','u','t','p','u','t',0};
     VfwCapture *object;
 
     if (!(object = CoTaskMemAlloc(sizeof(*object))))
-    {
-        *phr = E_OUTOFMEMORY;
-        return NULL;
-    }
+        return E_OUTOFMEMORY;
 
-    strmbase_filter_init(&object->filter, &VfwCapture_Vtbl, outer, &CLSID_VfwCapture, &filter_ops);
+    strmbase_filter_init(&object->filter, outer, &CLSID_VfwCapture, &filter_ops);
 
     object->IAMStreamConfig_iface.lpVtbl = &IAMStreamConfig_VTable;
     object->IAMVideoProcAmp_iface.lpVtbl = &IAMVideoProcAmp_VTable;
     object->IPersistPropertyBag_iface.lpVtbl = &IPersistPropertyBag_VTable;
     object->init = FALSE;
 
-    strmbase_source_init(&object->source, &VfwPin_Vtbl, &object->filter,
-            source_name, &source_ops);
+    strmbase_source_init(&object->source, &object->filter, source_name, &source_ops);
 
     object->IKsPropertySet_iface.lpVtbl = &IKsPropertySet_VTable;
 
     TRACE("Created VFW capture filter %p.\n", object);
     ObjectRefCount(TRUE);
-    *phr = S_OK;
-    return &object->filter.IUnknown_inner;
+    *out = &object->filter.IUnknown_inner;
+    return S_OK;
 }
