@@ -482,8 +482,7 @@ static void test_media_types(void)
     IBaseFilter_FindPin(filter, L"Out", &source);
 
     hr = IPin_EnumMediaTypes(sink, &enummt);
-    todo_wine ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
-    if (hr == S_OK) IEnumMediaTypes_Release(enummt);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
 
     hr = IPin_EnumMediaTypes(source, &enummt);
     ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
@@ -497,8 +496,7 @@ static void test_media_types(void)
     ok(hr == S_OK, "Got hr %#x.\n", hr);
 
     hr = IPin_EnumMediaTypes(sink, &enummt);
-    todo_wine ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
-    if (hr == S_OK) IEnumMediaTypes_Release(enummt);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
 
     hr = IPin_EnumMediaTypes(source, &enummt);
     ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
@@ -513,13 +511,13 @@ static void test_media_types(void)
     hr = ISampleGrabber_SetMediaType(grabber, &match_mt);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(sink, &mt);
-    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(source, &mt);
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
 
     mt.majortype = GUID_NULL;
     hr = IPin_QueryAccept(sink, &mt);
-    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(source, &mt);
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     mt.majortype = match_mt.majortype;
@@ -533,13 +531,13 @@ static void test_media_types(void)
     hr = ISampleGrabber_SetMediaType(grabber, &match_mt);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(sink, &mt);
-    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(source, &mt);
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
 
     mt.subtype = GUID_NULL;
     hr = IPin_QueryAccept(sink, &mt);
-    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(source, &mt);
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     mt.subtype = match_mt.subtype;
@@ -552,13 +550,13 @@ static void test_media_types(void)
     hr = ISampleGrabber_SetMediaType(grabber, &match_mt);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(sink, &mt);
-    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(source, &mt);
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
 
     mt.formattype = GUID_NULL;
     hr = IPin_QueryAccept(sink, &mt);
-    todo_wine ok(hr == S_FALSE, "Got hr %#x.\n", hr);
+    ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     hr = IPin_QueryAccept(source, &mt);
     ok(hr == S_FALSE, "Got hr %#x.\n", hr);
     mt.formattype = match_mt.formattype;
@@ -824,8 +822,7 @@ static void test_connect_pin(void)
     ok(compare_media_types(&mt, &req_mt), "Media types didn't match.\n");
 
     hr = IPin_EnumMediaTypes(sink, &enummt);
-    todo_wine ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
-    if (hr == S_OK) IEnumMediaTypes_Release(enummt);
+    ok(hr == VFW_E_NOT_CONNECTED, "Got hr %#x.\n", hr);
     hr = IPin_EnumMediaTypes(source, &enummt);
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     hr = IEnumMediaTypes_Next(enummt, 1, &pmt, NULL);
@@ -1000,6 +997,14 @@ static void test_connect_pin(void)
     ok(hr == S_OK, "Got hr %#x.\n", hr);
     ok(compare_media_types(&testsink.sink.pin.mt, &req_mt), "Media types didn't match.\n");
     ok(compare_media_types(&testsource.source.pin.mt, &testsink.sink.pin.mt), "Media types didn't match.\n");
+
+    hr = IPin_EnumMediaTypes(sink, &enummt);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    hr = IEnumMediaTypes_Next(enummt, 1, &pmt, NULL);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(compare_media_types(pmt, testsink.sink_mt), "Media types didn't match.\n");
+    IEnumMediaTypes_Release(enummt);
+
     IFilterGraph2_Disconnect(graph, source);
     IFilterGraph2_Disconnect(graph, &testsink.sink.pin.IPin_iface);
 
