@@ -503,6 +503,8 @@ static void dsound_render_destroy(struct strmbase_renderer *iface)
 
     strmbase_renderer_cleanup(&filter->renderer);
     CoTaskMemFree(filter);
+
+    InterlockedDecrement(&object_locks);
 }
 
 static HRESULT dsound_render_query_interface(struct strmbase_renderer *iface, REFIID iid, void **out)
@@ -808,7 +810,7 @@ static const IAMDirectSoundVtbl IAMDirectSound_Vtbl =
     AMDirectSound_GetFocusWindow
 };
 
-HRESULT dsound_render_create(IUnknown *outer, void **out)
+HRESULT dsound_render_create(IUnknown *outer, IUnknown **out)
 {
     static const DSBUFFERDESC buffer_desc = {
         .dwSize = sizeof(DSBUFFERDESC),
@@ -830,8 +832,7 @@ HRESULT dsound_render_create(IUnknown *outer, void **out)
         return hr;
     }
 
-    if (FAILED(hr = QUARTZ_CreateSystemClock(&object->renderer.filter.IUnknown_inner,
-            (void **)&object->system_clock)))
+    if (FAILED(hr = system_clock_create(&object->renderer.filter.IUnknown_inner, &object->system_clock)))
     {
         strmbase_renderer_cleanup(&object->renderer);
         CoTaskMemFree(object);
