@@ -27,6 +27,13 @@
 #include "initguid.h"
 #include "wine/fil_data.h"
 
+static ULONG get_refcount(void *iface)
+{
+    IUnknown *unknown = iface;
+    IUnknown_AddRef(unknown);
+    return IUnknown_Release(unknown);
+}
+
 /* Helper function, checks if filter with given name was enumerated. */
 static BOOL enum_find_filter(const WCHAR *wszFilterName, IEnumMoniker *pEnum)
 {
@@ -34,7 +41,6 @@ static BOOL enum_find_filter(const WCHAR *wszFilterName, IEnumMoniker *pEnum)
     BOOL found = FALSE;
     ULONG nb;
     HRESULT hr;
-    static const WCHAR wszFriendlyName[] = {'F','r','i','e','n','d','l','y','N','a','m','e',0};
 
     while(!found && IEnumMoniker_Next(pEnum, 1, &pMoniker, &nb) == S_OK)
     {
@@ -46,10 +52,10 @@ static BOOL enum_find_filter(const WCHAR *wszFilterName, IEnumMoniker *pEnum)
         hr = IMoniker_BindToStorage(pMoniker, NULL, NULL, &IID_IPropertyBag, (LPVOID*)&pPropBagCat);
         ok(SUCCEEDED(hr), "IMoniker_BindToStorage failed with %x\n", hr);
 
-        hr = IPropertyBag_Read(pPropBagCat, wszFriendlyName, &var, NULL);
+        hr = IPropertyBag_Read(pPropBagCat, L"FriendlyName", &var, NULL);
         ok(SUCCEEDED(hr), "IPropertyBag_Read failed with %x\n", hr);
 
-        if (!lstrcmpW(V_BSTR(&var), wszFilterName))
+        if (!wcscmp(V_BSTR(&var), wszFilterName))
             found = TRUE;
 
         IPropertyBag_Release(pPropBagCat);
@@ -69,8 +75,6 @@ static void test_fm2_enummatchingfilters(void)
     REGFILTER2 rgf2;
     REGFILTERPINS2 rgPins2[2];
     REGPINTYPES rgPinType;
-    static const WCHAR wszFilterName1[] = {'T', 'e', 's', 't', 'f', 'i', 'l', 't', 'e', 'r', '1', 0 };
-    static const WCHAR wszFilterName2[] = {'T', 'e', 's', 't', 'f', 'i', 'l', 't', 'e', 'r', '2', 0 };
     CLSID clsidFilter1;
     CLSID clsidFilter2;
     IEnumMoniker *pEnum = NULL;
@@ -107,7 +111,7 @@ static void test_fm2_enummatchingfilters(void)
     rgPinType.clsMajorType = &GUID_NULL;
     rgPinType.clsMinorType = &GUID_NULL;
 
-    hr = IFilterMapper2_RegisterFilter(pMapper, &clsidFilter1, wszFilterName1, NULL,
+    hr = IFilterMapper2_RegisterFilter(pMapper, &clsidFilter1, L"Testfilter1", NULL,
                     &CLSID_LegacyAmFilterCategory, NULL, &rgf2);
     if (hr == E_ACCESSDENIED)
     {
@@ -130,7 +134,7 @@ static void test_fm2_enummatchingfilters(void)
 
         S2(U(rgf2)).cPins2 = 2;
 
-        hr = IFilterMapper2_RegisterFilter(pMapper, &clsidFilter2, wszFilterName2, NULL,
+        hr = IFilterMapper2_RegisterFilter(pMapper, &clsidFilter2, L"Testfilter2", NULL,
                     &CLSID_LegacyAmFilterCategory, NULL, &rgf2);
         ok(hr == S_OK, "IFilterMapper2_RegisterFilter failed with %x\n", hr);
 
@@ -139,7 +143,7 @@ static void test_fm2_enummatchingfilters(void)
         ok(hr == S_OK, "IFilterMapper2_EnumMatchingFilters failed with %x\n", hr);
         if (SUCCEEDED(hr) && pEnum)
         {
-            found = enum_find_filter(wszFilterName1, pEnum);
+            found = enum_find_filter(L"Testfilter1", pEnum);
             ok(found, "EnumMatchingFilters failed to return the test filter 1\n");
         }
 
@@ -151,7 +155,7 @@ static void test_fm2_enummatchingfilters(void)
         ok(hr == S_OK, "IFilterMapper2_EnumMatchingFilters failed with %x\n", hr);
         if (SUCCEEDED(hr) && pEnum)
         {
-            found = enum_find_filter(wszFilterName2, pEnum);
+            found = enum_find_filter(L"Testfilter2", pEnum);
             ok(found, "EnumMatchingFilters failed to return the test filter 2\n");
         }
 
@@ -166,7 +170,7 @@ static void test_fm2_enummatchingfilters(void)
 
         if (SUCCEEDED(hr) && pEnum)
         {
-            found = enum_find_filter(wszFilterName1, pEnum);
+            found = enum_find_filter(L"Testfilter1", pEnum);
             ok(found, "EnumMatchingFilters failed to return the test filter 1\n");
         }
 
@@ -179,7 +183,7 @@ static void test_fm2_enummatchingfilters(void)
         ok(hr == S_OK, "IFilterMapper_EnumMatchingFilters failed: %#x\n", hr);
         while (!found && IEnumRegFilters_Next(enum_reg, 1, &regfilter, &count) == S_OK)
         {
-            if (!lstrcmpW(regfilter->Name, wszFilterName1) && IsEqualGUID(&clsidFilter1, &regfilter->Clsid))
+            if (!wcscmp(regfilter->Name, L"Testfilter1") && IsEqualGUID(&clsidFilter1, &regfilter->Clsid))
                 found = TRUE;
         }
         IEnumRegFilters_Release(enum_reg);
@@ -195,7 +199,7 @@ static void test_fm2_enummatchingfilters(void)
 
     if (SUCCEEDED(hr) && pEnum)
     {
-        found = enum_find_filter(wszFilterName2, pEnum);
+        found = enum_find_filter(L"Testfilter2", pEnum);
         ok(!found, "EnumMatchingFilters should not return the test filter 2\n");
     }
 
@@ -218,9 +222,6 @@ static void test_fm2_enummatchingfilters(void)
 
 static void test_legacy_filter_registration(void)
 {
-    static const WCHAR testfilterW[] = {'T','e','s','t','f','i','l','t','e','r',0};
-    static const WCHAR clsidW[] = {'C','L','S','I','D','\\',0};
-    static const WCHAR pinW[] = {'P','i','n','1',0};
     IEnumRegFilters *enum_reg;
     IEnumMoniker *enum_mon;
     IFilterMapper2 *mapper2;
@@ -239,8 +240,8 @@ static void test_legacy_filter_registration(void)
      * type data to. Create a bogus class key for it. */
     CoCreateGuid(&clsid);
     StringFromGUID2(&clsid, clsidstring, ARRAY_SIZE(clsidstring));
-    lstrcpyW(key_name, clsidW);
-    lstrcatW(key_name, clsidstring);
+    wcscpy(key_name, L"CLSID\\");
+    wcscat(key_name, clsidstring);
     ret = RegCreateKeyExW(HKEY_CLASSES_ROOT, key_name, 0, NULL, 0, KEY_WRITE, NULL, &hkey, NULL);
     if (ret == ERROR_ACCESS_DENIED)
     {
@@ -258,22 +259,22 @@ static void test_legacy_filter_registration(void)
     ok(hr == S_OK, "IFilterMapper2_QueryInterface failed with %x\n", hr);
 
     /* Set default value - this is interpreted as "friendly name" later. */
-    RegSetValueExW(hkey, NULL, 0, REG_SZ, (BYTE *)testfilterW, sizeof(testfilterW));
+    RegSetValueExW(hkey, NULL, 0, REG_SZ, (const BYTE *)L"Testfilter", sizeof(L"Testfilter"));
     RegCloseKey(hkey);
 
-    hr = IFilterMapper_RegisterFilter(mapper, clsid, testfilterW, MERIT_UNLIKELY);
+    hr = IFilterMapper_RegisterFilter(mapper, clsid, L"Testfilter", MERIT_UNLIKELY);
     ok(hr == S_OK, "RegisterFilter failed: %#x\n", hr);
 
-    hr = IFilterMapper_RegisterPin(mapper, clsid, pinW, TRUE, FALSE, FALSE, FALSE, GUID_NULL, NULL);
+    hr = IFilterMapper_RegisterPin(mapper, clsid, L"Pin1", TRUE, FALSE, FALSE, FALSE, GUID_NULL, NULL);
     ok(hr == S_OK, "RegisterPin failed: %#x\n", hr);
 
-    hr = IFilterMapper_RegisterPinType(mapper, clsid, pinW, GUID_NULL, GUID_NULL);
+    hr = IFilterMapper_RegisterPinType(mapper, clsid, L"Pin1", GUID_NULL, GUID_NULL);
     ok(hr == S_OK, "RegisterPinType failed: %#x\n", hr);
 
     hr = IFilterMapper2_EnumMatchingFilters(mapper2, &enum_mon, 0, TRUE, MERIT_UNLIKELY, TRUE,
             0, NULL, NULL, &GUID_NULL, FALSE, FALSE, 0, NULL, NULL, &GUID_NULL);
     ok(hr == S_OK, "IFilterMapper2_EnumMatchingFilters failed: %x\n", hr);
-    ok(enum_find_filter(testfilterW, enum_mon), "IFilterMapper2 didn't find filter\n");
+    ok(enum_find_filter(L"Testfilter", enum_mon), "IFilterMapper2 didn't find filter\n");
     IEnumMoniker_Release(enum_mon);
 
     found = FALSE;
@@ -282,7 +283,7 @@ static void test_legacy_filter_registration(void)
     ok(hr == S_OK, "IFilterMapper_EnumMatchingFilters failed with %x\n", hr);
     while(!found && IEnumRegFilters_Next(enum_reg, 1, &regfilter, &count) == S_OK)
     {
-        if (!lstrcmpW(regfilter->Name, testfilterW) && IsEqualGUID(&clsid, &regfilter->Clsid))
+        if (!wcscmp(regfilter->Name, L"Testfilter") && IsEqualGUID(&clsid, &regfilter->Clsid))
             found = TRUE;
     }
     IEnumRegFilters_Release(enum_reg);
@@ -294,7 +295,7 @@ static void test_legacy_filter_registration(void)
     hr = IFilterMapper2_EnumMatchingFilters(mapper2, &enum_mon, 0, TRUE, MERIT_UNLIKELY, TRUE,
             0, NULL, NULL, &GUID_NULL, FALSE, FALSE, 0, NULL, NULL, &GUID_NULL);
     ok(hr == S_OK, "IFilterMapper2_EnumMatchingFilters failed: %x\n", hr);
-    ok(!enum_find_filter(testfilterW, enum_mon), "IFilterMapper2 shouldn't find filter\n");
+    ok(!enum_find_filter(L"Testfilter", enum_mon), "IFilterMapper2 shouldn't find filter\n");
     IEnumMoniker_Release(enum_mon);
 
     found = FALSE;
@@ -303,7 +304,7 @@ static void test_legacy_filter_registration(void)
     ok(hr == S_OK, "IFilterMapper_EnumMatchingFilters failed with %x\n", hr);
     while(!found && IEnumRegFilters_Next(enum_reg, 1, &regfilter, &count) == S_OK)
     {
-        if (!lstrcmpW(regfilter->Name, testfilterW) && IsEqualGUID(&clsid, &regfilter->Clsid))
+        if (!wcscmp(regfilter->Name, L"Testfilter") && IsEqualGUID(&clsid, &regfilter->Clsid))
             found = TRUE;
     }
     IEnumRegFilters_Release(enum_reg);
@@ -312,7 +313,7 @@ static void test_legacy_filter_registration(void)
     ret = RegDeleteKeyW(HKEY_CLASSES_ROOT, key_name);
     ok(!ret, "RegDeleteKeyA failed: %lu\n", ret);
 
-    hr = IFilterMapper_RegisterFilter(mapper, clsid, testfilterW, MERIT_UNLIKELY);
+    hr = IFilterMapper_RegisterFilter(mapper, clsid, L"Testfilter", MERIT_UNLIKELY);
     ok(hr == S_OK, "RegisterFilter failed: %#x\n", hr);
 
     hr = IFilterMapper_UnregisterFilter(mapper, clsid);
@@ -381,15 +382,13 @@ static void test_ifiltermapper_from_filtergraph(void)
 
 static void test_register_filter_with_null_clsMinorType(void)
 {
+    static WCHAR wszPinName[] = L"Pin";
     IFilterMapper2 *pMapper = NULL;
     HRESULT hr;
     REGFILTER2 rgf2;
     REGFILTERPINS rgPins;
     REGFILTERPINS2 rgPins2;
     REGPINTYPES rgPinType;
-    static WCHAR wszPinName[] = {'P', 'i', 'n', 0 };
-    static const WCHAR wszFilterName1[] = {'T', 'e', 's', 't', 'f', 'i', 'l', 't', 'e', 'r', '1', 0 };
-    static const WCHAR wszFilterName2[] = {'T', 'e', 's', 't', 'f', 'i', 'l', 't', 'e', 'r', '2', 0 };
     CLSID clsidFilter1;
     CLSID clsidFilter2;
 
@@ -424,7 +423,7 @@ static void test_register_filter_with_null_clsMinorType(void)
     rgPins.nMediaTypes = 1;
     rgPins.lpMediaType = &rgPinType;
 
-    hr = IFilterMapper2_RegisterFilter(pMapper, &clsidFilter1, wszFilterName1, NULL,
+    hr = IFilterMapper2_RegisterFilter(pMapper, &clsidFilter1, L"Testfilter1", NULL,
                     &CLSID_LegacyAmFilterCategory, NULL, &rgf2);
     if (hr == E_ACCESSDENIED)
     {
@@ -451,7 +450,7 @@ static void test_register_filter_with_null_clsMinorType(void)
     rgPins2.lpMedium = NULL;
     rgPins2.clsPinCategory = NULL;
 
-    hr = IFilterMapper2_RegisterFilter(pMapper, &clsidFilter2, wszFilterName2, NULL,
+    hr = IFilterMapper2_RegisterFilter(pMapper, &clsidFilter2, L"Testfilter2", NULL,
                     &CLSID_LegacyAmFilterCategory, NULL, &rgf2);
     ok(hr == S_OK, "IFilterMapper2_RegisterFilter failed with %x\n", hr);
 
@@ -527,96 +526,98 @@ out:
         IFilterMapper2_Release(pMapper);
 }
 
-typedef struct IUnknownImpl
-{
-    IUnknown IUnknown_iface;
-    int AddRef_called;
-    int Release_called;
-} IUnknownImpl;
+static const GUID test_iid = {0x33333333};
+static LONG outer_ref = 1;
 
-static IUnknownImpl *IUnknownImpl_from_iface(IUnknown * iface)
+static HRESULT WINAPI outer_QueryInterface(IUnknown *iface, REFIID iid, void **out)
 {
-    return CONTAINING_RECORD(iface, IUnknownImpl, IUnknown_iface);
-}
-
-static HRESULT WINAPI IUnknownImpl_QueryInterface(IUnknown * iface, REFIID riid, LPVOID * ppv)
-{
-    ok(0, "QueryInterface should not be called for %s\n", wine_dbgstr_guid(riid));
+    if (IsEqualGUID(iid, &IID_IUnknown)
+            || IsEqualGUID(iid, &IID_IFilterMapper3)
+            || IsEqualGUID(iid, &test_iid))
+    {
+        *out = (IUnknown *)0xdeadbeef;
+        return S_OK;
+    }
+    ok(0, "unexpected call %s\n", wine_dbgstr_guid(iid));
     return E_NOINTERFACE;
 }
 
-static ULONG WINAPI IUnknownImpl_AddRef(IUnknown * iface)
+static ULONG WINAPI outer_AddRef(IUnknown *iface)
 {
-    IUnknownImpl *This = IUnknownImpl_from_iface(iface);
-    This->AddRef_called++;
-    return 2;
+    return InterlockedIncrement(&outer_ref);
 }
 
-static ULONG WINAPI IUnknownImpl_Release(IUnknown * iface)
+static ULONG WINAPI outer_Release(IUnknown *iface)
 {
-    IUnknownImpl *This = IUnknownImpl_from_iface(iface);
-    This->Release_called++;
-    return 1;
+    return InterlockedDecrement(&outer_ref);
 }
 
-static CONST_VTBL IUnknownVtbl IUnknownImpl_Vtbl =
+static const IUnknownVtbl outer_vtbl =
 {
-    IUnknownImpl_QueryInterface,
-    IUnknownImpl_AddRef,
-    IUnknownImpl_Release
+    outer_QueryInterface,
+    outer_AddRef,
+    outer_Release,
 };
 
-static void test_aggregate_filter_mapper(void)
+static IUnknown test_outer = {&outer_vtbl};
+
+static void test_aggregation(void)
 {
+    IFilterMapper3 *mapper, *mapper2;
+    IUnknown *unk, *unk2;
     HRESULT hr;
-    IUnknown *pmapper;
-    IUnknown *punk;
-    IUnknownImpl unk_outer = { { &IUnknownImpl_Vtbl }, 0, 0 };
+    ULONG ref;
 
-    hr = CoCreateInstance(&CLSID_FilterMapper2, &unk_outer.IUnknown_iface, CLSCTX_INPROC_SERVER,
-                          &IID_IUnknown, (void **)&pmapper);
-    ok(hr == S_OK, "CoCreateInstance returned %x\n", hr);
-    ok(pmapper != &unk_outer.IUnknown_iface, "pmapper = %p, expected not %p\n", pmapper, &unk_outer.IUnknown_iface);
+    mapper = (IFilterMapper3 *)0xdeadbeef;
+    hr = CoCreateInstance(&CLSID_FilterMapper2, &test_outer, CLSCTX_INPROC_SERVER,
+            &IID_IFilterMapper3, (void **)&mapper);
+    ok(hr == E_NOINTERFACE, "Got hr %#x.\n", hr);
+    ok(!mapper, "Got interface %p.\n", mapper);
 
-    hr = IUnknown_QueryInterface(pmapper, &IID_IUnknown, (void **)&punk);
-    ok(hr == S_OK, "IUnknown_QueryInterface returned %x\n", hr);
-    ok(punk != &unk_outer.IUnknown_iface, "punk = %p, expected not %p\n", punk, &unk_outer.IUnknown_iface);
-    IUnknown_Release(punk);
+    hr = CoCreateInstance(&CLSID_FilterMapper2, &test_outer, CLSCTX_INPROC_SERVER,
+            &IID_IUnknown, (void **)&unk);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(outer_ref == 1, "Got unexpected refcount %d.\n", outer_ref);
+    ok(unk != &test_outer, "Returned IUnknown should not be outer IUnknown.\n");
+    ref = get_refcount(unk);
+    ok(ref == 1, "Got unexpected refcount %d.\n", ref);
 
-    ok(unk_outer.AddRef_called == 0, "IUnknownImpl_AddRef called %d times\n", unk_outer.AddRef_called);
-    ok(unk_outer.Release_called == 0, "IUnknownImpl_Release called %d times\n", unk_outer.Release_called);
-    unk_outer.AddRef_called = 0;
-    unk_outer.Release_called = 0;
+    ref = IUnknown_AddRef(unk);
+    ok(ref == 2, "Got unexpected refcount %d.\n", ref);
+    ok(outer_ref == 1, "Got unexpected refcount %d.\n", outer_ref);
 
-    hr = IUnknown_QueryInterface(pmapper, &IID_IFilterMapper, (void **)&punk);
-    ok(hr == S_OK, "IUnknown_QueryInterface returned %x\n", hr);
-    ok(punk != &unk_outer.IUnknown_iface, "punk = %p, expected not %p\n", punk, &unk_outer.IUnknown_iface);
-    IUnknown_Release(punk);
+    ref = IUnknown_Release(unk);
+    ok(ref == 1, "Got unexpected refcount %d.\n", ref);
+    ok(outer_ref == 1, "Got unexpected refcount %d.\n", outer_ref);
 
-    ok(unk_outer.AddRef_called == 1, "IUnknownImpl_AddRef called %d times\n", unk_outer.AddRef_called);
-    ok(unk_outer.Release_called == 1, "IUnknownImpl_Release called %d times\n", unk_outer.Release_called);
-    unk_outer.AddRef_called = 0;
-    unk_outer.Release_called = 0;
+    hr = IUnknown_QueryInterface(unk, &IID_IUnknown, (void **)&unk2);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(unk2 == unk, "Got unexpected IUnknown %p.\n", unk2);
+    IUnknown_Release(unk2);
 
-    hr = IUnknown_QueryInterface(pmapper, &IID_IFilterMapper2, (void **)&punk);
-    ok(hr == S_OK, "IUnknown_QueryInterface returned %x\n", hr);
-    ok(punk != &unk_outer.IUnknown_iface, "punk = %p, expected not %p\n", punk, &unk_outer.IUnknown_iface);
-    IUnknown_Release(punk);
+    hr = IUnknown_QueryInterface(unk, &IID_IFilterMapper3, (void **)&mapper);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
 
-    ok(unk_outer.AddRef_called == 1, "IUnknownImpl_AddRef called %d times\n", unk_outer.AddRef_called);
-    ok(unk_outer.Release_called == 1, "IUnknownImpl_Release called %d times\n", unk_outer.Release_called);
-    unk_outer.AddRef_called = 0;
-    unk_outer.Release_called = 0;
+    hr = IFilterMapper3_QueryInterface(mapper, &IID_IUnknown, (void **)&unk2);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(unk2 == (IUnknown *)0xdeadbeef, "Got unexpected IUnknown %p.\n", unk2);
 
-    hr = IUnknown_QueryInterface(pmapper, &IID_IFilterMapper3, (void **)&punk);
-    ok(hr == S_OK, "IUnknown_QueryInterface returned %x\n", hr);
-    ok(punk != &unk_outer.IUnknown_iface, "punk = %p, expected not %p\n", punk, &unk_outer.IUnknown_iface);
-    IUnknown_Release(punk);
+    hr = IFilterMapper3_QueryInterface(mapper, &IID_IFilterMapper3, (void **)&mapper2);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(mapper2 == (IFilterMapper3 *)0xdeadbeef, "Got unexpected IFilterMapper3 %p.\n", mapper2);
 
-    ok(unk_outer.AddRef_called == 1, "IUnknownImpl_AddRef called %d times\n", unk_outer.AddRef_called);
-    ok(unk_outer.Release_called == 1, "IUnknownImpl_Release called %d times\n", unk_outer.Release_called);
+    hr = IUnknown_QueryInterface(unk, &test_iid, (void **)&unk2);
+    ok(hr == E_NOINTERFACE, "Got hr %#x.\n", hr);
+    ok(!unk2, "Got unexpected IUnknown %p.\n", unk2);
 
-    IUnknown_Release(pmapper);
+    hr = IFilterMapper3_QueryInterface(mapper, &test_iid, (void **)&unk2);
+    ok(hr == S_OK, "Got hr %#x.\n", hr);
+    ok(unk2 == (IUnknown *)0xdeadbeef, "Got unexpected IUnknown %p.\n", unk2);
+
+    IFilterMapper3_Release(mapper);
+    ref = IUnknown_Release(unk);
+    ok(!ref, "Got unexpected refcount %d.\n", ref);
+    ok(outer_ref == 1, "Got unexpected refcount %d.\n", outer_ref);
 }
 
 START_TEST(filtermapper)
@@ -628,7 +629,7 @@ START_TEST(filtermapper)
     test_ifiltermapper_from_filtergraph();
     test_register_filter_with_null_clsMinorType();
     test_parse_filter_data();
-    test_aggregate_filter_mapper();
+    test_aggregation();
 
     CoUninitialize();
 }
