@@ -69,12 +69,6 @@ static LPWSTR  (WINAPI *pStrChrNW)(LPCWSTR,WCHAR,UINT);
 static BOOL    (WINAPI *pStrToInt64ExA)(LPCSTR,DWORD,LONGLONG*);
 static BOOL    (WINAPI *pStrToInt64ExW)(LPCWSTR,DWORD,LONGLONG*);
 
-static int strcmpW(const WCHAR *str1, const WCHAR *str2)
-{
-    while (*str1 && (*str1 == *str2)) { str1++; str2++; }
-    return *str1 - *str2;
-}
-
 /* StrToInt/StrToIntEx results */
 typedef struct tagStrToIntResult
 {
@@ -82,6 +76,7 @@ typedef struct tagStrToIntResult
   int str_to_int;
   LONGLONG str_to_int64_ex;
   LONGLONG str_to_int64_hex;
+  BOOL failure;
 } StrToIntResult;
 
 static const StrToIntResult StrToInt_results[] = {
@@ -95,11 +90,12 @@ static const StrToIntResult StrToInt_results[] = {
      { "0x2bdc546291f4b1", 0, 0, ((LONGLONG)0x2bdc54 << 32) | 0x6291f4b1 },
      { "+0x44f4", 0, 0, 0x44f4 },
      { "-0x44fd", 0, 0, 0x44fd },
-     { "+ 88987", 0, 0, 0 },
-     { "- 55", 0, 0, 0 },
-     { "- 0", 0, 0, 0 },
-     { "+ 0x44f4", 0, 0, 0 },
-     { "--0x44fd", 0, 0, 0 },
+     { "+ 88987", 0, 0, 0, TRUE },
+
+     { "- 55", 0, 0, 0, TRUE },
+     { "- 0", 0, 0, 0, TRUE },
+     { "+ 0x44f4", 0, 0, 0, TRUE },
+     { "--0x44fd", 0, 0, 0, TRUE },
      { " 1999", 0, 1999, 1999 },
      { " +88987", 0, 88987, 88987 },
      { " 012", 0, 12, 12 },
@@ -107,6 +103,10 @@ static const StrToIntResult StrToInt_results[] = {
      { " 0x44ff", 0, 0, 0x44ff },
      { " +0x44f4", 0, 0, 0x44f4 },
      { " -0x44fd", 0, 0, 0x44fd },
+     { "\t\n +3", 0, 3, 3 },
+     { "\v+4", 0, 0, 0, TRUE },
+     { "\f+5", 0, 0, 0, TRUE },
+     { "\r+6", 0, 0, 0, TRUE },
      { NULL, 0, 0, 0 }
 };
 
@@ -412,7 +412,7 @@ static void test_StrCpyW(void)
 
   while(result->value)
   {
-    MultiByteToWideChar(CP_ACP,0,result->byte_size_64,-1,szSrc,sizeof(szSrc)/sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, result->byte_size_64, -1, szSrc, ARRAY_SIZE(szSrc));
 
     lpRes = StrCpyW(szBuff, szSrc);
     ok(!StrCmpW(szSrc, szBuff) && lpRes == szBuff, "Copied string %s wrong\n", result->byte_size_64);
@@ -476,7 +476,7 @@ static void test_StrToIntW(void)
 
   while (result->string)
   {
-    MultiByteToWideChar(CP_ACP,0,result->string,-1,szBuff,sizeof(szBuff)/sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, result->string, -1, szBuff, ARRAY_SIZE(szBuff));
     return_val = StrToIntW(szBuff);
     ok(return_val == result->str_to_int, "converted '%s' wrong (%d)\n",
        result->string, return_val);
@@ -494,8 +494,10 @@ static void test_StrToIntExA(void)
   {
     return_val = -1;
     bRet = StrToIntExA(result->string,0,&return_val);
-    ok(!bRet || return_val != -1, "No result returned from '%s'\n",
-       result->string);
+    if (result->failure)
+        ok(!bRet, "Got %d instead of failure for '%s'\n", return_val, result->string);
+    else
+        ok(bRet, "Failed for '%s'\n", result->string);
     if (bRet)
       ok(return_val == (int)result->str_to_int64_ex, "converted '%s' wrong (%d)\n",
          result->string, return_val);
@@ -507,8 +509,10 @@ static void test_StrToIntExA(void)
   {
     return_val = -1;
     bRet = StrToIntExA(result->string,STIF_SUPPORT_HEX,&return_val);
-    ok(!bRet || return_val != -1, "No result returned from '%s'\n",
-       result->string);
+    if (result->failure)
+        ok(!bRet, "Got %d instead of failure for '%s'\n", return_val, result->string);
+    else
+        ok(bRet, "Failed for '%s'\n", result->string);
     if (bRet)
       ok(return_val == (int)result->str_to_int64_hex, "converted '%s' wrong (%d)\n",
          result->string, return_val);
@@ -526,10 +530,12 @@ static void test_StrToIntExW(void)
   while (result->string)
   {
     return_val = -1;
-    MultiByteToWideChar(CP_ACP,0,result->string,-1,szBuff,sizeof(szBuff)/sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, result->string, -1, szBuff, ARRAY_SIZE(szBuff));
     bRet = StrToIntExW(szBuff, 0, &return_val);
-    ok(!bRet || return_val != -1, "No result returned from '%s'\n",
-       result->string);
+    if (result->failure)
+        ok(!bRet, "Got %d instead of failure for '%s'\n", return_val, result->string);
+    else
+        ok(bRet, "Failed for '%s'\n", result->string);
     if (bRet)
       ok(return_val == (int)result->str_to_int64_ex, "converted '%s' wrong (%d)\n",
          result->string, return_val);
@@ -540,15 +546,25 @@ static void test_StrToIntExW(void)
   while (result->string)
   {
     return_val = -1;
-    MultiByteToWideChar(CP_ACP,0,result->string,-1,szBuff,sizeof(szBuff)/sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, result->string, -1, szBuff, ARRAY_SIZE(szBuff));
     bRet = StrToIntExW(szBuff, STIF_SUPPORT_HEX, &return_val);
-    ok(!bRet || return_val != -1, "No result returned from '%s'\n",
-       result->string);
+    if (result->failure)
+        ok(!bRet, "Got %d instead of failure for '%s'\n", return_val, result->string);
+    else
+        ok(bRet, "Failed for '%s'\n", result->string);
     if (bRet)
       ok(return_val == (int)result->str_to_int64_hex, "converted '%s' wrong (%d)\n",
          result->string, return_val);
     result++;
   }
+
+  return_val = -1;
+  bRet = StrToIntExW(L"\x0661\x0662", 0, &return_val);
+  ok( !bRet, "Returned %d for Unicode digits\n", return_val );
+  bRet = StrToIntExW(L"\x07c3\x07c4", 0, &return_val);
+  ok( !bRet, "Returned %d for Unicode digits\n", return_val );
+  bRet = StrToIntExW(L"\xa0-2", 0, &return_val);
+  ok( !bRet, "Returned %d for Unicode space\n", return_val );
 }
 
 static void test_StrToInt64ExA(void)
@@ -567,8 +583,11 @@ static void test_StrToInt64ExA(void)
   {
     return_val = -1;
     bRet = pStrToInt64ExA(result->string,0,&return_val);
-    ok(!bRet || return_val != -1, "No result returned from '%s'\n",
-       result->string);
+    if (result->failure)
+        ok(!bRet, "Got %s instead of failure for '%s'\n",
+           wine_dbgstr_longlong(return_val), result->string);
+    else
+        ok(bRet, "Failed for '%s'\n", result->string);
     if (bRet)
       ok(return_val == result->str_to_int64_ex, "converted '%s' wrong (%s)\n",
          result->string, wine_dbgstr_longlong(return_val));
@@ -580,8 +599,11 @@ static void test_StrToInt64ExA(void)
   {
     return_val = -1;
     bRet = pStrToInt64ExA(result->string,STIF_SUPPORT_HEX,&return_val);
-    ok(!bRet || return_val != -1, "No result returned from '%s'\n",
-       result->string);
+    if (result->failure)
+        ok(!bRet, "Got %s instead of failure for '%s'\n",
+           wine_dbgstr_longlong(return_val), result->string);
+    else
+        ok(bRet, "Failed for '%s'\n", result->string);
     if (bRet)
       ok(return_val == result->str_to_int64_hex, "converted '%s' wrong (%s)\n",
          result->string, wine_dbgstr_longlong(return_val));
@@ -605,10 +627,13 @@ static void test_StrToInt64ExW(void)
   while (result->string)
   {
     return_val = -1;
-    MultiByteToWideChar(CP_ACP,0,result->string,-1,szBuff,sizeof(szBuff)/sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, result->string, -1, szBuff, ARRAY_SIZE(szBuff));
     bRet = pStrToInt64ExW(szBuff, 0, &return_val);
-    ok(!bRet || return_val != -1, "No result returned from '%s'\n",
-       result->string);
+    if (result->failure)
+        ok(!bRet, "Got %s instead of failure for '%s'\n",
+           wine_dbgstr_longlong(return_val), result->string);
+    else
+        ok(bRet, "Failed for '%s'\n", result->string);
     if (bRet)
       ok(return_val == result->str_to_int64_ex, "converted '%s' wrong (%s)\n",
          result->string, wine_dbgstr_longlong(return_val));
@@ -619,15 +644,26 @@ static void test_StrToInt64ExW(void)
   while (result->string)
   {
     return_val = -1;
-    MultiByteToWideChar(CP_ACP,0,result->string,-1,szBuff,sizeof(szBuff)/sizeof(WCHAR));
+    MultiByteToWideChar(CP_ACP, 0, result->string, -1, szBuff, ARRAY_SIZE(szBuff));
     bRet = pStrToInt64ExW(szBuff, STIF_SUPPORT_HEX, &return_val);
-    ok(!bRet || return_val != -1, "No result returned from '%s'\n",
-       result->string);
+    if (result->failure)
+        ok(!bRet, "Got %s instead of failure for '%s'\n",
+           wine_dbgstr_longlong(return_val), result->string);
+    else
+        ok(bRet, "Failed for '%s'\n", result->string);
     if (bRet)
       ok(return_val == result->str_to_int64_hex, "converted '%s' wrong (%s)\n",
          result->string, wine_dbgstr_longlong(return_val));
     result++;
   }
+
+  return_val = -1;
+  bRet = pStrToInt64ExW(L"\x0661\x0662", 0, &return_val);
+  ok( !bRet, "Returned %s for Unicode digits\n", wine_dbgstr_longlong(return_val) );
+  bRet = pStrToInt64ExW(L"\x07c3\x07c4", 0, &return_val);
+  ok( !bRet, "Returned %s for Unicode digits\n", wine_dbgstr_longlong(return_val) );
+  bRet = pStrToInt64ExW(L"\xa0-2", 0, &return_val);
+  ok( !bRet, "Returned %s for Unicode space\n", wine_dbgstr_longlong(return_val) );
 }
 
 static void test_StrDupA(void)
@@ -694,7 +730,7 @@ static void test_StrFormatKBSizeW(void)
   while(result->value)
   {
     pStrFormatKBSizeW(result->value, szBuffW, 256);
-    WideCharToMultiByte(CP_ACP,0,szBuffW,-1,szBuff,sizeof(szBuff)/sizeof(WCHAR),NULL,NULL);
+    WideCharToMultiByte(CP_ACP, 0, szBuffW, -1, szBuff, ARRAY_SIZE(szBuff), NULL, NULL);
 
     ok(!strcmp(result->kb_size, szBuff), "Formatted %s wrong: got %s, expected %s\n",
        wine_dbgstr_longlong(result->value), szBuff, result->kb_size);
@@ -830,14 +866,14 @@ static void test_StrRetToBSTR(void)
     U(strret).pOleStr = CoDupStrW("Test");
     bstr = 0;
     ret = pStrRetToBSTR(&strret, NULL, &bstr);
-    ok(ret == S_OK && bstr && !strcmpW(bstr, szTestW),
+    ok(ret == S_OK && bstr && !wcscmp(bstr, szTestW),
        "STRRET_WSTR: dup failed, ret=0x%08x, bstr %p\n", ret, bstr);
     SysFreeString(bstr);
 
     strret.uType = STRRET_CSTR;
     lstrcpyA(U(strret).cStr, "Test");
     ret = pStrRetToBSTR(&strret, NULL, &bstr);
-    ok(ret == S_OK && bstr && !strcmpW(bstr, szTestW),
+    ok(ret == S_OK && bstr && !wcscmp(bstr, szTestW),
        "STRRET_CSTR: dup failed, ret=0x%08x, bstr %p\n", ret, bstr);
     SysFreeString(bstr);
 
@@ -845,7 +881,7 @@ static void test_StrRetToBSTR(void)
     U(strret).uOffset = 1;
     strcpy((char*)&iidl, " Test");
     ret = pStrRetToBSTR(&strret, iidl, &bstr);
-    ok(ret == S_OK && bstr && !strcmpW(bstr, szTestW),
+    ok(ret == S_OK && bstr && !wcscmp(bstr, szTestW),
        "STRRET_OFFSET: dup failed, ret=0x%08x, bstr %p\n", ret, bstr);
     SysFreeString(bstr);
 
@@ -865,7 +901,7 @@ static void test_StrCpyNXA(void)
   }
 
   memset(dest, '\n', sizeof(dest));
-  lpszRes = pStrCpyNXA(dest, lpSrc, sizeof(dest)/sizeof(dest[0]));
+  lpszRes = pStrCpyNXA(dest, lpSrc, ARRAY_SIZE(dest));
   ok(lpszRes == dest + 5 && !memcmp(dest, "hello\0\n\n", sizeof(dest)),
        "StrCpyNXA: expected %p, \"hello\\0\\n\\n\", got %p, \"%d,%d,%d,%d,%d,%d,%d,%d\"\n",
        dest + 5, lpszRes, dest[0], dest[1], dest[2], dest[3], dest[4], dest[5], dest[6], dest[7]);
@@ -886,7 +922,7 @@ static void test_StrCpyNXW(void)
   }
 
   memcpy(dest, lpInit, sizeof(lpInit));
-  lpszRes = pStrCpyNXW(dest, lpSrc, sizeof(dest)/sizeof(dest[0]));
+  lpszRes = pStrCpyNXW(dest, lpSrc, ARRAY_SIZE(dest));
   ok(lpszRes == dest + 5 && !memcmp(dest, lpRes, sizeof(dest)),
        "StrCpyNXW: expected %p, \"hello\\0\\n\\n\", got %p, \"%d,%d,%d,%d,%d,%d,%d,%d\"\n",
        dest + 5, lpszRes, dest[0], dest[1], dest[2], dest[3], dest[4], dest[5], dest[6], dest[7]);
@@ -949,7 +985,7 @@ static void test_SHAnsiToAnsi(void)
   }
 
   memset(dest, '\n', sizeof(dest));
-  dwRet = pSHAnsiToAnsi("hello", dest, sizeof(dest)/sizeof(dest[0]));
+  dwRet = pSHAnsiToAnsi("hello", dest, ARRAY_SIZE(dest));
   ok(dwRet == 6 && !memcmp(dest, "hello\0\n\n", sizeof(dest)),
      "SHAnsiToAnsi: expected 6, \"hello\\0\\n\\n\", got %d, \"%d,%d,%d,%d,%d,%d,%d,%d\"\n",
      dwRet, dest[0], dest[1], dest[2], dest[3], dest[4], dest[5], dest[6], dest[7]);
@@ -976,7 +1012,7 @@ static void test_SHUnicodeToUnicode(void)
   }
 
   memcpy(dest, lpInit, sizeof(lpInit));
-  dwRet = pSHUnicodeToUnicode(lpSrc, dest, sizeof(dest)/sizeof(dest[0]));
+  dwRet = pSHUnicodeToUnicode(lpSrc, dest, ARRAY_SIZE(dest));
   ok(dwRet == 6 && !memcmp(dest, lpRes, sizeof(dest)),
      "SHUnicodeToUnicode: expected 6, \"hello\\0\\n\\n\", got %d, \"%d,%d,%d,%d,%d,%d,%d,%d\"\n",
      dwRet, dest[0], dest[1], dest[2], dest[3], dest[4], dest[5], dest[6], dest[7]);
@@ -1158,7 +1194,7 @@ static void test_StrStrA(void)
     ret = StrStrA("", "");
     ok(!ret, "Expected StrStrA to return NULL, got %p\n", ret);
 
-    for (i = 0; i < sizeof(StrStrA_cases)/sizeof(StrStrA_cases[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(StrStrA_cases); i++)
     {
         ret = StrStrA(deadbeefA, StrStrA_cases[i].search);
         ok(ret == StrStrA_cases[i].expect,
@@ -1212,7 +1248,7 @@ static void test_StrStrW(void)
     ret = StrStrW(emptyW, emptyW);
     ok(!ret, "Expected StrStrW to return NULL, got %p\n", ret);
 
-    for (i = 0; i < sizeof(StrStrW_cases)/sizeof(StrStrW_cases[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(StrStrW_cases); i++)
     {
         ret = StrStrW(deadbeefW, StrStrW_cases[i].search);
         ok(ret == StrStrW_cases[i].expect,
@@ -1260,7 +1296,7 @@ static void test_StrStrIA(void)
     ret = StrStrIA("", "");
     ok(!ret, "Expected StrStrIA to return NULL, got %p\n", ret);
 
-    for (i = 0; i < sizeof(StrStrIA_cases)/sizeof(StrStrIA_cases[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(StrStrIA_cases); i++)
     {
         ret = StrStrIA(deadbeefA, StrStrIA_cases[i].search);
         ok(ret == StrStrIA_cases[i].expect,
@@ -1316,7 +1352,7 @@ static void test_StrStrIW(void)
     ret = StrStrIW(emptyW, emptyW);
     ok(!ret, "Expected StrStrIW to return NULL, got %p\n", ret);
 
-    for (i = 0; i < sizeof(StrStrIW_cases)/sizeof(StrStrIW_cases[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(StrStrIW_cases); i++)
     {
         ret = StrStrIW(deadbeefW, StrStrIW_cases[i].search);
         ok(ret == StrStrIW_cases[i].expect,
@@ -1343,13 +1379,13 @@ static void test_StrStrNW(void)
         const WCHAR *expect;
     } StrStrNW_cases[] =
     {
-        {emptyW, sizeof(deadbeefW)/sizeof(WCHAR), NULL},
-        {deadW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW},
-        {dead_lowerW, sizeof(deadbeefW)/sizeof(WCHAR), NULL},
-        {adbeW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW + 2},
-        {adbe_lowerW, sizeof(deadbeefW)/sizeof(WCHAR), NULL},
-        {beefW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW + 4},
-        {beef_lowerW, sizeof(deadbeefW)/sizeof(WCHAR), NULL},
+        {emptyW, ARRAY_SIZE(deadbeefW), NULL},
+        {deadW, ARRAY_SIZE(deadbeefW), deadbeefW},
+        {dead_lowerW, ARRAY_SIZE(deadbeefW), NULL},
+        {adbeW, ARRAY_SIZE(deadbeefW), deadbeefW + 2},
+        {adbe_lowerW, ARRAY_SIZE(deadbeefW), NULL},
+        {beefW, ARRAY_SIZE(deadbeefW), deadbeefW + 4},
+        {beef_lowerW, ARRAY_SIZE(deadbeefW), NULL},
         {beefW, 0, NULL},
         {beefW, 1, NULL},
         {beefW, 2, NULL},
@@ -1386,7 +1422,7 @@ static void test_StrStrNW(void)
     ret = pStrStrNW(emptyW, emptyW, 10);
     ok(!ret, "Expected StrStrNW to return NULL, got %p\n", ret);
 
-    for (i = 0; i < sizeof(StrStrNW_cases)/sizeof(StrStrNW_cases[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(StrStrNW_cases); i++)
     {
         ret = pStrStrNW(deadbeefW, StrStrNW_cases[i].search, StrStrNW_cases[i].count);
         ok(ret == StrStrNW_cases[i].expect,
@@ -1426,14 +1462,14 @@ static void test_StrStrNIW(void)
         const WCHAR *expect;
     } StrStrNIW_cases[] =
     {
-        {emptyW, sizeof(deadbeefW)/sizeof(WCHAR), NULL},
-        {deadW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW},
-        {dead_lowerW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW},
-        {adbeW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW + 2},
-        {adbe_lowerW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW + 2},
-        {beefW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW + 4},
-        {beef_lowerW, sizeof(deadbeefW)/sizeof(WCHAR), deadbeefW + 4},
-        {cafeW, sizeof(deadbeefW)/sizeof(WCHAR), NULL},
+        {emptyW, ARRAY_SIZE(deadbeefW), NULL},
+        {deadW, ARRAY_SIZE(deadbeefW), deadbeefW},
+        {dead_lowerW, ARRAY_SIZE(deadbeefW), deadbeefW},
+        {adbeW, ARRAY_SIZE(deadbeefW), deadbeefW + 2},
+        {adbe_lowerW, ARRAY_SIZE(deadbeefW), deadbeefW + 2},
+        {beefW, ARRAY_SIZE(deadbeefW), deadbeefW + 4},
+        {beef_lowerW, ARRAY_SIZE(deadbeefW), deadbeefW + 4},
+        {cafeW, ARRAY_SIZE(deadbeefW), NULL},
         {beefW, 0, NULL},
         {beefW, 1, NULL},
         {beefW, 2, NULL},
@@ -1480,7 +1516,7 @@ static void test_StrStrNIW(void)
     ret = pStrStrNIW(emptyW, emptyW, 10);
     ok(!ret, "Expected StrStrNIW to return NULL, got %p\n", ret);
 
-    for (i = 0; i < sizeof(StrStrNIW_cases)/sizeof(StrStrNIW_cases[0]); i++)
+    for (i = 0; i < ARRAY_SIZE(StrStrNIW_cases); i++)
     {
         ret = pStrStrNIW(deadbeefW, StrStrNIW_cases[i].search, StrStrNIW_cases[i].count);
         ok(ret == StrStrNIW_cases[i].expect,
