@@ -422,19 +422,21 @@ static inline struct strmbase_source *impl_source_from_IPin( IPin *iface )
     return CONTAINING_RECORD(iface, struct strmbase_source, pin.IPin_iface);
 }
 
-static BOOL compare_media_types(const AM_MEDIA_TYPE *a, const AM_MEDIA_TYPE *b)
+static BOOL compare_media_types(const AM_MEDIA_TYPE *req_mt, const AM_MEDIA_TYPE *pin_mt)
 {
-    if (!a)
+    if (!req_mt)
         return TRUE;
 
-    if (!IsEqualGUID(&a->majortype, &b->majortype)
-            && !IsEqualGUID(&a->majortype, &GUID_NULL)
-            && !IsEqualGUID(&b->majortype, &GUID_NULL))
+    if (!IsEqualGUID(&req_mt->majortype, &pin_mt->majortype)
+            && !IsEqualGUID(&req_mt->majortype, &GUID_NULL))
         return FALSE;
 
-    if (!IsEqualGUID(&a->subtype, &b->subtype)
-            && !IsEqualGUID(&a->subtype, &GUID_NULL)
-            && !IsEqualGUID(&b->subtype, &GUID_NULL))
+    if (!IsEqualGUID(&req_mt->subtype, &pin_mt->subtype)
+            && !IsEqualGUID(&req_mt->subtype, &GUID_NULL))
+        return FALSE;
+
+    if (!IsEqualGUID(&req_mt->formattype, &pin_mt->formattype)
+            && !IsEqualGUID(&req_mt->formattype, &GUID_NULL))
         return FALSE;
 
     return TRUE;
@@ -477,7 +479,11 @@ static HRESULT WINAPI source_Connect(IPin *iface, IPin *peer, const AM_MEDIA_TYP
         return VFW_E_NOT_STOPPED;
     }
 
-    if (mt && !IsEqualGUID(&mt->majortype, &GUID_NULL) && !IsEqualGUID(&mt->subtype, &GUID_NULL))
+    /* We don't check the subtype here. The rationale (as given by the DirectX
+     * documentation) is that the format type is supposed to provide at least
+     * as much information as the subtype. */
+    if (mt && !IsEqualGUID(&mt->majortype, &GUID_NULL)
+            && !IsEqualGUID(&mt->formattype, &GUID_NULL))
     {
         hr = pin->pFuncsTable->pfnAttemptConnection(pin, peer, mt);
         LeaveCriticalSection(&pin->pin.filter->csFilter);
@@ -546,6 +552,9 @@ static HRESULT WINAPI source_Disconnect(IPin *iface)
             return VFW_E_NOT_STOPPED;
         }
 
+        if (This->pFuncsTable->source_disconnect)
+            This->pFuncsTable->source_disconnect(This);
+
         if (This->pMemInputPin)
         {
             IMemInputPin_Release(This->pMemInputPin);
