@@ -30,6 +30,9 @@
 #include <time.h>
 #include <direct.h>
 #include <locale.h>
+#include <process.h>
+#include <fenv.h>
+#include <malloc.h>
 
 #include <windef.h>
 #include <winbase.h>
@@ -71,40 +74,9 @@ static inline double __port_max_double(void)
     return *(const double *)&__max_double;
 }
 
-#define M_PI_2 1.57079632679489661923
-
-#define FE_TONEAREST 0
-
-#define _DOMAIN         1       /* domain error in argument */
-#define _SING           2       /* singularity */
-#define _OVERFLOW       3       /* range overflow */
-#define _UNDERFLOW      4       /* range underflow */
-
 DEFINE_EXPECT(global_invalid_parameter_handler);
 DEFINE_EXPECT(thread_invalid_parameter_handler);
 
-typedef int (CDECL *MSVCRT__onexit_t)(void);
-
-typedef struct MSVCRT__onexit_table_t
-{
-    MSVCRT__onexit_t *_first;
-    MSVCRT__onexit_t *_last;
-    MSVCRT__onexit_t *_end;
-} MSVCRT__onexit_table_t;
-
-typedef struct MSVCRT__lldiv_t {
-    LONGLONG quot;  /* quotient */
-    LONGLONG rem;   /* remainder */
-} MSVCRT_lldiv_t;
-
-typedef struct MSVCRT__exception {
-    int     type;
-    char*   name;
-    double  arg1;
-    double  arg2;
-    double  retval;
-} MSVCRT__exception;
-
 typedef struct {
     const char *short_wday[7];
     const char *wday[7];
@@ -128,77 +100,36 @@ typedef struct {
     const wchar_t *locnameW;
 } __lc_time_data;
 
-typedef int (CDECL *MSVCRT_matherr_func)(struct MSVCRT__exception *);
+static LONGLONG crt_init_end;
 
-static HMODULE module;
-static LONGLONG crt_init_start, crt_init_end;
-
-static int (CDECL *p_initialize_onexit_table)(MSVCRT__onexit_table_t *table);
-static int (CDECL *p_register_onexit_function)(MSVCRT__onexit_table_t *table, MSVCRT__onexit_t func);
-static int (CDECL *p_execute_onexit_table)(MSVCRT__onexit_table_t *table);
-static int (CDECL *p_o__initialize_onexit_table)(MSVCRT__onexit_table_t *table);
-static int (CDECL *p_o__register_onexit_function)(MSVCRT__onexit_table_t *table, MSVCRT__onexit_t func);
-static int (CDECL *p_o__execute_onexit_table)(MSVCRT__onexit_table_t *table);
-static int (CDECL *p___fpe_flt_rounds)(void);
-static unsigned int (CDECL *p__controlfp)(unsigned int, unsigned int);
-static _invalid_parameter_handler (CDECL *p__set_invalid_parameter_handler)(_invalid_parameter_handler);
-static _invalid_parameter_handler (CDECL *p__get_invalid_parameter_handler)(void);
-static _invalid_parameter_handler (CDECL *p__set_thread_local_invalid_parameter_handler)(_invalid_parameter_handler);
-static _invalid_parameter_handler (CDECL *p__get_thread_local_invalid_parameter_handler)(void);
-static int (CDECL *p__ltoa_s)(LONG, char*, size_t, int);
-static char* (CDECL *p__get_narrow_winmain_command_line)(void);
-static int (CDECL *p_sopen_dispatch)(const char *, int, int, int, int *, int);
-static int (CDECL *p_sopen_s)(int *, const char *, int, int, int);
-static int (WINAPIV *p__open)(const char*, int, ...);
-static MSVCRT_lldiv_t* (CDECL *p_lldiv)(MSVCRT_lldiv_t*,LONGLONG,LONGLONG);
-static int (CDECL *p__isctype)(int,int);
-static int (CDECL *p_isblank)(int);
-static int (CDECL *p__isblank_l)(int,_locale_t);
-static int (CDECL *p__iswctype_l)(int,int,_locale_t);
-static int (CDECL *p_iswblank)(int);
-static int (CDECL *p__iswblank_l)(wint_t,_locale_t);
-static int (CDECL *p_fesetround)(int);
-static void (CDECL *p___setusermatherr)(MSVCRT_matherr_func);
-static int* (CDECL *p_errno)(void);
-static char* (CDECL *p_asctime)(const struct tm *);
-static size_t (__cdecl *p_strftime)(char *, size_t, const char *, const struct tm *);
-static size_t (__cdecl *p__Strftime)(char*, size_t, const char*, const struct tm*, void*);
-static char* (__cdecl *p_setlocale)(int, const char*);
-static struct tm*  (__cdecl *p__gmtime32)(const __time32_t*);
-static void (CDECL *p_exit)(int);
-static int (CDECL *p__crt_atexit)(void (CDECL*)(void));
-static int (__cdecl *p_crt_at_quick_exit)(void (__cdecl *func)(void));
-static void (__cdecl *p_quick_exit)(int exitcode);
-static int (__cdecl *p__stat32)(const char*, struct _stat32 *buf);
-static int (__cdecl *p__close)(int);
-static void* (__cdecl *p__o_malloc)(size_t);
-static size_t (__cdecl *p__msize)(void*);
-static void (__cdecl *p_free)(void*);
-static clock_t (__cdecl *p_clock)(void);
+_ACRTIMP int __cdecl _o__initialize_onexit_table(_onexit_table_t *table);
+_ACRTIMP int __cdecl _o__register_onexit_function(_onexit_table_t *table, _onexit_t func);
+_ACRTIMP int __cdecl _o__execute_onexit_table(_onexit_table_t *table);
+_ACRTIMP void *__cdecl _o_malloc(size_t);
 
 static void test__initialize_onexit_table(void)
 {
-    MSVCRT__onexit_table_t table, table2;
+    _onexit_table_t table, table2;
     int ret;
 
-    ret = p_initialize_onexit_table(NULL);
+    ret = _initialize_onexit_table(NULL);
     ok(ret == -1, "got %d\n", ret);
 
     memset(&table, 0, sizeof(table));
-    ret = p_initialize_onexit_table(&table);
+    ret = _initialize_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(table._first == table._last && table._first == table._end, "got first %p, last %p, end %p\n",
         table._first, table._last, table._end);
 
     memset(&table2, 0, sizeof(table2));
-    ret = p_initialize_onexit_table(&table2);
+    ret = _initialize_onexit_table(&table2);
     ok(ret == 0, "got %d\n", ret);
     ok(table2._first == table._first, "got %p, %p\n", table2._first, table._first);
     ok(table2._last == table._last, "got %p, %p\n", table2._last, table._last);
     ok(table2._end == table._end, "got %p, %p\n", table2._end, table._end);
 
     memset(&table2, 0, sizeof(table2));
-    ret = p_o__initialize_onexit_table(&table2);
+    ret = _o__initialize_onexit_table(&table2);
     ok(ret == 0, "got %d\n", ret);
     ok(table2._first == table._first, "got %p, %p\n", table2._first, table._first);
     ok(table2._last == table._last, "got %p, %p\n", table2._last, table._last);
@@ -206,7 +137,7 @@ static void test__initialize_onexit_table(void)
 
     /* uninitialized table */
     table._first = table._last = table._end = (void*)0x123;
-    ret = p_initialize_onexit_table(&table);
+    ret = _initialize_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(table._first == table._last && table._first == table._end, "got first %p, last %p, end %p\n",
         table._first, table._last, table._end);
@@ -215,7 +146,7 @@ static void test__initialize_onexit_table(void)
     table._first = (void*)0x123;
     table._last = (void*)0x456;
     table._end = (void*)0x123;
-    ret = p_initialize_onexit_table(&table);
+    ret = _initialize_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(table._first == table._last && table._first == table._end, "got first %p, last %p, end %p\n",
         table._first, table._last, table._end);
@@ -224,7 +155,7 @@ static void test__initialize_onexit_table(void)
     table._first = (void*)0x123;
     table._last = (void*)0x456;
     table._end = (void*)0x789;
-    ret = p_initialize_onexit_table(&table);
+    ret = _initialize_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(table._first == (void*)0x123, "got %p\n", table._first);
     ok(table._last == (void*)0x456, "got %p\n", table._last);
@@ -233,7 +164,7 @@ static void test__initialize_onexit_table(void)
     table._first = NULL;
     table._last = (void*)0x456;
     table._end = NULL;
-    ret = p_initialize_onexit_table(&table);
+    ret = _initialize_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(table._first == table._last && table._first == table._end, "got first %p, last %p, end %p\n",
         table._first, table._last, table._end);
@@ -255,128 +186,128 @@ static int CDECL onexit_func2(void)
 
 static void test__register_onexit_function(void)
 {
-    MSVCRT__onexit_table_t table;
-    MSVCRT__onexit_t *f;
+    _onexit_table_t table;
+    _PVFV *f;
     int ret;
 
     memset(&table, 0, sizeof(table));
-    ret = p_initialize_onexit_table(&table);
+    ret = _initialize_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_register_onexit_function(NULL, NULL);
+    ret = _register_onexit_function(NULL, NULL);
     ok(ret == -1, "got %d\n", ret);
 
-    ret = p_register_onexit_function(NULL, onexit_func);
+    ret = _register_onexit_function(NULL, onexit_func);
     ok(ret == -1, "got %d\n", ret);
 
     f = table._last;
-    ret = p_register_onexit_function(&table, NULL);
+    ret = _register_onexit_function(&table, NULL);
     ok(ret == 0, "got %d\n", ret);
     ok(f != table._last, "got %p, initial %p\n", table._last, f);
 
-    ret = p_register_onexit_function(&table, onexit_func);
+    ret = _register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
 
     f = table._last;
-    ret = p_register_onexit_function(&table, onexit_func);
+    ret = _register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
     ok(f != table._last, "got %p, initial %p\n", table._last, f);
 
     f = table._last;
-    ret = p_o__register_onexit_function(&table, NULL);
+    ret = _o__register_onexit_function(&table, NULL);
     ok(ret == 0, "got %d\n", ret);
     ok(f != table._last, "got %p, initial %p\n", table._last, f);
 
     f = table._last;
-    ret = p_o__register_onexit_function(&table, onexit_func);
+    ret = _o__register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
     ok(f != table._last, "got %p, initial %p\n", table._last, f);
 
-    ret = p_execute_onexit_table(&table);
+    ret = _execute_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
 }
 
 static void test__execute_onexit_table(void)
 {
-    MSVCRT__onexit_table_t table;
+    _onexit_table_t table;
     int ret;
 
-    ret = p_execute_onexit_table(NULL);
+    ret = _execute_onexit_table(NULL);
     ok(ret == -1, "got %d\n", ret);
 
     memset(&table, 0, sizeof(table));
-    ret = p_initialize_onexit_table(&table);
+    ret = _initialize_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
 
     /* execute empty table */
-    ret = p_execute_onexit_table(&table);
+    ret = _execute_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
 
     /* same function registered multiple times */
-    ret = p_register_onexit_function(&table, onexit_func);
+    ret = _register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_register_onexit_function(&table, NULL);
+    ret = _register_onexit_function(&table, NULL);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_register_onexit_function(&table, onexit_func);
+    ret = _register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_o__register_onexit_function(&table, onexit_func);
+    ret = _o__register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
 
     ok(table._first != table._end, "got %p, %p\n", table._first, table._end);
     g_onexit_called = 0;
-    ret = p_execute_onexit_table(&table);
+    ret = _execute_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(g_onexit_called == 3, "got %d\n", g_onexit_called);
     ok(table._first == table._end, "got %p, %p\n", table._first, table._end);
 
-    ret = p_register_onexit_function(&table, onexit_func);
+    ret = _register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_register_onexit_function(&table, NULL);
+    ret = _register_onexit_function(&table, NULL);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_register_onexit_function(&table, onexit_func);
+    ret = _register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_o__register_onexit_function(&table, onexit_func);
+    ret = _o__register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
 
     ok(table._first != table._end, "got %p, %p\n", table._first, table._end);
     g_onexit_called = 0;
-    ret = p_o__execute_onexit_table(&table);
+    ret = _o__execute_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(g_onexit_called == 3, "got %d\n", g_onexit_called);
     ok(table._first == table._end, "got %p, %p\n", table._first, table._end);
 
     /* execute again, table is already empty */
     g_onexit_called = 0;
-    ret = p_execute_onexit_table(&table);
+    ret = _execute_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(g_onexit_called == 0, "got %d\n", g_onexit_called);
 
     /* check call order */
     memset(&table, 0, sizeof(table));
-    ret = p_initialize_onexit_table(&table);
+    ret = _initialize_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_register_onexit_function(&table, onexit_func);
+    ret = _register_onexit_function(&table, onexit_func);
     ok(ret == 0, "got %d\n", ret);
 
-    ret = p_register_onexit_function(&table, onexit_func2);
+    ret = _register_onexit_function(&table, onexit_func2);
     ok(ret == 0, "got %d\n", ret);
 
     g_onexit_called = 0;
-    ret = p_execute_onexit_table(&table);
+    ret = _execute_onexit_table(&table);
     ok(ret == 0, "got %d\n", ret);
     ok(g_onexit_called == 2, "got %d\n", g_onexit_called);
 }
 
 static void test___fpe_flt_rounds(void)
 {
-    unsigned int cfp = p__controlfp(0, 0);
+    unsigned int cfp = _controlfp(0, 0);
     int ret;
 
     if(!cfp) {
@@ -384,20 +315,20 @@ static void test___fpe_flt_rounds(void)
         return;
     }
 
-    ok((p__controlfp(_RC_NEAR, _RC_CHOP) & _RC_CHOP) == _RC_NEAR, "_controlfp(_RC_NEAR, _RC_CHOP) failed\n");
-    ret = p___fpe_flt_rounds();
+    ok((_controlfp(_RC_NEAR, _RC_CHOP) & _RC_CHOP) == _RC_NEAR, "_controlfp(_RC_NEAR, _RC_CHOP) failed\n");
+    ret = __fpe_flt_rounds();
     ok(ret == 1, "__fpe_flt_rounds returned %d\n", ret);
 
-    ok((p__controlfp(_RC_UP, _RC_CHOP) & _RC_CHOP) == _RC_UP, "_controlfp(_RC_UP, _RC_CHOP) failed\n");
-    ret = p___fpe_flt_rounds();
+    ok((_controlfp(_RC_UP, _RC_CHOP) & _RC_CHOP) == _RC_UP, "_controlfp(_RC_UP, _RC_CHOP) failed\n");
+    ret = __fpe_flt_rounds();
     ok(ret == 2 || broken(ret == 3) /* w1064v1507 */, "__fpe_flt_rounds returned %d\n", ret);
 
-    ok((p__controlfp(_RC_DOWN, _RC_CHOP) & _RC_CHOP) == _RC_DOWN, "_controlfp(_RC_DOWN, _RC_CHOP) failed\n");
-    ret = p___fpe_flt_rounds();
+    ok((_controlfp(_RC_DOWN, _RC_CHOP) & _RC_CHOP) == _RC_DOWN, "_controlfp(_RC_DOWN, _RC_CHOP) failed\n");
+    ret = __fpe_flt_rounds();
     ok(ret == 3 || broken(ret == 2) /* w1064v1507 */, "__fpe_flt_rounds returned %d\n", ret);
 
-    ok((p__controlfp(_RC_CHOP, _RC_CHOP) & _RC_CHOP) == _RC_CHOP, "_controlfp(_RC_CHOP, _RC_CHOP) failed\n");
-    ret = p___fpe_flt_rounds();
+    ok((_controlfp(_RC_CHOP, _RC_CHOP) & _RC_CHOP) == _RC_CHOP, "_controlfp(_RC_CHOP, _RC_CHOP) failed\n");
+    ret = __fpe_flt_rounds();
     ok(ret == 0, "__fpe_flt_rounds returned %d\n", ret);
 }
 
@@ -419,45 +350,45 @@ static void test_invalid_parameter_handler(void)
 {
     _invalid_parameter_handler ret;
 
-    ret = p__get_invalid_parameter_handler();
+    ret = _get_invalid_parameter_handler();
     ok(!ret, "ret != NULL\n");
 
-    ret = p__get_thread_local_invalid_parameter_handler();
+    ret = _get_thread_local_invalid_parameter_handler();
     ok(!ret, "ret != NULL\n");
 
-    ret = p__set_thread_local_invalid_parameter_handler(thread_invalid_parameter_handler);
+    ret = _set_thread_local_invalid_parameter_handler(thread_invalid_parameter_handler);
     ok(!ret, "ret != NULL\n");
 
-    ret = p__get_thread_local_invalid_parameter_handler();
+    ret = _get_thread_local_invalid_parameter_handler();
     ok(ret == thread_invalid_parameter_handler, "ret = %p\n", ret);
 
-    ret = p__get_invalid_parameter_handler();
+    ret = _get_invalid_parameter_handler();
     ok(!ret, "ret != NULL\n");
 
-    ret = p__set_invalid_parameter_handler(global_invalid_parameter_handler);
+    ret = _set_invalid_parameter_handler(global_invalid_parameter_handler);
     ok(!ret, "ret != NULL\n");
 
-    ret = p__get_invalid_parameter_handler();
+    ret = _get_invalid_parameter_handler();
     ok(ret == global_invalid_parameter_handler, "ret = %p\n", ret);
 
-    ret = p__get_thread_local_invalid_parameter_handler();
+    ret = _get_thread_local_invalid_parameter_handler();
     ok(ret == thread_invalid_parameter_handler, "ret = %p\n", ret);
 
     SET_EXPECT(thread_invalid_parameter_handler);
-    p__ltoa_s(0, NULL, 0, 0);
+    _ltoa_s(0, NULL, 0, 0);
     CHECK_CALLED(thread_invalid_parameter_handler);
 
-    ret = p__set_thread_local_invalid_parameter_handler(NULL);
+    ret = _set_thread_local_invalid_parameter_handler(NULL);
     ok(ret == thread_invalid_parameter_handler, "ret = %p\n", ret);
 
     SET_EXPECT(global_invalid_parameter_handler);
-    p__ltoa_s(0, NULL, 0, 0);
+    _ltoa_s(0, NULL, 0, 0);
     CHECK_CALLED(global_invalid_parameter_handler);
 
-    ret = p__set_invalid_parameter_handler(NULL);
+    ret = _set_invalid_parameter_handler(NULL);
     ok(ret == global_invalid_parameter_handler, "ret = %p\n", ret);
 
-    ret = p__set_invalid_parameter_handler(global_invalid_parameter_handler);
+    ret = _set_invalid_parameter_handler(global_invalid_parameter_handler);
     ok(!ret, "ret != NULL\n");
 }
 
@@ -469,7 +400,7 @@ static void test__get_narrow_winmain_command_line(char *path)
     char *ret, *cmdline, *name;
     int len;
 
-    ret = p__get_narrow_winmain_command_line();
+    ret = _get_narrow_winmain_command_line();
     cmdline = GetCommandLineA();
     len = strlen(cmdline);
     ok(ret>cmdline && ret<cmdline+len, "ret = %p, cmdline = %p (len = %d)\n", ret, cmdline, len);
@@ -490,72 +421,11 @@ static void test__get_narrow_winmain_command_line(char *path)
     CreateProcessA(path, cmd, NULL, NULL, TRUE,
             CREATE_DEFAULT_ERROR_MODE|NORMAL_PRIORITY_CLASS,
             NULL, NULL, &startup, &proc);
-    winetest_wait_child_process(proc.hProcess);
+    wait_child_process(proc.hProcess);
     CloseHandle(proc.hProcess);
     CloseHandle(proc.hThread);
 }
 
-static BOOL init(void)
-{
-    FILETIME cur;
-
-    GetSystemTimeAsFileTime(&cur);
-    crt_init_start = ((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime;
-    module = LoadLibraryA("ucrtbase.dll");
-    GetSystemTimeAsFileTime(&cur);
-    crt_init_end = ((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime;
-
-    if(!module) {
-        win_skip("ucrtbase.dll not available\n");
-        return FALSE;
-    }
-
-    p_initialize_onexit_table = (void*)GetProcAddress(module, "_initialize_onexit_table");
-    p_register_onexit_function = (void*)GetProcAddress(module, "_register_onexit_function");
-    p_execute_onexit_table = (void*)GetProcAddress(module, "_execute_onexit_table");
-    p_o__initialize_onexit_table = (void*)GetProcAddress(module, "_o__initialize_onexit_table");
-    p_o__register_onexit_function = (void*)GetProcAddress(module, "_o__register_onexit_function");
-    p_o__execute_onexit_table = (void*)GetProcAddress(module, "_o__execute_onexit_table");
-    p___fpe_flt_rounds = (void*)GetProcAddress(module, "__fpe_flt_rounds");
-    p__controlfp = (void*)GetProcAddress(module, "_controlfp");
-    p__set_invalid_parameter_handler = (void*)GetProcAddress(module, "_set_invalid_parameter_handler");
-    p__get_invalid_parameter_handler = (void*)GetProcAddress(module, "_get_invalid_parameter_handler");
-    p__set_thread_local_invalid_parameter_handler = (void*)GetProcAddress(module, "_set_thread_local_invalid_parameter_handler");
-    p__get_thread_local_invalid_parameter_handler = (void*)GetProcAddress(module, "_get_thread_local_invalid_parameter_handler");
-    p__ltoa_s = (void*)GetProcAddress(module, "_ltoa_s");
-    p__get_narrow_winmain_command_line = (void*)GetProcAddress(GetModuleHandleA("ucrtbase.dll"), "_get_narrow_winmain_command_line");
-    p_sopen_dispatch = (void*)GetProcAddress(module, "_sopen_dispatch");
-    p_sopen_s = (void*)GetProcAddress(module, "_sopen_s");
-    p__open = (void*)GetProcAddress(module, "_open");
-    p_lldiv = (void*)GetProcAddress(module, "lldiv");
-    p__isctype = (void*)GetProcAddress(module, "_isctype");
-    p_isblank = (void*)GetProcAddress(module, "isblank");
-    p__isblank_l = (void*)GetProcAddress(module, "_isblank_l");
-    p__iswctype_l = (void*)GetProcAddress(module, "_iswctype_l");
-    p_iswblank = (void*)GetProcAddress(module, "iswblank");
-    p__iswblank_l = (void*)GetProcAddress(module, "_iswblank_l");
-    p_fesetround = (void*)GetProcAddress(module, "fesetround");
-    p___setusermatherr = (void*)GetProcAddress(module, "__setusermatherr");
-    p_errno = (void*)GetProcAddress(module, "_errno");
-    p_asctime = (void*)GetProcAddress(module, "asctime");
-    p_strftime = (void*)GetProcAddress(module, "strftime");
-    p__Strftime = (void*)GetProcAddress(module, "_Strftime");
-    p_setlocale = (void*)GetProcAddress(module, "setlocale");
-    p__gmtime32 = (void*)GetProcAddress(module, "_gmtime32");
-    p__crt_atexit = (void*)GetProcAddress(module, "_crt_atexit");
-    p_exit = (void*)GetProcAddress(module, "exit");
-    p_crt_at_quick_exit = (void*)GetProcAddress(module, "_crt_at_quick_exit");
-    p_quick_exit = (void*)GetProcAddress(module, "quick_exit");
-    p__stat32 = (void*)GetProcAddress(module, "_stat32");
-    p__close = (void*)GetProcAddress(module, "_close");
-    p__o_malloc = (void*)GetProcAddress(module, "_o_malloc");
-    p__msize = (void*)GetProcAddress(module, "_msize");
-    p_free = (void*)GetProcAddress(module, "free");
-    p_clock = (void*)GetProcAddress(module, "clock");
-
-    return TRUE;
-}
-
 static void test__sopen_dispatch(void)
 {
     int ret, fd;
@@ -564,21 +434,21 @@ static void test__sopen_dispatch(void)
     tempf = _tempnam(".", "wne");
 
     fd = 0;
-    ret = p_sopen_dispatch(tempf, _O_CREAT, _SH_DENYWR, 0xff, &fd, 0);
+    ret = _sopen_dispatch(tempf, _O_CREAT, _SH_DENYWR, 0xff, &fd, 0);
     ok(!ret, "got %d\n", ret);
     ok(fd > 0, "got fd %d\n", fd);
-    p__close(fd);
+    _close(fd);
     unlink(tempf);
 
     SET_EXPECT(global_invalid_parameter_handler);
     fd = 0;
-    ret = p_sopen_dispatch(tempf, _O_CREAT, _SH_DENYWR, 0xff, &fd, 1);
+    ret = _sopen_dispatch(tempf, _O_CREAT, _SH_DENYWR, 0xff, &fd, 1);
     ok(ret == EINVAL, "got %d\n", ret);
     ok(fd == -1, "got fd %d\n", fd);
     CHECK_CALLED(global_invalid_parameter_handler);
     if (fd > 0)
     {
-        p__close(fd);
+        _close(fd);
         unlink(tempf);
     }
 
@@ -593,22 +463,22 @@ static void test__sopen_s(void)
     tempf = _tempnam(".", "wne");
 
     fd = 0;
-    ret = p_sopen_s(&fd, tempf, _O_CREAT, _SH_DENYWR, 0);
+    ret = _sopen_s(&fd, tempf, _O_CREAT, _SH_DENYWR, 0);
     ok(!ret, "got %d\n", ret);
     ok(fd > 0, "got fd %d\n", fd);
-    p__close(fd);
+    _close(fd);
     unlink(tempf);
 
     /* _open() does not validate pmode */
-    fd = p__open(tempf, _O_CREAT, 0xff);
+    fd = _open(tempf, _O_CREAT, 0xff);
     ok(fd > 0, "got fd %d\n", fd);
-    p__close(fd);
+    _close(fd);
     unlink(tempf);
 
     /* _sopen_s() invokes invalid parameter handler on invalid pmode */
     SET_EXPECT(global_invalid_parameter_handler);
     fd = 0;
-    ret = p_sopen_s(&fd, tempf, _O_CREAT, _SH_DENYWR, 0xff);
+    ret = _sopen_s(&fd, tempf, _O_CREAT, _SH_DENYWR, 0xff);
     ok(ret == EINVAL, "got %d\n", ret);
     ok(fd == -1, "got fd %d\n", fd);
     CHECK_CALLED(global_invalid_parameter_handler);
@@ -618,7 +488,8 @@ static void test__sopen_s(void)
 
 static void test_lldiv(void)
 {
-    MSVCRT_lldiv_t r;
+    static lldiv_t* (CDECL *p_lldiv)(lldiv_t*,LONGLONG,LONGLONG) = (void*)lldiv;
+    lldiv_t r;
 
     p_lldiv(&r, (LONGLONG)0x111 << 32 | 0x222, (LONGLONG)1 << 32);
     ok(r.quot == 0x111, "quot = %s\n", wine_dbgstr_longlong(r.quot));
@@ -632,41 +503,41 @@ static void test_isblank(void)
     for(c = 0; c <= 0xff; c++) {
         if(c == '\t' || c == ' ') {
             if(c == '\t')
-                ok(!p__isctype(c, _BLANK), "tab shouldn't be blank\n");
+                ok(!_isctype(c, _BLANK), "tab shouldn't be blank\n");
             else
-                ok(p__isctype(c, _BLANK), "space should be blank\n");
-            ok(p_isblank(c), "%d should be blank\n", c);
-            ok(p__isblank_l(c, NULL), "%d should be blank\n", c);
+                ok(_isctype(c, _BLANK), "space should be blank\n");
+            ok(isblank(c), "%d should be blank\n", c);
+            ok(_isblank_l(c, NULL), "%d should be blank\n", c);
         } else {
-            ok(!p__isctype(c, _BLANK), "%d shouldn't be blank\n", c);
-            ok(!p_isblank(c), "%d shouldn't be blank\n", c);
-            ok(!p__isblank_l(c, NULL), "%d shouldn't be blank\n", c);
+            ok(!_isctype(c, _BLANK), "%d shouldn't be blank\n", c);
+            ok(!isblank(c), "%d shouldn't be blank\n", c);
+            ok(!_isblank_l(c, NULL), "%d shouldn't be blank\n", c);
         }
     }
 
     for(c = 0; c <= 0xffff; c++) {
         if(c == '\t' || c == ' ' || c == 0x3000 || c == 0xfeff) {
             if(c == '\t')
-                todo_wine ok(!p__iswctype_l(c, _BLANK, NULL), "tab shouldn't be blank\n");
+                todo_wine ok(!_iswctype_l(c, _BLANK, NULL), "tab shouldn't be blank\n");
             else
-                ok(p__iswctype_l(c, _BLANK, NULL), "%d should be blank\n", c);
-            ok(p_iswblank(c), "%d should be blank\n", c);
-            ok(p__iswblank_l(c, NULL), "%d should be blank\n", c);
+                ok(_iswctype_l(c, _BLANK, NULL), "%d should be blank\n", c);
+            ok(iswblank(c), "%d should be blank\n", c);
+            ok(_iswblank_l(c, NULL), "%d should be blank\n", c);
         } else {
             todo_wine_if(c == 0xa0) {
-                ok(!p__iswctype_l(c, _BLANK, NULL), "%d shouldn't be blank\n", c);
-                ok(!p_iswblank(c), "%d shouldn't be blank\n", c);
-                ok(!p__iswblank_l(c, NULL), "%d shouldn't be blank\n", c);
+                ok(!_iswctype_l(c, _BLANK, NULL), "%d shouldn't be blank\n", c);
+                ok(!iswblank(c), "%d shouldn't be blank\n", c);
+                ok(!_iswblank_l(c, NULL), "%d shouldn't be blank\n", c);
             }
         }
     }
 }
 
-static struct MSVCRT__exception exception;
+static struct _exception exception;
 
-static int CDECL matherr_callback(struct MSVCRT__exception *e)
+static int CDECL matherr_callback(struct _exception *e)
 {
-    memcpy(&exception, e, sizeof(MSVCRT__exception));
+    exception = *e;
     return 0;
 }
 
@@ -839,20 +710,22 @@ static void test_math_errors(void)
     double (CDECL *p_func2d)(double, double);
     double (CDECL *p_func3d)(double, double, double);
     double (CDECL *p_funcdl)(double, long);
+    HMODULE module;
     int i;
 
-    p___setusermatherr(matherr_callback);
+    __setusermatherr(matherr_callback);
+    module = GetModuleHandleW(L"ucrtbase.dll");
 
     /* necessary so that exp(1e100)==INFINITY on glibc, we can remove this if we change our implementation */
-    p_fesetround(FE_TONEAREST);
+    fesetround(FE_TONEAREST);
 
     for(i = 0; i < ARRAY_SIZE(testsd); i++) {
         p_funcd = (void*)GetProcAddress(module, testsd[i].func);
-        *p_errno() = -1;
+        errno = -1;
         exception.type = -1;
         p_funcd(testsd[i].x);
-        ok(*p_errno() == testsd[i].error,
-           "%s(%f) got errno %d\n", testsd[i].func, testsd[i].x, *p_errno());
+        ok(errno == testsd[i].error,
+           "%s(%f) got errno %d\n", testsd[i].func, testsd[i].x, errno);
         ok(exception.type == testsd[i].exception,
            "%s(%f) got exception type %d\n", testsd[i].func, testsd[i].x, exception.type);
         if(exception.type == -1) continue;
@@ -862,11 +735,11 @@ static void test_math_errors(void)
 
     for(i = 0; i < ARRAY_SIZE(tests2d); i++) {
         p_func2d = (void*)GetProcAddress(module, tests2d[i].func);
-        *p_errno() = -1;
+        errno = -1;
         exception.type = -1;
         p_func2d(tests2d[i].a, tests2d[i].b);
-        ok(*p_errno() == tests2d[i].error,
-           "%s(%f, %f) got errno %d\n", tests2d[i].func, tests2d[i].a, tests2d[i].b, *p_errno());
+        ok(errno == tests2d[i].error,
+           "%s(%f, %f) got errno %d\n", tests2d[i].func, tests2d[i].a, tests2d[i].b, errno);
         ok(exception.type == tests2d[i].exception,
            "%s(%f, %f) got exception type %d\n", tests2d[i].func, tests2d[i].a, tests2d[i].b, exception.type);
         if(exception.type == -1) continue;
@@ -878,11 +751,11 @@ static void test_math_errors(void)
 
     for(i = 0; i < ARRAY_SIZE(tests3d); i++) {
         p_func3d = (void*)GetProcAddress(module, tests3d[i].func);
-        *p_errno() = -1;
+        errno = -1;
         exception.type = -1;
         p_func3d(tests3d[i].a, tests3d[i].b, tests3d[i].c);
-        ok(*p_errno() == tests3d[i].error,
-           "%s(%f, %f, %f) got errno %d\n", tests3d[i].func, tests3d[i].a, tests3d[i].b, tests3d[i].c, *p_errno());
+        ok(errno == tests3d[i].error,
+           "%s(%f, %f, %f) got errno %d\n", tests3d[i].func, tests3d[i].a, tests3d[i].b, tests3d[i].c, errno);
         ok(exception.type == tests3d[i].exception,
            "%s(%f, %f, %f) got exception type %d\n", tests3d[i].func, tests3d[i].a, tests3d[i].b, tests3d[i].c, exception.type);
         if(exception.type == -1) continue;
@@ -894,11 +767,11 @@ static void test_math_errors(void)
 
     for(i = 0; i < ARRAY_SIZE(testsdl); i++) {
         p_funcdl = (void*)GetProcAddress(module, testsdl[i].func);
-        *p_errno() = -1;
+        errno = -1;
         exception.type = -1;
         p_funcdl(testsdl[i].a, testsdl[i].b);
-        ok(*p_errno() == testsdl[i].error,
-           "%s(%f, %ld) got errno %d\n", testsdl[i].func, testsdl[i].a, testsdl[i].b, *p_errno());
+        ok(errno == testsdl[i].error,
+           "%s(%f, %ld) got errno %d\n", testsdl[i].func, testsdl[i].a, testsdl[i].b, errno);
         ok(exception.type == testsdl[i].exception,
            "%s(%f, %ld) got exception type %d\n", testsdl[i].func, testsdl[i].a, testsdl[i].b, exception.type);
         if(exception.type == -1) continue;
@@ -914,13 +787,7 @@ static void test_asctime(void)
     const struct tm epoch = { 0, 0, 0, 1, 0, 70, 4, 0, 0 };
     char *ret;
 
-    if(!p_asctime)
-    {
-        win_skip("asctime is not available\n");
-        return;
-    }
-
-    ret = p_asctime(&epoch);
+    ret = asctime(&epoch);
     ok(!strcmp(ret, "Thu Jan  1 00:00:00 1970\n"), "asctime returned %s\n", ret);
 }
 
@@ -1144,7 +1011,7 @@ static void test_strftime(void)
         todo_wine_if(tests[i].todo_handler) {
             if (!tests[i].ret[0])
                 SET_EXPECT(global_invalid_parameter_handler);
-            ret = p_strftime(buf, sizeof(buf), tests[i].format, &tests[i].tm);
+            ret = strftime(buf, sizeof(buf), tests[i].format, &tests[i].tm);
             if (!tests[i].ret[0])
                 CHECK_CALLED(global_invalid_parameter_handler);
         }
@@ -1156,7 +1023,7 @@ static void test_strftime(void)
         }
     }
 
-    ret = p_strftime(buf, sizeof(buf), "%z", &epoch);
+    ret = strftime(buf, sizeof(buf), "%z", &epoch);
     ok(ret == 5, "expected 5, got %d\n", ret);
     ok((buf[0] == '+' || buf[0] == '-') &&
         isdigit(buf[1]) && isdigit(buf[2]) &&
@@ -1167,7 +1034,7 @@ static void test_strftime(void)
         time_data.short_dateW = tests_td[i].short_date;
         time_data.dateW = tests_td[i].date;
         time_data.timeW = tests_td[i].time;
-        ret = p__Strftime(buf, sizeof(buf), tests_td[i].format, &tests_td[i].tm, &time_data);
+        ret = _Strftime(buf, sizeof(buf), tests_td[i].format, &tests_td[i].tm, &time_data);
         ok(ret == strlen(buf), "%d) ret = %d\n", i, ret);
         todo_wine_if(tests_td[i].todo) {
             ok(!strcmp(buf, tests_td[i].ret), "%d) buf = \"%s\", expected \"%s\"\n",
@@ -1183,23 +1050,23 @@ static void test_strftime(void)
         for (j=0; j<7; j++)
         {
             tm_yweek.tm_wday = j;
-            p_strftime(buf, sizeof(buf), "%g %V", &tm_yweek);
+            strftime(buf, sizeof(buf), "%g %V", &tm_yweek);
             ok(!strcmp(buf, tests_yweek[i].ret[j]), "%d,%d) buf = \"%s\", expected \"%s\"\n",
                     i, j, buf, tests_yweek[i].ret[j]);
         }
     }
 
-    if(!p_setlocale(LC_ALL, "fr-FR")) {
+    if(!setlocale(LC_ALL, "fr-FR")) {
         win_skip("fr-FR locale not available\n");
         return;
     }
-    ret = p_strftime(buf, sizeof(buf), "%c", &epoch);
+    ret = strftime(buf, sizeof(buf), "%c", &epoch);
     ok(ret == 19, "ret = %d\n", ret);
     ok(!strcmp(buf, "01/01/1970 00:00:00"), "buf = \"%s\", expected \"%s\"\n", buf, "01/01/1970 00:00:00");
-    ret = p_strftime(buf, sizeof(buf), "%r", &epoch);
+    ret = strftime(buf, sizeof(buf), "%r", &epoch);
     ok(ret == 8, "ret = %d\n", ret);
     ok(!strcmp(buf, "00:00:00"), "buf = \"%s\", expected \"%s\"\n", buf, "00:00:00");
-    p_setlocale(LC_ALL, "C");
+    setlocale(LC_ALL, "C");
 }
 
 static LONG* get_failures_counter(HANDLE *map)
@@ -1304,26 +1171,26 @@ static void CDECL at_quick_exit_func2(void)
 
 static void test_call_exit(void)
 {
-    ok(!p__crt_atexit(at_exit_func1), "_crt_atexit failed\n");
-    ok(!p__crt_atexit(at_exit_func2), "_crt_atexit failed\n");
+    ok(!_crt_atexit(at_exit_func1), "_crt_atexit failed\n");
+    ok(!_crt_atexit(at_exit_func2), "_crt_atexit failed\n");
 
-    ok(!p_crt_at_quick_exit(at_quick_exit_func1), "_crt_at_quick_exit failed\n");
-    ok(!p_crt_at_quick_exit(at_quick_exit_func2), "_crt_at_quick_exit failed\n");
+    ok(!_crt_at_quick_exit(at_quick_exit_func1), "_crt_at_quick_exit failed\n");
+    ok(!_crt_at_quick_exit(at_quick_exit_func2), "_crt_at_quick_exit failed\n");
 
     set_failures_counter(winetest_get_failures());
-    p_exit(1);
+    exit(1);
 }
 
 static void test_call_quick_exit(void)
 {
-    ok(!p__crt_atexit(at_exit_func1), "_crt_atexit failed\n");
-    ok(!p__crt_atexit(at_exit_func2), "_crt_atexit failed\n");
+    ok(!_crt_atexit(at_exit_func1), "_crt_atexit failed\n");
+    ok(!_crt_atexit(at_exit_func2), "_crt_atexit failed\n");
 
-    ok(!p_crt_at_quick_exit(at_quick_exit_func1), "_crt_at_quick_exit failed\n");
-    ok(!p_crt_at_quick_exit(at_quick_exit_func2), "_crt_at_quick_exit failed\n");
+    ok(!_crt_at_quick_exit(at_quick_exit_func1), "_crt_at_quick_exit failed\n");
+    ok(!_crt_at_quick_exit(at_quick_exit_func2), "_crt_at_quick_exit failed\n");
 
     set_failures_counter(winetest_get_failures());
-    p_quick_exit(2);
+    quick_exit(2);
 }
 
 static void test_quick_exit(const char *argv0)
@@ -1373,18 +1240,18 @@ static void test__stat32(void)
     len = GetTempPathA(MAX_PATH, path);
     ok(len, "GetTempPathA failed\n");
 
-    ret = p__stat32("c:", &buf);
+    ret = _stat32("c:", &buf);
     ok(ret == -1, "_stat32('c:') returned %d\n", ret);
-    ret = p__stat32("c:\\", &buf);
+    ret = _stat32("c:\\", &buf);
     ok(!ret, "_stat32('c:\\') returned %d\n", ret);
 
     memcpy(path+len, test_file, sizeof(test_file));
     if((fd = open(path, O_WRONLY | O_CREAT | O_BINARY, _S_IREAD |_S_IWRITE)) >= 0)
     {
-        ret = p__stat32(path, &buf);
+        ret = _stat32(path, &buf);
         ok(!ret, "_stat32('%s') returned %d\n", path, ret);
         strcat(path, "\\");
-        ret = p__stat32(path, &buf);
+        ret = _stat32(path, &buf);
         todo_wine ok(ret, "_stat32('%s') returned %d\n", path, ret);
         close(fd);
         remove(path);
@@ -1393,10 +1260,10 @@ static void test__stat32(void)
     memcpy(path+len, test_dir, sizeof(test_dir));
     if(!mkdir(path))
     {
-        ret = p__stat32(path, &buf);
+        ret = _stat32(path, &buf);
         ok(!ret, "_stat32('%s') returned %d\n", path, ret);
         strcat(path, "\\");
-        ret = p__stat32(path, &buf);
+        ret = _stat32(path, &buf);
         ok(!ret, "_stat32('%s') returned %d\n", path, ret);
         rmdir(path);
     }
@@ -1407,37 +1274,37 @@ static void test__o_malloc(void)
     void *m;
     size_t s;
 
-    m = p__o_malloc(1);
+    m = _o_malloc(1);
     ok(m != NULL, "p__o_malloc(1) returned NULL\n");
 
-    s = p__msize(m);
+    s = _msize(m);
     ok(s == 1, "_msize returned %d\n", (int)s);
 
-    p_free(m);
+    free(m);
 }
 
 static void test_clock(void)
 {
-    static const int thresh = 100;
-    int c, expect_min, expect_max;
+    static const int thresh = 100, max_load_delay = 1000;
+    int c, expect_min;
     FILETIME cur;
 
     GetSystemTimeAsFileTime(&cur);
-    c = p_clock();
+    c = clock();
 
     expect_min = (((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime - crt_init_end) / 10000;
-    expect_max = (((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime - crt_init_start) / 10000;
-    ok(c > expect_min-thresh && c < expect_max+thresh, "clock() = %d, expected range [%d, %d]\n",
-            c, expect_min, expect_max);
+    ok(c >= expect_min - thresh && c < expect_min + max_load_delay, "clock() = %d, expected range [%d, %d]\n",
+            c, expect_min - thresh, expect_min + max_load_delay);
 }
 
 START_TEST(misc)
 {
     int arg_c;
     char** arg_v;
+    FILETIME cur;
 
-    if(!init())
-        return;
+    GetSystemTimeAsFileTime(&cur);
+    crt_init_end = ((LONGLONG)cur.dwHighDateTime << 32) + cur.dwLowDateTime;
 
     arg_c = winetest_get_mainargs(&arg_v);
     if(arg_c == 3) {
