@@ -41,7 +41,7 @@ static char driver_load_error[80];
 
 static BOOL CDECL nodrv_CreateWindow( HWND hwnd );
 
-static HMODULE load_desktop_driver( HWND hwnd )
+static BOOL load_desktop_driver( HWND hwnd, HMODULE *module )
 {
     static const WCHAR display_device_guid_propW[] = {
         '_','_','w','i','n','e','_','d','i','s','p','l','a','y','_',
@@ -53,7 +53,8 @@ static HMODULE load_desktop_driver( HWND hwnd )
         'V','i','d','e','o','\\','{',0};
     static const WCHAR displayW[] = {'}','\\','0','0','0','0',0};
     static const WCHAR driverW[] = {'G','r','a','p','h','i','c','s','D','r','i','v','e','r',0};
-    HMODULE ret = 0;
+    static const WCHAR nullW[] = {'n','u','l','l',0};
+    BOOL ret = FALSE;
     HKEY hkey;
     DWORD size;
     WCHAR path[MAX_PATH];
@@ -76,12 +77,15 @@ static HMODULE load_desktop_driver( HWND hwnd )
     size = sizeof(path);
     if (!RegQueryValueExW( hkey, driverW, NULL, NULL, (BYTE *)path, &size ))
     {
-        if (!(ret = LoadLibraryW( path ))) ERR( "failed to load %s\n", debugstr_w(path) );
-        TRACE( "%s %p\n", debugstr_w(path), ret );
+        if ((ret = !strcmpW( path, nullW ))) *module = NULL;
+        else ret = (*module = LoadLibraryW( path )) != NULL;
+        if (!ret) ERR( "failed to load %s\n", debugstr_w(path) );
+        TRACE( "%s %p\n", debugstr_w(path), *module );
     }
     else
     {
         size = sizeof(driver_load_error);
+        *module = NULL;
         RegQueryValueExA( hkey, "DriverError", NULL, NULL, (BYTE *)driver_load_error, &size );
     }
     RegCloseKey( hkey );
@@ -98,15 +102,23 @@ static const USER_DRIVER *load_driver(void)
     driver = HeapAlloc( GetProcessHeap(), 0, sizeof(*driver) );
     *driver = null_driver;
 
-    graphics_driver = load_desktop_driver( GetDesktopWindow() );
-    if (graphics_driver)
+    if (!load_desktop_driver( GetDesktopWindow(), &graphics_driver ))
+    {
+        USEROBJECTFLAGS flags;
+        HWINSTA winstation;
+
+        winstation = GetProcessWindowStation();
+        if (!GetUserObjectInformationA(winstation, UOI_FLAGS, &flags, sizeof(flags), NULL)
+            || (flags.dwFlags & WSF_VISIBLE))
+            driver->pCreateWindow = nodrv_CreateWindow;
+    }
+    else if (graphics_driver)
     {
 #define GET_USER_FUNC(name) \
     do { if ((ptr = GetProcAddress( graphics_driver, #name ))) driver->p##name = ptr; } while(0)
 
         GET_USER_FUNC(ActivateKeyboardLayout);
         GET_USER_FUNC(Beep);
-        GET_USER_FUNC(GetAsyncKeyState);
         GET_USER_FUNC(GetKeyNameText);
         GET_USER_FUNC(GetKeyboardLayout);
         GET_USER_FUNC(GetKeyboardLayoutList);
@@ -154,16 +166,6 @@ static const USER_DRIVER *load_driver(void)
         GET_USER_FUNC(ThreadDetach);
 #undef GET_USER_FUNC
     }
-    else
-    {
-        USEROBJECTFLAGS flags;
-        HWINSTA winstation;
-
-        winstation = GetProcessWindowStation();
-        if (!GetUserObjectInformationA(winstation, UOI_FLAGS, &flags, sizeof(flags), NULL)
-            || (flags.dwFlags & WSF_VISIBLE))
-            driver->pCreateWindow = nodrv_CreateWindow;
-    }
 
     prev = InterlockedCompareExchangePointer( (void **)&USER_Driver, driver, &lazy_load_driver );
     if (prev != &lazy_load_driver)
@@ -206,11 +208,6 @@ static void CDECL nulldrv_Beep(void)
 {
 }
 
-static SHORT CDECL nulldrv_GetAsyncKeyState( INT key )
-{
-    return -1;
-}
-
 static UINT CDECL nulldrv_GetKeyboardLayoutList( INT size, HKL *layouts )
 {
     HKEY hKeyKeyboard;
@@ -494,7 +491,6 @@ static USER_DRIVER null_driver =
     /* keyboard functions */
     nulldrv_ActivateKeyboardLayout,
     nulldrv_Beep,
-    nulldrv_GetAsyncKeyState,
     nulldrv_GetKeyNameText,
     nulldrv_GetKeyboardLayout,
     nulldrv_GetKeyboardLayoutList,
@@ -566,11 +562,6 @@ static void CDECL loaderdrv_Beep(void)
     load_driver()->pBeep();
 }
 
-static SHORT CDECL loaderdrv_GetAsyncKeyState( INT key )
-{
-    return load_driver()->pGetAsyncKeyState( key );
-}
-
 static INT CDECL loaderdrv_GetKeyNameText( LONG lparam, LPWSTR buffer, INT size )
 {
     return load_driver()->pGetKeyNameText( lparam, buffer, size );
@@ -715,7 +706,6 @@ static USER_DRIVER lazy_load_driver =
     /* keyboard functions */
     loaderdrv_ActivateKeyboardLayout,
     loaderdrv_Beep,
-    loaderdrv_GetAsyncKeyState,
     loaderdrv_GetKeyNameText,
     loaderdrv_GetKeyboardLayout,
     loaderdrv_GetKeyboardLayoutList,
