@@ -865,7 +865,6 @@ static BOOL MDI_AugmentFrameMenu( HWND frame, HWND hChild )
     {
       HDC hMemDC;
       HBITMAP hBitmap, hOldBitmap;
-      HBRUSH hBrush;
       HDC hdc = GetDC(hChild);
 
       if (hdc)
@@ -877,10 +876,8 @@ static BOOL MDI_AugmentFrameMenu( HWND frame, HWND hChild )
         hBitmap = CreateCompatibleBitmap(hdc, cx, cy);
         hOldBitmap = SelectObject(hMemDC, hBitmap);
         SetMapMode(hMemDC, MM_TEXT);
-        hBrush = CreateSolidBrush(GetSysColor(COLOR_MENU));
-        DrawIconEx(hMemDC, 0, 0, hIcon, cx, cy, 0, hBrush, DI_NORMAL);
+        DrawIconEx(hMemDC, 0, 0, hIcon, cx, cy, 0, GetSysColorBrush(COLOR_MENU), DI_NORMAL);
         SelectObject (hMemDC, hOldBitmap);
-        DeleteObject(hBrush);
         DeleteDC(hMemDC);
         ReleaseDC(hChild, hdc);
         hSysMenuBitmap = hBitmap;
