@@ -255,8 +255,11 @@ static UINT WPRINTF_GetLen( WPRINTF_FORMAT *format, WPRINTF_DATA *arg,
     switch(format->type)
     {
     case WPR_CHAR:
-    case WPR_WCHAR:
         return (format->precision = 1);
+    case WPR_WCHAR:
+        if (dst_is_wide) len = 1;
+        else len = WideCharToMultiByte( CP_ACP, 0, &arg->wchar_view, 1, NULL, 0, NULL, NULL );
+        return (format->precision = len);
     case WPR_STRING:
         if (!arg->lpcstr_view) arg->lpcstr_view = null_stringA;
         if (dst_is_wide)
@@ -387,7 +390,14 @@ static INT wvsnprintfA( LPSTR buffer, UINT maxlen, LPCSTR spec, __ms_va_list arg
         switch(format.type)
         {
         case WPR_WCHAR:
-            *p++ = argData.wchar_view;
+            {
+                CHAR mb[5];
+                if (WideCharToMultiByte( CP_ACP, 0, &argData.wchar_view, 1, mb, sizeof(mb), NULL, NULL ))
+                {
+                    memcpy( p, mb, len );
+                    p += len;
+                }
+            }
             break;
         case WPR_CHAR:
             *p++ = argData.char_view;
@@ -503,8 +513,15 @@ static INT wvsnprintfW( LPWSTR buffer, UINT maxlen, LPCWSTR spec, __ms_va_list a
             *p++ = argData.wchar_view;
             break;
         case WPR_CHAR:
-            *p++ = argData.char_view;
-            break;
+            {
+                WCHAR wc;
+                if (!IsDBCSLeadByte( (BYTE)argData.char_view )
+                    && MultiByteToWideChar( CP_ACP, 0, &argData.char_view, 1, &wc, 1 ) > 0)
+                    *p++ = wc;
+                else
+                    *p++ = 0;
+                break;
+            }
         case WPR_STRING:
             {
                 LPCSTR ptr = argData.lpcstr_view;
