@@ -181,16 +181,6 @@ static HRESULT lookup_identifier(exec_ctx_t *ctx, BSTR name, vbdisp_invoke_type_
         }
     }
 
-    if(ctx->func->code_ctx->context) {
-        hres = disp_get_id(ctx->func->code_ctx->context, name, invoke_type, TRUE, &id);
-        if(SUCCEEDED(hres)) {
-            ref->type = REF_DISP;
-            ref->u.d.disp = ctx->func->code_ctx->context;
-            ref->u.d.id = id;
-            return S_OK;
-        }
-    }
-
     if(ctx->code->named_item) {
         if(lookup_global_vars(ctx->code->named_item->script_obj, name, ref))
             return S_OK;
@@ -198,6 +188,16 @@ static HRESULT lookup_identifier(exec_ctx_t *ctx, BSTR name, vbdisp_invoke_type_
             return S_OK;
     }
 
+    if(ctx->func->code_ctx->named_item && ctx->func->code_ctx->named_item->disp) {
+        hres = disp_get_id(ctx->func->code_ctx->named_item->disp, name, invoke_type, TRUE, &id);
+        if(SUCCEEDED(hres)) {
+            ref->type = REF_DISP;
+            ref->u.d.disp = ctx->func->code_ctx->named_item->disp;
+            ref->u.d.id = id;
+            return S_OK;
+        }
+    }
+
     if(lookup_global_vars(script_obj, name, ref))
         return S_OK;
     if(lookup_global_funcs(script_obj, name, ref))
