@@ -24,6 +24,8 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(vbscript);
 
+static const GUID GUID_VBScriptTypeInfo = {0xc59c6b12,0xf6c1,0x11cf,{0x88,0x35,0x00,0xa0,0xc9,0x11,0xe8,0xb2}};
+
 #define DISPID_FUNCTION_MASK 0x20000000
 #define FDEX_VERSION_MASK 0xf0000000
 
@@ -527,22 +529,61 @@ HRESULT create_vbdisp(const class_desc_t *desc, vbdisp_t **ret)
     return S_OK;
 }
 
+struct typeinfo_func {
+    function_t *func;
+    MEMBERID memid;
+};
+
 typedef struct {
     ITypeInfo ITypeInfo_iface;
+    ITypeComp ITypeComp_iface;
     LONG ref;
+
+    UINT num_vars;
+    UINT num_funcs;
+    struct typeinfo_func *funcs;
+
+    ScriptDisp *disp;
 } ScriptTypeInfo;
 
+static function_t *get_func_from_memid(const ScriptTypeInfo *typeinfo, MEMBERID memid)
+{
+    UINT a = 0, b = typeinfo->num_funcs;
+
+    if (!(memid & DISPID_FUNCTION_MASK)) return NULL;
+
+    while (a < b)
+    {
+        UINT i = (a + b - 1) / 2;
+
+        if (memid == typeinfo->funcs[i].memid)
+            return typeinfo->funcs[i].func;
+        else if (memid < typeinfo->funcs[i].memid)
+            b = i;
+        else
+            a = i + 1;
+    }
+    return NULL;
+}
+
 static inline ScriptTypeInfo *ScriptTypeInfo_from_ITypeInfo(ITypeInfo *iface)
 {
     return CONTAINING_RECORD(iface, ScriptTypeInfo, ITypeInfo_iface);
 }
 
+static inline ScriptTypeInfo *ScriptTypeInfo_from_ITypeComp(ITypeComp *iface)
+{
+    return CONTAINING_RECORD(iface, ScriptTypeInfo, ITypeComp_iface);
+}
+
 static HRESULT WINAPI ScriptTypeInfo_QueryInterface(ITypeInfo *iface, REFIID riid, void **ppv)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
 
     if (IsEqualGUID(&IID_IUnknown, riid) || IsEqualGUID(&IID_ITypeInfo, riid))
         *ppv = &This->ITypeInfo_iface;
+    else if (IsEqualGUID(&IID_ITypeComp, riid))
+        *ppv = &This->ITypeComp_iface;
     else
     {
         WARN("(%p)->(%s %p)\n", This, debugstr_guid(riid), ppv);
@@ -569,11 +610,17 @@ static ULONG WINAPI ScriptTypeInfo_Release(ITypeInfo *iface)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
     LONG ref = InterlockedDecrement(&This->ref);
+    UINT i;
 
     TRACE("(%p) ref=%d\n", This, ref);
 
     if (!ref)
     {
+        for (i = 0; i < This->num_funcs; i++)
+            release_vbscode(This->funcs[i].func->code_ctx);
+
+        IDispatchEx_Release(&This->disp->IDispatchEx_iface);
+        heap_free(This->funcs);
         heap_free(This);
     }
     return ref;
@@ -582,142 +629,408 @@ static ULONG WINAPI ScriptTypeInfo_Release(ITypeInfo *iface)
 static HRESULT WINAPI ScriptTypeInfo_GetTypeAttr(ITypeInfo *iface, TYPEATTR **ppTypeAttr)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    TYPEATTR *attr;
 
-    FIXME("(%p)->(%p)\n", This, ppTypeAttr);
+    TRACE("(%p)->(%p)\n", This, ppTypeAttr);
 
-    return E_NOTIMPL;
+    if (!ppTypeAttr) return E_INVALIDARG;
+
+    attr = heap_alloc_zero(sizeof(*attr));
+    if (!attr) return E_OUTOFMEMORY;
+
+    attr->guid = GUID_VBScriptTypeInfo;
+    attr->lcid = LOCALE_USER_DEFAULT;
+    attr->memidConstructor = MEMBERID_NIL;
+    attr->memidDestructor = MEMBERID_NIL;
+    attr->cbSizeInstance = 4;
+    attr->typekind = TKIND_DISPATCH;
+    attr->cFuncs = This->num_funcs;
+    attr->cVars = This->num_vars;
+    attr->cImplTypes = 1;
+    attr->cbSizeVft = sizeof(IDispatchVtbl);
+    attr->cbAlignment = 4;
+    attr->wTypeFlags = TYPEFLAG_FDISPATCHABLE;
+    attr->wMajorVerNum = VBSCRIPT_MAJOR_VERSION;
+    attr->wMinorVerNum = VBSCRIPT_MINOR_VERSION;
+
+    *ppTypeAttr = attr;
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetTypeComp(ITypeInfo *iface, ITypeComp **ppTComp)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
 
-    FIXME("(%p)->(%p)\n", This, ppTComp);
+    TRACE("(%p)->(%p)\n", This, ppTComp);
 
-    return E_NOTIMPL;
+    if (!ppTComp) return E_INVALIDARG;
+
+    *ppTComp = &This->ITypeComp_iface;
+    ITypeInfo_AddRef(iface);
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetFuncDesc(ITypeInfo *iface, UINT index, FUNCDESC **ppFuncDesc)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    function_t *func;
+    FUNCDESC *desc;
+    UINT i;
 
-    FIXME("(%p)->(%u %p)\n", This, index, ppFuncDesc);
+    TRACE("(%p)->(%u %p)\n", This, index, ppFuncDesc);
 
-    return E_NOTIMPL;
+    if (!ppFuncDesc) return E_INVALIDARG;
+    if (index >= This->num_funcs) return TYPE_E_ELEMENTNOTFOUND;
+    func = This->funcs[index].func;
+
+    /* Store the parameter array after the FUNCDESC structure */
+    desc = heap_alloc_zero(sizeof(*desc) + sizeof(ELEMDESC) * func->arg_cnt);
+    if (!desc) return E_OUTOFMEMORY;
+
+    desc->memid = This->funcs[index].memid;
+    desc->funckind = FUNC_DISPATCH;
+    desc->invkind = INVOKE_FUNC;
+    desc->callconv = CC_STDCALL;
+    desc->cParams = func->arg_cnt;
+    desc->elemdescFunc.tdesc.vt = (func->type == FUNC_SUB) ? VT_VOID : VT_VARIANT;
+
+    if (func->arg_cnt) desc->lprgelemdescParam = (ELEMDESC*)(desc + 1);
+    for (i = 0; i < func->arg_cnt; i++)
+        desc->lprgelemdescParam[i].tdesc.vt = VT_VARIANT;
+
+    *ppFuncDesc = desc;
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetVarDesc(ITypeInfo *iface, UINT index, VARDESC **ppVarDesc)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    VARDESC *desc;
 
-    FIXME("(%p)->(%u %p)\n", This, index, ppVarDesc);
+    TRACE("(%p)->(%u %p)\n", This, index, ppVarDesc);
 
-    return E_NOTIMPL;
+    if (!ppVarDesc) return E_INVALIDARG;
+    if (index >= This->num_vars) return TYPE_E_ELEMENTNOTFOUND;
+
+    desc = heap_alloc_zero(sizeof(*desc));
+    if (!desc) return E_OUTOFMEMORY;
+
+    desc->memid = index + 1;
+    desc->varkind = VAR_DISPATCH;
+    desc->elemdescVar.tdesc.vt = VT_VARIANT;
+
+    *ppVarDesc = desc;
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetNames(ITypeInfo *iface, MEMBERID memid, BSTR *rgBstrNames,
         UINT cMaxNames, UINT *pcNames)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    ITypeInfo *disp_typeinfo;
+    function_t *func;
+    HRESULT hr;
+    UINT i = 0;
 
-    FIXME("(%p)->(%d %p %u %p)\n", This, memid, rgBstrNames, cMaxNames, pcNames);
+    TRACE("(%p)->(%d %p %u %p)\n", This, memid, rgBstrNames, cMaxNames, pcNames);
 
-    return E_NOTIMPL;
+    if (!rgBstrNames || !pcNames) return E_INVALIDARG;
+    if (memid <= 0) return TYPE_E_ELEMENTNOTFOUND;
+
+    func = get_func_from_memid(This, memid);
+    if (!func && memid > This->num_vars)
+    {
+        hr = get_dispatch_typeinfo(&disp_typeinfo);
+        if (FAILED(hr)) return hr;
+
+        return ITypeInfo_GetNames(disp_typeinfo, memid, rgBstrNames, cMaxNames, pcNames);
+    }
+
+    *pcNames = 0;
+    if (!cMaxNames) return S_OK;
+
+    if (func)
+    {
+        UINT num = min(cMaxNames, func->arg_cnt + 1);
+
+        rgBstrNames[0] = SysAllocString(func->name);
+        if (!rgBstrNames[0]) return E_OUTOFMEMORY;
+
+        for (i = 1; i < num; i++)
+        {
+            if (!(rgBstrNames[i] = SysAllocString(func->args[i - 1].name)))
+            {
+                do SysFreeString(rgBstrNames[--i]); while (i);
+                return E_OUTOFMEMORY;
+            }
+        }
+    }
+    else
+    {
+        rgBstrNames[0] = SysAllocString(This->disp->global_vars[memid - 1]->name);
+        if (!rgBstrNames[0]) return E_OUTOFMEMORY;
+        i++;
+    }
+
+    *pcNames = i;
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetRefTypeOfImplType(ITypeInfo *iface, UINT index, HREFTYPE *pRefType)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
 
-    FIXME("(%p)->(%u %p)\n", This, index, pRefType);
+    TRACE("(%p)->(%u %p)\n", This, index, pRefType);
 
-    return E_NOTIMPL;
+    /* We only inherit from IDispatch */
+    if (!pRefType) return E_INVALIDARG;
+    if (index != 0) return TYPE_E_ELEMENTNOTFOUND;
+
+    *pRefType = 1;
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetImplTypeFlags(ITypeInfo *iface, UINT index, INT *pImplTypeFlags)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
 
-    FIXME("(%p)->(%u %p)\n", This, index, pImplTypeFlags);
+    TRACE("(%p)->(%u %p)\n", This, index, pImplTypeFlags);
 
-    return E_NOTIMPL;
+    if (!pImplTypeFlags) return E_INVALIDARG;
+    if (index != 0) return TYPE_E_ELEMENTNOTFOUND;
+
+    *pImplTypeFlags = 0;
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetIDsOfNames(ITypeInfo *iface, LPOLESTR *rgszNames, UINT cNames,
         MEMBERID *pMemId)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    ITypeInfo *disp_typeinfo;
+    const WCHAR *name;
+    HRESULT hr = S_OK;
+    int i, j, arg;
 
-    FIXME("(%p)->(%p %u %p)\n", This, rgszNames, cNames, pMemId);
+    TRACE("(%p)->(%p %u %p)\n", This, rgszNames, cNames, pMemId);
 
-    return E_NOTIMPL;
+    if (!rgszNames || !cNames || !pMemId) return E_INVALIDARG;
+
+    for (i = 0; i < cNames; i++) pMemId[i] = MEMBERID_NIL;
+    name = rgszNames[0];
+
+    for (i = 0; i < This->num_funcs; i++)
+    {
+        function_t *func = This->funcs[i].func;
+
+        if (wcsicmp(name, func->name)) continue;
+        pMemId[0] = This->funcs[i].memid;
+
+        for (j = 1; j < cNames; j++)
+        {
+            name = rgszNames[j];
+            for (arg = func->arg_cnt; --arg >= 0;)
+                if (!wcsicmp(name, func->args[arg].name))
+                    break;
+            if (arg >= 0)
+                pMemId[j] = arg;
+            else
+                hr = DISP_E_UNKNOWNNAME;
+        }
+        return hr;
+    }
+
+    for (i = 0; i < This->num_vars; i++)
+    {
+        if (wcsicmp(name, This->disp->global_vars[i]->name)) continue;
+        pMemId[0] = i + 1;
+        return S_OK;
+    }
+
+    /* Look into the inherited IDispatch */
+    hr = get_dispatch_typeinfo(&disp_typeinfo);
+    if (FAILED(hr)) return hr;
+
+    return ITypeInfo_GetIDsOfNames(disp_typeinfo, rgszNames, cNames, pMemId);
 }
 
 static HRESULT WINAPI ScriptTypeInfo_Invoke(ITypeInfo *iface, PVOID pvInstance, MEMBERID memid, WORD wFlags,
         DISPPARAMS *pDispParams, VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    ITypeInfo *disp_typeinfo;
+    IDispatch *disp;
+    HRESULT hr;
 
-    FIXME("(%p)->(%p %d %d %p %p %p %p)\n", This, pvInstance, memid, wFlags,
+    TRACE("(%p)->(%p %d %d %p %p %p %p)\n", This, pvInstance, memid, wFlags,
           pDispParams, pVarResult, pExcepInfo, puArgErr);
 
-    return E_NOTIMPL;
+    if (!pvInstance) return E_INVALIDARG;
+    if (memid <= 0) return TYPE_E_ELEMENTNOTFOUND;
+
+    if (!get_func_from_memid(This, memid) && memid > This->num_vars)
+    {
+        hr = get_dispatch_typeinfo(&disp_typeinfo);
+        if (FAILED(hr)) return hr;
+
+        return ITypeInfo_Invoke(disp_typeinfo, pvInstance, memid, wFlags, pDispParams,
+                                pVarResult, pExcepInfo, puArgErr);
+    }
+
+    hr = IUnknown_QueryInterface((IUnknown*)pvInstance, &IID_IDispatch, (void**)&disp);
+    if (FAILED(hr)) return hr;
+
+    hr = IDispatch_Invoke(disp, memid, &IID_NULL, LOCALE_USER_DEFAULT, wFlags,
+                          pDispParams, pVarResult, pExcepInfo, puArgErr);
+    IDispatch_Release(disp);
+
+    return hr;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetDocumentation(ITypeInfo *iface, MEMBERID memid, BSTR *pBstrName,
         BSTR *pBstrDocString, DWORD *pdwHelpContext, BSTR *pBstrHelpFile)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    ITypeInfo *disp_typeinfo;
+    function_t *func;
+    HRESULT hr;
 
-    FIXME("(%p)->(%d %p %p %p %p)\n", This, memid, pBstrName, pBstrDocString, pdwHelpContext, pBstrHelpFile);
+    TRACE("(%p)->(%d %p %p %p %p)\n", This, memid, pBstrName, pBstrDocString, pdwHelpContext, pBstrHelpFile);
 
-    return E_NOTIMPL;
+    if (pBstrDocString) *pBstrDocString = NULL;
+    if (pdwHelpContext) *pdwHelpContext = 0;
+    if (pBstrHelpFile) *pBstrHelpFile = NULL;
+
+    if (memid == MEMBERID_NIL)
+    {
+        if (pBstrName && !(*pBstrName = SysAllocString(L"VBScriptTypeInfo")))
+            return E_OUTOFMEMORY;
+        if (pBstrDocString &&
+            !(*pBstrDocString = SysAllocString(L"Visual Basic Scripting Type Info")))
+        {
+            if (pBstrName) SysFreeString(*pBstrName);
+            return E_OUTOFMEMORY;
+        }
+        return S_OK;
+    }
+    if (memid <= 0) return TYPE_E_ELEMENTNOTFOUND;
+
+    func = get_func_from_memid(This, memid);
+    if (!func && memid > This->num_vars)
+    {
+        hr = get_dispatch_typeinfo(&disp_typeinfo);
+        if (FAILED(hr)) return hr;
+
+        return ITypeInfo_GetDocumentation(disp_typeinfo, memid, pBstrName, pBstrDocString,
+                                          pdwHelpContext, pBstrHelpFile);
+    }
+
+    if (pBstrName)
+    {
+        *pBstrName = SysAllocString(func ? func->name : This->disp->global_vars[memid - 1]->name);
+        if (!*pBstrName) return E_OUTOFMEMORY;
+    }
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetDllEntry(ITypeInfo *iface, MEMBERID memid, INVOKEKIND invKind,
         BSTR *pBstrDllName, BSTR *pBstrName, WORD *pwOrdinal)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    ITypeInfo *disp_typeinfo;
+    HRESULT hr;
 
-    FIXME("(%p)->(%d %d %p %p %p)\n", This, memid, invKind, pBstrDllName, pBstrName, pwOrdinal);
+    TRACE("(%p)->(%d %d %p %p %p)\n", This, memid, invKind, pBstrDllName, pBstrName, pwOrdinal);
 
-    return E_NOTIMPL;
+    if (pBstrDllName) *pBstrDllName = NULL;
+    if (pBstrName) *pBstrName = NULL;
+    if (pwOrdinal) *pwOrdinal = 0;
+
+    if (!get_func_from_memid(This, memid) && memid > This->num_vars)
+    {
+        hr = get_dispatch_typeinfo(&disp_typeinfo);
+        if (FAILED(hr)) return hr;
+
+        return ITypeInfo_GetDllEntry(disp_typeinfo, memid, invKind, pBstrDllName, pBstrName, pwOrdinal);
+    }
+    return TYPE_E_BADMODULEKIND;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetRefTypeInfo(ITypeInfo *iface, HREFTYPE hRefType, ITypeInfo **ppTInfo)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    HRESULT hr;
 
-    FIXME("(%p)->(%x %p)\n", This, hRefType, ppTInfo);
+    TRACE("(%p)->(%x %p)\n", This, hRefType, ppTInfo);
 
-    return E_NOTIMPL;
+    if (!ppTInfo || (INT)hRefType < 0) return E_INVALIDARG;
+
+    if (hRefType & ~3) return E_FAIL;
+    if (hRefType & 1)
+    {
+        hr = get_dispatch_typeinfo(ppTInfo);
+        if (FAILED(hr)) return hr;
+    }
+    else
+        *ppTInfo = iface;
+
+    ITypeInfo_AddRef(*ppTInfo);
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_AddressOfMember(ITypeInfo *iface, MEMBERID memid, INVOKEKIND invKind, PVOID *ppv)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    ITypeInfo *disp_typeinfo;
+    HRESULT hr;
 
-    FIXME("(%p)->(%d %d %p)\n", This, memid, invKind, ppv);
+    TRACE("(%p)->(%d %d %p)\n", This, memid, invKind, ppv);
 
-    return E_NOTIMPL;
+    if (!ppv) return E_INVALIDARG;
+    *ppv = NULL;
+
+    if (!get_func_from_memid(This, memid) && memid > This->num_vars)
+    {
+        hr = get_dispatch_typeinfo(&disp_typeinfo);
+        if (FAILED(hr)) return hr;
+
+        return ITypeInfo_AddressOfMember(disp_typeinfo, memid, invKind, ppv);
+    }
+    return TYPE_E_BADMODULEKIND;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_CreateInstance(ITypeInfo *iface, IUnknown *pUnkOuter, REFIID riid, PVOID *ppvObj)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
 
-    FIXME("(%p)->(%p %s %p)\n", This, pUnkOuter, debugstr_guid(riid), ppvObj);
+    TRACE("(%p)->(%p %s %p)\n", This, pUnkOuter, debugstr_guid(riid), ppvObj);
 
-    return E_NOTIMPL;
+    if (!ppvObj) return E_INVALIDARG;
+
+    *ppvObj = NULL;
+    return TYPE_E_BADMODULEKIND;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetMops(ITypeInfo *iface, MEMBERID memid, BSTR *pBstrMops)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
+    ITypeInfo *disp_typeinfo;
+    HRESULT hr;
 
-    FIXME("(%p)->(%d %p)\n", This, memid, pBstrMops);
+    TRACE("(%p)->(%d %p)\n", This, memid, pBstrMops);
 
-    return E_NOTIMPL;
+    if (!pBstrMops) return E_INVALIDARG;
+
+    if (!get_func_from_memid(This, memid) && memid > This->num_vars)
+    {
+        hr = get_dispatch_typeinfo(&disp_typeinfo);
+        if (FAILED(hr)) return hr;
+
+        return ITypeInfo_GetMops(disp_typeinfo, memid, pBstrMops);
+    }
+
+    *pBstrMops = NULL;
+    return S_OK;
 }
 
 static HRESULT WINAPI ScriptTypeInfo_GetContainingTypeLib(ITypeInfo *iface, ITypeLib **ppTLib, UINT *pIndex)
@@ -733,21 +1046,27 @@ static void WINAPI ScriptTypeInfo_ReleaseTypeAttr(ITypeInfo *iface, TYPEATTR *pT
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
 
-    FIXME("(%p)->(%p)\n", This, pTypeAttr);
+    TRACE("(%p)->(%p)\n", This, pTypeAttr);
+
+    heap_free(pTypeAttr);
 }
 
 static void WINAPI ScriptTypeInfo_ReleaseFuncDesc(ITypeInfo *iface, FUNCDESC *pFuncDesc)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
 
-    FIXME("(%p)->(%p)\n", This, pFuncDesc);
+    TRACE("(%p)->(%p)\n", This, pFuncDesc);
+
+    heap_free(pFuncDesc);
 }
 
 static void WINAPI ScriptTypeInfo_ReleaseVarDesc(ITypeInfo *iface, VARDESC *pVarDesc)
 {
     ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeInfo(iface);
 
-    FIXME("(%p)->(%p)\n", This, pVarDesc);
+    TRACE("(%p)->(%p)\n", This, pVarDesc);
+
+    heap_free(pVarDesc);
 }
 
 static const ITypeInfoVtbl ScriptTypeInfoVtbl = {
@@ -775,6 +1094,113 @@ static const ITypeInfoVtbl ScriptTypeInfoVtbl = {
     ScriptTypeInfo_ReleaseVarDesc
 };
 
+static HRESULT WINAPI ScriptTypeComp_QueryInterface(ITypeComp *iface, REFIID riid, void **ppv)
+{
+    ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeComp(iface);
+    return ITypeInfo_QueryInterface(&This->ITypeInfo_iface, riid, ppv);
+}
+
+static ULONG WINAPI ScriptTypeComp_AddRef(ITypeComp *iface)
+{
+    ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeComp(iface);
+    return ITypeInfo_AddRef(&This->ITypeInfo_iface);
+}
+
+static ULONG WINAPI ScriptTypeComp_Release(ITypeComp *iface)
+{
+    ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeComp(iface);
+    return ITypeInfo_Release(&This->ITypeInfo_iface);
+}
+
+static HRESULT WINAPI ScriptTypeComp_Bind(ITypeComp *iface, LPOLESTR szName, ULONG lHashVal, WORD wFlags,
+        ITypeInfo **ppTInfo, DESCKIND *pDescKind, BINDPTR *pBindPtr)
+{
+    ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeComp(iface);
+    UINT flags = wFlags ? wFlags : ~0;
+    ITypeInfo *disp_typeinfo;
+    ITypeComp *disp_typecomp;
+    HRESULT hr;
+    UINT i;
+
+    TRACE("(%p)->(%s %08x %d %p %p %p)\n", This, debugstr_w(szName), lHashVal,
+          wFlags, ppTInfo, pDescKind, pBindPtr);
+
+    if (!szName || !ppTInfo || !pDescKind || !pBindPtr)
+        return E_INVALIDARG;
+
+    for (i = 0; i < This->num_funcs; i++)
+    {
+        if (wcsicmp(szName, This->funcs[i].func->name)) continue;
+        if (!(flags & INVOKE_FUNC)) return TYPE_E_TYPEMISMATCH;
+
+        hr = ITypeInfo_GetFuncDesc(&This->ITypeInfo_iface, i, &pBindPtr->lpfuncdesc);
+        if (FAILED(hr)) return hr;
+
+        *pDescKind = DESCKIND_FUNCDESC;
+        *ppTInfo = &This->ITypeInfo_iface;
+        ITypeInfo_AddRef(*ppTInfo);
+        return S_OK;
+    }
+
+    for (i = 0; i < This->num_vars; i++)
+    {
+        if (wcsicmp(szName, This->disp->global_vars[i]->name)) continue;
+        if (!(flags & INVOKE_PROPERTYGET)) return TYPE_E_TYPEMISMATCH;
+
+        hr = ITypeInfo_GetVarDesc(&This->ITypeInfo_iface, i, &pBindPtr->lpvardesc);
+        if (FAILED(hr)) return hr;
+
+        *pDescKind = DESCKIND_VARDESC;
+        *ppTInfo = &This->ITypeInfo_iface;
+        ITypeInfo_AddRef(*ppTInfo);
+        return S_OK;
+    }
+
+    /* Look into the inherited IDispatch */
+    hr = get_dispatch_typeinfo(&disp_typeinfo);
+    if (FAILED(hr)) return hr;
+
+    hr = ITypeInfo_GetTypeComp(disp_typeinfo, &disp_typecomp);
+    if (FAILED(hr)) return hr;
+
+    hr = ITypeComp_Bind(disp_typecomp, szName, lHashVal, wFlags, ppTInfo, pDescKind, pBindPtr);
+    ITypeComp_Release(disp_typecomp);
+    return hr;
+}
+
+static HRESULT WINAPI ScriptTypeComp_BindType(ITypeComp *iface, LPOLESTR szName, ULONG lHashVal,
+        ITypeInfo **ppTInfo, ITypeComp **ppTComp)
+{
+    ScriptTypeInfo *This = ScriptTypeInfo_from_ITypeComp(iface);
+    ITypeInfo *disp_typeinfo;
+    ITypeComp *disp_typecomp;
+    HRESULT hr;
+
+    TRACE("(%p)->(%s %08x %p %p)\n", This, debugstr_w(szName), lHashVal, ppTInfo, ppTComp);
+
+    if (!szName || !ppTInfo || !ppTComp)
+        return E_INVALIDARG;
+
+    /* Look into the inherited IDispatch */
+    hr = get_dispatch_typeinfo(&disp_typeinfo);
+    if (FAILED(hr)) return hr;
+
+    hr = ITypeInfo_GetTypeComp(disp_typeinfo, &disp_typecomp);
+    if (FAILED(hr)) return hr;
+
+    hr = ITypeComp_BindType(disp_typecomp, szName, lHashVal, ppTInfo, ppTComp);
+    ITypeComp_Release(disp_typecomp);
+    return hr;
+}
+
+static const ITypeCompVtbl ScriptTypeCompVtbl = {
+    ScriptTypeComp_QueryInterface,
+    ScriptTypeComp_AddRef,
+    ScriptTypeComp_Release,
+    ScriptTypeComp_Bind,
+    ScriptTypeComp_BindType
+};
+
 static inline ScriptDisp *ScriptDisp_from_IDispatchEx(IDispatchEx *iface)
 {
     return CONTAINING_RECORD(iface, ScriptDisp, IDispatchEx_iface);
@@ -817,11 +1243,19 @@ static ULONG WINAPI ScriptDisp_Release(IDispatchEx *iface)
 {
     ScriptDisp *This = ScriptDisp_from_IDispatchEx(iface);
     LONG ref = InterlockedDecrement(&This->ref);
+    unsigned i;
 
     TRACE("(%p) ref=%d\n", This, ref);
 
     if(!ref) {
         assert(!This->ctx);
+
+        for (i = 0; i < This->global_vars_cnt; i++)
+            release_dynamic_var(This->global_vars[i]);
+
+        heap_pool_free(&This->heap);
+        heap_free(This->global_vars);
+        heap_free(This->global_funcs);
         heap_free(This);
     }
 
@@ -842,6 +1276,8 @@ static HRESULT WINAPI ScriptDisp_GetTypeInfo(IDispatchEx *iface, UINT iTInfo, LC
 {
     ScriptDisp *This = ScriptDisp_from_IDispatchEx(iface);
     ScriptTypeInfo *type_info;
+    UINT num_funcs = 0;
+    unsigned i, j;
 
     TRACE("(%p)->(%u %u %p)\n", This, iTInfo, lcid, ret);
 
@@ -851,8 +1287,35 @@ static HRESULT WINAPI ScriptDisp_GetTypeInfo(IDispatchEx *iface, UINT iTInfo, LC
     if(!(type_info = heap_alloc(sizeof(*type_info))))
         return E_OUTOFMEMORY;
 
+    for(i = 0; i < This->global_funcs_cnt; i++)
+        if(This->global_funcs[i]->is_public)
+            num_funcs++;
+
     type_info->ITypeInfo_iface.lpVtbl = &ScriptTypeInfoVtbl;
+    type_info->ITypeComp_iface.lpVtbl = &ScriptTypeCompVtbl;
     type_info->ref = 1;
+    type_info->num_funcs = num_funcs;
+    type_info->num_vars = This->global_vars_cnt;
+    type_info->disp = This;
+
+    type_info->funcs = heap_alloc(sizeof(*type_info->funcs) * num_funcs);
+    if(!type_info->funcs)
+    {
+        heap_free(type_info);
+        return E_OUTOFMEMORY;
+    }
+
+    for(j = 0, i = 0; i < This->global_funcs_cnt; i++)
+    {
+        if(!This->global_funcs[i]->is_public) continue;
+
+        type_info->funcs[j].memid = i + 1 + DISPID_FUNCTION_MASK;
+        type_info->funcs[j].func = This->global_funcs[i];
+        grab_vbscode(This->global_funcs[i]->code_ctx);
+        j++;
+    }
+
+    IDispatchEx_AddRef(&This->IDispatchEx_iface);
 
     *ret = &type_info->ITypeInfo_iface;
     return S_OK;
@@ -899,15 +1362,15 @@ static HRESULT WINAPI ScriptDisp_GetDispID(IDispatchEx *iface, BSTR bstrName, DW
     if(!This->ctx)
         return E_UNEXPECTED;
 
-    for(i = 0; i < This->ctx->global_vars_cnt; i++) {
-        if(!wcsicmp(This->ctx->global_vars[i]->name, bstrName)) {
+    for(i = 0; i < This->global_vars_cnt; i++) {
+        if(!wcsicmp(This->global_vars[i]->name, bstrName)) {
             *pid = i + 1;
             return S_OK;
         }
     }
 
-    for(i = 0; i < This->ctx->global_funcs_cnt; i++) {
-        if(!wcsicmp(This->ctx->global_funcs[i]->name, bstrName)) {
+    for(i = 0; i < This->global_funcs_cnt; i++) {
+        if(!wcsicmp(This->global_funcs[i]->name, bstrName)) {
             *pid = i + 1 + DISPID_FUNCTION_MASK;
             return S_OK;
         }
@@ -925,17 +1388,20 @@ static HRESULT WINAPI ScriptDisp_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
 
     TRACE("(%p)->(%x %x %x %p %p %p %p)\n", This, id, lcid, wFlags, pdp, pvarRes, pei, pspCaller);
 
+    if (!This->ctx)
+        return E_UNEXPECTED;
+
     if (id & DISPID_FUNCTION_MASK)
     {
         id &= ~DISPID_FUNCTION_MASK;
-        if (id > This->ctx->global_funcs_cnt)
+        if (id > This->global_funcs_cnt)
             return DISP_E_MEMBERNOTFOUND;
 
         switch (wFlags)
         {
         case DISPATCH_METHOD:
         case DISPATCH_METHOD | DISPATCH_PROPERTYGET:
-            hres = exec_script(This->ctx, TRUE, This->ctx->global_funcs[id - 1], NULL, pdp, pvarRes);
+            hres = exec_script(This->ctx, TRUE, This->global_funcs[id - 1], NULL, pdp, pvarRes);
             break;
         default:
             FIXME("Unsupported flags %x\n", wFlags);
@@ -945,16 +1411,16 @@ static HRESULT WINAPI ScriptDisp_InvokeEx(IDispatchEx *iface, DISPID id, LCID lc
         return hres;
     }
 
-    if (id > This->ctx->global_vars_cnt)
+    if (id > This->global_vars_cnt)
         return DISP_E_MEMBERNOTFOUND;
 
-    if (This->ctx->global_vars[id - 1]->is_const)
+    if (This->global_vars[id - 1]->is_const)
     {
         FIXME("const not supported\n");
         return E_NOTIMPL;
     }
 
-    return invoke_variant_prop(This->ctx, &This->ctx->global_vars[id - 1]->v, wFlags, pdp, pvarRes);
+    return invoke_variant_prop(This->ctx, &This->global_vars[id - 1]->v, wFlags, pdp, pvarRes);
 }
 
 static HRESULT WINAPI ScriptDisp_DeleteMemberByName(IDispatchEx *iface, BSTR bstrName, DWORD grfdex)
@@ -1028,6 +1494,7 @@ HRESULT create_script_disp(script_ctx_t *ctx, ScriptDisp **ret)
     script_disp->IDispatchEx_iface.lpVtbl = &ScriptDispVtbl;
     script_disp->ref = 1;
     script_disp->ctx = ctx;
+    heap_pool_init(&script_disp->heap);
 
     *ret = script_disp;
     return S_OK;
