@@ -80,11 +80,6 @@ static inline BitmapImpl *impl_from_IMILUnknown1(IMILUnknown1 *iface)
     return CONTAINING_RECORD(iface, BitmapImpl, IMILUnknown1_iface);
 }
 
-static inline BitmapImpl *impl_from_IMILUnknown2(IMILUnknown2 *iface)
-{
-    return CONTAINING_RECORD(iface, BitmapImpl, IMILUnknown2_iface);
-}
-
 static inline BitmapLockImpl *impl_from_IWICBitmapLock(IWICBitmapLock *iface)
 {
     return CONTAINING_RECORD(iface, BitmapLockImpl, IWICBitmapLock_iface);
