@@ -22,7 +22,7 @@
 
 #include <stdarg.h>
 
-#ifdef HAVE_PNG_H
+#ifdef SONAME_LIBPNG
 #include <png.h>
 #endif
 
@@ -1772,7 +1772,9 @@ static HRESULT WINAPI PngFrameEncode_WriteSource(IWICBitmapFrameEncode *iface,
     if (SUCCEEDED(hr))
     {
         hr = write_source(iface, pIBitmapSource, prc,
-            This->format->guid, This->format->bpp, This->width, This->height);
+            This->format->guid, This->format->bpp,
+            !This->colors && This->format->color_type == PNG_COLOR_TYPE_PALETTE,
+            This->width, This->height);
     }
 
     return hr;
@@ -2183,7 +2185,7 @@ HRESULT PngEncoder_CreateInstance(REFIID iid, void** ppv)
     return ret;
 }
 
-#else /* !HAVE_PNG_H */
+#else /* !SONAME_LIBPNG */
 
 HRESULT PngDecoder_CreateInstance(REFIID iid, void** ppv)
 {
