@@ -98,10 +98,6 @@ static int deviceloop_control[2];
 static const WCHAR hidraw_busidW[] = {'H','I','D','R','A','W',0};
 static const WCHAR lnxev_busidW[] = {'L','N','X','E','V',0};
 
-#include "initguid.h"
-DEFINE_GUID(GUID_DEVCLASS_HIDRAW, 0x3def44ad,0x242e,0x46e5,0x82,0x6d,0x70,0x72,0x13,0xf3,0xaa,0x81);
-DEFINE_GUID(GUID_DEVCLASS_LINUXEVENT, 0x1b932c0d,0xfea7,0x42cd,0x8e,0xaa,0x0e,0x48,0x79,0xb6,0x9e,0xaa);
-
 struct platform_private
 {
     struct udev_device *udev_device;
@@ -143,8 +139,8 @@ static const BYTE ABS_TO_HID_MAP[][2] = {
     {HID_USAGE_PAGE_SIMULATION, HID_USAGE_SIMULATION_THROTTLE}, /*ABS_THROTTLE*/
     {HID_USAGE_PAGE_SIMULATION, HID_USAGE_SIMULATION_RUDDER},   /*ABS_RUDDER*/
     {HID_USAGE_PAGE_GENERIC, HID_USAGE_GENERIC_WHEEL},          /*ABS_WHEEL*/
-    {HID_USAGE_PAGE_SIMULATION, 0xC4},                          /*ABS_GAS*/
-    {HID_USAGE_PAGE_SIMULATION, 0xC5},                          /*ABS_BRAKE*/
+    {HID_USAGE_PAGE_SIMULATION, HID_USAGE_SIMULATION_ACCELLERATOR}, /*ABS_GAS*/
+    {HID_USAGE_PAGE_SIMULATION, HID_USAGE_SIMULATION_BRAKE},    /*ABS_BRAKE*/
     {0,0},                                                      /*ABS_HAT0X*/
     {0,0},                                                      /*ABS_HAT0Y*/
     {0,0},                                                      /*ABS_HAT1X*/
@@ -153,15 +149,15 @@ static const BYTE ABS_TO_HID_MAP[][2] = {
     {0,0},                                                      /*ABS_HAT2Y*/
     {0,0},                                                      /*ABS_HAT3X*/
     {0,0},                                                      /*ABS_HAT3Y*/
-    {HID_USAGE_PAGE_DIGITIZER, 0x30},                           /*ABS_PRESSURE*/
+    {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_TIP_PRESSURE}, /*ABS_PRESSURE*/
     {0, 0},                                                     /*ABS_DISTANCE*/
-    {HID_USAGE_PAGE_DIGITIZER, 0x3D},                           /*ABS_TILT_X*/
-    {HID_USAGE_PAGE_DIGITIZER, 0x3F},                           /*ABS_TILT_Y*/
+    {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_X_TILT},     /*ABS_TILT_X*/
+    {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_Y_TILT},     /*ABS_TILT_Y*/
     {0, 0},                                                     /*ABS_TOOL_WIDTH*/
     {0, 0},
     {0, 0},
     {0, 0},
-    {HID_USAGE_PAGE_CONSUMER, 0xE0}                             /*ABS_VOLUME*/
+    {HID_USAGE_PAGE_CONSUMER, HID_USAGE_CONSUMER_VOLUME}        /*ABS_VOLUME*/
 };
 #define HID_ABS_MAX (ABS_VOLUME+1)
 #define TOP_ABS_PAGE (HID_USAGE_PAGE_DIGITIZER+1)
@@ -249,9 +245,9 @@ static const BYTE* what_am_I(struct udev_device *dev)
     static const BYTE Keyboard[2]    = {HID_USAGE_PAGE_GENERIC, HID_USAGE_GENERIC_KEYBOARD};
     static const BYTE Gamepad[2]     = {HID_USAGE_PAGE_GENERIC, HID_USAGE_GENERIC_GAMEPAD};
     static const BYTE Keypad[2]      = {HID_USAGE_PAGE_GENERIC, HID_USAGE_GENERIC_KEYPAD};
-    static const BYTE Tablet[2]      = {HID_USAGE_PAGE_DIGITIZER, 0x2};
-    static const BYTE Touchscreen[2] = {HID_USAGE_PAGE_DIGITIZER, 0x4};
-    static const BYTE Touchpad[2]    = {HID_USAGE_PAGE_DIGITIZER, 0x5};
+    static const BYTE Tablet[2]      = {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_PEN};
+    static const BYTE Touchscreen[2] = {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_TOUCH_SCREEN};
+    static const BYTE Touchpad[2]    = {HID_USAGE_PAGE_DIGITIZER, HID_USAGE_DIGITIZER_TOUCH_PAD};
 
     struct udev_device *parent = dev;
 
@@ -1241,13 +1237,13 @@ static void try_add_device(struct udev_device *dev)
     if (strcmp(subsystem, "hidraw") == 0)
     {
         device = bus_create_hid_device(hidraw_busidW, vid, pid, input, version, 0, serial, is_gamepad,
-                                       &GUID_DEVCLASS_HIDRAW, &hidraw_vtbl, sizeof(struct platform_private));
+                                       &hidraw_vtbl, sizeof(struct platform_private));
     }
 #ifdef HAS_PROPER_INPUT_HEADER
     else if (strcmp(subsystem, "input") == 0)
     {
         device = bus_create_hid_device(lnxev_busidW, vid, pid, input, version, 0, serial, is_gamepad,
-                                       &GUID_DEVCLASS_LINUXEVENT, &lnxev_vtbl, sizeof(struct wine_input_private));
+                                       &lnxev_vtbl, sizeof(struct wine_input_private));
     }
 #endif
 
@@ -1263,12 +1259,13 @@ static void try_add_device(struct udev_device *dev)
                 ERR("Building report descriptor failed, removing device\n");
                 close(fd);
                 udev_device_unref(dev);
+                bus_unlink_hid_device(device);
                 bus_remove_hid_device(device);
                 HeapFree(GetProcessHeap(), 0, serial);
                 return;
             }
 #endif
-        IoInvalidateDeviceRelations(device, BusRelations);
+        IoInvalidateDeviceRelations(bus_pdo, BusRelations);
     }
     else
     {
@@ -1297,7 +1294,8 @@ static void try_remove_device(struct udev_device *dev)
 #endif
     if (!device) return;
 
-    IoInvalidateDeviceRelations(device, RemovalRelations);
+    bus_unlink_hid_device(device);
+    IoInvalidateDeviceRelations(bus_pdo, BusRelations);
 
     private = impl_from_DEVICE_OBJECT(device);
 
