@@ -18,7 +18,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#ifndef _WIN32
+#if 0
+#pragma makedep unix
+#endif
 
 #include <stdarg.h>
 #include "windef.h"
@@ -52,5 +54,3 @@ int __cdecl wmain( int argc, WCHAR *argv[] )
     if (!(info.dwFlags & STARTF_USESHOWWINDOW)) info.wShowWindow = SW_SHOWNORMAL;
     return wWinMain( GetModuleHandleW(0), 0, cmdline, info.wShowWindow );
 }
-
-#endif  /* _WIN32 */
