@@ -139,11 +139,13 @@ static void wined3d_buffer_gl_bind(struct wined3d_buffer_gl *buffer_gl, struct w
 }
 
 /* Context activation is done by the caller. */
-void wined3d_buffer_gl_destroy_buffer_object(struct wined3d_buffer_gl *buffer_gl,
+static void wined3d_buffer_gl_destroy_buffer_object(struct wined3d_buffer_gl *buffer_gl,
         struct wined3d_context_gl *context_gl)
 {
     const struct wined3d_gl_info *gl_info = context_gl->gl_info;
     struct wined3d_resource *resource = &buffer_gl->b.resource;
+    struct wined3d_buffer *buffer = &buffer_gl->b;
+    struct wined3d_cs *cs = resource->device->cs;
     GLuint bo;
 
     if (!buffer_gl->b.buffer_object)
@@ -158,7 +160,8 @@ void wined3d_buffer_gl_destroy_buffer_object(struct wined3d_buffer_gl *buffer_gl
     {
         if (resource->bind_flags & WINED3D_BIND_VERTEX_BUFFER)
             device_invalidate_state(resource->device, STATE_STREAMSRC);
-        if (resource->bind_flags & WINED3D_BIND_INDEX_BUFFER)
+        if (resource->bind_flags & WINED3D_BIND_INDEX_BUFFER
+                && cs->state.index_buffer == buffer)
             device_invalidate_state(resource->device, STATE_INDEXBUFFER);
         if (resource->bind_flags & WINED3D_BIND_CONSTANT_BUFFER)
         {
@@ -602,10 +605,16 @@ static BOOL wined3d_buffer_prepare_location(struct wined3d_buffer *buffer,
     return buffer->buffer_ops->buffer_prepare_location(buffer, context, location);
 }
 
+static void wined3d_buffer_unload_location(struct wined3d_buffer *buffer,
+        struct wined3d_context *context, unsigned int location)
+{
+    buffer->buffer_ops->buffer_unload_location(buffer, context, location);
+}
+
 BOOL wined3d_buffer_load_location(struct wined3d_buffer *buffer,
         struct wined3d_context *context, DWORD location)
 {
-    struct wined3d_map_range range;
+    struct wined3d_range range;
 
     TRACE("buffer %p, context %p, location %s.\n",
             buffer, context, wined3d_debug_location(location));
@@ -699,7 +708,7 @@ DWORD wined3d_buffer_get_memory(struct wined3d_buffer *buffer,
     return 0;
 }
 
-static void buffer_unload(struct wined3d_resource *resource)
+static void buffer_resource_unload(struct wined3d_resource *resource)
 {
     struct wined3d_buffer *buffer = buffer_from_resource(resource);
 
@@ -713,7 +722,7 @@ static void buffer_unload(struct wined3d_resource *resource)
 
         wined3d_buffer_load_location(buffer, context, WINED3D_LOCATION_SYSMEM);
         wined3d_buffer_invalidate_location(buffer, WINED3D_LOCATION_BUFFER);
-        wined3d_buffer_gl_destroy_buffer_object(wined3d_buffer_gl(buffer), wined3d_context_gl(context));
+        wined3d_buffer_unload_location(buffer, context, WINED3D_LOCATION_BUFFER);
         buffer_clear_dirty_areas(buffer);
 
         context_release(context);
@@ -731,21 +740,28 @@ static void buffer_unload(struct wined3d_resource *resource)
 static void wined3d_buffer_drop_bo(struct wined3d_buffer *buffer)
 {
     buffer->flags &= ~WINED3D_BUFFER_USE_BO;
-    buffer_unload(&buffer->resource);
+    buffer_resource_unload(&buffer->resource);
 }
 
 static void wined3d_buffer_destroy_object(void *object)
 {
     struct wined3d_buffer *buffer = object;
+    struct wined3d_context *context;
 
+    if (buffer->buffer_object)
+    {
+        context = context_acquire(buffer->resource.device, NULL, 0);
+        wined3d_buffer_unload_location(buffer, context, WINED3D_LOCATION_BUFFER);
+        context_release(context);
+    }
     heap_free(buffer->conversion_map);
     heap_free(buffer->maps);
 }
 
 void wined3d_buffer_cleanup(struct wined3d_buffer *buffer)
 {
-    resource_cleanup(&buffer->resource);
     wined3d_cs_destroy_object(buffer->resource.device->cs, wined3d_buffer_destroy_object, buffer);
+    resource_cleanup(&buffer->resource);
 }
 
 ULONG CDECL wined3d_buffer_decref(struct wined3d_buffer *buffer)
@@ -1202,7 +1218,7 @@ void wined3d_buffer_copy(struct wined3d_buffer *dst_buffer, unsigned int dst_off
 void wined3d_buffer_upload_data(struct wined3d_buffer *buffer, struct wined3d_context *context,
         const struct wined3d_box *box, const void *data)
 {
-    struct wined3d_map_range range;
+    struct wined3d_range range;
 
     if (box)
     {
@@ -1264,7 +1280,7 @@ static const struct wined3d_resource_ops buffer_resource_ops =
     buffer_resource_incref,
     buffer_resource_decref,
     buffer_resource_preload,
-    buffer_unload,
+    buffer_resource_unload,
     buffer_resource_sub_resource_map,
     buffer_resource_sub_resource_unmap,
 };
@@ -1287,6 +1303,10 @@ GLenum wined3d_buffer_gl_binding_from_bind_flags(const struct wined3d_gl_info *g
     if (bind_flags & WINED3D_BIND_STREAM_OUTPUT)
         return GL_TRANSFORM_FEEDBACK_BUFFER;
 
+    if (bind_flags & WINED3D_BIND_INDIRECT_BUFFER
+            && gl_info->supported[ARB_DRAW_INDIRECT])
+        return GL_DRAW_INDIRECT_BUFFER;
+
     if (bind_flags & ~(WINED3D_BIND_VERTEX_BUFFER | WINED3D_BIND_INDEX_BUFFER))
         FIXME("Unhandled bind flags %#x.\n", bind_flags);
 
@@ -1298,9 +1318,7 @@ static HRESULT wined3d_buffer_init(struct wined3d_buffer *buffer, struct wined3d
         void *parent, const struct wined3d_parent_ops *parent_ops, const struct wined3d_buffer_ops *buffer_ops)
 {
     const struct wined3d_format *format = wined3d_get_format(device->adapter, WINED3DFMT_UNKNOWN, desc->bind_flags);
-    const struct wined3d_gl_info *gl_info = &device->adapter->gl_info;
     struct wined3d_resource *resource = &buffer->resource;
-    BOOL dynamic_buffer_ok;
     HRESULT hr;
 
     TRACE("buffer %p, device %p, desc byte_width %u, usage %s, bind_flags %s, "
@@ -1351,28 +1369,6 @@ static HRESULT wined3d_buffer_init(struct wined3d_buffer *buffer, struct wined3d
         buffer->locations = WINED3D_LOCATION_SYSMEM;
     }
 
-    /* Observations show that draw_primitive_immediate_mode() is faster on
-     * dynamic vertex buffers than converting + draw_primitive_arrays().
-     * (Half-Life 2 and others.) */
-    dynamic_buffer_ok = gl_info->supported[APPLE_FLUSH_BUFFER_RANGE] || gl_info->supported[ARB_MAP_BUFFER_RANGE];
-
-    if (!gl_info->supported[ARB_VERTEX_BUFFER_OBJECT])
-    {
-        TRACE("Not creating a BO because GL_ARB_vertex_buffer is not supported.\n");
-    }
-    else if (!(desc->access & WINED3D_RESOURCE_ACCESS_GPU))
-    {
-        TRACE("Not creating a BO because the buffer is not GPU accessible.\n");
-    }
-    else if (!dynamic_buffer_ok && (resource->usage & WINED3DUSAGE_DYNAMIC))
-    {
-        TRACE("Not creating a BO because the buffer has dynamic usage and no GL support.\n");
-    }
-    else
-    {
-        buffer->flags |= WINED3D_BUFFER_USE_BO;
-    }
-
     if (buffer->locations & WINED3D_LOCATION_SYSMEM || !(buffer->flags & WINED3D_BUFFER_USE_BO))
     {
         if (!wined3d_resource_prepare_sysmem(&buffer->resource))
@@ -1382,7 +1378,7 @@ static HRESULT wined3d_buffer_init(struct wined3d_buffer *buffer, struct wined3d
     if (!(buffer->maps = heap_alloc(sizeof(*buffer->maps))))
     {
         ERR("Out of memory.\n");
-        buffer_unload(resource);
+        buffer_resource_unload(resource);
         resource_cleanup(resource);
         wined3d_resource_wait_idle(resource);
         return E_OUTOFMEMORY;
@@ -1406,14 +1402,20 @@ static BOOL wined3d_buffer_no3d_prepare_location(struct wined3d_buffer *buffer,
     return FALSE;
 }
 
+static void wined3d_buffer_no3d_unload_location(struct wined3d_buffer *buffer,
+        struct wined3d_context *context, unsigned int location)
+{
+    TRACE("buffer %p, context %p, location %s.\n", buffer, context, wined3d_debug_location(location));
+}
+
 static void wined3d_buffer_no3d_upload_ranges(struct wined3d_buffer *buffer, struct wined3d_context *context,
-        const void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_map_range *ranges)
+        const void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_range *ranges)
 {
     FIXME("Not implemented.\n");
 }
 
 static void wined3d_buffer_no3d_download_ranges(struct wined3d_buffer *buffer, struct wined3d_context *context,
-        void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_map_range *ranges)
+        void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_range *ranges)
 {
     FIXME("Not implemented.\n");
 }
@@ -1421,6 +1423,7 @@ static void wined3d_buffer_no3d_download_ranges(struct wined3d_buffer *buffer, s
 static const struct wined3d_buffer_ops wined3d_buffer_no3d_ops =
 {
     wined3d_buffer_no3d_prepare_location,
+    wined3d_buffer_no3d_unload_location,
     wined3d_buffer_no3d_upload_ranges,
     wined3d_buffer_no3d_download_ranges,
 };
@@ -1463,14 +1466,31 @@ static BOOL wined3d_buffer_gl_prepare_location(struct wined3d_buffer *buffer,
     }
 }
 
+static void wined3d_buffer_gl_unload_location(struct wined3d_buffer *buffer,
+        struct wined3d_context *context, unsigned int location)
+{
+    TRACE("buffer %p, context %p, location %s.\n", buffer, context, wined3d_debug_location(location));
+
+    switch (location)
+    {
+        case WINED3D_LOCATION_BUFFER:
+            wined3d_buffer_gl_destroy_buffer_object(wined3d_buffer_gl(buffer), wined3d_context_gl(context));
+            break;
+
+        default:
+            ERR("Unhandled location %s.\n", wined3d_debug_location(location));
+            break;
+    }
+}
+
 /* Context activation is done by the caller. */
 static void wined3d_buffer_gl_upload_ranges(struct wined3d_buffer *buffer, struct wined3d_context *context,
-        const void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_map_range *ranges)
+        const void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_range *ranges)
 {
     struct wined3d_context_gl *context_gl = wined3d_context_gl(context);
     struct wined3d_buffer_gl *buffer_gl = wined3d_buffer_gl(buffer);
     const struct wined3d_gl_info *gl_info = context_gl->gl_info;
-    const struct wined3d_map_range *range;
+    const struct wined3d_range *range;
 
     wined3d_buffer_gl_bind(buffer_gl, context_gl);
 
@@ -1485,12 +1505,12 @@ static void wined3d_buffer_gl_upload_ranges(struct wined3d_buffer *buffer, struc
 
 /* Context activation is done by the caller. */
 static void wined3d_buffer_gl_download_ranges(struct wined3d_buffer *buffer, struct wined3d_context *context,
-        void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_map_range *ranges)
+        void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_range *ranges)
 {
     struct wined3d_context_gl *context_gl = wined3d_context_gl(context);
     struct wined3d_buffer_gl *buffer_gl = wined3d_buffer_gl(buffer);
     const struct wined3d_gl_info *gl_info = context_gl->gl_info;
-    const struct wined3d_map_range *range;
+    const struct wined3d_range *range;
 
     wined3d_buffer_gl_bind(buffer_gl, context_gl);
 
@@ -1506,6 +1526,7 @@ static void wined3d_buffer_gl_download_ranges(struct wined3d_buffer *buffer, str
 static const struct wined3d_buffer_ops wined3d_buffer_gl_ops =
 {
     wined3d_buffer_gl_prepare_location,
+    wined3d_buffer_gl_unload_location,
     wined3d_buffer_gl_upload_ranges,
     wined3d_buffer_gl_download_ranges,
 };
@@ -1519,11 +1540,61 @@ HRESULT wined3d_buffer_gl_init(struct wined3d_buffer_gl *buffer_gl, struct wined
     TRACE("buffer_gl %p, device %p, desc %p, data %p, parent %p, parent_ops %p.\n",
             buffer_gl, device, desc, data, parent, parent_ops);
 
+    /* Observations show that draw_primitive_immediate_mode() is faster on
+     * dynamic vertex buffers than converting + draw_primitive_arrays().
+     * (Half-Life 2 and others.) */
+    if (!(desc->access & WINED3D_RESOURCE_ACCESS_GPU))
+        TRACE("Not creating a BO because the buffer is not GPU accessible.\n");
+    else if (!gl_info->supported[ARB_VERTEX_BUFFER_OBJECT])
+        TRACE("Not creating a BO because GL_ARB_vertex_buffer is not supported.\n");
+    else if (!(gl_info->supported[APPLE_FLUSH_BUFFER_RANGE] || gl_info->supported[ARB_MAP_BUFFER_RANGE])
+            && (desc->usage & WINED3DUSAGE_DYNAMIC))
+        TRACE("Not creating a BO because the buffer has dynamic usage and no GL support.\n");
+    else
+        buffer_gl->b.flags |= WINED3D_BUFFER_USE_BO;
     buffer_gl->buffer_type_hint = wined3d_buffer_gl_binding_from_bind_flags(gl_info, desc->bind_flags);
 
     return wined3d_buffer_init(&buffer_gl->b, device, desc, data, parent, parent_ops, &wined3d_buffer_gl_ops);
 }
 
+static BOOL wined3d_buffer_vk_create_buffer_object(struct wined3d_buffer_vk *buffer_vk,
+        struct wined3d_context_vk *context_vk)
+{
+    struct wined3d_resource *resource = &buffer_vk->b.resource;
+    uint32_t bind_flags = resource->bind_flags;
+    VkMemoryPropertyFlags memory_type;
+    VkBufferUsageFlags usage;
+
+    usage = VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT;
+    if (bind_flags & WINED3D_BIND_VERTEX_BUFFER)
+        usage |= VK_BUFFER_USAGE_VERTEX_BUFFER_BIT;
+    if (bind_flags & WINED3D_BIND_INDEX_BUFFER)
+        usage |= VK_BUFFER_USAGE_INDEX_BUFFER_BIT;
+    if (bind_flags & WINED3D_BIND_CONSTANT_BUFFER)
+        usage |= VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT;
+    if (bind_flags & WINED3D_BIND_SHADER_RESOURCE)
+        usage |= VK_BUFFER_USAGE_UNIFORM_TEXEL_BUFFER_BIT;
+    if (bind_flags & WINED3D_BIND_UNORDERED_ACCESS)
+        usage |= VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT;
+    if (bind_flags & WINED3D_BIND_INDIRECT_BUFFER)
+        usage |= VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT;
+    if (bind_flags & (WINED3D_BIND_STREAM_OUTPUT | WINED3D_BIND_RENDER_TARGET | WINED3D_BIND_DEPTH_STENCIL))
+        FIXME("Ignoring some bind flags %#x.\n", bind_flags);
+    memory_type = VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT
+            | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT
+            | VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
+    if (!(wined3d_context_vk_create_bo(context_vk, resource->size, usage, memory_type, &buffer_vk->bo)))
+    {
+        WARN("Failed to create Vulkan buffer.\n");
+        return FALSE;
+    }
+
+    buffer_vk->b.buffer_object = (uintptr_t)&buffer_vk->bo;
+    buffer_invalidate_bo_range(&buffer_vk->b, 0, 0);
+
+    return TRUE;
+}
+
 static BOOL wined3d_buffer_vk_prepare_location(struct wined3d_buffer *buffer,
         struct wined3d_context *context, unsigned int location)
 {
@@ -1533,8 +1604,10 @@ static BOOL wined3d_buffer_vk_prepare_location(struct wined3d_buffer *buffer,
             return wined3d_resource_prepare_sysmem(&buffer->resource);
 
         case WINED3D_LOCATION_BUFFER:
-            /* The Vulkan buffer is created during resource creation. */
-            return TRUE;
+            if (buffer->buffer_object)
+                return TRUE;
+
+            return wined3d_buffer_vk_create_buffer_object(wined3d_buffer_vk(buffer), wined3d_context_vk(context));
 
         default:
             FIXME("Unhandled location %s.\n", wined3d_debug_location(location));
@@ -1542,14 +1615,37 @@ static BOOL wined3d_buffer_vk_prepare_location(struct wined3d_buffer *buffer,
     }
 }
 
+static void wined3d_buffer_vk_unload_location(struct wined3d_buffer *buffer,
+        struct wined3d_context *context, unsigned int location)
+{
+    struct wined3d_context_vk *context_vk = wined3d_context_vk(context);
+    struct wined3d_buffer_vk *buffer_vk = wined3d_buffer_vk(buffer);
+
+    TRACE("buffer %p, context %p, location %s.\n", buffer, context, wined3d_debug_location(location));
+
+    switch (location)
+    {
+        case WINED3D_LOCATION_BUFFER:
+            wined3d_context_vk_destroy_bo(context_vk, &buffer_vk->bo);
+            buffer_vk->bo.vk_buffer = VK_NULL_HANDLE;
+            buffer_vk->bo.vk_memory = VK_NULL_HANDLE;
+            buffer_vk->b.buffer_object = 0u;
+            break;
+
+        default:
+            ERR("Unhandled location %s.\n", wined3d_debug_location(location));
+            break;
+    }
+}
+
 static void wined3d_buffer_vk_upload_ranges(struct wined3d_buffer *buffer, struct wined3d_context *context,
-        const void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_map_range *ranges)
+        const void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_range *ranges)
 {
     FIXME("Not implemented.\n");
 }
 
 static void wined3d_buffer_vk_download_ranges(struct wined3d_buffer *buffer, struct wined3d_context *context,
-        void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_map_range *ranges)
+        void *data, unsigned int data_offset, unsigned int range_count, const struct wined3d_range *ranges)
 {
     FIXME("Not implemented.\n");
 }
@@ -1557,6 +1653,7 @@ static void wined3d_buffer_vk_download_ranges(struct wined3d_buffer *buffer, str
 static const struct wined3d_buffer_ops wined3d_buffer_vk_ops =
 {
     wined3d_buffer_vk_prepare_location,
+    wined3d_buffer_vk_unload_location,
     wined3d_buffer_vk_upload_ranges,
     wined3d_buffer_vk_download_ranges,
 };
@@ -1568,6 +1665,9 @@ HRESULT wined3d_buffer_vk_init(struct wined3d_buffer_vk *buffer_vk, struct wined
     TRACE("buffer_vk %p, device %p, desc %p, data %p, parent %p, parent_ops %p.\n",
             buffer_vk, device, desc, data, parent, parent_ops);
 
+    if (desc->access & WINED3D_RESOURCE_ACCESS_GPU)
+        buffer_vk->b.flags |= WINED3D_BUFFER_USE_BO;
+
     return wined3d_buffer_init(&buffer_vk->b, device, desc, data, parent, parent_ops, &wined3d_buffer_vk_ops);
 }
 
