@@ -852,9 +852,9 @@ void CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock,
     }
 
     map = stateblock->changed.streamSource;
-    for (i = 0; map; map >>= 1, ++i)
+    while (map)
     {
-        if (!(map & 1)) continue;
+        i = wined3d_bit_scan(&map);
 
         if (stateblock->stateblock_state.streams[i].stride != state->streams[i].stride
                 || stateblock->stateblock_state.streams[i].offset != state->streams[i].offset
@@ -877,9 +877,9 @@ void CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock,
     }
 
     map = stateblock->changed.streamFreq;
-    for (i = 0; map; map >>= 1, ++i)
+    while (map)
     {
-        if (!(map & 1)) continue;
+        i = wined3d_bit_scan(&map);
 
         if (stateblock->stateblock_state.streams[i].frequency != state->streams[i].frequency
                 || stateblock->stateblock_state.streams[i].flags != state->streams[i].flags)
@@ -893,9 +893,9 @@ void CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock,
     }
 
     map = stateblock->changed.clipplane;
-    for (i = 0; map; map >>= 1, ++i)
+    while (map)
     {
-        if (!(map & 1)) continue;
+        i = wined3d_bit_scan(&map);
 
         if (memcmp(&stateblock->stateblock_state.clip_planes[i], &state->clip_planes[i], sizeof(state->clip_planes[i])))
         {
@@ -929,9 +929,9 @@ void CDECL wined3d_stateblock_capture(struct wined3d_stateblock *stateblock,
 
     /* Samplers */
     map = stateblock->changed.textures;
-    for (i = 0; map; map >>= 1, ++i)
+    while (map)
     {
-        if (!(map & 1)) continue;
+        i = wined3d_bit_scan(&map);
 
         TRACE("Updating texture %u to %p (was %p).\n",
                 i, state->textures[i], stateblock->stateblock_state.textures[i]);
@@ -985,6 +985,8 @@ void CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblock,
 
     if (stateblock->changed.vertexShader)
         wined3d_stateblock_set_vertex_shader(device_state, state->vs);
+    if (stateblock->changed.pixelShader)
+        wined3d_stateblock_set_pixel_shader(device_state, state->ps);
 
     for (start = 0; ; start = range.offset + range.size)
     {
@@ -1007,23 +1009,6 @@ void CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblock,
         wined3d_stateblock_set_vs_consts_b(device_state, range.offset, range.size, &state->vs_consts_b[range.offset]);
     }
 
-    if (stateblock->changed.lights)
-    {
-        for (i = 0; i < ARRAY_SIZE(state->light_state->light_map); ++i)
-        {
-            const struct wined3d_light_info *light;
-
-            LIST_FOR_EACH_ENTRY(light, &state->light_state->light_map[i], struct wined3d_light_info, entry)
-            {
-                wined3d_stateblock_set_light(device_state, light->OriginalIndex, &light->OriginalParms);
-                wined3d_stateblock_set_light_enable(device_state, light->OriginalIndex, light->glIndex != -1);
-            }
-        }
-    }
-
-    if (stateblock->changed.pixelShader)
-        wined3d_stateblock_set_pixel_shader(device_state, state->ps);
-
     for (start = 0; ; start = range.offset + range.size)
     {
         if (!wined3d_bitmap_get_range(stateblock->changed.ps_consts_f, WINED3D_MAX_PS_CONSTS_F, start, &range))
@@ -1045,6 +1030,30 @@ void CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblock,
         wined3d_stateblock_set_ps_consts_b(device_state, range.offset, range.size, &state->ps_consts_b[range.offset]);
     }
 
+    if (stateblock->changed.transforms)
+    {
+        for (i = 0; i < stateblock->num_contained_transform_states; ++i)
+        {
+            enum wined3d_transform_state transform = stateblock->contained_transform_states[i];
+
+            wined3d_stateblock_set_transform(device_state, transform, &state->transforms[transform]);
+        }
+    }
+
+    if (stateblock->changed.lights)
+    {
+        for (i = 0; i < ARRAY_SIZE(state->light_state->light_map); ++i)
+        {
+            const struct wined3d_light_info *light;
+
+            LIST_FOR_EACH_ENTRY(light, &state->light_state->light_map[i], struct wined3d_light_info, entry)
+            {
+                wined3d_stateblock_set_light(device_state, light->OriginalIndex, &light->OriginalParms);
+                wined3d_stateblock_set_light_enable(device_state, light->OriginalIndex, light->glIndex != -1);
+            }
+        }
+    }
+
     if (stateblock->changed.alpha_to_coverage)
     {
         device_state->stateblock_state.alpha_to_coverage = state->alpha_to_coverage;
@@ -1075,21 +1084,10 @@ void CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblock,
         DWORD stage = stateblock->contained_sampler_states[i].stage;
         DWORD sampler_state = stateblock->contained_sampler_states[i].state;
 
-        if (stage >= WINED3D_MAX_FRAGMENT_SAMPLERS) stage += WINED3DVERTEXTEXTURESAMPLER0 - WINED3D_MAX_FRAGMENT_SAMPLERS;
         wined3d_stateblock_set_sampler_state(device_state, stage, sampler_state,
                 state->sampler_states[stage][sampler_state]);
     }
 
-    if (stateblock->changed.transforms)
-    {
-        for (i = 0; i < stateblock->num_contained_transform_states; ++i)
-        {
-            enum wined3d_transform_state transform = stateblock->contained_transform_states[i];
-
-            wined3d_stateblock_set_transform(device_state, transform, &state->transforms[transform]);
-        }
-    }
-
     if (stateblock->changed.indices)
     {
         wined3d_stateblock_set_index_buffer(device_state, state->index_buffer, state->index_format);
@@ -1109,38 +1107,33 @@ void CDECL wined3d_stateblock_apply(const struct wined3d_stateblock *stateblock,
         wined3d_stateblock_set_scissor_rect(device_state, &state->scissor_rect);
 
     map = stateblock->changed.streamSource;
-    for (i = 0; map; map >>= 1, ++i)
+    while (map)
     {
-        if (map & 1)
-            wined3d_stateblock_set_stream_source(device_state, i, state->streams[i].buffer,
-                    state->streams[i].offset, state->streams[i].stride);
+        i = wined3d_bit_scan(&map);
+        wined3d_stateblock_set_stream_source(device_state, i, state->streams[i].buffer,
+                state->streams[i].offset, state->streams[i].stride);
     }
 
     map = stateblock->changed.streamFreq;
-    for (i = 0; map; map >>= 1, ++i)
+    while (map)
     {
-        if (map & 1)
-            wined3d_stateblock_set_stream_source_freq(device_state, i,
-                    state->streams[i].frequency | state->streams[i].flags);
+        i = wined3d_bit_scan(&map);
+        wined3d_stateblock_set_stream_source_freq(device_state, i,
+                state->streams[i].frequency | state->streams[i].flags);
     }
 
     map = stateblock->changed.textures;
-    for (i = 0; map; map >>= 1, ++i)
+    while (map)
     {
-        DWORD stage = i;
-
-        if (stage >= WINED3D_MAX_FRAGMENT_SAMPLERS)
-            stage += WINED3DVERTEXTEXTURESAMPLER0 - WINED3D_MAX_FRAGMENT_SAMPLERS;
-
-        if (map & 1)
-            wined3d_stateblock_set_texture(device_state, stage, state->textures[i]);
+        i = wined3d_bit_scan(&map);
+        wined3d_stateblock_set_texture(device_state, i, state->textures[i]);
     }
 
     map = stateblock->changed.clipplane;
-    for (i = 0; map; map >>= 1, ++i)
+    while (map)
     {
-        if (map & 1)
-            wined3d_stateblock_set_clip_plane(device_state, i, &state->clip_planes[i]);
+        i = wined3d_bit_scan(&map);
+        wined3d_stateblock_set_clip_plane(device_state, i, &state->clip_planes[i]);
     }
 
     TRACE("Applied stateblock %p.\n", stateblock);
@@ -1353,9 +1346,6 @@ void CDECL wined3d_stateblock_set_sampler_state(struct wined3d_stateblock *state
     TRACE("stateblock %p, sampler_idx %u, state %s, value %#x.\n",
             stateblock, sampler_idx, debug_d3dsamplerstate(state), value);
 
-    if (sampler_idx >= WINED3DVERTEXTEXTURESAMPLER0 && sampler_idx <= WINED3DVERTEXTEXTURESAMPLER3)
-        sampler_idx -= (WINED3DVERTEXTEXTURESAMPLER0 - WINED3D_MAX_FRAGMENT_SAMPLERS);
-
     if (sampler_idx >= ARRAY_SIZE(stateblock->stateblock_state.sampler_states))
     {
         WARN("Invalid sampler %u.\n", sampler_idx);
@@ -1394,9 +1384,6 @@ void CDECL wined3d_stateblock_set_texture(struct wined3d_stateblock *stateblock,
 {
     TRACE("stateblock %p, stage %u, texture %p.\n", stateblock, stage, texture);
 
-    if (stage >= WINED3DVERTEXTEXTURESAMPLER0 && stage <= WINED3DVERTEXTEXTURESAMPLER3)
-        stage -= (WINED3DVERTEXTEXTURESAMPLER0 - WINED3D_MAX_FRAGMENT_SAMPLERS);
-
     if (stage >= ARRAY_SIZE(stateblock->stateblock_state.textures))
     {
         WARN("Ignoring invalid stage %u.\n", stage);
@@ -1864,13 +1851,11 @@ static void state_init_default(struct wined3d_state *state, const struct wined3d
         state->streams[i].frequency = 1;
 }
 
-void state_init(struct wined3d_state *state, struct wined3d_fb_state *fb,
-        const struct wined3d_d3d_info *d3d_info, DWORD flags)
+void state_init(struct wined3d_state *state, const struct wined3d_d3d_info *d3d_info, DWORD flags)
 {
     unsigned int i;
 
     state->flags = flags;
-    state->fb = fb;
 
     for (i = 0; i < LIGHTMAP_SIZE; i++)
     {
