@@ -57,6 +57,7 @@ enum wined3d_gl_extension
     ARB_DEPTH_TEXTURE,
     ARB_DERIVATIVE_CONTROL,
     ARB_DRAW_BUFFERS,
+    ARB_DRAW_BUFFERS_BLEND,
     ARB_DRAW_ELEMENTS_BASE_VERTEX,
     ARB_DRAW_INDIRECT,
     ARB_DRAW_INSTANCED,
@@ -142,6 +143,7 @@ enum wined3d_gl_extension
     ARB_VERTEX_SHADER,
     ARB_VERTEX_TYPE_2_10_10_10_REV,
     ARB_VIEWPORT_ARRAY,
+    ARB_TEXTURE_BARRIER,
     /* ATI */
     ATI_FRAGMENT_SHADER,
     ATI_SEPARATE_STENCIL,
@@ -159,6 +161,7 @@ enum wined3d_gl_extension
     EXT_FOG_COORD,
     EXT_FRAMEBUFFER_BLIT,
     EXT_FRAMEBUFFER_MULTISAMPLE,
+    EXT_FRAMEBUFFER_MULTISAMPLE_BLIT_SCALED,
     EXT_FRAMEBUFFER_OBJECT,
     EXT_GPU_PROGRAM_PARAMETERS,
     EXT_GPU_SHADER4,
@@ -204,6 +207,7 @@ enum wined3d_gl_extension
     NV_VERTEX_PROGRAM2,
     NV_VERTEX_PROGRAM2_OPTION,
     NV_VERTEX_PROGRAM3,
+    NV_TEXTURE_BARRIER,
     /* WGL extensions */
     WGL_ARB_PIXEL_FORMAT,
     WGL_EXT_SWAP_CONTROL,
