@@ -541,6 +541,7 @@ static LRESULT CALLBACK wined3d_hook_proc(int code, WPARAM wparam, LPARAM lparam
     struct wined3d_swapchain_desc swapchain_desc;
     struct wined3d_swapchain *swapchain;
     struct wined3d_wndproc *entry;
+    struct wined3d_output *output;
     MSG *msg = (MSG *)lparam;
     unsigned int i;
 
@@ -564,8 +565,13 @@ static LRESULT CALLBACK wined3d_hook_proc(int code, WPARAM wparam, LPARAM lparam
 
             wined3d_swapchain_get_desc(swapchain, &swapchain_desc);
             swapchain_desc.windowed = !swapchain_desc.windowed;
-            wined3d_swapchain_state_set_fullscreen(&swapchain->state, &swapchain_desc,
-                    swapchain->device->wined3d, swapchain->device->adapter->ordinal, NULL);
+            if (!(output = wined3d_swapchain_get_output(swapchain)))
+            {
+                ERR("Failed to get output from swapchain %p.\n", swapchain);
+                break;
+            }
+            wined3d_swapchain_state_set_fullscreen(&swapchain->state, &swapchain_desc, output,
+                    NULL);
 
             wined3d_wndproc_mutex_unlock();
 
