@@ -23,6 +23,13 @@
 
 #include "config.h"
 
+#include <IOKit/pwr_mgt/IOPMLib.h>
+#define GetCurrentThread Mac_GetCurrentThread
+#define LoadResource Mac_LoadResource
+#include <CoreServices/CoreServices.h>
+#undef GetCurrentThread
+#undef LoadResource
+
 #include "macdrv.h"
 #include "winuser.h"
 #include "wine/unicode.h"
@@ -50,12 +57,15 @@ static DWORD activate_on_focus_time;
  */
 static void get_cocoa_window_features(struct macdrv_win_data *data,
                                       DWORD style, DWORD ex_style,
-                                      struct macdrv_window_features* wf)
+                                      struct macdrv_window_features* wf,
+                                      const RECT *window_rect,
+                                      const RECT *client_rect)
 {
     memset(wf, 0, sizeof(*wf));
 
     if (disable_window_decorations) return;
-    if (IsRectEmpty(&data->window_rect)) return;
+    if (IsRectEmpty(window_rect)) return;
+    if (EqualRect(window_rect, client_rect)) return;
 
     if ((style & WS_CAPTION) == WS_CAPTION && !(ex_style & WS_EX_LAYERED))
     {
@@ -123,7 +133,8 @@ static void get_cocoa_window_state(struct macdrv_win_data *data,
  *
  * Helper for macdrv_window_to_mac_rect and macdrv_mac_to_window_rect.
  */
-static void get_mac_rect_offset(struct macdrv_win_data *data, DWORD style, RECT *rect)
+static void get_mac_rect_offset(struct macdrv_win_data *data, DWORD style, RECT *rect,
+                                const RECT *window_rect, const RECT *client_rect)
 {
     DWORD ex_style, style_mask = 0, ex_style_mask = 0;
 
@@ -134,7 +145,7 @@ static void get_mac_rect_offset(struct macdrv_win_data *data, DWORD style, RECT
     if (!data->shaped)
     {
         struct macdrv_window_features wf;
-        get_cocoa_window_features(data, style, ex_style, &wf);
+        get_cocoa_window_features(data, style, ex_style, &wf, window_rect, client_rect);
 
         if (wf.title_bar)
         {
@@ -160,14 +171,15 @@ static void get_mac_rect_offset(struct macdrv_win_data *data, DWORD style, RECT
  *
  * Convert a rect from client to Mac window coordinates
  */
-static void macdrv_window_to_mac_rect(struct macdrv_win_data *data, DWORD style, RECT *rect)
+static void macdrv_window_to_mac_rect(struct macdrv_win_data *data, DWORD style, RECT *rect,
+                                      const RECT *window_rect, const RECT *client_rect)
 {
     RECT rc;
 
     if ((style & (WS_POPUP|WS_CHILD)) == WS_CHILD) return;
     if (IsRectEmpty(rect)) return;
 
-    get_mac_rect_offset(data, style, &rc);
+    get_mac_rect_offset(data, style, &rc, window_rect, client_rect);
 
     rect->left   -= rc.left;
     rect->right  -= rc.right;
@@ -191,7 +203,7 @@ static void macdrv_mac_to_window_rect(struct macdrv_win_data *data, RECT *rect)
     if ((style & (WS_POPUP|WS_CHILD)) == WS_CHILD) return;
     if (IsRectEmpty(rect)) return;
 
-    get_mac_rect_offset(data, style, &rc);
+    get_mac_rect_offset(data, style, &rc, &data->window_rect, &data->client_rect);
 
     rect->left   += rc.left;
     rect->right  += rc.right;
@@ -344,7 +356,7 @@ static void set_cocoa_window_properties(struct macdrv_win_data *data)
     owner_win = macdrv_get_cocoa_window(owner, TRUE);
     macdrv_set_cocoa_parent_window(data->cocoa_window, owner_win);
 
-    get_cocoa_window_features(data, style, ex_style, &wf);
+    get_cocoa_window_features(data, style, ex_style, &wf, &data->window_rect, &data->client_rect);
     macdrv_set_cocoa_window_features(data->cocoa_window, &wf);
 
     get_cocoa_window_state(data, style, ex_style, &state);
@@ -613,7 +625,7 @@ static void sync_window_min_max_info(HWND hwnd)
         CGSize min_size, max_size;
 
         SetRect(&min_rect, 0, 0, minmax.ptMinTrackSize.x, minmax.ptMinTrackSize.y);
-        macdrv_window_to_mac_rect(data, style, &min_rect);
+        macdrv_window_to_mac_rect(data, style, &min_rect, &data->window_rect, &data->client_rect);
         min_size = CGSizeMake(min_rect.right - min_rect.left, min_rect.bottom - min_rect.top);
 
         if (minmax.ptMaxTrackSize.x == GetSystemMetrics(SM_CXMAXTRACK) &&
@@ -622,7 +634,7 @@ static void sync_window_min_max_info(HWND hwnd)
         else
         {
             SetRect(&max_rect, 0, 0, minmax.ptMaxTrackSize.x, minmax.ptMaxTrackSize.y);
-            macdrv_window_to_mac_rect(data, style, &max_rect);
+            macdrv_window_to_mac_rect(data, style, &max_rect, &data->window_rect, &data->client_rect);
             max_size = CGSizeMake(max_rect.right - max_rect.left, max_rect.bottom - max_rect.top);
         }
 
@@ -684,9 +696,9 @@ static void create_cocoa_window(struct macdrv_win_data *data)
     ex_style = GetWindowLongW(data->hwnd, GWL_EXSTYLE);
 
     data->whole_rect = data->window_rect;
-    macdrv_window_to_mac_rect(data, style, &data->whole_rect);
+    macdrv_window_to_mac_rect(data, style, &data->whole_rect, &data->window_rect, &data->client_rect);
 
-    get_cocoa_window_features(data, style, ex_style, &wf);
+    get_cocoa_window_features(data, style, ex_style, &wf, &data->window_rect, &data->client_rect);
 
     frame = cgrect_from_rect(data->whole_rect);
     constrain_window_frame(&frame);
@@ -703,7 +715,7 @@ static void create_cocoa_window(struct macdrv_win_data *data)
     set_cocoa_window_properties(data);
 
     /* set the window text */
-    if (!InternalGetWindowText(data->hwnd, text, sizeof(text)/sizeof(WCHAR))) text[0] = 0;
+    if (!InternalGetWindowText(data->hwnd, text, ARRAY_SIZE(text))) text[0] = 0;
     macdrv_set_cocoa_window_title(data->cocoa_window, text, strlenW(text));
 
     /* set the window region */
@@ -1551,13 +1563,44 @@ BOOL CDECL macdrv_CreateDesktopWindow(HWND hwnd)
 }
 
 
+static WNDPROC desktop_orig_wndproc;
+
+#define WM_WINE_NOTIFY_ACTIVITY WM_USER
+
+static LRESULT CALLBACK desktop_wndproc_wrapper( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp )
+{
+    switch (msg)
+    {
+    case WM_WINE_NOTIFY_ACTIVITY:
+    {
+        /* This wakes from display sleep, but doesn't affect the screen saver. */
+        static IOPMAssertionID assertion;
+        IOPMAssertionDeclareUserActivity(CFSTR("Wine user input"), kIOPMUserActiveLocal, &assertion);
+
+        /* This prevents the screen saver, but doesn't wake from display sleep. */
+        /* It's deprecated, but there's no better alternative. */
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+        UpdateSystemActivity(UsrActivity);
+#pragma clang diagnostic pop
+        break;
+    }
+    }
+    return desktop_orig_wndproc( hwnd, msg, wp, lp );
+}
+
 /**********************************************************************
  *              CreateWindow   (MACDRV.@)
  */
 BOOL CDECL macdrv_CreateWindow(HWND hwnd)
 {
     if (hwnd == GetDesktopWindow())
+    {
+        desktop_orig_wndproc = (WNDPROC)SetWindowLongPtrW( GetDesktopWindow(),
+            GWLP_WNDPROC, (LONG_PTR)desktop_wndproc_wrapper );
+
         macdrv_init_clipboard();
+    }
     return TRUE;
 }
 
@@ -2025,7 +2068,7 @@ void CDECL macdrv_WindowPosChanging(HWND hwnd, HWND insert_after, UINT swp_flags
     if (!data && !(data = macdrv_create_win_data(hwnd, window_rect, client_rect))) return;
 
     *visible_rect = *window_rect;
-    macdrv_window_to_mac_rect(data, style, visible_rect);
+    macdrv_window_to_mac_rect(data, style, visible_rect, window_rect, client_rect);
     TRACE("visible_rect %s -> %s\n", wine_dbgstr_rect(window_rect),
           wine_dbgstr_rect(visible_rect));
 
@@ -2414,6 +2457,7 @@ void macdrv_window_did_unminimize(HWND hwnd)
     {
         TRACE("restoring win %p/%p\n", hwnd, data->cocoa_window);
         release_win_data(data);
+        SetActiveWindow(hwnd);
         SendMessageW(hwnd, WM_SYSCOMMAND, SC_RESTORE, 0);
         return;
     }
@@ -2806,7 +2850,8 @@ BOOL query_resize_size(HWND hwnd, macdrv_query *query)
 
     if (SendMessageW(hwnd, WM_SIZING, corner, (LPARAM)&rect))
     {
-        macdrv_window_to_mac_rect(data, GetWindowLongW(hwnd, GWL_STYLE), &rect);
+        macdrv_window_to_mac_rect(data, GetWindowLongW(hwnd, GWL_STYLE), &rect,
+                                  &data->window_rect, &data->client_rect);
         query->resize_size.rect = cgrect_from_rect(rect);
         ret = TRUE;
     }
