@@ -124,20 +124,21 @@
 #include "wine/strmbase.h"
 
 #include "qtprivate.h"
+#include "wineqtdecoder_classes.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(qtsplitter);
-extern CLSID CLSID_QTSplitter;
 
 typedef struct QTOutPin {
     struct strmbase_source pin;
     IQualityControl IQualityControl_iface;
+    SourceSeeking seeking;
 
     AM_MEDIA_TYPE * pmt;
     OutputQueue * queue;
 } QTOutPin;
 
 typedef struct QTInPin {
-    struct strmbase_pin pin;
+    struct strmbase_sink pin;
     GUID subType;
 
     IAsyncReader *pReader;
@@ -160,10 +161,8 @@ typedef struct QTSplitter {
     HANDLE runEvent;
 
     DWORD outputSize;
-    FILTER_STATE state;
     CRITICAL_SECTION csReceive;
 
-    SourceSeeking sourceSeeking;
     TimeValue movie_time;
     TimeValue movie_start;
     TimeScale movie_scale;
@@ -172,8 +171,6 @@ typedef struct QTSplitter {
     HANDLE splitterThread;
 } QTSplitter;
 
-static const IPinVtbl QT_OutputPin_Vtbl;
-static const IPinVtbl QT_InputPin_Vtbl;
 static const IBaseFilterVtbl QT_Vtbl;
 static const IMediaSeekingVtbl QT_Seeking_Vtbl;
 
@@ -184,9 +181,9 @@ static HRESULT WINAPI QTSplitter_ChangeStart(IMediaSeeking *iface);
 static HRESULT WINAPI QTSplitter_ChangeStop(IMediaSeeking *iface);
 static HRESULT WINAPI QTSplitter_ChangeRate(IMediaSeeking *iface);
 
-static inline QTSplitter *impl_from_IMediaSeeking( IMediaSeeking *iface )
+static inline QTOutPin *impl_from_IMediaSeeking(IMediaSeeking *iface)
 {
-    return CONTAINING_RECORD(iface, QTSplitter, sourceSeeking.IMediaSeeking_iface);
+    return CONTAINING_RECORD(iface, QTOutPin, seeking.IMediaSeeking_iface);
 }
 
 static inline QTSplitter *impl_from_strmbase_filter(struct strmbase_filter *iface)
@@ -201,7 +198,7 @@ static inline QTSplitter *impl_from_IBaseFilter( IBaseFilter *iface )
 
 static inline QTInPin *impl_from_IPin(IPin *iface)
 {
-    return CONTAINING_RECORD(iface, QTInPin, pin.IPin_iface);
+    return CONTAINING_RECORD(iface, QTInPin, pin.pin.IPin_iface);
 }
 
 /*
@@ -213,7 +210,7 @@ static struct strmbase_pin *qt_splitter_get_pin(struct strmbase_filter *base, un
     QTSplitter *filter = impl_from_strmbase_filter(base);
 
     if (index == 0)
-        return &filter->pInputPin.pin;
+        return &filter->pInputPin.pin.pin;
     else if (index == 1)
     {
         if (filter->pVideo_Pin)
@@ -230,22 +227,18 @@ static struct strmbase_pin *qt_splitter_get_pin(struct strmbase_filter *base, un
 static void qt_splitter_destroy(struct strmbase_filter *iface)
 {
     QTSplitter *filter = impl_from_strmbase_filter(iface);
-    IPin *peer = NULL;
 
     EnterCriticalSection(&filter->csReceive);
     /* Don't need to clean up output pins, disconnecting input pin will do that */
+    if (filter->pInputPin.pin.pin.peer)
+        IPin_Disconnect(filter->pInputPin.pin.pin.peer);
 
-    if (filter->pInputPin.pin.peer)
-        IPin_Disconnect(filter->pInputPin.pin.peer);
-
-    FreeMediaType(&filter->pInputPin.pin.mt);
     if (filter->pInputPin.pAlloc)
         IMemAllocator_Release(filter->pInputPin.pAlloc);
     filter->pInputPin.pAlloc = NULL;
     if (filter->pInputPin.pReader)
         IAsyncReader_Release(filter->pInputPin.pReader);
     filter->pInputPin.pReader = NULL;
-    filter->pInputPin.pin.IPin_iface.lpVtbl = NULL;
 
     if (filter->pQTMovie)
     {
@@ -276,15 +269,54 @@ static void qt_splitter_destroy(struct strmbase_filter *iface)
 
     filter->csReceive.DebugInfo->Spare[0] = 0;
     DeleteCriticalSection(&filter->csReceive);
+    strmbase_sink_cleanup(&filter->pInputPin.pin);
     strmbase_filter_cleanup(&filter->filter);
 
     CoTaskMemFree(filter);
 }
 
+static HRESULT qt_splitter_start_stream(struct strmbase_filter *iface, REFERENCE_TIME time)
+{
+    QTSplitter *filter = impl_from_strmbase_filter(iface);
+    HRESULT hr = VFW_E_NOT_CONNECTED, pin_hr;
+
+    EnterCriticalSection(&filter->csReceive);
+
+    if (filter->pVideo_Pin)
+    {
+        if (SUCCEEDED(pin_hr = BaseOutputPinImpl_Active(&filter->pVideo_Pin->pin)))
+            hr = pin_hr;
+    }
+    if (filter->pAudio_Pin)
+    {
+        if (SUCCEEDED(pin_hr = BaseOutputPinImpl_Active(&filter->pAudio_Pin->pin)))
+            hr = pin_hr;
+    }
+    SetEvent(filter->runEvent);
+
+    LeaveCriticalSection(&filter->csReceive);
+
+    return hr;
+}
+
+static HRESULT qt_splitter_cleanup_stream(struct strmbase_filter *iface)
+{
+    QTSplitter *filter = impl_from_strmbase_filter(iface);
+
+    EnterCriticalSection(&filter->csReceive);
+    IAsyncReader_BeginFlush(filter->pInputPin.pReader);
+    IAsyncReader_EndFlush(filter->pInputPin.pReader);
+    LeaveCriticalSection(&filter->csReceive);
+
+    return S_OK;
+}
+
 static const struct strmbase_filter_ops filter_ops =
 {
     .filter_get_pin = qt_splitter_get_pin,
     .filter_destroy = qt_splitter_destroy,
+    .filter_start_stream = qt_splitter_start_stream,
+    .filter_cleanup_stream = qt_splitter_cleanup_stream,
 };
 
 static HRESULT sink_query_accept(struct strmbase_pin *iface, const AM_MEDIA_TYPE *mt)
@@ -299,13 +331,78 @@ static HRESULT sink_query_accept(struct strmbase_pin *iface, const AM_MEDIA_TYPE
     return S_FALSE;
 }
 
-static const BasePinFuncTable sink_ops =
+static HRESULT QT_Process_Movie(QTSplitter *filter);
+
+static HRESULT qt_splitter_sink_connect(struct strmbase_sink *iface, IPin *peer, const AM_MEDIA_TYPE *mt)
 {
-    .pin_query_accept = sink_query_accept,
-    .pin_get_media_type = strmbase_pin_get_media_type,
+    QTSplitter *filter = impl_from_strmbase_filter(iface->pin.filter);
+    ALLOCATOR_PROPERTIES props;
+    IMemAllocator *allocator;
+    HRESULT hr = S_OK;
+
+    filter->pInputPin.pReader = NULL;
+
+    if (FAILED(hr = IPin_QueryInterface(peer, &IID_IAsyncReader, (void **)&filter->pInputPin.pReader)))
+        return hr;
+
+    if (FAILED(hr = QT_Process_Movie(filter)))
+    {
+        IAsyncReader_Release(filter->pInputPin.pReader);
+        filter->pInputPin.pReader = NULL;
+        return hr;
+    }
+
+    filter->pInputPin.pAlloc = NULL;
+    props.cBuffers = 8;
+    props.cbAlign = 1;
+    props.cbBuffer = filter->outputSize + props.cbAlign;
+    props.cbPrefix = 0;
+
+    /* Some applications depend on IAsyncReader::RequestAllocator() passing a
+     * non-NULL preferred allocator. */
+    hr = CoCreateInstance(&CLSID_MemoryAllocator, NULL, CLSCTX_INPROC,
+            &IID_IMemAllocator, (void **)&allocator);
+    if (FAILED(hr))
+        goto err;
+
+    hr = IAsyncReader_RequestAllocator(filter->pInputPin.pReader, allocator, &props, &filter->pInputPin.pAlloc);
+    IMemAllocator_Release(allocator);
+    if (FAILED(hr))
+    {
+        WARN("Failed to get allocator, hr %#x.\n", hr);
+        goto err;
+    }
+
+    if (FAILED(hr = IMemAllocator_Commit(filter->pInputPin.pAlloc)))
+    {
+        WARN("Failed to commit allocator, hr %#x.\n", hr);
+        goto err;
+    }
+
+    return S_OK;
+err:
+    QT_RemoveOutputPins(filter);
+    IAsyncReader_Release(filter->pInputPin.pReader);
+    return hr;
+}
+
+static void qt_splitter_sink_disconnect(struct strmbase_sink *iface)
+{
+    QTSplitter *filter = impl_from_strmbase_filter(iface->pin.filter);
+
+    IMemAllocator_Decommit(filter->pInputPin.pAlloc);
+    QT_RemoveOutputPins(filter);
+}
+
+static const struct strmbase_sink_ops sink_ops =
+{
+    .base.pin_query_accept = sink_query_accept,
+    .base.pin_get_media_type = strmbase_pin_get_media_type,
+    .sink_connect = qt_splitter_sink_connect,
+    .sink_disconnect = qt_splitter_sink_disconnect,
 };
 
-IUnknown * CALLBACK QTSplitter_create(IUnknown *outer, HRESULT *phr)
+HRESULT qt_splitter_create(IUnknown *outer, IUnknown **out)
 {
     QTSplitter *This;
     static const WCHAR wcsInputPinName[] = {'I','n','p','u','t',' ','P','i','n',0};
@@ -314,91 +411,26 @@ IUnknown * CALLBACK QTSplitter_create(IUnknown *outer, HRESULT *phr)
 
     RegisterWineDataHandler();
 
-    This = CoTaskMemAlloc(sizeof(*This));
-    if (!This)
-    {
-        *phr = E_OUTOFMEMORY;
-        return NULL;
-    }
+    if (!(This = CoTaskMemAlloc(sizeof(*This))))
+        return E_OUTOFMEMORY;
     ZeroMemory(This,sizeof(*This));
 
-    strmbase_filter_init(&This->filter, &QT_Vtbl, outer, &CLSID_QTSplitter, &filter_ops);
+    strmbase_filter_init(&This->filter, outer, &CLSID_QTSplitter, &filter_ops);
+    strmbase_sink_init(&This->pInputPin.pin, &This->filter, wcsInputPinName, &sink_ops, NULL);
 
     InitializeCriticalSection(&This->csReceive);
     This->csReceive.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__": QTSplitter.csReceive");
 
     This->pVideo_Pin = NULL;
     This->pAudio_Pin = NULL;
-    This->state = State_Stopped;
     This->aSession = NULL;
     This->runEvent = CreateEventW(NULL, 0, 0, NULL);
 
-    This->pInputPin.pin.dir = PINDIR_INPUT;
-    This->pInputPin.pin.filter = &This->filter;
-    lstrcpynW(This->pInputPin.pin.name, wcsInputPinName, ARRAY_SIZE(This->pInputPin.pin.name));
-    This->pInputPin.pin.IPin_iface.lpVtbl = &QT_InputPin_Vtbl;
-    This->pInputPin.pin.peer = NULL;
-    This->pInputPin.pin.pFuncsTable = &sink_ops;
-
-    SourceSeeking_Init(&This->sourceSeeking, &QT_Seeking_Vtbl, QTSplitter_ChangeStop, QTSplitter_ChangeStart, QTSplitter_ChangeRate,  &This->filter.csFilter);
-
-    *phr = S_OK;
-    return &This->filter.IUnknown_inner;
-}
-
-static HRESULT WINAPI QT_QueryInterface(IBaseFilter *iface, REFIID riid, LPVOID *ppv)
-{
-    QTSplitter *This = impl_from_IBaseFilter(iface);
-    TRACE("(%s, %p)\n", debugstr_guid(riid), ppv);
-
-    *ppv = NULL;
-
-    if (IsEqualIID(riid, &IID_IUnknown))
-        *ppv = This;
-    else if (IsEqualIID(riid, &IID_IPersist))
-        *ppv = This;
-    else if (IsEqualIID(riid, &IID_IMediaFilter))
-        *ppv = This;
-    else if (IsEqualIID(riid, &IID_IBaseFilter))
-        *ppv = This;
-    else if (IsEqualIID(riid, &IID_IMediaSeeking))
-        *ppv = &This->sourceSeeking;
-
-    if (*ppv)
-    {
-        IUnknown_AddRef((IUnknown *)(*ppv));
-        return S_OK;
-    }
-
-    if (!IsEqualIID(riid, &IID_IPin) && !IsEqualIID(riid, &IID_IVideoWindow) &&
-        !IsEqualIID(riid, &IID_IAMFilterMiscFlags))
-        FIXME("No interface for %s!\n", debugstr_guid(riid));
-
-    return E_NOINTERFACE;
-}
-
-static HRESULT WINAPI QT_Stop(IBaseFilter *iface)
-{
-    QTSplitter *This = impl_from_IBaseFilter(iface);
-
-    TRACE("()\n");
-
-    EnterCriticalSection(&This->csReceive);
-    IAsyncReader_BeginFlush(This->pInputPin.pReader);
-    IAsyncReader_EndFlush(This->pInputPin.pReader);
-    LeaveCriticalSection(&This->csReceive);
-
+    TRACE("Created QT splitter %p.\n", This);
+    *out = &This->filter.IUnknown_inner;
     return S_OK;
 }
 
-static HRESULT WINAPI QT_Pause(IBaseFilter *iface)
-{
-    HRESULT hr = S_OK;
-    TRACE("()\n");
-
-    return hr;
-}
-
 static OSErr QT_Create_Extract_Session(QTSplitter *filter)
 {
     AudioStreamBasicDescription aDesc;
@@ -533,7 +565,6 @@ static DWORD WINAPI QTSplitter_thread(LPVOID data)
         return 0;
     }
 
-    This->state = State_Running;
     /* Prime the pump:  Needed for MPEG streams */
     GetMovieNextInterestingTime(This->pQTMovie, nextTimeEdgeOK | nextTimeStep, 0, NULL, This->movie_time, 1, &next_time, NULL);
 
@@ -732,7 +763,6 @@ audio_error:
         LeaveCriticalSection(&This->csReceive);
     } while (hr == S_OK);
 
-    This->state = State_Stopped;
     if (This->pAudio_Pin)
         OutputQueue_EOS(This->pAudio_Pin->queue);
     if (This->pVideo_Pin)
@@ -741,61 +771,6 @@ audio_error:
     return hr;
 }
 
-static HRESULT WINAPI QT_Run(IBaseFilter *iface, REFERENCE_TIME tStart)
-{
-    HRESULT hr = S_OK;
-    QTSplitter *This = impl_from_IBaseFilter(iface);
-    HRESULT hr_any = VFW_E_NOT_CONNECTED;
-
-    TRACE("(%s)\n", wine_dbgstr_longlong(tStart));
-
-    EnterCriticalSection(&This->csReceive);
-
-    if (This->pVideo_Pin)
-        hr = BaseOutputPinImpl_Active(&This->pVideo_Pin->pin);
-    if (SUCCEEDED(hr))
-        hr_any = hr;
-    if (This->pAudio_Pin)
-        hr = BaseOutputPinImpl_Active(&This->pAudio_Pin->pin);
-    if (SUCCEEDED(hr))
-        hr_any = hr;
-
-    hr = hr_any;
-
-    SetEvent(This->runEvent);
-    LeaveCriticalSection(&This->csReceive);
-
-    return hr;
-}
-
-static HRESULT WINAPI QT_GetState(IBaseFilter *iface, DWORD dwMilliSecsTimeout, FILTER_STATE *pState)
-{
-    QTSplitter *This = impl_from_IBaseFilter(iface);
-    TRACE("(%d, %p)\n", dwMilliSecsTimeout, pState);
-
-    *pState = This->state;
-
-    return S_OK;
-}
-
-static const IBaseFilterVtbl QT_Vtbl = {
-    QT_QueryInterface,
-    BaseFilterImpl_AddRef,
-    BaseFilterImpl_Release,
-    BaseFilterImpl_GetClassID,
-    QT_Stop,
-    QT_Pause,
-    QT_Run,
-    QT_GetState,
-    BaseFilterImpl_SetSyncSource,
-    BaseFilterImpl_GetSyncSource,
-    BaseFilterImpl_EnumPins,
-    BaseFilterImpl_FindPin,
-    BaseFilterImpl_QueryFilterInfo,
-    BaseFilterImpl_JoinFilterGraph,
-    BaseFilterImpl_QueryVendorInfo
-};
-
 static void free_source_pin(QTOutPin *pin)
 {
     if (pin->pin.pin.peer)
@@ -806,6 +781,7 @@ static void free_source_pin(QTOutPin *pin)
     }
 
     DeleteMediaType(pin->pmt);
+    strmbase_seeking_cleanup(&pin->seeking);
     strmbase_source_cleanup(&pin->pin);
     heap_free(pin);
 }
@@ -989,7 +965,7 @@ static HRESULT QT_Process_Movie(QTSplitter* filter)
     Track trk;
     short id = 0;
     DWORD tid;
-    LONGLONG time;
+    LONGLONG time, duration;
 
     TRACE("Trying movie connect\n");
 
@@ -1030,10 +1006,12 @@ static HRESULT QT_Process_Movie(QTSplitter* filter)
 
     time = GetMovieDuration(filter->pQTMovie);
     filter->movie_scale = GetMovieTimeScale(filter->pQTMovie);
-    filter->sourceSeeking.llDuration = ((double)time / filter->movie_scale) * 10000000;
-    filter->sourceSeeking.llStop = filter->sourceSeeking.llDuration;
-
-    TRACE("Movie duration is %s\n",wine_dbgstr_longlong(filter->sourceSeeking.llDuration));
+    duration = ((double)time / filter->movie_scale) * 10000000;
+    TRACE("Movie duration is %s.\n", wine_dbgstr_longlong(duration));
+    if (filter->pVideo_Pin)
+        filter->pVideo_Pin->seeking.llStop = filter->pVideo_Pin->seeking.llDuration = duration;
+    if (filter->pAudio_Pin)
+        filter->pAudio_Pin->seeking.llStop = filter->pAudio_Pin->seeking.llDuration = duration;
 
     filter->loaderThread = CreateThread(NULL, 0, QTSplitter_loading_thread, filter, 0, &tid);
     if (filter->loaderThread)
@@ -1047,209 +1025,7 @@ static HRESULT QT_Process_Movie(QTSplitter* filter)
     return hr;
 }
 
-static HRESULT WINAPI QTInPin_ReceiveConnection(IPin *iface, IPin *pReceivePin, const AM_MEDIA_TYPE *pmt)
-{
-    HRESULT hr = S_OK;
-    ALLOCATOR_PROPERTIES props;
-    QTInPin *This = impl_from_IPin(iface);
-    QTSplitter *filter = impl_from_strmbase_filter(This->pin.filter);
-    IMemAllocator *pAlloc;
-
-    TRACE("(%p/%p)->(%p, %p)\n", This, iface, pReceivePin, pmt);
-
-    EnterCriticalSection(&filter->filter.csFilter);
-    This->pReader = NULL;
-
-    if (This->pin.peer)
-        hr = VFW_E_ALREADY_CONNECTED;
-    else if (IPin_QueryAccept(iface, pmt) != S_OK)
-        hr = VFW_E_TYPE_NOT_ACCEPTED;
-    else
-    {
-        PIN_DIRECTION pindirReceive;
-        IPin_QueryDirection(pReceivePin, &pindirReceive);
-        if (pindirReceive != PINDIR_OUTPUT)
-            hr = VFW_E_INVALID_DIRECTION;
-    }
-
-    if (FAILED(hr))
-    {
-        LeaveCriticalSection(&filter->filter.csFilter);
-        return hr;
-    }
-
-    hr = IPin_QueryInterface(pReceivePin, &IID_IAsyncReader, (LPVOID *)&This->pReader);
-    if (FAILED(hr))
-    {
-        LeaveCriticalSection(&filter->filter.csFilter);
-        TRACE("Input source is not an AsyncReader\n");
-        return hr;
-    }
-
-    LeaveCriticalSection(&filter->filter.csFilter);
-    EnterCriticalSection(&filter->filter.csFilter);
-    hr = QT_Process_Movie(filter);
-    if (FAILED(hr))
-    {
-        IAsyncReader_Release(This->pReader);
-        This->pReader = NULL;
-        LeaveCriticalSection(&filter->filter.csFilter);
-        TRACE("Unable to process movie\n");
-        return hr;
-    }
-
-    This->pAlloc = NULL;
-    props.cBuffers = 8;
-    props.cbAlign = 1;
-    props.cbBuffer = filter->outputSize + props.cbAlign;
-    props.cbPrefix = 0;
-    hr = CoCreateInstance(&CLSID_MemoryAllocator, NULL, CLSCTX_INPROC,
-                          &IID_IMemAllocator, (LPVOID *)&pAlloc);
-    if (SUCCEEDED(hr))
-    {
-        /* A certain IAsyncReader::RequestAllocator expects to be passed
-           non-NULL preferred allocator */
-        hr = IAsyncReader_RequestAllocator(This->pReader, pAlloc, &props, &This->pAlloc);
-        if (FAILED(hr))
-            WARN("Can't get an allocator, got %08x\n", hr);
-        IMemAllocator_Release(pAlloc);
-    }
-
-    if (SUCCEEDED(hr))
-    {
-        CopyMediaType(&This->pin.mt, pmt);
-        This->pin.peer = pReceivePin;
-        IPin_AddRef(pReceivePin);
-        hr = IMemAllocator_Commit(This->pAlloc);
-    }
-    else
-    {
-        QT_RemoveOutputPins(filter);
-        if (This->pReader)
-            IAsyncReader_Release(This->pReader);
-        This->pReader = NULL;
-        if (This->pAlloc)
-            IMemAllocator_Release(This->pAlloc);
-        This->pAlloc = NULL;
-    }
-    TRACE("Size: %i\n", props.cbBuffer);
-    LeaveCriticalSection(&filter->filter.csFilter);
-
-    return hr;
-}
-
-static HRESULT WINAPI QTInPin_Disconnect(IPin *iface)
-{
-    HRESULT hr;
-    QTInPin *This = impl_from_IPin(iface);
-    QTSplitter *filter = impl_from_strmbase_filter(This->pin.filter);
-    FILTER_STATE state;
-    TRACE("()\n");
-
-    hr = IBaseFilter_GetState(&filter->filter.IBaseFilter_iface, INFINITE, &state);
-    EnterCriticalSection(&filter->filter.csFilter);
-    if (This->pin.peer)
-    {
-        QTSplitter *Parser = impl_from_strmbase_filter(This->pin.filter);
-
-        if (SUCCEEDED(hr) && state == State_Stopped)
-        {
-            IMemAllocator_Decommit(This->pAlloc);
-            IPin_Disconnect(This->pin.peer);
-            IPin_Release(This->pin.peer);
-            This->pin.peer = NULL;
-            hr = QT_RemoveOutputPins(Parser);
-        }
-        else
-            hr = VFW_E_NOT_STOPPED;
-    }
-    else
-        hr = S_FALSE;
-    LeaveCriticalSection(&filter->filter.csFilter);
-    return hr;
-}
-
-static HRESULT WINAPI QTInPin_EndOfStream(IPin *iface)
-{
-    FIXME("iface %p, stub!\n", iface);
-    return S_OK;
-}
-
-static HRESULT WINAPI QTInPin_BeginFlush(IPin *iface)
-{
-    FIXME("iface %p, stub!\n", iface);
-    return S_OK;
-}
-
-static HRESULT WINAPI QTInPin_EndFlush(IPin *iface)
-{
-    FIXME("iface %p, stub!\n", iface);
-    return S_OK;
-}
-
-static HRESULT WINAPI QTInPin_NewSegment(IPin *iface, REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate)
-{
-    BasePinImpl_NewSegment(iface, tStart, tStop, dRate);
-    FIXME("iface %p, stub!\n", iface);
-    return S_OK;
-}
-
-static HRESULT WINAPI QTInPin_QueryInterface(IPin * iface, REFIID riid, LPVOID * ppv)
-{
-    QTInPin *This = impl_from_IPin(iface);
-
-    TRACE("(%p/%p)->(%s, %p)\n", This, iface, debugstr_guid(riid), ppv);
-
-    *ppv = NULL;
-
-    if (IsEqualIID(riid, &IID_IUnknown))
-        *ppv = iface;
-    else if (IsEqualIID(riid, &IID_IPin))
-        *ppv = iface;
-    else if (IsEqualIID(riid, &IID_IMediaSeeking))
-        return IBaseFilter_QueryInterface(&This->pin.filter->IBaseFilter_iface, &IID_IMediaSeeking, ppv);
-
-    if (*ppv)
-    {
-        IUnknown_AddRef((IUnknown *)(*ppv));
-        return S_OK;
-    }
-
-    FIXME("No interface for %s!\n", debugstr_guid(riid));
-
-    return E_NOINTERFACE;
-}
-
-static const IPinVtbl QT_InputPin_Vtbl = {
-    QTInPin_QueryInterface,
-    BasePinImpl_AddRef,
-    BasePinImpl_Release,
-    BaseInputPinImpl_Connect,
-    QTInPin_ReceiveConnection,
-    QTInPin_Disconnect,
-    BasePinImpl_ConnectedTo,
-    BasePinImpl_ConnectionMediaType,
-    BasePinImpl_QueryPinInfo,
-    BasePinImpl_QueryDirection,
-    BasePinImpl_QueryId,
-    BasePinImpl_QueryAccept,
-    BasePinImpl_EnumMediaTypes,
-    BasePinImpl_QueryInternalConnections,
-    QTInPin_EndOfStream,
-    QTInPin_BeginFlush,
-    QTInPin_EndFlush,
-    QTInPin_NewSegment
-};
-
-/*
- * Output Pin
- */
-static inline QTOutPin *impl_QTOutPin_from_IPin( IPin *iface )
-{
-    return CONTAINING_RECORD(iface, QTOutPin, pin.pin.IPin_iface);
-}
-
-static inline QTOutPin *impl_sink_from_strmbase_pin(struct strmbase_pin *iface)
+static inline QTOutPin *impl_source_from_strmbase_pin(struct strmbase_pin *iface)
 {
     return CONTAINING_RECORD(iface, QTOutPin, pin.pin);
 }
@@ -1259,30 +1035,19 @@ static inline QTOutPin *impl_QTOutPin_from_BaseOutputPin(struct strmbase_source
     return CONTAINING_RECORD(iface, QTOutPin, pin);
 }
 
-static HRESULT WINAPI QTOutPin_QueryInterface(IPin *iface, REFIID riid, void **ppv)
+static HRESULT source_query_interface(struct strmbase_pin *iface, REFIID iid, void **out)
 {
-    QTOutPin *This = impl_QTOutPin_from_IPin(iface);
+    QTOutPin *pin = impl_source_from_strmbase_pin(&iface->IPin_iface);
 
-    TRACE("(%s, %p)\n", debugstr_guid(riid), ppv);
+    if (IsEqualGUID(iid, &IID_IMediaSeeking))
+        *out = &pin->seeking.IMediaSeeking_iface;
+    else if (IsEqualGUID(iid, &IID_IQualityControl))
+        *out = &pin->IQualityControl_iface;
+    else
+        return E_NOINTERFACE;
 
-    *ppv = NULL;
-
-    if (IsEqualIID(riid, &IID_IUnknown))
-        *ppv = iface;
-    else if (IsEqualIID(riid, &IID_IPin))
-        *ppv = iface;
-    else if (IsEqualIID(riid, &IID_IMediaSeeking))
-        return IBaseFilter_QueryInterface(&This->pin.pin.filter->IBaseFilter_iface, &IID_IMediaSeeking, ppv);
-    else if (IsEqualIID(riid, &IID_IQualityControl))
-        *ppv = &This->IQualityControl_iface;
-
-    if (*ppv)
-    {
-        IUnknown_AddRef((IUnknown *)(*ppv));
-        return S_OK;
-    }
-    FIXME("No interface for %s!\n", debugstr_guid(riid));
-    return E_NOINTERFACE;
+    IUnknown_AddRef((IUnknown *)*out);
+    return S_OK;
 }
 
 static HRESULT source_query_accept(struct strmbase_pin *base, const AM_MEDIA_TYPE *amt)
@@ -1293,7 +1058,7 @@ static HRESULT source_query_accept(struct strmbase_pin *base, const AM_MEDIA_TYP
 
 static HRESULT source_get_media_type(struct strmbase_pin *iface, unsigned int iPosition, AM_MEDIA_TYPE *pmt)
 {
-    QTOutPin *This = impl_sink_from_strmbase_pin(iface);
+    QTOutPin *This = impl_source_from_strmbase_pin(iface);
 
     if (iPosition > 0)
         return VFW_S_NO_MORE_ITEMS;
@@ -1331,27 +1096,6 @@ static HRESULT WINAPI QTOutPin_DecideAllocator(struct strmbase_source *iface,
     return hr;
 }
 
-static const IPinVtbl QT_OutputPin_Vtbl = {
-    QTOutPin_QueryInterface,
-    BasePinImpl_AddRef,
-    BasePinImpl_Release,
-    BaseOutputPinImpl_Connect,
-    BaseOutputPinImpl_ReceiveConnection,
-    BaseOutputPinImpl_Disconnect,
-    BasePinImpl_ConnectedTo,
-    BasePinImpl_ConnectionMediaType,
-    BasePinImpl_QueryPinInfo,
-    BasePinImpl_QueryDirection,
-    BasePinImpl_QueryId,
-    BasePinImpl_QueryAccept,
-    BasePinImpl_EnumMediaTypes,
-    BasePinImpl_QueryInternalConnections,
-    BaseOutputPinImpl_EndOfStream,
-    BaseOutputPinImpl_BeginFlush,
-    BaseOutputPinImpl_EndFlush,
-    BasePinImpl_NewSegment
-};
-
 static inline QTOutPin *impl_from_IQualityControl( IQualityControl *iface )
 {
     return CONTAINING_RECORD(iface, QTOutPin, IQualityControl_iface);
@@ -1424,11 +1168,12 @@ static HRESULT QT_AddPin(QTSplitter *filter, const WCHAR *name,
     else
         filter->pAudio_Pin = pin;
 
-    strmbase_source_init(&pin->pin, &QT_OutputPin_Vtbl, &filter->filter, name,
-            &source_ops);
+    strmbase_source_init(&pin->pin, &filter->filter, name, &source_ops);
     pin->pmt = CoTaskMemAlloc(sizeof(AM_MEDIA_TYPE));
     CopyMediaType(pin->pmt, mt);
     pin->IQualityControl_iface.lpVtbl = &QTOutPin_QualityControl_Vtbl;
+    strmbase_seeking_init(&pin->seeking, &QT_Seeking_Vtbl,
+            QTSplitter_ChangeStop, QTSplitter_ChangeStart, QTSplitter_ChangeRate);
     BaseFilterImpl_IncrementPinVersion(&filter->filter);
 
     return OutputQueue_Construct(&pin->pin, TRUE, TRUE, 5, FALSE,
@@ -1437,12 +1182,12 @@ static HRESULT QT_AddPin(QTSplitter *filter, const WCHAR *name,
 
 static HRESULT WINAPI QTSplitter_ChangeStart(IMediaSeeking *iface)
 {
-    QTSplitter *This = impl_from_IMediaSeeking(iface);
+    QTOutPin *pin = impl_from_IMediaSeeking(iface);
+    QTSplitter *filter = impl_from_strmbase_filter(pin->pin.pin.filter);
     TRACE("(%p)\n", iface);
-    EnterCriticalSection(&This->csReceive);
-    This->movie_time = (This->sourceSeeking.llCurrent * This->movie_scale)/10000000;
-    This->movie_start = This->movie_time;
-    LeaveCriticalSection(&This->csReceive);
+    EnterCriticalSection(&filter->csReceive);
+    filter->movie_start = filter->movie_time = (pin->seeking.llCurrent * filter->movie_scale)/10000000;
+    LeaveCriticalSection(&filter->csReceive);
     return S_OK;
 }
 
@@ -1458,25 +1203,22 @@ static HRESULT WINAPI QTSplitter_ChangeRate(IMediaSeeking *iface)
     return S_OK;
 }
 
-static HRESULT WINAPI QT_Seeking_QueryInterface(IMediaSeeking * iface, REFIID riid, LPVOID * ppv)
+static HRESULT WINAPI QT_Seeking_QueryInterface(IMediaSeeking *iface, REFIID iid, void **out)
 {
-    QTSplitter *This = impl_from_IMediaSeeking(iface);
-
-    return IBaseFilter_QueryInterface(&This->filter.IBaseFilter_iface, riid, ppv);
+    QTOutPin *pin = impl_from_IMediaSeeking(iface);
+    return IPin_QueryInterface(&pin->pin.pin.IPin_iface, iid, out);
 }
 
 static ULONG WINAPI QT_Seeking_AddRef(IMediaSeeking * iface)
 {
-    QTSplitter *This = impl_from_IMediaSeeking(iface);
-
-    return IBaseFilter_AddRef(&This->filter.IBaseFilter_iface);
+    QTOutPin *pin = impl_from_IMediaSeeking(iface);
+    return IPin_AddRef(&pin->pin.pin.IPin_iface);
 }
 
 static ULONG WINAPI QT_Seeking_Release(IMediaSeeking * iface)
 {
-    QTSplitter *This = impl_from_IMediaSeeking(iface);
-
-    return IBaseFilter_Release(&This->filter.IBaseFilter_iface);
+    QTOutPin *pin = impl_from_IMediaSeeking(iface);
+    return IPin_Release(&pin->pin.pin.IPin_iface);
 }
 
 static const IMediaSeekingVtbl QT_Seeking_Vtbl =
