@@ -27,6 +27,7 @@
 /* avoid conflict with field names in included win32 headers */
 #undef Status
 #include "wine/debug.h"
+#include "wine/heap.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(x11drv);
 
@@ -36,6 +37,8 @@ static unsigned int dd_mode_count;
 
 static unsigned int max_width;
 static unsigned int max_height;
+static unsigned int desktop_width;
+static unsigned int desktop_height;
 
 static struct screen_size {
     unsigned int width;
@@ -77,6 +80,12 @@ static struct screen_size {
 #define _NET_WM_STATE_REMOVE 0
 #define _NET_WM_STATE_ADD 1
 
+/* Return TRUE if Wine is currently in virtual desktop mode */
+BOOL is_virtual_desktop(void)
+{
+    return root_window != DefaultRootWindow( gdi_display );
+}
+
 /* create the mode structures */
 static void make_modes(void)
 {
@@ -140,6 +149,88 @@ static LONG X11DRV_desktop_SetCurrentMode(int mode)
     return DISP_CHANGE_SUCCESSFUL;
 }
 
+static void query_desktop_work_area( RECT *rc_work )
+{
+    static const WCHAR trayW[] = {'S','h','e','l','l','_','T','r','a','y','W','n','d',0};
+    RECT rect;
+    HWND hwnd = FindWindowW( trayW, NULL );
+
+    if (!hwnd || !IsWindowVisible( hwnd )) return;
+    if (!GetWindowRect( hwnd, &rect )) return;
+    if (rect.top) rc_work->bottom = rect.top;
+    else rc_work->top = rect.bottom;
+    TRACE( "found tray %p %s work area %s\n", hwnd, wine_dbgstr_rect( &rect ), wine_dbgstr_rect( rc_work ) );
+}
+
+static BOOL X11DRV_desktop_get_gpus( struct x11drv_gpu **new_gpus, int *count )
+{
+    static const WCHAR wine_adapterW[] = {'W','i','n','e',' ','A','d','a','p','t','e','r',0};
+    struct x11drv_gpu *gpu;
+
+    gpu = heap_calloc( 1, sizeof(*gpu) );
+    if (!gpu) return FALSE;
+
+    lstrcpyW( gpu->name, wine_adapterW );
+    *new_gpus = gpu;
+    *count = 1;
+    return TRUE;
+}
+
+static void X11DRV_desktop_free_gpus( struct x11drv_gpu *gpus )
+{
+    heap_free( gpus );
+}
+
+/* TODO: Support multi-head virtual desktop */
+static BOOL X11DRV_desktop_get_adapters( ULONG_PTR gpu_id, struct x11drv_adapter **new_adapters, int *count )
+{
+    struct x11drv_adapter *adapter;
+
+    adapter = heap_calloc( 1, sizeof(*adapter) );
+    if (!adapter) return FALSE;
+
+    adapter->state_flags = DISPLAY_DEVICE_PRIMARY_DEVICE;
+    if (desktop_width && desktop_height)
+        adapter->state_flags |= DISPLAY_DEVICE_ATTACHED_TO_DESKTOP;
+
+    *new_adapters = adapter;
+    *count = 1;
+    return TRUE;
+}
+
+static void X11DRV_desktop_free_adapters( struct x11drv_adapter *adapters )
+{
+    heap_free( adapters );
+}
+
+static BOOL X11DRV_desktop_get_monitors( ULONG_PTR adapter_id, struct x11drv_monitor **new_monitors, int *count )
+{
+    static const WCHAR generic_nonpnp_monitorW[] = {
+        'G','e','n','e','r','i','c',' ',
+        'N','o','n','-','P','n','P',' ','M','o','n','i','t','o','r',0};
+    struct x11drv_monitor *monitor;
+
+    monitor = heap_calloc( 1, sizeof(*monitor) );
+    if (!monitor) return FALSE;
+
+    lstrcpyW( monitor->name, generic_nonpnp_monitorW );
+    SetRect( &monitor->rc_monitor, 0, 0, desktop_width, desktop_height );
+    SetRect( &monitor->rc_work, 0, 0, desktop_width, desktop_height );
+    query_desktop_work_area( &monitor->rc_work );
+    monitor->state_flags = DISPLAY_DEVICE_ATTACHED;
+    if (desktop_width && desktop_height)
+        monitor->state_flags |= DISPLAY_DEVICE_ACTIVE;
+
+    *new_monitors = monitor;
+    *count = 1;
+    return TRUE;
+}
+
+static void X11DRV_desktop_free_monitors( struct x11drv_monitor *monitors )
+{
+    heap_free( monitors );
+}
+
 /***********************************************************************
  *		X11DRV_init_desktop
  *
@@ -147,18 +238,27 @@ static LONG X11DRV_desktop_SetCurrentMode(int mode)
  */
 void X11DRV_init_desktop( Window win, unsigned int width, unsigned int height )
 {
-    RECT primary_rect;
+    RECT primary_rect = get_host_primary_monitor_rect();
 
     root_window = win;
     managed_mode = FALSE;  /* no managed windows in desktop mode */
+    desktop_width = width;
+    desktop_height = height;
+    max_width = primary_rect.right;
+    max_height = primary_rect.bottom;
 
-    xinerama_init( width, height );
+    /* Initialize virtual desktop mode display device handler */
+    desktop_handler.name = "Virtual Desktop";
+    desktop_handler.get_gpus = X11DRV_desktop_get_gpus;
+    desktop_handler.get_adapters = X11DRV_desktop_get_adapters;
+    desktop_handler.get_monitors = X11DRV_desktop_get_monitors;
+    desktop_handler.free_gpus = X11DRV_desktop_free_gpus;
+    desktop_handler.free_adapters = X11DRV_desktop_free_adapters;
+    desktop_handler.free_monitors = X11DRV_desktop_free_monitors;
+    desktop_handler.register_event_handlers = NULL;
+    TRACE("Display device functions are now handled by: Virtual Desktop\n");
     X11DRV_DisplayDevices_Init( TRUE );
 
-    primary_rect = get_primary_monitor_rect();
-    max_width = primary_rect.right - primary_rect.left;
-    max_height = primary_rect.bottom - primary_rect.top;
-
     /* initialize the available resolutions */
     dd_modes = X11DRV_Settings_SetHandlers("desktop", 
                                            X11DRV_desktop_GetCurrentMode, 
@@ -181,7 +281,6 @@ BOOL CDECL X11DRV_create_desktop( UINT width, UINT height )
     XSetWindowAttributes win_attr;
     Window win;
     Display *display = thread_init_display();
-    RECT rect;
     WCHAR name[MAX_PATH];
 
     if (!GetUserObjectInformationW( GetThreadDesktop( GetCurrentThreadId() ),
@@ -208,41 +307,30 @@ BOOL CDECL X11DRV_create_desktop( UINT width, UINT height )
                          0, 0, width, height, 0, default_visual.depth, InputOutput, default_visual.visual,
                          CWEventMask | CWCursor | CWColormap, &win_attr );
     if (!win) return FALSE;
+    if (!create_desktop_win_data( win )) return FALSE;
 
-    SetRect( &rect, 0, 0, width, height );
-    if (is_window_rect_fullscreen( &rect ))
+    X11DRV_init_desktop( win, width, height );
+    if (is_desktop_fullscreen())
     {
         TRACE("setting desktop to fullscreen\n");
         XChangeProperty( display, win, x11drv_atom(_NET_WM_STATE), XA_ATOM, 32,
             PropModeReplace, (unsigned char*)&x11drv_atom(_NET_WM_STATE_FULLSCREEN),
             1);
     }
-    if (!create_desktop_win_data( win )) return FALSE;
     XFlush( display );
-    X11DRV_init_desktop( win, width, height );
     return TRUE;
 }
 
-
-struct desktop_resize_data
-{
-    RECT  old_virtual_rect;
-    RECT  new_virtual_rect;
-};
-
 static BOOL CALLBACK update_windows_on_desktop_resize( HWND hwnd, LPARAM lparam )
 {
     struct x11drv_win_data *data;
-    struct desktop_resize_data *resize_data = (struct desktop_resize_data *)lparam;
-    int mask = 0;
+    UINT mask = (UINT)lparam;
 
     if (!(data = get_win_data( hwnd ))) return TRUE;
 
     /* update the full screen state */
     update_net_wm_states( data );
 
-    if (resize_data->old_virtual_rect.left != resize_data->new_virtual_rect.left) mask |= CWX;
-    if (resize_data->old_virtual_rect.top != resize_data->new_virtual_rect.top) mask |= CWY;
     if (mask && data->whole_window)
     {
         POINT pos = virtual_screen_to_root( data->whole_rect.left, data->whole_rect.top );
@@ -268,7 +356,7 @@ static void update_desktop_fullscreen( unsigned int width, unsigned int height)
     Display *display = thread_display();
     XEvent xev;
 
-    if (!display || root_window == DefaultRootWindow( display )) return;
+    if (!display || !is_virtual_desktop()) return;
 
     xev.xclient.type = ClientMessage;
     xev.xclient.window = root_window;
@@ -301,14 +389,18 @@ static void update_desktop_fullscreen( unsigned int width, unsigned int height)
  */
 void X11DRV_resize_desktop( unsigned int width, unsigned int height )
 {
+    RECT old_virtual_rect, new_virtual_rect;
     HWND hwnd = GetDesktopWindow();
-    struct desktop_resize_data resize_data;
+    UINT mask = 0;
 
-    resize_data.old_virtual_rect = get_virtual_screen_rect();
-
-    xinerama_init( width, height );
+    old_virtual_rect = get_virtual_screen_rect();
+    desktop_width = width;
+    desktop_height = height;
     X11DRV_DisplayDevices_Init( TRUE );
-    resize_data.new_virtual_rect = get_virtual_screen_rect();
+    new_virtual_rect = get_virtual_screen_rect();
+
+    if (old_virtual_rect.left != new_virtual_rect.left) mask |= CWX;
+    if (old_virtual_rect.top != new_virtual_rect.top) mask |= CWY;
 
     if (GetWindowThreadProcessId( hwnd, NULL ) != GetCurrentThreadId())
     {
@@ -318,14 +410,13 @@ void X11DRV_resize_desktop( unsigned int width, unsigned int height )
     {
         TRACE( "desktop %p change to (%dx%d)\n", hwnd, width, height );
         update_desktop_fullscreen( width, height );
-        SetWindowPos( hwnd, 0, resize_data.new_virtual_rect.left, resize_data.new_virtual_rect.top,
-                      resize_data.new_virtual_rect.right - resize_data.new_virtual_rect.left,
-                      resize_data.new_virtual_rect.bottom - resize_data.new_virtual_rect.top,
+        SetWindowPos( hwnd, 0, new_virtual_rect.left, new_virtual_rect.top,
+                      new_virtual_rect.right - new_virtual_rect.left, new_virtual_rect.bottom - new_virtual_rect.top,
                       SWP_NOZORDER | SWP_NOACTIVATE | SWP_DEFERERASE );
         ungrab_clipping_window();
         SendMessageTimeoutW( HWND_BROADCAST, WM_DISPLAYCHANGE, screen_bpp,
                              MAKELPARAM( width, height ), SMTO_ABORTIFHUNG, 2000, NULL );
     }
 
-    EnumWindows( update_windows_on_desktop_resize, (LPARAM)&resize_data );
+    EnumWindows( update_windows_on_desktop_resize, (LPARAM)mask );
 }
