@@ -25,6 +25,7 @@
 #include "winbase.h"
 #include "winuser.h"
 #include "wininet.h"
+#include "winineti.h"
 #include "winerror.h"
 #include "winreg.h"
 
@@ -376,7 +377,6 @@ static void test_complicated_cookie(void)
   BOOL ret;
 
   CHAR buffer[1024];
-  CHAR user[256];
   WCHAR wbuf[1024];
 
   static const WCHAR testing_example_comW[] =
@@ -530,25 +530,16 @@ static void test_complicated_cookie(void)
   ret = InternetSetCookieA("http://testing.example.com", NULL, "A=B; expires=Fri, 01-Jan-2038 00:00:00 GMT");
   ok(ret, "InternetSetCookie failed with error %d\n", GetLastError());
 
-  len = sizeof(user);
-  ret = GetUserNameA(user, &len);
-  ok(ret, "GetUserName failed with error %d\n", GetLastError());
-  for(; len>0; len--)
-      user[len-1] = tolower(user[len-1]);
-
-  sprintf(buffer, "Cookie:%s@testing.example.com/", user);
-  ret = GetUrlCacheEntryInfoA(buffer, NULL, &len);
-  ok(!ret, "GetUrlCacheEntryInfo succeeded\n");
-  ok(GetLastError() == ERROR_INSUFFICIENT_BUFFER, "GetLastError() = %d\n", GetLastError());
+  len = sizeof(buffer);
+  ret = InternetGetCookieA("http://testing.example.com/foobar", NULL, buffer, &len);
+  ok(ret, "got error %u\n", GetLastError());
+  ok(len == 24, "got len %u\n", len);
+  ok(!!strstr(buffer, "A=B"), "cookie is not present\n");
 
   /* remove persistent cookie */
   ret = InternetSetCookieA("http://testing.example.com", NULL, "A=B");
   ok(ret, "InternetSetCookie failed with error %d\n", GetLastError());
 
-  ret = GetUrlCacheEntryInfoA(buffer, NULL, &len);
-  ok(!ret, "GetUrlCacheEntryInfo succeeded\n");
-  ok(GetLastError() == ERROR_FILE_NOT_FOUND, "GetLastError() = %d\n", GetLastError());
-
   /* try setting cookie for different domain */
   ret = InternetSetCookieA("http://www.aaa.example.com/bar",NULL,"E=F; domain=different.com");
   ok(!ret, "InternetSetCookie succeeded\n");
@@ -629,7 +620,7 @@ static void test_cookie_url(void)
     ok(!res && GetLastError() == ERROR_INVALID_PARAMETER,
        "InternetGetCookeA failed: %u, expected ERROR_INVALID_PARAMETER\n", GetLastError());
 
-    len = sizeof(bufw)/sizeof(*bufw);
+    len = ARRAY_SIZE(bufw);
     res = InternetGetCookieW(about_blankW, NULL, bufw, &len);
     ok(!res && GetLastError() == ERROR_INVALID_PARAMETER,
        "InternetGetCookeW failed: %u, expected ERROR_INVALID_PARAMETER\n", GetLastError());
@@ -639,7 +630,7 @@ static void test_cookie_url(void)
     ok(!res && GetLastError() == ERROR_INVALID_PARAMETER,
        "InternetGetCookeExA failed: %u, expected ERROR_INVALID_PARAMETER\n", GetLastError());
 
-    len = sizeof(bufw)/sizeof(*bufw);
+    len = ARRAY_SIZE(bufw);
     res = pInternetGetCookieExW(about_blankW, NULL, bufw, &len, 0, NULL);
     ok(!res && GetLastError() == ERROR_INVALID_PARAMETER,
        "InternetGetCookeExW failed: %u, expected ERROR_INVALID_PARAMETER\n", GetLastError());
@@ -887,7 +878,7 @@ static void InternetTimeFromSystemTimeW_test(void)
 
     /* test too small buffer size */
     SetLastError(0xdeadbeef);
-    ret = pInternetTimeFromSystemTimeW( &time, INTERNET_RFC1123_FORMAT, string, sizeof(string)/sizeof(string[0]) );
+    ret = pInternetTimeFromSystemTimeW( &time, INTERNET_RFC1123_FORMAT, string, ARRAY_SIZE(string));
     error = GetLastError();
     ok( !ret, "InternetTimeFromSystemTimeW should have returned FALSE\n" );
     ok( error == ERROR_INSUFFICIENT_BUFFER,
@@ -1084,7 +1075,7 @@ static void test_PrivacyGetSetZonePreferenceW(void)
     trace("template %u\n", old_template);
 
     if(old_template == PRIVACY_TEMPLATE_ADVANCED) {
-        pref_size = sizeof(pref)/sizeof(WCHAR);
+        pref_size = ARRAY_SIZE(pref);
         ret = pPrivacyGetZonePreferenceW(zone, type, &old_template, pref, &pref_size);
         ok(ret == 0, "expected ret == 0, got %u\n", ret);
     }
@@ -1469,7 +1460,7 @@ static void test_Option_PerConnectionOptionA(void)
 static void test_InternetErrorDlg(void)
 {
     HINTERNET ses, con, req;
-    DWORD res, flags;
+    DWORD res;
     HWND hwnd;
     ULONG i;
     static const struct {
@@ -1496,12 +1487,9 @@ static void test_InternetErrorDlg(void)
         { ERROR_INTERNET_UNABLE_TO_DOWNLOAD_SCRIPT, ERROR_CANCELLED, FLAG_TODO },
         { ERROR_HTTP_REDIRECT_NEEDS_CONFIRMATION, ERROR_CANCELLED, FLAG_TODO },
         { ERROR_INTERNET_SEC_CERT_REVOKED       , ERROR_CANCELLED, 0 },
-        { 0, ERROR_NOT_SUPPORTED }
     };
 
-    flags = 0;
-
-    res = InternetErrorDlg(NULL, NULL, 12055, flags, NULL);
+    res = InternetErrorDlg(NULL, NULL, ERROR_INTERNET_SEC_CERT_ERRORS, 0, NULL);
     ok(res == ERROR_INVALID_HANDLE, "Got %d\n", res);
 
     ses = InternetOpenA(NULL, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);
@@ -1511,30 +1499,24 @@ static void test_InternetErrorDlg(void)
     req = HttpOpenRequestA(con, "GET", "/", NULL, NULL, NULL, 0, 0);
     ok(req != 0, "HttpOpenRequest failed: 0x%08x\n", GetLastError());
 
-    /* NULL hwnd and FLAGS_ERROR_UI_FLAGS_NO_UI not set */
-    for(i = INTERNET_ERROR_BASE; i < INTERNET_ERROR_LAST; i++)
-    {
-        res = InternetErrorDlg(NULL, req, i, flags, NULL);
-        ok(res == ERROR_INVALID_HANDLE, "Got %d (%d)\n", res, i);
-    }
-
     hwnd = GetDesktopWindow();
     ok(hwnd != NULL, "GetDesktopWindow failed (%d)\n", GetLastError());
 
-    flags = FLAGS_ERROR_UI_FLAGS_NO_UI;
     for(i = INTERNET_ERROR_BASE; i < INTERNET_ERROR_LAST; i++)
     {
-        DWORD expected, test_flags, j;
+        DWORD expected = ERROR_NOT_SUPPORTED, test_flags = 0, j;
 
-        for(j = 0; no_ui_res[j].error != 0; ++j)
-            if(no_ui_res[j].error == i)
-                break;
-
-        test_flags = no_ui_res[j].test_flags;
-        expected = no_ui_res[j].res;
+        for (j = 0; j < ARRAY_SIZE(no_ui_res); ++j)
+        {
+            if (no_ui_res[j].error == i)
+            {
+                test_flags = no_ui_res[j].test_flags;
+                expected = no_ui_res[j].res;
+            }
+        }
 
         /* Try an invalid request handle */
-        res = InternetErrorDlg(hwnd, (HANDLE)0xdeadbeef, i, flags, NULL);
+        res = InternetErrorDlg(hwnd, (HANDLE)0xdeadbeef, i, FLAGS_ERROR_UI_FLAGS_NO_UI, NULL);
         if(res == ERROR_CALL_NOT_IMPLEMENTED)
         {
             ok(test_flags & FLAG_UNIMPL, "%i is unexpectedly unimplemented.\n", i);
@@ -1549,29 +1531,23 @@ static void test_InternetErrorDlg(void)
 
         if(i == ERROR_INTERNET_SEC_CERT_REVOKED)
             continue; /* Interactive (XP, Win7) */
+        if (i == ERROR_INTERNET_PROXY_ALERT)
+            continue; /* Interactive (Win10 1607+) */
 
-        res = InternetErrorDlg(hwnd, req, i, flags, NULL);
+        res = InternetErrorDlg(hwnd, req, i, FLAGS_ERROR_UI_FLAGS_NO_UI, NULL);
 
         /* Handle some special cases */
         switch(i)
         {
-        case ERROR_INTERNET_HTTP_TO_HTTPS_ON_REDIR:
-        case ERROR_INTERNET_HTTPS_TO_HTTP_ON_REDIR:
+        case ERROR_INTERNET_HTTP_TO_HTTPS_ON_REDIR: /* later 9.x versions */
+        case ERROR_INTERNET_HTTPS_TO_HTTP_ON_REDIR: /* later 9.x versions */
+        case ERROR_INTERNET_SEC_CERT_WEAK_SIGNATURE: /* later 11.x versions */
             if(res == ERROR_CANCELLED)
-            {
-                /* Some windows XP, w2k3 x64, W2K8 */
-                win_skip("Skipping some tests for %d\n", i);
-                continue;
-            }
+                expected = ERROR_CANCELLED;
             break;
         case ERROR_INTERNET_FORTEZZA_LOGIN_NEEDED:
-            if(res != expected)
-            {
-                /* Windows XP, W2K3 */
-                ok(res == NTE_PROV_TYPE_NOT_DEF, "Got %d\n", res);
-                win_skip("Skipping some tests for %d\n", i);
-                continue;
-            }
+            if (res == NTE_PROV_TYPE_NOT_DEF) /* XP, 2003 */
+                expected = NTE_PROV_TYPE_NOT_DEF;
             break;
         case ERROR_INTERNET_CHG_POST_IS_NON_SECURE:
             if(res == ERROR_SUCCESS) /* win10 returns ERROR_SUCCESS */
@@ -1580,11 +1556,14 @@ static void test_InternetErrorDlg(void)
         default: break;
         }
 
+        if (expected == ERROR_NOT_SUPPORTED && res == ERROR_CANCELLED) /* Win10 1607+ */
+            expected = ERROR_CANCELLED;
+
         todo_wine_if(test_flags & FLAG_TODO)
             ok(res == expected, "Got %d, expected %d (%d)\n", res, expected, i);
 
         /* Same thing with NULL hwnd */
-        res = InternetErrorDlg(NULL, req, i, flags, NULL);
+        res = InternetErrorDlg(NULL, req, i, FLAGS_ERROR_UI_FLAGS_NO_UI, NULL);
         todo_wine_if(test_flags & FLAG_TODO)
             ok(res == expected, "Got %d, expected %d (%d)\n", res, expected, i);
 
@@ -1593,7 +1572,7 @@ static void test_InternetErrorDlg(void)
         if(test_flags & FLAG_NEEDREQ)
             expected = ERROR_INVALID_PARAMETER;
 
-        res = InternetErrorDlg(hwnd, NULL, i, flags, NULL);
+        res = InternetErrorDlg(hwnd, NULL, i, FLAGS_ERROR_UI_FLAGS_NO_UI, NULL);
         todo_wine_if( test_flags & FLAG_TODO || i == ERROR_INTERNET_INCORRECT_PASSWORD)
             ok(res == expected, "Got %d, expected %d (%d)\n", res, expected, i);
     }
@@ -1721,7 +1700,7 @@ static void test_InternetGetConnectedStateExW(void)
 
     flags = 0;
     buffer[0] = 0;
-    res = pInternetGetConnectedStateExW(&flags, buffer, sizeof(buffer) / sizeof(buffer[0]), 0);
+    res = pInternetGetConnectedStateExW(&flags, buffer, ARRAY_SIZE(buffer), 0);
     trace("Internet Connection: Flags 0x%02x - Name '%s'\n", flags, wine_dbgstr_w(buffer));
 todo_wine
     ok (flags & INTERNET_RAS_INSTALLED, "Missing RAS flag\n");
@@ -1755,21 +1734,21 @@ todo_wine
     ok(!buffer[0], "Buffer must not change, got %02X\n", buffer[0]);
 
     buffer[0] = 0;
-    res = pInternetGetConnectedStateExW(NULL, buffer, sizeof(buffer) / sizeof(buffer[0]), 0);
+    res = pInternetGetConnectedStateExW(NULL, buffer, ARRAY_SIZE(buffer), 0);
     ok(res == TRUE, "Expected TRUE, got %d\n", res);
     sz = lstrlenW(buffer);
     ok(sz > 0, "Expected a connection name\n");
 
     buffer[0] = 0;
     flags = 0;
-    res = pInternetGetConnectedStateExW(&flags, buffer, sizeof(buffer) / sizeof(buffer[0]), 0);
+    res = pInternetGetConnectedStateExW(&flags, buffer, ARRAY_SIZE(buffer), 0);
     ok(res == TRUE, "Expected TRUE, got %d\n", res);
     ok(flags, "Expected at least one flag set\n");
     sz = lstrlenW(buffer);
     ok(sz > 0, "Expected a connection name\n");
 
     flags = 0;
-    res = pInternetGetConnectedStateExW(&flags, NULL, sizeof(buffer) / sizeof(buffer[0]), 0);
+    res = pInternetGetConnectedStateExW(&flags, NULL, ARRAY_SIZE(buffer), 0);
     ok(res == TRUE, "Expected TRUE, got %d\n", res);
     ok(flags, "Expected at least one flag set\n");
 
