@@ -524,7 +524,7 @@ static void test_buffer_callbacks(IXAudio2 *xa)
 
     if(xaudio27){
         hr = IXAudio27SourceVoice_SetSourceSampleRate((IXAudio27SourceVoice*)src, 48000);
-        todo_wine ok(hr == S_OK, "SetSourceSampleRate failed: %08x\n", hr);
+        ok(hr == S_OK, "SetSourceSampleRate failed: %08x\n", hr);
     }else{
         hr = IXAudio2SourceVoice_SetSourceSampleRate(src, 48000);
         ok(hr == XAUDIO2_E_INVALID_CALL, "SetSourceSampleRate should have failed: %08x\n", hr);
@@ -948,23 +948,21 @@ static void test_xapo_creation_legacy(const char *module, unsigned int version)
     HRESULT hr;
     IUnknown *fx_unk;
     unsigned int i;
+    ULONG rc;
 
     HRESULT (CDECL *pCreateFX)(REFCLSID,IUnknown**) = NULL;
 
     /* CLSIDs are the same across all versions */
-    static struct {
-        const GUID *clsid;
-        BOOL todo;
-    } const_clsids[] = {
-        { &CLSID_FXEQ27, FALSE },
-        { &CLSID_FXMasteringLimiter27, TRUE },
-        { &CLSID_FXReverb27, FALSE },
-        { &CLSID_FXEcho27, TRUE},
+    static const GUID *const_clsids[] = {
+        &CLSID_FXEQ27,
+        &CLSID_FXMasteringLimiter27,
+        &CLSID_FXReverb27,
+        &CLSID_FXEcho27,
         /* older versions of xapofx actually have support for new clsids */
-        { &CLSID_FXEQ, FALSE },
-        { &CLSID_FXMasteringLimiter, TRUE },
-        { &CLSID_FXReverb, FALSE },
-        { &CLSID_FXEcho, TRUE}
+        &CLSID_FXEQ,
+        &CLSID_FXMasteringLimiter,
+        &CLSID_FXReverb,
+        &CLSID_FXEcho
     };
 
     /* different CLSID for each version */
@@ -1003,48 +1001,48 @@ static void test_xapo_creation_legacy(const char *module, unsigned int version)
         return;
     }
 
-    if(pCreateFX){
-        for(i = 0; i < ARRAY_SIZE(const_clsids); ++i){
-            hr = pCreateFX(const_clsids[i].clsid, &fx_unk);
-            todo_wine_if(const_clsids[i].todo)
-                ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(const_clsids[i].clsid), hr);
-            if(SUCCEEDED(hr)){
-                IXAPO *xapo;
-                hr = IUnknown_QueryInterface(fx_unk, &IID_IXAPO27, (void**)&xapo);
-                ok(hr == S_OK, "Couldn't get IXAPO27 interface: %08x\n", hr);
-                if(SUCCEEDED(hr))
-                    IXAPO_Release(xapo);
-                IUnknown_Release(fx_unk);
-            }
-
-            hr = CoCreateInstance(const_clsids[i].clsid, NULL, CLSCTX_INPROC_SERVER,
-                    &IID_IUnknown, (void**)&fx_unk);
-            ok(hr == REGDB_E_CLASSNOTREG, "CoCreateInstance should have failed: %08x\n", hr);
-            if(SUCCEEDED(hr))
-                IUnknown_Release(fx_unk);
-        }
-
-        hr = pCreateFX(avm_clsids[version - 20], &fx_unk);
-        ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(avm_clsids[version - 20]), hr);
+    for(i = 0; i < ARRAY_SIZE(const_clsids); ++i){
+        hr = pCreateFX(const_clsids[i], &fx_unk);
+        ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(const_clsids[i]), hr);
         if(SUCCEEDED(hr)){
             IXAPO *xapo;
             hr = IUnknown_QueryInterface(fx_unk, &IID_IXAPO27, (void**)&xapo);
             ok(hr == S_OK, "Couldn't get IXAPO27 interface: %08x\n", hr);
             if(SUCCEEDED(hr))
                 IXAPO_Release(xapo);
-            IUnknown_Release(fx_unk);
+            rc = IUnknown_Release(fx_unk);
+            ok(rc == 0, "XAPO via CreateFX should have been released, got refcount: %u\n", rc);
         }
 
-        hr = pCreateFX(ar_clsids[version - 20], &fx_unk);
-        ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(ar_clsids[version - 20]), hr);
-        if(SUCCEEDED(hr)){
-            IXAPO *xapo;
-            hr = IUnknown_QueryInterface(fx_unk, &IID_IXAPO27, (void**)&xapo);
-            ok(hr == S_OK, "Couldn't get IXAPO27 interface: %08x\n", hr);
-            if(SUCCEEDED(hr))
-                IXAPO_Release(xapo);
+        hr = CoCreateInstance(const_clsids[i], NULL, CLSCTX_INPROC_SERVER,
+                &IID_IUnknown, (void**)&fx_unk);
+        ok(hr == REGDB_E_CLASSNOTREG, "CoCreateInstance should have failed: %08x\n", hr);
+        if(SUCCEEDED(hr))
             IUnknown_Release(fx_unk);
-        }
+    }
+
+    hr = pCreateFX(avm_clsids[version - 20], &fx_unk);
+    ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(avm_clsids[version - 20]), hr);
+    if(SUCCEEDED(hr)){
+        IXAPO *xapo;
+        hr = IUnknown_QueryInterface(fx_unk, &IID_IXAPO27, (void**)&xapo);
+        ok(hr == S_OK, "Couldn't get IXAPO27 interface: %08x\n", hr);
+        if(SUCCEEDED(hr))
+            IXAPO_Release(xapo);
+        rc = IUnknown_Release(fx_unk);
+        ok(rc == 0, "AudioVolumeMeter via CreateFX should have been released, got refcount: %u\n", rc);
+    }
+
+    hr = pCreateFX(ar_clsids[version - 20], &fx_unk);
+    ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(ar_clsids[version - 20]), hr);
+    if(SUCCEEDED(hr)){
+        IXAPO *xapo;
+        hr = IUnknown_QueryInterface(fx_unk, &IID_IXAPO27, (void**)&xapo);
+        ok(hr == S_OK, "Couldn't get IXAPO27 interface: %08x\n", hr);
+        if(SUCCEEDED(hr))
+            IXAPO_Release(xapo);
+        rc = IUnknown_Release(fx_unk);
+        ok(rc == 0, "AudioReverb via CreateFX should have been released, got refcount: %u\n", rc);
     }
 
     FreeLibrary(xapofxdll);
@@ -1056,26 +1054,25 @@ static void test_xapo_creation_modern(const char *module)
     HRESULT hr;
     IUnknown *fx_unk;
     unsigned int i;
+    ULONG rc;
 
     HRESULT (CDECL *pCreateFX)(REFCLSID,IUnknown**,void*,UINT32) = NULL;
     HRESULT (WINAPI *pCAVM)(IUnknown**) = NULL;
     HRESULT (WINAPI *pCAR)(IUnknown**) = NULL;
 
     /* CLSIDs are the same across all versions */
-    static struct {
-        const GUID *clsid;
-        BOOL todo;
-    } const_clsids[] = {
-        { &CLSID_FXEQ27, FALSE },
-        { &CLSID_FXMasteringLimiter27, TRUE },
-        { &CLSID_FXReverb27, FALSE },
-        { &CLSID_FXEcho27, TRUE},
-        { &CLSID_FXEQ, FALSE },
-        { &CLSID_FXMasteringLimiter, TRUE },
-        { &CLSID_FXReverb, FALSE },
-        { &CLSID_FXEcho, TRUE}
+    static const GUID *const_clsids[] = {
+        &CLSID_FXEQ27,
+        &CLSID_FXMasteringLimiter27,
+        &CLSID_FXReverb27,
+        &CLSID_FXEcho27,
+        &CLSID_FXEQ,
+        &CLSID_FXMasteringLimiter,
+        &CLSID_FXReverb,
+        &CLSID_FXEcho
     };
 
+
     xaudio2dll = LoadLibraryA(module);
     if(xaudio2dll){
         pCreateFX = (void*)GetProcAddress(xaudio2dll, "CreateFX");
@@ -1089,38 +1086,37 @@ static void test_xapo_creation_modern(const char *module)
         return;
     }
 
-    if(pCreateFX){
-        for(i = 0; i < ARRAY_SIZE(const_clsids); ++i){
-            hr = pCreateFX(const_clsids[i].clsid, &fx_unk, NULL, 0);
-            todo_wine_if(const_clsids[i].todo)
-                ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(const_clsids[i].clsid), hr);
-            if(SUCCEEDED(hr)){
-                IXAPO *xapo;
-                hr = IUnknown_QueryInterface(fx_unk, &IID_IXAPO, (void**)&xapo);
-                ok(hr == S_OK, "Couldn't get IXAPO interface: %08x\n", hr);
-                if(SUCCEEDED(hr))
-                    IXAPO_Release(xapo);
-                IUnknown_Release(fx_unk);
-            }
-
-            hr = CoCreateInstance(const_clsids[i].clsid, NULL, CLSCTX_INPROC_SERVER,
-                    &IID_IUnknown, (void**)&fx_unk);
-            ok(hr == REGDB_E_CLASSNOTREG, "CoCreateInstance should have failed: %08x\n", hr);
-            if(SUCCEEDED(hr))
-                IUnknown_Release(fx_unk);
-        }
-
-        /* test legacy CLSID */
-        hr = pCreateFX(&CLSID_AudioVolumeMeter27, &fx_unk, NULL, 0);
-        ok(hr == S_OK, "%s: CreateFX(CLSID_AudioVolumeMeter) failed: %08x\n", module, hr);
+    for(i = 0; i < ARRAY_SIZE(const_clsids); ++i){
+        hr = pCreateFX(const_clsids[i], &fx_unk, NULL, 0);
+        ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(const_clsids[i]), hr);
         if(SUCCEEDED(hr)){
             IXAPO *xapo;
             hr = IUnknown_QueryInterface(fx_unk, &IID_IXAPO, (void**)&xapo);
             ok(hr == S_OK, "Couldn't get IXAPO interface: %08x\n", hr);
             if(SUCCEEDED(hr))
                 IXAPO_Release(xapo);
-            IUnknown_Release(fx_unk);
+            rc = IUnknown_Release(fx_unk);
+            ok(rc == 0, "XAPO via CreateFX should have been released, got refcount: %u\n", rc);
         }
+
+        hr = CoCreateInstance(const_clsids[i], NULL, CLSCTX_INPROC_SERVER,
+                &IID_IUnknown, (void**)&fx_unk);
+        ok(hr == REGDB_E_CLASSNOTREG, "CoCreateInstance should have failed: %08x\n", hr);
+        if(SUCCEEDED(hr))
+            IUnknown_Release(fx_unk);
+    }
+
+    /* test legacy CLSID */
+    hr = pCreateFX(&CLSID_AudioVolumeMeter27, &fx_unk, NULL, 0);
+    ok(hr == S_OK, "%s: CreateFX(CLSID_AudioVolumeMeter) failed: %08x\n", module, hr);
+    if(SUCCEEDED(hr)){
+        IXAPO *xapo;
+        hr = IUnknown_QueryInterface(fx_unk, &IID_IXAPO, (void**)&xapo);
+        ok(hr == S_OK, "Couldn't get IXAPO interface: %08x\n", hr);
+        if(SUCCEEDED(hr))
+            IXAPO_Release(xapo);
+        rc = IUnknown_Release(fx_unk);
+        ok(rc == 0, "XAPO via legacy CreateFX should have been released, got refcount: %u\n", rc);
     }
 
     pCAVM = (void*)GetProcAddress(xaudio2dll, "CreateAudioVolumeMeter");
@@ -1134,7 +1130,8 @@ static void test_xapo_creation_modern(const char *module)
         ok(hr == S_OK, "Couldn't get IXAPO interface: %08x\n", hr);
         if(SUCCEEDED(hr))
             IXAPO_Release(xapo);
-        IUnknown_Release(fx_unk);
+        rc = IUnknown_Release(fx_unk);
+        ok(rc == 0, "XAPO via CreateAudioVolumeMeter should have been released, got refcount: %u\n", rc);
     }
 
     pCAR = (void*)GetProcAddress(xaudio2dll, "CreateAudioReverb");
@@ -1148,7 +1145,8 @@ static void test_xapo_creation_modern(const char *module)
         ok(hr == S_OK, "Couldn't get IXAPO interface: %08x\n", hr);
         if(SUCCEEDED(hr))
             IXAPO_Release(xapo);
-        IUnknown_Release(fx_unk);
+        rc = IUnknown_Release(fx_unk);
+        ok(rc == 0, "XAPO via CreateAudioReverb should have been released, got refcount: %u\n", rc);
     }
 
     FreeLibrary(xaudio2dll);
@@ -1256,6 +1254,7 @@ START_TEST(xaudio2)
     IXAudio2 *xa = NULL;
     HANDLE xa28dll;
     UINT32 has_devices;
+    ULONG rc;
 
     CoInitialize(NULL);
 
@@ -1290,7 +1289,8 @@ START_TEST(xaudio2)
         }else
             skip("No audio devices available\n");
 
-        IXAudio27_Release(xa27);
+        rc = IXAudio27_Release(xa27);
+        ok(rc == 0, "IXAudio2.7 object should have been released, got refcount %u\n", rc);
     }else
         win_skip("XAudio 2.7 not available\n");
 
@@ -1315,7 +1315,8 @@ START_TEST(xaudio2)
         }else
             skip("No audio devices available\n");
 
-        IXAudio2_Release(xa);
+        rc = IXAudio2_Release(xa);
+        ok(rc == 0, "IXAudio2 object should have been released, got refcount %u\n", rc);
     }else
         win_skip("XAudio 2.8 not available\n");
 
