@@ -336,6 +336,45 @@ enum ImageFlags
     ImageFlagsCaching           = 0x00020000
 };
 
+enum EncoderParameterValueType {
+    EncoderParameterValueTypeByte = 1,
+    EncoderParameterValueTypeASCII = 2,
+    EncoderParameterValueTypeShort = 3,
+    EncoderParameterValueTypeLong = 4,
+    EncoderParameterValueTypeRational = 5,
+    EncoderParameterValueTypeLongRange = 6,
+    EncoderParameterValueTypeUndefined = 7,
+    EncoderParameterValueTypeRationalRange = 8,
+    EncoderParameterValueTypePointer = 9
+};
+
+enum EncoderValue {
+    EncoderValueColorTypeCMYK = 0,
+    EncoderValueColorTypeYCCK = 1,
+    EncoderValueCompressionLZW = 2,
+    EncoderValueCompressionCCITT3 = 3,
+    EncoderValueCompressionCCITT4 = 4,
+    EncoderValueCompressionRle = 5,
+    EncoderValueCompressionNone = 6,
+    EncoderValueScanMethodInterlaced = 7,
+    EncoderValueScanMethodNonInterlaced = 8,
+    EncoderValueVersionGif87 = 9,
+    EncoderValueVersionGif89 = 10,
+    EncoderValueRenderProgressive = 11,
+    EncoderValueRenderNonProgressive = 12,
+    EncoderValueTransformRotate90 = 13,
+    EncoderValueTransformRotate180 = 14,
+    EncoderValueTransformRotate270 = 15,
+    EncoderValueTransformFlipHorizontal = 16,
+    EncoderValueTransformFlipVertical = 17,
+    EncoderValueMultiFrame = 18,
+    EncoderValueLastFrame = 19,
+    EncoderValueFlush = 20,
+    EncoderValueFrameDimensionTime = 21,
+    EncoderValueFrameDimensionResolution = 22,
+    EncoderValueFrameDimensionPage = 23
+};
+
 enum CombineMode
 {
     CombineModeReplace,
@@ -747,6 +786,8 @@ typedef enum HotkeyPrefix HotkeyPrefix;
 typedef enum PenAlignment PenAlignment;
 typedef enum PaletteFlags PaletteFlags;
 typedef enum ImageCodecFlags ImageCodecFlags;
+typedef enum EncoderParameterValueType EncoderParameterValueType;
+typedef enum EncoderValue EncoderValue;
 typedef enum CombineMode CombineMode;
 typedef enum FlushIntention FlushIntention;
 typedef enum CoordinateSpace CoordinateSpace;
