@@ -22,11 +22,15 @@ int __cdecl __isascii(int);
 int __cdecl __iscsym(int);
 int __cdecl __iscsymf(int);
 int __cdecl __toascii(int);
+int __cdecl _isblank_l(int,_locale_t);
 int __cdecl _isctype(int,int);
 int __cdecl _tolower(int);
+int __cdecl _tolower_l(int,_locale_t);
 int __cdecl _toupper(int);
+int __cdecl _toupper_l(int,_locale_t);
 int __cdecl isalnum(int);
 int __cdecl isalpha(int);
+int __cdecl isblank(int);
 int __cdecl iscntrl(int);
 int __cdecl isdigit(int);
 int __cdecl isgraph(int);
