@@ -19,6 +19,8 @@
 #ifndef __WINE_SHELLSCALINGAPI_H
 #define __WINE_SHELLSCALINGAPI_H
 
+#include <shtypes.h>
+
 typedef enum MONITOR_DPI_TYPE
 {
     MDT_EFFECTIVE_DPI   = 0,
@@ -36,6 +38,7 @@ typedef enum PROCESS_DPI_AWARENESS
 
 HRESULT WINAPI GetDpiForMonitor(HMONITOR,MONITOR_DPI_TYPE,UINT*,UINT*);
 HRESULT WINAPI GetProcessDpiAwareness(HANDLE,PROCESS_DPI_AWARENESS*);
+HRESULT WINAPI GetScaleFactorForMonitor(HMONITOR,DEVICE_SCALE_FACTOR*);
 HRESULT WINAPI SetProcessDpiAwareness(PROCESS_DPI_AWARENESS);
 
 #endif /* __WINE_SHELLSCALINGAPI_H */
