@@ -33,7 +33,7 @@
 # define __ASM_STDCALL(name,args) __ASM_NAME(name)
 #endif
 
-#ifdef __GCC_HAVE_DWARF2_CFI_ASM
+#if defined(__GCC_HAVE_DWARF2_CFI_ASM) || defined(__APPLE__)
 # define __ASM_CFI(str) str
 #else
 # define __ASM_CFI(str)
@@ -128,4 +128,10 @@
 
 #endif  /* __i386__ */
 
+#if defined(__GNUC__) && !defined(_WIN32) && !defined(__APPLE__) && !defined(__ANDROID__)
+#define __ASM_OBSOLETE(func) __asm__( ".symver " #func "_obsolete," #func "@WINE_1.0" )
+#else
+#undef __ASM_OBSOLETE
+#endif
+
 #endif  /* __WINE_WINE_ASM_H */
