@@ -47,7 +47,7 @@
  * Hard-coded values for the Windows platform
  */
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(__CYGWIN__)
 
 #include <direct.h>
 #include <io.h>
