@@ -31,16 +31,12 @@
 #error This file should not be used in Wine tests
 #endif
 
-#ifdef __cplusplus
-extern "C" {
+#ifdef __WINE_USE_MSVCRT
+#error This file should not be used with msvcrt headers
 #endif
 
-#ifndef WINE_UNICODE_API
-# if defined(_MSC_VER) || defined(__MINGW32__)
-#  define WINE_UNICODE_API DECLSPEC_IMPORT
-# else
-#  define WINE_UNICODE_API
-# endif
+#ifdef __cplusplus
+extern "C" {
 #endif
 
 #ifndef WINE_UNICODE_INLINE
@@ -83,29 +79,6 @@ union cptable
     struct dbcs_table dbcs;
 };
 
-extern const union cptable *wine_cp_get_table( unsigned int codepage );
-extern const union cptable *wine_cp_enum_table( unsigned int index );
-
-extern int wine_cp_mbstowcs( const union cptable *table, int flags,
-                             const char *src, int srclen,
-                             WCHAR *dst, int dstlen );
-extern int wine_cp_wcstombs( const union cptable *table, int flags,
-                             const WCHAR *src, int srclen,
-                             char *dst, int dstlen, const char *defchar, int *used );
-extern int wine_cpsymbol_mbstowcs( const char *src, int srclen, WCHAR *dst, int dstlen );
-extern int wine_cpsymbol_wcstombs( const WCHAR *src, int srclen, char *dst, int dstlen );
-extern int wine_utf8_mbstowcs( int flags, const char *src, int srclen, WCHAR *dst, int dstlen );
-extern int wine_utf8_wcstombs( int flags, const WCHAR *src, int srclen, char *dst, int dstlen );
-
-extern int wine_compare_string( int flags, const WCHAR *str1, int len1, const WCHAR *str2, int len2 );
-extern int wine_get_sortkey( int flags, const WCHAR *src, int srclen, char *dst, int dstlen );
-extern int wine_fold_string( int flags, const WCHAR *src, int srclen , WCHAR *dst, int dstlen );
-
-extern unsigned int wine_compose_string( WCHAR *str, unsigned int len );
-extern unsigned int wine_decompose_string( int flags, const WCHAR *src, unsigned int srclen, WCHAR *dst, unsigned int dstlen );
-#define WINE_DECOMPOSE_COMPAT     1
-#define WINE_DECOMPOSE_REORDER    2
-
 extern int strcmpiW( const WCHAR *str1, const WCHAR *str2 );
 extern int strncmpiW( const WCHAR *str1, const WCHAR *str2, int n );
 extern int memicmpW( const WCHAR *str1, const WCHAR *str2, int n );
@@ -117,20 +90,15 @@ extern int snprintfW( WCHAR *str, size_t len, const WCHAR *format, ... );
 extern int vsprintfW( WCHAR *str, const WCHAR *format, va_list valist );
 extern int vsnprintfW( WCHAR *str, size_t len, const WCHAR *format, va_list valist );
 
-WINE_UNICODE_INLINE int wine_is_dbcs_leadbyte( const union cptable *table, unsigned char ch )
-{
-    return (table->info.char_size == 2) && (table->dbcs.cp2uni_leadbytes[ch]);
-}
-
 WINE_UNICODE_INLINE WCHAR tolowerW( WCHAR ch )
 {
-    extern WINE_UNICODE_API const WCHAR wine_casemap_lower[];
+    extern const WCHAR wine_casemap_lower[];
     return ch + wine_casemap_lower[wine_casemap_lower[ch >> 8] + (ch & 0xff)];
 }
 
 WINE_UNICODE_INLINE WCHAR toupperW( WCHAR ch )
 {
-    extern WINE_UNICODE_API const WCHAR wine_casemap_upper[];
+    extern const WCHAR wine_casemap_upper[];
     return ch + wine_casemap_upper[wine_casemap_upper[ch >> 8] + (ch & 0xff)];
 }
 
@@ -138,7 +106,7 @@ WINE_UNICODE_INLINE WCHAR toupperW( WCHAR ch )
 /* and the C2_* type in the high 4 bits */
 WINE_UNICODE_INLINE unsigned short get_char_typeW( WCHAR ch )
 {
-    extern WINE_UNICODE_API const unsigned short wine_wctype_table[];
+    extern const unsigned short wine_wctype_table[];
     return wine_wctype_table[wine_wctype_table[ch >> 8] + (ch & 0xff)];
 }
 
