@@ -61,7 +61,7 @@ static void fatal_error( const char *err, ... )  __attribute__((noreturn,format(
 static void fatal_perror( const char *err, ... )  __attribute__((noreturn,format(printf,1,2)));
 #endif
 
-#if defined(__linux__) || defined(__FreeBSD_kernel__ )
+#if defined(__linux__) || defined(__FreeBSD_kernel__) || defined(__NetBSD__)
 static const char exe_link[] = "/proc/self/exe";
 #elif defined (__FreeBSD__) || defined(__DragonFly__)
 static const char exe_link[] = "/proc/curproc/file";
@@ -164,33 +164,15 @@ static char *get_runtime_libdir(void)
 /* read a symlink and return its directory */
 static char *symlink_dirname( const char *name )
 {
-    char *p, *buffer, *absdir = NULL;
-    int ret, size;
+    char *p, *fullpath = realpath( name, NULL );
 
-    for (size = 256; ; size *= 2)
+    if (fullpath)
     {
-        if (!(buffer = malloc( size ))) return NULL;
-        if ((ret = readlink( name, buffer, size )) == -1) break;
-        if (ret != size)
-        {
-            buffer[ret] = 0;
-            if (!(p = strrchr( buffer, '/' ))) break;
-            if (p == buffer) p++;
-            *p = 0;
-            if (buffer[0] == '/') return buffer;
-            /* make it absolute */
-            absdir = xmalloc( strlen(name) + strlen(buffer) + 1 );
-            strcpy( absdir, name );
-            if (!(p = strrchr( absdir, '/' ))) break;
-            strcpy( p + 1, buffer );
-            free( buffer );
-            return absdir;
-        }
-        free( buffer );
+        p = strrchr( fullpath, '/' );
+        if (p == fullpath) p++;
+        if (p) *p = 0;
     }
-    free( buffer );
-    free( absdir );
-    return NULL;
+    return fullpath;
 }
 
 /* return the directory that contains the main exe at run-time */
@@ -246,11 +228,9 @@ static void init_server_dir( dev_t dev, ino_t ino )
 
 #ifdef __ANDROID__  /* there's no /tmp dir on Android */
     root = build_path( config_dir, ".wineserver" );
-#elif defined(HAVE_GETUID)
+#else
     root = xmalloc( sizeof(server_root_prefix) + 12 );
     sprintf( root, "%s-%u", server_root_prefix, getuid() );
-#else
-    root = xstrdup( server_root_prefix );
 #endif
 
     server_dir = xmalloc( strlen(root) + sizeof(server_dir_prefix) + 2*sizeof(dev) + 2*sizeof(ino) + 2 );
@@ -285,8 +265,6 @@ static void init_paths(void)
     const char *home = getenv( "HOME" );
     const char *user = NULL;
     const char *prefix = getenv( "WINEPREFIX" );
-
-#ifdef HAVE_GETPWUID
     char uid_str[32];
     struct passwd *pwd = getpwuid( getuid() );
 
@@ -300,10 +278,6 @@ static void init_paths(void)
         sprintf( uid_str, "%lu", (unsigned long)getuid() );
         user = uid_str;
     }
-#else  /* HAVE_GETPWUID */
-    if (!(user = getenv( "USER" )))
-        fatal_error( "cannot determine your user name, set the USER environment variable\n" );
-#endif  /* HAVE_GETPWUID */
     user_name = xstrdup( user );
 
     /* build config_dir */
@@ -335,10 +309,7 @@ static void init_paths(void)
         }
     }
     if (!S_ISDIR(st.st_mode)) fatal_error( "%s is not a directory\n", config_dir );
-#ifdef HAVE_GETUID
     if (st.st_uid != getuid()) fatal_error( "%s is not owned by you\n", config_dir );
-#endif
-
     init_server_dir( st.st_dev, st.st_ino );
 }
 
