@@ -61,6 +61,7 @@
 #include <unistd.h>
 #include <windows.h>
 #include <winternl.h>
+#include <sddl.h>
 #include <wine/svcctl.h>
 #include <wine/asm.h>
 #include <wine/debug.h>
@@ -159,21 +160,21 @@ done:
 }
 
 /* print the config directory in a more Unix-ish way */
-static const char *prettyprint_configdir(void)
+static const WCHAR *prettyprint_configdir(void)
 {
-    static char buffer[MAX_PATH];
-    WCHAR *path = _wgetenv( wineconfigdirW );
-    char *p;
+    static WCHAR buffer[MAX_PATH];
+    WCHAR *p, *path = _wgetenv( wineconfigdirW );
 
-    if (!WideCharToMultiByte( CP_UNIXCP, 0, path, -1, buffer, ARRAY_SIZE(buffer), NULL, NULL ))
-        strcpy( buffer + ARRAY_SIZE(buffer) - 4, "..." );
+    lstrcpynW( buffer, path, ARRAY_SIZE(buffer) );
+    if (lstrlenW( wineconfigdirW ) >= ARRAY_SIZE(buffer) )
+        lstrcpyW( buffer + ARRAY_SIZE(buffer) - 4, L"..." );
 
-    if (!strncmp( buffer, "\\??\\unix\\", 9 ))
+    if (!wcsncmp( buffer, L"\\??\\unix\\", 9 ))
     {
         for (p = buffer + 9; *p; p++) if (*p == '\\') *p = '/';
         return buffer + 9;
     }
-    else if (!strncmp( buffer, "\\??\\Z:\\", 7 ))
+    else if (!wcsncmp( buffer, L"\\??\\Z:\\", 7 ))
     {
         for (p = buffer + 6; *p; p++) if (*p == '\\') *p = '/';
         return buffer + 6;
@@ -187,6 +188,11 @@ static DWORD set_reg_value( HKEY hkey, const WCHAR *name, const WCHAR *value )
     return RegSetValueExW( hkey, name, 0, REG_SZ, (const BYTE *)value, (lstrlenW(value) + 1) * sizeof(WCHAR) );
 }
 
+static DWORD set_reg_value_dword( HKEY hkey, const WCHAR *name, DWORD value )
+{
+    return RegSetValueExW( hkey, name, 0, REG_DWORD, (const BYTE *)&value, sizeof(value) );
+}
+
 #if defined(__i386__) || defined(__x86_64__)
 
 #if defined(_MSC_VER)
@@ -298,6 +304,239 @@ static void get_namestring( WCHAR *buf ) { }
 
 #endif  /* __i386__ || __x86_64__ */
 
+#include "pshpack1.h"
+struct smbios_prologue
+{
+    BYTE  calling_method;
+    BYTE  major_version;
+    BYTE  minor_version;
+    BYTE  revision;
+    DWORD length;
+};
+
+enum smbios_type
+{
+    SMBIOS_TYPE_BIOS,
+    SMBIOS_TYPE_SYSTEM,
+    SMBIOS_TYPE_BASEBOARD,
+};
+
+struct smbios_header
+{
+    BYTE type;
+    BYTE length;
+    WORD handle;
+};
+
+struct smbios_baseboard
+{
+    struct smbios_header hdr;
+    BYTE                 vendor;
+    BYTE                 product;
+    BYTE                 version;
+    BYTE                 serial;
+};
+
+struct smbios_bios
+{
+    struct smbios_header hdr;
+    BYTE                 vendor;
+    BYTE                 version;
+    WORD                 start;
+    BYTE                 date;
+    BYTE                 size;
+    UINT64               characteristics;
+    BYTE                 characteristics_ext[2];
+    BYTE                 system_bios_major_release;
+    BYTE                 system_bios_minor_release;
+    BYTE                 ec_firmware_major_release;
+    BYTE                 ec_firmware_minor_release;
+};
+
+struct smbios_system
+{
+    struct smbios_header hdr;
+    BYTE                 vendor;
+    BYTE                 product;
+    BYTE                 version;
+    BYTE                 serial;
+    BYTE                 uuid[16];
+    BYTE                 wake_up_type;
+    BYTE                 sku;
+    BYTE                 family;
+};
+#include "poppack.h"
+
+#define RSMB (('R' << 24) | ('S' << 16) | ('M' << 8) | 'B')
+
+static const struct smbios_header *find_smbios_entry( enum smbios_type type, const char *buf, UINT len )
+{
+    const char *ptr, *start;
+    const struct smbios_prologue *prologue;
+    const struct smbios_header *hdr;
+
+    if (len < sizeof(struct smbios_prologue)) return NULL;
+    prologue = (const struct smbios_prologue *)buf;
+    if (prologue->length > len - sizeof(*prologue) || prologue->length < sizeof(*hdr)) return NULL;
+
+    start = (const char *)(prologue + 1);
+    hdr = (const struct smbios_header *)start;
+
+    for (;;)
+    {
+        if ((const char *)hdr - start >= prologue->length - sizeof(*hdr)) return NULL;
+
+        if (!hdr->length)
+        {
+            WARN( "invalid entry\n" );
+            return NULL;
+        }
+
+        if (hdr->type == type)
+        {
+            if ((const char *)hdr - start + hdr->length > prologue->length) return NULL;
+            break;
+        }
+        else /* skip other entries and their strings */
+        {
+            for (ptr = (const char *)hdr + hdr->length; ptr - buf < len && *ptr; ptr++)
+            {
+                for (; ptr - buf < len; ptr++) if (!*ptr) break;
+            }
+            if (ptr == (const char *)hdr + hdr->length) ptr++;
+            hdr = (const struct smbios_header *)(ptr + 1);
+        }
+    }
+
+    return hdr;
+}
+
+static inline WCHAR *heap_strdupAW( const char *src )
+{
+    int len;
+    WCHAR *dst;
+    if (!src) return NULL;
+    len = MultiByteToWideChar( CP_ACP, 0, src, -1, NULL, 0 );
+    if ((dst = HeapAlloc( GetProcessHeap(), 0, len * sizeof(*dst) ))) MultiByteToWideChar( CP_ACP, 0, src, -1, dst, len );
+    return dst;
+}
+
+static WCHAR *get_smbios_string( BYTE id, const char *buf, UINT offset, UINT buflen )
+{
+    const char *ptr = buf + offset;
+    UINT i = 0;
+
+    if (!id || offset >= buflen) return NULL;
+    for (ptr = buf + offset; ptr - buf < buflen && *ptr; ptr++)
+    {
+        if (++i == id) return heap_strdupAW( ptr );
+        for (; ptr - buf < buflen; ptr++) if (!*ptr) break;
+    }
+    return NULL;
+}
+
+static void set_value_from_smbios_string( HKEY key, const WCHAR *value, BYTE id, const char *buf, UINT offset, UINT buflen )
+{
+    WCHAR *str;
+    str = get_smbios_string( id, buf, offset, buflen );
+    set_reg_value( key, value, str ? str : L"" );
+    HeapFree( GetProcessHeap(), 0, str );
+}
+
+static void create_bios_baseboard_values( HKEY bios_key, const char *buf, UINT len )
+{
+    const struct smbios_header *hdr;
+    const struct smbios_baseboard *baseboard;
+    UINT offset;
+
+    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_BASEBOARD, buf, len ))) return;
+    baseboard = (const struct smbios_baseboard *)hdr;
+    offset = (const char *)baseboard - buf + baseboard->hdr.length;
+
+    set_value_from_smbios_string( bios_key, L"BaseBoardManufacturer", baseboard->vendor, buf, offset, len );
+    set_value_from_smbios_string( bios_key, L"BaseBoardProduct", baseboard->product, buf, offset, len );
+    set_value_from_smbios_string( bios_key, L"BaseBoardVersion", baseboard->version, buf, offset, len );
+}
+
+static void create_bios_bios_values( HKEY bios_key, const char *buf, UINT len )
+{
+    const struct smbios_header *hdr;
+    const struct smbios_bios *bios;
+    UINT offset;
+
+    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_BIOS, buf, len ))) return;
+    bios = (const struct smbios_bios *)hdr;
+    offset = (const char *)bios - buf + bios->hdr.length;
+
+    set_value_from_smbios_string( bios_key, L"BIOSVendor", bios->vendor, buf, offset, len );
+    set_value_from_smbios_string( bios_key, L"BIOSVersion", bios->version, buf, offset, len );
+    set_value_from_smbios_string( bios_key, L"BIOSReleaseDate", bios->date, buf, offset, len );
+
+    if (bios->hdr.length >= 0x18)
+    {
+        set_reg_value_dword( bios_key, L"BiosMajorRelease", bios->system_bios_major_release );
+        set_reg_value_dword( bios_key, L"BiosMinorRelease", bios->system_bios_minor_release );
+        set_reg_value_dword( bios_key, L"ECFirmwareMajorVersion", bios->ec_firmware_major_release );
+        set_reg_value_dword( bios_key, L"ECFirmwareMinorVersion", bios->ec_firmware_minor_release );
+    }
+    else
+    {
+        set_reg_value_dword( bios_key, L"BiosMajorRelease", 0xFF );
+        set_reg_value_dword( bios_key, L"BiosMinorRelease", 0xFF );
+        set_reg_value_dword( bios_key, L"ECFirmwareMajorVersion", 0xFF );
+        set_reg_value_dword( bios_key, L"ECFirmwareMinorVersion", 0xFF );
+    }
+}
+
+static void create_bios_system_values( HKEY bios_key, const char *buf, UINT len )
+{
+    const struct smbios_header *hdr;
+    const struct smbios_system *system;
+    UINT offset;
+
+    if (!(hdr = find_smbios_entry( SMBIOS_TYPE_SYSTEM, buf, len ))) return;
+    system = (const struct smbios_system *)hdr;
+    offset = (const char *)system - buf + system->hdr.length;
+
+    set_value_from_smbios_string( bios_key, L"SystemManufacturer", system->vendor, buf, offset, len );
+    set_value_from_smbios_string( bios_key, L"SystemProductName", system->product, buf, offset, len );
+    set_value_from_smbios_string( bios_key, L"SystemVersion", system->version, buf, offset, len );
+
+    if (system->hdr.length >= 0x1B)
+    {
+        set_value_from_smbios_string( bios_key, L"SystemSKU", system->sku, buf, offset, len );
+        set_value_from_smbios_string( bios_key, L"SystemFamily", system->family, buf, offset, len );
+    }
+    else
+    {
+        set_value_from_smbios_string( bios_key, L"SystemSKU", 0, buf, offset, len );
+        set_value_from_smbios_string( bios_key, L"SystemFamily", 0, buf, offset, len );
+    }
+}
+
+static void create_bios_key( HKEY system_key )
+{
+    HKEY bios_key;
+    UINT len;
+    char *buf;
+
+    if (RegCreateKeyExW( system_key, L"BIOS", 0, NULL, REG_OPTION_VOLATILE,
+                         KEY_ALL_ACCESS, NULL, &bios_key, NULL ))
+        return;
+
+    len = GetSystemFirmwareTable( RSMB, 0, NULL, 0 );
+    if (!(buf = HeapAlloc( GetProcessHeap(), 0, len ))) goto done;
+    len = GetSystemFirmwareTable( RSMB, 0, buf, len );
+
+    create_bios_baseboard_values( bios_key, buf, len );
+    create_bios_bios_values( bios_key, buf, len );
+    create_bios_system_values( bios_key, buf, len );
+
+done:
+    HeapFree( GetProcessHeap(), 0, buf );
+    RegCloseKey( bios_key );
+}
+
 /* create the volatile hardware registry keys */
 static void create_hardware_registry_keys(void)
 {
@@ -408,6 +647,9 @@ static void create_hardware_registry_keys(void)
             RegCloseKey( hkey );
         }
     }
+
+    create_bios_key( system_key );
+
     RegCloseKey( fpu_key );
     RegCloseKey( cpu_key );
     RegCloseKey( system_key );
@@ -1063,18 +1305,9 @@ static INT_PTR CALLBACK wait_dlgproc( HWND hwnd, UINT msg, WPARAM wp, LPARAM lp
 
 static HWND show_wait_window(void)
 {
-    const char *config_dir = prettyprint_configdir();
-    WCHAR *name;
-    HWND hwnd;
-    DWORD len;
-
-    len = MultiByteToWideChar( CP_UNIXCP, 0, config_dir, -1, NULL, 0 );
-    name = HeapAlloc( GetProcessHeap(), 0, len * sizeof(WCHAR) );
-    MultiByteToWideChar( CP_UNIXCP, 0, config_dir, -1, name, len );
-    hwnd = CreateDialogParamW( GetModuleHandleW(0), MAKEINTRESOURCEW(IDD_WAITDLG), 0,
-                               wait_dlgproc, (LPARAM)name );
+    HWND hwnd = CreateDialogParamW( GetModuleHandleW(0), MAKEINTRESOURCEW(IDD_WAITDLG), 0,
+                                    wait_dlgproc, (LPARAM)prettyprint_configdir() );
     ShowWindow( hwnd, SW_SHOWNORMAL );
-    HeapFree( GetProcessHeap(), 0, name );
     return hwnd;
 }
 
@@ -1174,6 +1407,48 @@ static void install_root_pnp_devices(void)
     SetupDiDestroyDeviceInfoList(set);
 }
 
+static void update_user_profile(void)
+{
+    static const WCHAR profile_list[] = {'S','o','f','t','w','a','r','e','\\',
+                                         'M','i','c','r','o','s','o','f','t','\\',
+                                         'W','i','n','d','o','w','s',' ','N','T','\\',
+                                         'C','u','r','r','e','n','t','V','e','r','s','i','o','n','\\',
+                                         'P','r','o','f','i','l','e','L','i','s','t',0};
+    static const WCHAR profile_image_path[] = {'P','r','o','f','i','l','e','I','m','a','g','e','P','a','t','h',0};
+    char token_buf[sizeof(TOKEN_USER) + sizeof(SID) + sizeof(DWORD) * SID_MAX_SUB_AUTHORITIES];
+    HANDLE token;
+    WCHAR profile[MAX_PATH], *sid;
+    DWORD size;
+    HKEY hkey, profile_hkey;
+
+    if (!OpenProcessToken(GetCurrentProcess(), TOKEN_READ, &token))
+        return;
+
+    size = sizeof(token_buf);
+    GetTokenInformation(token, TokenUser, token_buf, size, &size);
+    CloseHandle(token);
+
+    ConvertSidToStringSidW(((TOKEN_USER *)token_buf)->User.Sid, &sid);
+
+    if (!RegCreateKeyExW(HKEY_LOCAL_MACHINE, profile_list, 0, NULL, 0,
+                         KEY_ALL_ACCESS, NULL, &hkey, NULL))
+    {
+        if (!RegCreateKeyExW(hkey, sid, 0, NULL, 0,
+                             KEY_ALL_ACCESS, NULL, &profile_hkey, NULL))
+        {
+            DWORD flags = 0;
+            if (SHGetSpecialFolderPathW(NULL, profile, CSIDL_PROFILE, TRUE))
+                set_reg_value(profile_hkey, profile_image_path, profile);
+            RegSetValueExW( profile_hkey, L"Flags", 0, REG_DWORD, (const BYTE *)&flags, sizeof(flags) );
+            RegCloseKey(profile_hkey);
+        }
+
+        RegCloseKey(hkey);
+    }
+
+    LocalFree(sid);
+}
+
 /* execute rundll32 on the wine.inf file if necessary */
 static void update_wineprefix( BOOL force )
 {
@@ -1184,13 +1459,13 @@ static void update_wineprefix( BOOL force )
 
     if (!inf_path)
     {
-        WINE_MESSAGE( "wine: failed to update %s, wine.inf not found\n", prettyprint_configdir() );
+        WINE_MESSAGE( "wine: failed to update %s, wine.inf not found\n", debugstr_w( config_dir ));
         return;
     }
     if ((fd = _wopen( inf_path, O_RDONLY )) == -1)
     {
         WINE_MESSAGE( "wine: failed to update %s with %s: %s\n",
-                      prettyprint_configdir(), debugstr_w(inf_path), strerror(errno) );
+                      debugstr_w(config_dir), debugstr_w(inf_path), strerror(errno) );
         goto done;
     }
     fstat( fd, &st );
@@ -1218,8 +1493,9 @@ static void update_wineprefix( BOOL force )
             DestroyWindow( hwnd );
         }
         install_root_pnp_devices();
+        update_user_profile();
 
-        WINE_MESSAGE( "wine: configuration in '%s' has been updated.\n", prettyprint_configdir() );
+        WINE_MESSAGE( "wine: configuration in %s has been updated.\n", debugstr_w(prettyprint_configdir()) );
     }
 
 done:
@@ -1232,7 +1508,6 @@ static BOOL ProcessStartupItems(void)
 {
     BOOL ret = FALSE;
     HRESULT hr;
-    IMalloc *ppM = NULL;
     IShellFolder *psfDesktop = NULL, *psfStartup = NULL;
     LPITEMIDLIST pidlStartup = NULL, pidlItem;
     ULONG NumPIDLs;
@@ -1242,13 +1517,6 @@ static BOOL ProcessStartupItems(void)
 
     WINE_TRACE("Processing items in the StartUp folder.\n");
 
-    hr = SHGetMalloc(&ppM);
-    if (FAILED(hr))
-    {
-	WINE_ERR("Couldn't get IMalloc object.\n");
-	goto done;
-    }
-
     hr = SHGetDesktopFolder(&psfDesktop);
     if (FAILED(hr))
     {
@@ -1298,7 +1566,7 @@ static BOOL ProcessStartupItems(void)
             }
 	}
 
-	IMalloc_Free(ppM, pidlItem);
+        ILFree(pidlItem);
     }
 
     /* Return success */
@@ -1307,7 +1575,7 @@ static BOOL ProcessStartupItems(void)
 done:
     if (iEnumList) IEnumIDList_Release(iEnumList);
     if (psfStartup) IShellFolder_Release(psfStartup);
-    if (pidlStartup) IMalloc_Free(ppM, pidlStartup);
+    if (pidlStartup) ILFree(pidlStartup);
 
     return ret;
 }
