@@ -403,6 +403,7 @@ static int CALLBACK get_first_font_enum_2(const LOGFONTW* lf, const TEXTMETRICW*
          */
         mlf.lfWidth  = fc->data->curcfg.cell_width;
         mlf.lfHeight = fc->data->curcfg.cell_height;
+        if (!mlf.lfHeight) mlf.lfHeight = MulDiv( 16, GetDpiForSystem(), USER_DEFAULT_SCREEN_DPI );
         if (WCUSER_SetFont(fc->data, &mlf))
         {
             struct      config_data     defcfg;
@@ -1309,8 +1310,8 @@ static LRESULT CALLBACK WCUSER_Proc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM
 	    break;
 	case IDS_SELECTALL:
             PRIVATE(data)->selectPt1.X = PRIVATE(data)->selectPt1.Y = 0;
-            PRIVATE(data)->selectPt2.X = (data->curcfg.sb_width - 1) * data->curcfg.cell_width;
-            PRIVATE(data)->selectPt2.Y = (data->curcfg.sb_height - 1) * data->curcfg.cell_height;
+            PRIVATE(data)->selectPt2.X = data->curcfg.sb_width - 1;
+            PRIVATE(data)->selectPt2.Y = data->curcfg.sb_height - 1;
             WCUSER_SetSelection(data, 0);
             PRIVATE(data)->has_selection = TRUE;
 	    break;
@@ -1436,7 +1437,7 @@ enum init_return WCUSER_InitBackend(struct inner_data* data)
     RegisterClassW(&wndclass);
 
     data->hWnd = CreateWindowW(wndclass.lpszClassName, NULL,
-                               WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_THICKFRAME|WS_MINIMIZEBOX|WS_HSCROLL|WS_VSCROLL,
+                               WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_THICKFRAME|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|WS_HSCROLL|WS_VSCROLL,
                                CW_USEDEFAULT, CW_USEDEFAULT, 0, 0, 0, 0, wndclass.hInstance, data);
     if (!data->hWnd) return init_not_supported;
 
