@@ -112,7 +112,7 @@ static void LoadBoard( BOARD *p_board )
         size = sizeof( data );
         if( RegQueryValueExW( hkey, key_name, NULL, &type,
                 (LPBYTE) data, &size ) == ERROR_SUCCESS )
-            lstrcpynW( p_board->best_name[i], data, sizeof(p_board->best_name[i])/sizeof(WCHAR) );
+            lstrcpynW( p_board->best_name[i], data, ARRAY_SIZE(p_board->best_name[i]));
         else
             LoadStringW( p_board->hInst, IDS_NOBODY, p_board->best_name[i], MAX_PLAYER_NAME_SIZE+1 );
     }
@@ -179,7 +179,7 @@ void SaveBoard( BOARD *p_board )
 
     for( i = 0; i < 3; i++ ) {
         wsprintfW( key_name, nameW, i+1 );
-        lstrcpynW( data, p_board->best_name[i], sizeof(data)/sizeof(WCHAR) );
+        lstrcpynW( data, p_board->best_name[i], ARRAY_SIZE(data));
         RegSetValueExW( hkey, key_name, 0, REG_SZ, (LPBYTE) data, (lstrlenW(data)+1) * sizeof(WCHAR) );
     }
 
@@ -1003,8 +1003,8 @@ static LRESULT WINAPI MainProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lPara
         case IDM_ABOUT:
         {
             WCHAR appname[256], other[256];
-            LoadStringW( board.hInst, IDS_APPNAME, appname, sizeof(appname)/sizeof(WCHAR) );
-            LoadStringW( board.hInst, IDS_ABOUT, other, sizeof(other)/sizeof(WCHAR) );
+            LoadStringW( board.hInst, IDS_APPNAME, appname, ARRAY_SIZE(appname));
+            LoadStringW( board.hInst, IDS_ABOUT, other, ARRAY_SIZE(other));
             ShellAboutW( hWnd, appname, other,
                          LoadImageW(board.hInst, MAKEINTRESOURCEW(IDI_WINEMINE), IMAGE_ICON, 48, 48, LR_SHARED));
             return 0;
@@ -1025,7 +1025,7 @@ int WINAPI wWinMain( HINSTANCE hInst, HINSTANCE hPrevInst, LPWSTR cmdline, int c
     HACCEL haccel;
     WCHAR appname[20];
 
-    LoadStringW( hInst, IDS_APPNAME, appname, sizeof(appname)/sizeof(WCHAR));
+    LoadStringW( hInst, IDS_APPNAME, appname, ARRAY_SIZE(appname));
 
     wc.cbSize = sizeof(wc);
     wc.style = 0;
