@@ -25,9 +25,6 @@
  *
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #define COBJMACROS
 #include <stdio.h>
 #include <assert.h>
@@ -205,7 +202,7 @@ static BOOL running_on_visible_desktop (void)
         HWINSTA wstation;
         USEROBJECTFLAGS uoflags;
 
-        wstation = (HWINSTA)pGetProcessWindowStation();
+        wstation = pGetProcessWindowStation();
         assert(pGetUserObjectInformationA(wstation, UOI_FLAGS, &uoflags, sizeof(uoflags), &len));
         return (uoflags.dwFlags & WSF_VISIBLE) != 0;
     }
@@ -295,6 +292,7 @@ static int running_elevated (void)
 /* check for native dll when running under wine */
 static BOOL is_native_dll( HMODULE module )
 {
+    static const char builtin_signature[] = "Wine builtin DLL";
     static const char fakedll_signature[] = "Wine placeholder DLL";
     const IMAGE_DOS_HEADER *dos;
 
@@ -303,8 +301,11 @@ static BOOL is_native_dll( HMODULE module )
     /* builtin dlls can't be loaded as datafile, so we must have native or fake dll */
     dos = (const IMAGE_DOS_HEADER *)((const char *)module - 1);
     if (dos->e_magic != IMAGE_DOS_SIGNATURE) return FALSE;
-    if (dos->e_lfanew >= sizeof(*dos) + sizeof(fakedll_signature) &&
-        !memcmp( dos + 1, fakedll_signature, sizeof(fakedll_signature) )) return FALSE;
+    if (dos->e_lfanew >= sizeof(*dos) + 32)
+    {
+        if (!memcmp( dos + 1, builtin_signature, sizeof(builtin_signature) )) return FALSE;
+        if (!memcmp( dos + 1, fakedll_signature, sizeof(fakedll_signature) )) return FALSE;
+    }
     return TRUE;
 }
 
@@ -1236,7 +1237,7 @@ usage (void)
 " -x DIR    Extract tests to DIR (default: .\\wct) and exit\n");
 }
 
-int main( int argc, char *argv[] )
+int __cdecl main( int argc, char *argv[] )
 {
     BOOL (WINAPI *pIsWow64Process)(HANDLE hProcess, PBOOL Wow64Process);
     char *logname = NULL, *outdir = NULL;
