@@ -38,7 +38,7 @@
 #include "thread.h"
 #include "request.h"
 
-enum debug_event_state { EVENT_QUEUED, EVENT_SENT, EVENT_CONTINUED };
+enum debug_event_state { EVENT_QUEUED, EVENT_SENT, EVENT_DELAYED, EVENT_CONTINUED };
 
 /* debug event */
 struct debug_event
@@ -84,6 +84,7 @@ static const struct object_ops debug_event_ops =
     no_link_name,                  /* link_name */
     NULL,                          /* unlink_name */
     no_open_file,                  /* open_file */
+    no_kernel_obj_list,            /* get_kernel_obj_list */
     no_close_handle,               /* close_handle */
     debug_event_destroy            /* destroy */
 };
@@ -110,6 +111,7 @@ static const struct object_ops debug_ctx_ops =
     no_link_name,                  /* link_name */
     NULL,                          /* unlink_name */
     no_open_file,                  /* open_file */
+    no_kernel_obj_list,            /* get_kernel_obj_list */
     no_close_handle,               /* close_handle */
     debug_ctx_destroy              /* destroy */
 };
@@ -147,6 +149,7 @@ static int fill_create_process_event( struct debug_event *event, const void *arg
     struct process *process = thread->process;
     struct process_dll *exe_module = get_process_exe_module( process );
     const client_ptr_t *entry = arg;
+    struct file *file;
     obj_handle_t handle;
 
     /* documented: PROCESS_VM_READ | PROCESS_VM_WRITE */
@@ -170,8 +173,12 @@ static int fill_create_process_event( struct debug_event *event, const void *arg
     event->data.create_process.unicode    = 1;
 
     /* the doc says write access too, but this doesn't seem a good idea */
-    event->data.create_process.file = open_mapping_file( debugger, exe_module->base, GENERIC_READ,
-                                                         FILE_SHARE_READ | FILE_SHARE_WRITE );
+    if ((file = get_mapping_file( process, exe_module->base, GENERIC_READ,
+                                  FILE_SHARE_READ | FILE_SHARE_WRITE )))
+    {
+        event->data.create_process.file = alloc_handle( debugger, file, GENERIC_READ, 0 );
+        release_object( file );
+    }
     return 1;
 }
 
@@ -191,8 +198,10 @@ static int fill_exit_process_event( struct debug_event *event, const void *arg )
 
 static int fill_load_dll_event( struct debug_event *event, const void *arg )
 {
+    struct process *process = event->sender->process;
     struct process *debugger = event->debugger->process;
     const struct process_dll *dll = arg;
+    struct file *file;
 
     event->data.load_dll.handle     = 0;
     event->data.load_dll.base       = dll->base;
@@ -200,8 +209,11 @@ static int fill_load_dll_event( struct debug_event *event, const void *arg )
     event->data.load_dll.dbg_size   = dll->dbg_size;
     event->data.load_dll.name       = dll->name;
     event->data.load_dll.unicode    = 1;
-    event->data.load_dll.handle     = open_mapping_file( debugger, dll->base, GENERIC_READ,
-                                                         FILE_SHARE_READ | FILE_SHARE_WRITE );
+    if ((file = get_mapping_file( process, dll->base, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE )))
+    {
+        event->data.load_dll.handle = alloc_handle( debugger, file, GENERIC_READ, 0 );
+        release_object( file );
+    }
     return 1;
 }
 
@@ -232,7 +244,7 @@ static const fill_event_func fill_debug_event[NB_DEBUG_EVENTS] =
 static void unlink_event( struct debug_ctx *debug_ctx, struct debug_event *event )
 {
     list_remove( &event->entry );
-    if (event->sender->debug_event == event) event->sender->debug_event = NULL;
+    if (event->sender->process->debug_event == event) event->sender->process->debug_event = NULL;
     release_object( event );
 }
 
@@ -244,7 +256,7 @@ static void link_event( struct debug_event *event )
     assert( debug_ctx );
     grab_object( event );
     list_add_tail( &debug_ctx->event_queue, &event->entry );
-    if (!event->sender->debug_event)
+    if (!event->sender->process->debug_event)
     {
         /* grab reference since debugger could be killed while trying to wake up */
         grab_object( debug_ctx );
@@ -253,6 +265,31 @@ static void link_event( struct debug_event *event )
     }
 }
 
+/* resume a delayed debug event already in the queue */
+static void resume_event( struct debug_event *event )
+{
+    struct debug_ctx *debug_ctx = event->debugger->debug_ctx;
+
+    assert( debug_ctx );
+    event->state = EVENT_QUEUED;
+    if (!event->sender->process->debug_event)
+    {
+        grab_object( debug_ctx );
+        wake_up( &debug_ctx->obj, 0 );
+        release_object( debug_ctx );
+    }
+}
+
+/* delay a debug event already in the queue to be replayed when thread wakes up */
+static void delay_event( struct debug_event *event )
+{
+    struct debug_ctx *debug_ctx = event->debugger->debug_ctx;
+
+    assert( debug_ctx );
+    event->state = EVENT_DELAYED;
+    if (event->sender->process->debug_event == event) event->sender->process->debug_event = NULL;
+}
+
 /* find the next event that we can send to the debugger */
 static struct debug_event *find_event_to_send( struct debug_ctx *debug_ctx )
 {
@@ -261,7 +298,8 @@ static struct debug_event *find_event_to_send( struct debug_ctx *debug_ctx )
     LIST_FOR_EACH_ENTRY( event, &debug_ctx->event_queue, struct debug_event, entry )
     {
         if (event->state == EVENT_SENT) continue;  /* already sent */
-        if (event->sender->debug_event) continue;  /* thread busy with another one */
+        if (event->state == EVENT_DELAYED) continue;  /* delayed until thread resumes */
+        if (event->sender->process->debug_event) continue;  /* process busy with another one */
         return event;
     }
     return NULL;
@@ -353,14 +391,36 @@ static int continue_debug_event( struct process *process, struct thread *thread,
     {
         struct debug_event *event;
 
+        if (status == DBG_REPLY_LATER)
+        {
+            /* if thread is suspended, delay all its events and resume process
+             * if not, reset the event for immediate replay */
+            LIST_FOR_EACH_ENTRY( event, &debug_ctx->event_queue, struct debug_event, entry )
+            {
+                if (event->sender != thread) continue;
+                if (thread->suspend)
+                {
+                    delay_event( event );
+                    resume_process( process );
+                }
+                else if (event->state == EVENT_SENT)
+                {
+                    assert( event->sender->process->debug_event == event );
+                    event->sender->process->debug_event = NULL;
+                    resume_event( event );
+                    return 1;
+                }
+            }
+            return 1;
+        }
+
         /* find the event in the queue */
         LIST_FOR_EACH_ENTRY( event, &debug_ctx->event_queue, struct debug_event, entry )
         {
             if (event->state != EVENT_SENT) continue;
             if (event->sender == thread)
             {
-                assert( event->sender->debug_event == event );
-
+                assert( event->sender->process->debug_event == event );
                 event->status = status;
                 event->state  = EVENT_CONTINUED;
                 wake_up( &event->obj, 0 );
@@ -418,6 +478,24 @@ void generate_debug_event( struct thread *thread, int code, const void *arg )
     }
 }
 
+void resume_delayed_debug_events( struct thread *thread )
+{
+    struct thread *debugger = thread->process->debugger;
+    struct debug_event *event;
+
+    if (debugger)
+    {
+        assert( debugger->debug_ctx );
+        LIST_FOR_EACH_ENTRY( event, &debugger->debug_ctx->event_queue, struct debug_event, entry )
+        {
+            if (event->sender != thread) continue;
+            if (event->state != EVENT_DELAYED) continue;
+            resume_event( event );
+            suspend_process( thread->process );
+        }
+    }
+}
+
 /* attach a process to a debugger thread and suspend it */
 static int debugger_attach( struct process *process, struct thread *debugger )
 {
@@ -504,22 +582,28 @@ void generate_startup_debug_events( struct process *process, client_ptr_t entry
     struct list *ptr;
     struct thread *thread, *first_thread = get_process_first_thread( process );
 
-    /* generate creation events */
-    LIST_FOR_EACH_ENTRY( thread, &process->thread_list, struct thread, proc_entry )
-    {
-        if (thread == first_thread)
-            generate_debug_event( thread, CREATE_PROCESS_DEBUG_EVENT, &entry );
-        else
-            generate_debug_event( thread, CREATE_THREAD_DEBUG_EVENT, NULL );
-    }
+    generate_debug_event( first_thread, CREATE_PROCESS_DEBUG_EVENT, &entry );
+    ptr = list_head( &process->dlls ); /* skip main module reported in create process event */
 
-    /* generate dll events (in loading order, i.e. reverse list order) */
-    ptr = list_tail( &process->dlls );
-    while (ptr != list_head( &process->dlls ))
+    /* generate ntdll.dll load event */
+    if (ptr && (ptr = list_next( &process->dlls, ptr )))
+    {
+        struct process_dll *dll = LIST_ENTRY( ptr, struct process_dll, entry );
+        generate_debug_event( first_thread, LOAD_DLL_DEBUG_EVENT, dll );
+    }
+
+    /* generate creation events */
+    LIST_FOR_EACH_ENTRY( thread, &process->thread_list, struct thread, proc_entry )
+    {
+        if (thread != first_thread)
+            generate_debug_event( thread, CREATE_THREAD_DEBUG_EVENT, NULL );
+    }
+
+    /* generate dll events (in loading order) */
+    while (ptr && (ptr = list_next( &process->dlls, ptr )))
     {
         struct process_dll *dll = LIST_ENTRY( ptr, struct process_dll, entry );
         generate_debug_event( first_thread, LOAD_DLL_DEBUG_EVENT, dll );
-        ptr = list_prev( &process->dlls, ptr );
     }
 }
 
@@ -576,7 +660,7 @@ DECL_HANDLER(wait_debug_event)
     {
         data_size_t size = get_reply_max_size();
         event->state = EVENT_SENT;
-        event->sender->debug_event = event;
+        event->sender->process->debug_event = event;
         reply->pid = get_process_id( event->sender->process );
         reply->tid = get_thread_id( event->sender );
         if (size > sizeof(debug_event_t)) size = sizeof(debug_event_t);
@@ -594,8 +678,17 @@ DECL_HANDLER(wait_debug_event)
 /* Continue a debug event */
 DECL_HANDLER(continue_debug_event)
 {
-    struct process *process = get_process_from_id( req->pid );
-    if (process)
+    struct process *process;
+
+    if (req->status != DBG_EXCEPTION_NOT_HANDLED &&
+        req->status != DBG_CONTINUE &&
+        req->status != DBG_REPLY_LATER)
+    {
+        set_error( STATUS_INVALID_PARAMETER );
+        return;
+    }
+
+    if ((process = get_process_from_id( req->pid )))
     {
         struct thread *thread = get_thread_from_id( req->tid );
         if (thread)
@@ -620,7 +713,6 @@ DECL_HANDLER(debug_process)
     else if (debugger_attach( process, current ))
     {
         generate_startup_debug_events( process, 0 );
-        break_process( process );
         resume_process( process );
     }
     release_object( process );
@@ -696,20 +788,6 @@ DECL_HANDLER(get_exception_status)
     }
 }
 
-/* simulate a breakpoint in a process */
-DECL_HANDLER(debug_break)
-{
-    struct process *process;
-
-    reply->self = 0;
-    if ((process = get_process_from_handle( req->handle, PROCESS_SET_INFORMATION /*FIXME*/ )))
-    {
-        if (process != current->process) break_process( process );
-        else reply->self = 1;
-        release_object( process );
-    }
-}
-
 /* set debugger kill on exit flag */
 DECL_HANDLER(set_debugger_kill_on_exit)
 {
