@@ -54,11 +54,12 @@
 
 struct file
 {
-    struct object       obj;        /* object header */
-    struct fd          *fd;         /* file descriptor for this file */
-    unsigned int        access;     /* file access (FILE_READ_DATA etc.) */
-    mode_t              mode;       /* file stat.st_mode */
-    uid_t               uid;        /* file stat.st_uid */
+    struct object       obj;            /* object header */
+    struct fd          *fd;             /* file descriptor for this file */
+    unsigned int        access;         /* file access (FILE_READ_DATA etc.) */
+    mode_t              mode;           /* file stat.st_mode */
+    uid_t               uid;            /* file stat.st_uid */
+    struct list         kernel_object;  /* list of kernel object pointers */
 };
 
 static unsigned int generic_file_map_access( unsigned int access );
@@ -70,6 +71,7 @@ static int file_set_sd( struct object *obj, const struct security_descriptor *sd
 static struct object *file_lookup_name( struct object *obj, struct unicode_str *name, unsigned int attr );
 static struct object *file_open_file( struct object *obj, unsigned int access,
                                       unsigned int sharing, unsigned int options );
+static struct list *file_get_kernel_obj_list( struct object *obj );
 static void file_destroy( struct object *obj );
 
 static int file_get_poll_events( struct fd *fd );
@@ -94,7 +96,7 @@ static const struct object_ops file_ops =
     no_link_name,                 /* link_name */
     NULL,                         /* unlink_name */
     file_open_file,               /* open_file */
-    no_kernel_obj_list,           /* get_kernel_obj_list */
+    file_get_kernel_obj_list,     /* get_kernel_obj_list */
     fd_close_handle,              /* close_handle */
     file_destroy                  /* destroy */
 };
@@ -136,6 +138,7 @@ struct file *create_file_for_fd( int fd, unsigned int access, unsigned int shari
 
     file->mode = st.st_mode;
     file->access = default_fd_map_access( &file->obj, access );
+    list_init( &file->kernel_object );
     if (!(file->fd = create_anonymous_fd( &file_fd_ops, fd, &file->obj,
                                           FILE_SYNCHRONOUS_IO_NONALERT )))
     {
@@ -162,6 +165,7 @@ struct file *create_file_for_fd_obj( struct fd *fd, unsigned int access, unsigne
     {
         file->mode = st.st_mode;
         file->access = default_fd_map_access( &file->obj, access );
+        list_init( &file->kernel_object );
         if (!(file->fd = dup_fd_object( fd, access, sharing, FILE_SYNCHRONOUS_IO_NONALERT )))
         {
             release_object( file );
@@ -181,11 +185,18 @@ static struct object *create_file_obj( struct fd *fd, unsigned int access, mode_
     file->mode    = mode;
     file->uid     = ~(uid_t)0;
     file->fd      = fd;
+    list_init( &file->kernel_object );
     grab_object( fd );
     set_fd_user( fd, &file_fd_ops, &file->obj );
     return &file->obj;
 }
 
+int is_file_executable( const char *name )
+{
+    int len = strlen( name );
+    return len >= 4 && (!strcasecmp( name + len - 4, ".exe") || !strcasecmp( name + len - 4, ".com" ));
+}
+
 static struct object *create_file( struct fd *root, const char *nameptr, data_size_t len,
                                    unsigned int access, unsigned int sharing, int create,
                                    unsigned int options, unsigned int attrs,
@@ -231,8 +242,7 @@ static struct object *create_file( struct fd *root, const char *nameptr, data_si
     else
         mode = (attrs & FILE_ATTRIBUTE_READONLY) ? 0444 : 0666;
 
-    if (len >= 4 &&
-        (!strcasecmp( name + len - 4, ".exe" ) || !strcasecmp( name + len - 4, ".com" )))
+    if (is_file_executable( name ))
     {
         if (mode & S_IRUSR)
             mode |= S_IXUSR;
@@ -631,6 +641,12 @@ static struct object *file_open_file( struct object *obj, unsigned int access,
     return new_file;
 }
 
+static struct list *file_get_kernel_obj_list( struct object *obj )
+{
+    struct file *file = (struct file *)obj;
+    return &file->kernel_object;
+}
+
 static void file_destroy( struct object *obj )
 {
     struct file *file = (struct file *)obj;
@@ -667,6 +683,7 @@ void file_set_error(void)
     case ENODEV:    set_error( STATUS_NO_SUCH_DEVICE ); break;
     case ENXIO:     set_error( STATUS_NO_SUCH_DEVICE ); break;
     case EXDEV:     set_error( STATUS_NOT_SAME_DEVICE ); break;
+    case ELOOP:     set_error( STATUS_REPARSE_POINT_NOT_RESOLVED ); break;
 #ifdef EOVERFLOW
     case EOVERFLOW: set_error( STATUS_INVALID_PARAMETER ); break;
 #endif
