@@ -48,7 +48,6 @@
 #include "security.h"
 
 #include "winternl.h"
-#include "wine/library.h"
 
 struct notify
 {
@@ -178,8 +177,7 @@ static const struct object_ops key_ops =
 
 static inline int is_wow6432node( const WCHAR *name, unsigned int len )
 {
-    return (len == sizeof(wow6432node) &&
-            !memicmpW( name, wow6432node, ARRAY_SIZE( wow6432node )));
+    return (len == sizeof(wow6432node) && !memicmp_strW( name, wow6432node, sizeof( wow6432node )));
 }
 
 /*
@@ -199,7 +197,7 @@ static void dump_path( const struct key *key, const struct key *base, FILE *f )
         dump_path( key->parent, base, f );
         fprintf( f, "\\\\" );
     }
-    dump_strW( key->name, key->namelen / sizeof(WCHAR), f, "[]" );
+    dump_strW( key->name, key->namelen, f, "[]" );
 }
 
 /* dump a value to a text file */
@@ -211,7 +209,7 @@ static void dump_value( const struct key_value *value, FILE *f )
     if (value->namelen)
     {
         fputc( '\"', f );
-        count = 1 + dump_strW( value->name, value->namelen / sizeof(WCHAR), f, "\"\"" );
+        count = 1 + dump_strW( value->name, value->namelen, f, "\"\"" );
         count += fprintf( f, "\"=" );
     }
     else count = fprintf( f, "@=" );
@@ -227,7 +225,7 @@ static void dump_value( const struct key_value *value, FILE *f )
         if (((WCHAR *)value->data)[value->len / sizeof(WCHAR) - 1]) break;
         if (value->type != REG_SZ) fprintf( f, "str(%x):", value->type );
         fputc( '\"', f );
-        dump_strW( (WCHAR *)value->data, value->len / sizeof(WCHAR), f, "\"\"" );
+        dump_strW( (WCHAR *)value->data, value->len, f, "\"\"" );
         fprintf( f, "\"\n" );
         return;
 
@@ -273,7 +271,7 @@ static void save_subkeys( const struct key *key, const struct key *base, FILE *f
         if (key->class)
         {
             fprintf( f, "#class=\"" );
-            dump_strW( key->class, key->classlen / sizeof(WCHAR), f, "\"\"" );
+            dump_strW( key->class, key->classlen, f, "\"\"" );
             fprintf( f, "\"\n" );
         }
         if (key->flags & KEY_SYMLINK) fputs( "#link\n", f );
@@ -439,8 +437,7 @@ static inline void get_req_path( struct unicode_str *str, int skip_root )
     str->str = get_req_data();
     str->len = (get_req_data_size() / sizeof(WCHAR)) * sizeof(WCHAR);
 
-    if (skip_root && str->len >= sizeof(root_name) &&
-        !memicmpW( str->str, root_name, ARRAY_SIZE( root_name )))
+    if (skip_root && str->len >= sizeof(root_name) && !memicmp_strW( str->str, root_name, sizeof(root_name) ))
     {
         str->str += ARRAY_SIZE( root_name );
         str->len -= sizeof(root_name);
@@ -648,7 +645,7 @@ static struct key *find_subkey( const struct key *key, const struct unicode_str
     {
         i = (min + max) / 2;
         len = min( key->subkeys[i]->namelen, name->len );
-        res = memicmpW( key->subkeys[i]->name, name->str, len / sizeof(WCHAR) );
+        res = memicmp_strW( key->subkeys[i]->name, name->str, len );
         if (!res) res = key->subkeys[i]->namelen - name->len;
         if (!res)
         {
@@ -692,7 +689,7 @@ static struct key *follow_symlink( struct key *key, int iteration )
     path.str = value->data;
     path.len = (value->len / sizeof(WCHAR)) * sizeof(WCHAR);
     if (path.len <= sizeof(root_name)) return NULL;
-    if (memicmpW( path.str, root_name, ARRAY_SIZE( root_name ))) return NULL;
+    if (memicmp_strW( path.str, root_name, sizeof(root_name) )) return NULL;
     path.str += ARRAY_SIZE( root_name );
     path.len -= sizeof(root_name);
 
@@ -1055,7 +1052,7 @@ static struct key_value *find_value( const struct key *key, const struct unicode
     {
         i = (min + max) / 2;
         len = min( key->values[i].namelen, name->len );
-        res = memicmpW( key->values[i].name, name->str, len / sizeof(WCHAR) );
+        res = memicmp_strW( key->values[i].name, name->str, len );
         if (!res) res = key->values[i].namelen - name->len;
         if (!res)
         {
@@ -1117,7 +1114,7 @@ static void set_value( struct key *key, const struct unicode_str *name,
     if (key->flags & KEY_SYMLINK)
     {
         if (type != REG_LINK || name->len != symlink_str.len ||
-            memicmpW( name->str, symlink_str.str, name->len / sizeof(WCHAR) ))
+            memicmp_strW( name->str, symlink_str.str, name->len ))
         {
             set_error( STATUS_ACCESS_DENIED );
             return;
@@ -1615,7 +1612,7 @@ static int get_prefix_len( struct key *key, const char *name, struct file_load_i
     len = (p - info->tmp) * sizeof(WCHAR);
     for (res = 1; key != root_key; res++)
     {
-        if (len == key->namelen && !memicmpW( info->tmp, key->name, len / sizeof(WCHAR) )) break;
+        if (len == key->namelen && !memicmp_strW( info->tmp, key->name, len )) break;
         key = key->parent;
     }
     if (key == root_key) res = 0;  /* no matching name */
@@ -1741,25 +1738,16 @@ static int load_init_registry_from_file( const char *filename, struct key *key )
 
 static WCHAR *format_user_registry_path( const SID *sid, struct unicode_str *path )
 {
-    static const WCHAR prefixW[] = {'U','s','e','r','\\','S',0};
-    static const WCHAR formatW[] = {'-','%','u',0};
-    WCHAR buffer[7 + 10 + 10 + 10 * SID_MAX_SUB_AUTHORITIES];
-    WCHAR *p = buffer;
+    char buffer[7 + 11 + 11 + 11 * SID_MAX_SUB_AUTHORITIES], *p = buffer;
     unsigned int i;
 
-    strcpyW( p, prefixW );
-    p += strlenW( prefixW );
-    p += sprintfW( p, formatW, sid->Revision );
-    p += sprintfW( p, formatW, MAKELONG( MAKEWORD( sid->IdentifierAuthority.Value[5],
-                                                   sid->IdentifierAuthority.Value[4] ),
-                                         MAKEWORD( sid->IdentifierAuthority.Value[3],
-                                                   sid->IdentifierAuthority.Value[2] )));
-    for (i = 0; i < sid->SubAuthorityCount; i++)
-        p += sprintfW( p, formatW, sid->SubAuthority[i] );
-
-    path->len = (p - buffer) * sizeof(WCHAR);
-    path->str = p = memdup( buffer, path->len );
-    return p;
+    p += sprintf( p, "User\\S-%u-%u", sid->Revision,
+                  MAKELONG( MAKEWORD( sid->IdentifierAuthority.Value[5],
+                                      sid->IdentifierAuthority.Value[4] ),
+                            MAKEWORD( sid->IdentifierAuthority.Value[3],
+                                      sid->IdentifierAuthority.Value[2] )));
+    for (i = 0; i < sid->SubAuthorityCount; i++) p += sprintf( p, "-%u", sid->SubAuthority[i] );
+    return ascii_to_unicode_str( buffer, path );
 }
 
 /* get the cpu architectures that can be supported in the current prefix */
@@ -2061,7 +2049,7 @@ DECL_HANDLER(create_key)
     class.len = (class.len / sizeof(WCHAR)) * sizeof(WCHAR);
 
     if (!objattr->rootdir && name.len >= sizeof(root_name) &&
-        !memicmpW( name.str, root_name, ARRAY_SIZE( root_name )))
+        !memicmp_strW( name.str, root_name, sizeof(root_name) ))
     {
         name.str += ARRAY_SIZE( root_name );
         name.len -= sizeof(root_name);
@@ -2219,7 +2207,7 @@ DECL_HANDLER(load_registry)
     }
 
     if (!objattr->rootdir && name.len >= sizeof(root_name) &&
-        !memicmpW( name.str, root_name, ARRAY_SIZE( root_name )))
+        !memicmp_strW( name.str, root_name, sizeof(root_name) ))
     {
         name.str += ARRAY_SIZE( root_name );
         name.len -= sizeof(root_name);
