@@ -38,7 +38,6 @@
 #include "user.h"
 #include "file.h"
 #include "security.h"
-#include "wine/unicode.h"
 
 
 static struct list winstation_list = LIST_INIT(winstation_list);
@@ -163,7 +162,7 @@ static struct object *winstation_lookup_name( struct object *obj, struct unicode
 
     if (!name) return NULL;  /* open the winstation itself */
 
-    if (memchrW( name->str, '\\', name->len / sizeof(WCHAR) ))  /* no backslash allowed in name */
+    if (get_path_element( name->str, name->len ) < name->len)  /* no backslash allowed in name */
     {
         set_error( STATUS_OBJECT_PATH_SYNTAX_BAD );
         return NULL;
@@ -262,7 +261,7 @@ static int desktop_link_name( struct object *obj, struct object_name *name, stru
         set_error( STATUS_OBJECT_TYPE_MISMATCH );
         return 0;
     }
-    if (memchrW( name->name, '\\', name->len / sizeof(WCHAR) ))  /* no backslash allowed in name */
+    if (get_path_element( name->name, name->len ) < name->len)  /* no backslash allowed in name */
     {
         set_error( STATUS_OBJECT_PATH_SYNTAX_BAD );
         return 0;
@@ -372,7 +371,8 @@ void set_process_default_desktop( struct process *process, struct desktop *deskt
 }
 
 /* connect a process to its window station */
-void connect_process_winstation( struct process *process, struct thread *parent )
+void connect_process_winstation( struct process *process, struct thread *parent_thread,
+                                 struct process *parent_process )
 {
     struct winstation *winstation = NULL;
     struct desktop *desktop = NULL;
@@ -383,9 +383,9 @@ void connect_process_winstation( struct process *process, struct thread *parent
     {
         winstation = (struct winstation *)get_handle_obj( process, handle, 0, &winstation_ops );
     }
-    else if (parent && parent->process->winstation)
+    else if (parent_process->winstation)
     {
-        handle = duplicate_handle( parent->process, parent->process->winstation,
+        handle = duplicate_handle( parent_process, parent_process->winstation,
                                    process, 0, 0, DUP_HANDLE_SAME_ACCESS );
         winstation = (struct winstation *)get_handle_obj( process, handle, 0, &winstation_ops );
     }
@@ -397,14 +397,21 @@ void connect_process_winstation( struct process *process, struct thread *parent
         desktop = get_desktop_obj( process, handle, 0 );
         if (!desktop || desktop->winstation != winstation) goto done;
     }
-    else if (parent && parent->desktop)
+    else
     {
-        desktop = get_desktop_obj( parent->process, parent->desktop, 0 );
-        if (!desktop || desktop->winstation != winstation) goto done;
-        handle = duplicate_handle( parent->process, parent->desktop,
-                                   process, 0, 0, DUP_HANDLE_SAME_ACCESS );
-    }
+        if (parent_thread && parent_thread->desktop)
+            handle = parent_thread->desktop;
+        else if (parent_process->desktop)
+            handle = parent_process->desktop;
+        else
+            goto done;
 
+        desktop = get_desktop_obj( parent_process, handle, 0 );
+
+        if (!desktop || desktop->winstation != winstation) goto done;
+
+        handle = duplicate_handle( parent_process, handle, process, 0, 0, DUP_HANDLE_SAME_ACCESS );
+    }
     if (handle) set_process_default_desktop( process, desktop, handle );
 
 done:
