@@ -113,6 +113,8 @@ enum exec_mode_values
     MODE_EXE,
     MODE_DEF,
     MODE_IMPLIB,
+    MODE_STATICLIB,
+    MODE_BUILTIN,
     MODE_RESOURCES
 };
 
@@ -196,7 +198,7 @@ static void set_subsystem( const char *subsystem, DLLSPEC *spec )
 static void set_target( const char *target )
 {
     unsigned int i;
-    char *p, *platform, *spec = xstrdup( target );
+    char *p, *spec = xstrdup( target );
 
     /* target specification is in the form CPU-MANUFACTURER-OS or CPU-MANUFACTURER-KERNEL-OS */
 
@@ -212,13 +214,11 @@ static void set_target( const char *target )
         cpu = get_cpu_from_name( spec );
         if (cpu == -1) fatal_error( "Unrecognized CPU '%s'\n", spec );
         target_cpu = cpu;
-        platform = p;
-        if ((p = strrchr( p, '-' ))) platform = p + 1;
     }
     else if (!strcmp( spec, "mingw32" ))
     {
         target_cpu = CPU_x86;
-        platform = spec;
+        p = spec;
     }
     else
         fatal_error( "Invalid target specification '%s'\n", target );
@@ -226,13 +226,18 @@ static void set_target( const char *target )
     /* get the OS part */
 
     target_platform = PLATFORM_UNSPECIFIED;  /* default value */
-    for (i = 0; i < ARRAY_SIZE(platform_names); i++)
+    for (;;)
     {
-        if (!strncmp( platform_names[i].name, platform, strlen(platform_names[i].name) ))
+        for (i = 0; i < ARRAY_SIZE(platform_names); i++)
         {
-            target_platform = platform_names[i].platform;
-            break;
+            if (!strncmp( platform_names[i].name, p, strlen(platform_names[i].name) ))
+            {
+                target_platform = platform_names[i].platform;
+                break;
+            }
         }
+        if (target_platform != PLATFORM_UNSPECIFIED || !(p = strchr( p, '-' ))) break;
+        p++;
     }
 
     free( spec );
@@ -291,11 +296,13 @@ static const char usage_str[] =
 "       --version             Print the version and exit\n"
 "   -w, --warnings            Turn on warnings\n"
 "\nMode options:\n"
-"       --dll                 Build a .c file from a .spec or .def file\n"
+"       --dll                 Build a library from a .spec file and object files\n"
 "       --def                 Build a .def file from a .spec file\n"
-"       --exe                 Build a .c file for an executable\n"
+"       --exe                 Build an executable from object files\n"
 "       --implib              Build an import library\n"
-"       --resources           Build a .o file for the resource files\n\n"
+"       --staticlib           Build a static library\n"
+"       --builtin             Mark a library as a Wine builtin\n"
+"       --resources           Build a .o or .res file for the resource files\n\n"
 "The mode options are mutually exclusive; you must specify one and only one.\n\n";
 
 enum long_options_values
@@ -304,6 +311,7 @@ enum long_options_values
     LONG_OPT_DEF,
     LONG_OPT_EXE,
     LONG_OPT_IMPLIB,
+    LONG_OPT_BUILTIN,
     LONG_OPT_ASCMD,
     LONG_OPT_CCCMD,
     LONG_OPT_EXTERNAL_SYMS,
@@ -314,6 +322,7 @@ enum long_options_values
     LONG_OPT_NXCOMPAT,
     LONG_OPT_RESOURCES,
     LONG_OPT_SAVE_TEMPS,
+    LONG_OPT_STATICLIB,
     LONG_OPT_SUBSYSTEM,
     LONG_OPT_VERSION
 };
@@ -326,6 +335,8 @@ static const struct option long_options[] =
     { "def",           0, 0, LONG_OPT_DEF },
     { "exe",           0, 0, LONG_OPT_EXE },
     { "implib",        0, 0, LONG_OPT_IMPLIB },
+    { "staticlib",     0, 0, LONG_OPT_STATICLIB },
+    { "builtin",       0, 0, LONG_OPT_BUILTIN },
     { "as-cmd",        1, 0, LONG_OPT_ASCMD },
     { "cc-cmd",        1, 0, LONG_OPT_CCCMD },
     { "external-symbols", 0, 0, LONG_OPT_EXTERNAL_SYMS },
@@ -494,6 +505,12 @@ static char **parse_options( int argc, char **argv, DLLSPEC *spec )
         case LONG_OPT_IMPLIB:
             set_exec_mode( MODE_IMPLIB );
             break;
+        case LONG_OPT_STATICLIB:
+            set_exec_mode( MODE_STATICLIB );
+            break;
+        case LONG_OPT_BUILTIN:
+            set_exec_mode( MODE_BUILTIN );
+            break;
         case LONG_OPT_ASCMD:
             as_command = strarray_fromstring( optarg, " " );
             break;
@@ -675,7 +692,14 @@ int main(int argc, char **argv)
     case MODE_IMPLIB:
         if (!spec_file_name) fatal_error( "missing .spec file\n" );
         if (!parse_input_file( spec )) break;
-        output_import_lib( spec, argv );
+        output_static_lib( spec, argv );
+        break;
+    case MODE_STATICLIB:
+        output_static_lib( NULL, argv );
+        break;
+    case MODE_BUILTIN:
+        if (!argv[0]) fatal_error( "missing file argument for --builtin option\n" );
+        make_builtin_files( argv );
         break;
     case MODE_RESOURCES:
         load_resources( argv, spec );
